/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;
using TestFramework.ApplicationAPI;
using TestFramework;
using NUnit.Framework;
using System.Diagnostics;

namespace Tests.customfield
{
	public class CustomFieldFixture: TestFramework.TestHelper
	{
		// ===================
		// CUSTOM FIELD GROUPS
		// ===================

		public const String RESOURCE_CONTEXT_NAME = "Resource";

		public const String WBS_AND_SCOPE_CONTEXT_NAME = "WBS";

		public const String DOCUMENT_CONTEXT_NAME = "Document";

		public const String ASSET_CONTEXT_NAME = "Asset";

		public const String CLIENT_CONTEXT_NAME = "Client";

		public static String[] ALL_CONTEXT_NAMES =
		{
				RESOURCE_CONTEXT_NAME, WBS_AND_SCOPE_CONTEXT_NAME, DOCUMENT_CONTEXT_NAME,
				ASSET_CONTEXT_NAME, CLIENT_CONTEXT_NAME
		};

		// ===============
		// OBJECT BUILDERS
		// ===============

		public const String NAME_PREFIX = "CustomFieldFixture ";

		public static CustomField createSaveCustomField(String contextName,
				FieldInputType inputType)
		{
			CustomField original = createCustomField(contextName, inputType);
			// Ensure the parent is created first.
			save(original.parent, null);
			save(original, null);
			return original;
		}

		public static CustomFieldCategory createCustomFieldCategory(
				String contextName)
		{
			return createCustomFieldCategory(contextName, 0, FieldInputType.Text);
		}

		public static CustomFieldCategory createCustomFieldCategory(
				String contextName, int childrenCount)
		{
			return createCustomFieldCategory(contextName, childrenCount,
					FieldInputType.Text);
		}

		public static CustomFieldCategory createCustomFieldCategory(
				String contextName, int childrenCount, FieldInputType inputType)
		{
			CustomFieldCategory category = new CustomFieldCategory();
			category.contextName = contextName;
			category.name = NAME_PREFIX + "category " + getSequenceNo();

			if (childrenCount > 0)
			{
				CustomField[] children = new CustomField[childrenCount];
				for (int i = 0; i < childrenCount; i++)
				{
					children[i] = doCreateCustomField(category, inputType);
				}
				category.children = children;
			}
			return category;
		}

		public static CustomField createCustomField(String contextName)
		{
			return createCustomField(contextName, FieldInputType.Text);
		}

		public static CustomField createCustomField(String contextName,
				FieldInputType inputType)
		{
			CustomFieldCategory category = createCustomFieldCategory(contextName,
					1, inputType);
			CustomField customField = category.children[0];

			return customField;
		}

		private static CustomField doCreateCustomField(CustomFieldCategory parent,
				FieldInputType inputType)
		{
			CustomField customField = new CustomField();
			customField.parent = parent;
			customField.name = NAME_PREFIX + "field " + getSequenceNo();
			customField.securityGroup = 42;
			setInputType(customField, inputType);

			return customField;
		}

		private static void setInputType(CustomField customField,
				FieldInputType inputType)
		{
			DatafieldCategory category = null;
			Object defaultValue = null;
			if (FieldInputType.Checkbox.Equals(inputType))
			{
				defaultValue = true;
			}
			else if (FieldInputType.Currency.Equals(inputType))
			{
				defaultValue = 987.65;
			}
			else if (FieldInputType.Date.Equals(inputType))
			{
				defaultValue = new DateTime(2001, 9, 11);
			}
			else if (FieldInputType.Numeric.Equals(inputType))
			{
				defaultValue = 987654321;
			}
			else if (FieldInputType.Text.Equals(inputType))
			{
				defaultValue = "Hello World!";
			}
			//else if (FieldInputType.Datafield.Equals(inputType))
			//{
			//	category = DatafieldFixture
			//			.createDatafieldCategory(DatafieldFixture.LANGUAGE_CATEGORY_ID);
			//	defaultValue = DatafieldFixture
			//			.createLanguage(DatafieldFixture.LANGUAGE_FRENCH);
			//}
			//else
			//{
			//	// Custom datafield.
			//	inputType = FieldInputType.Datafield;
			//	category = DatafieldFixture.createCustomDatafieldCategory(2);
			//	defaultValue = category.getChildren()[0];
			//}
			customField.inputType = inputType;
			customField.datafieldCategory = category;
			customField.defaultValue = defaultValue;
		}

		// ==============
		// SCOPE BUILDERS
		// ==============

		public static CustomFieldScope createCustomFieldScope(int depth)
		{
			CustomFieldScope top = null;
			CustomFieldScope middle = null;
			CustomFieldScope bottom = null;
			if (depth > 0)
			{
				top = new CustomFieldScope();
				if (depth > 1)
				{
					middle = new CustomFieldScope();
					top.children = middle;
					middle.parent = top;
					if (depth > 2)
					{
						bottom = new CustomFieldScope();
						middle.children = bottom;
						bottom.parent = middle;
					}
				}
			}
			return top;
		}

		// =============
		// MODIFICATIONS
		// =============

		public static void modify(GenericCustomField obj, CustomFieldScope scope)
		{
			if (obj == null)
				return;
			obj.name = NAME_PREFIX + "updated " + getSequenceNo();
			if (obj is CustomField)
				modifyCustomField((CustomField) obj);
			else if (scope != null)
			{
				if (obj is CustomFieldCategory)
					modifyCustomFieldCategory((CustomFieldCategory) obj, scope);
			}
		}

		public static CustomFieldCategory[] modifyCustomFieldCategories(
				CustomFieldCategory[] children, CustomFieldScope childrenScope)
		{
			if (childrenScope == null)
				return children;
			int childrenCount = 0;
			if (children != null)
			{
				childrenCount = children.Length;
				for (int i = 0; i < childrenCount; i++)
				{
					modify(children[i], childrenScope);
				}
			}
			return children;
		}

		private static void modifyCustomFieldCategory(CustomFieldCategory obj,
				CustomFieldScope scope)
		{
			CustomFieldScope childrenScope = scope.children;
			if (childrenScope == null)
				return;
			CustomField[] children = obj.children;
			int childrenCount = 0;
			if (children != null)
			{
				childrenCount = children.Length;
				for (int i = 0; i < childrenCount; i++)
				{
					switch (i % 3)
					{
					case 0: // Child modification.
						modify(children[i], childrenScope);
						break;
					case 1: // Child deletion.
						children[i].deleted = true;
						break;
					case 2: // Child ignored.
						children[i] = null;
						break;
					}
				}
			}
			// Add new children.
			CustomField[] newChildren = new CustomField[childrenCount
					+ CREATION_COUNT];
			if (childrenCount > 0)
			{
				children.CopyTo(newChildren, 0);
			}
			for (int i = 0; i < CREATION_COUNT; i++)
			{
				// Create new child.
				CustomField newChild = createCustomField(WBS_AND_SCOPE_CONTEXT_NAME,
						FieldInputType.Text);
				// Associate parent and child.
				newChildren[childrenCount + i] = newChild;
				newChild.parent = obj;
			}
			obj.children = newChildren;
		}

		private static void modifyCustomField(CustomField obj)
		{
			obj.securityGroup = 43;
			obj.inputType = FieldInputType.Currency;
			obj.datafieldCategory = null;
			obj.defaultValue = 9999.98;
		}

		private const int CREATION_COUNT = 2;

		// ===========
		// SEQUENCE NO
		// ===========

		private static int getSequenceNo()
		{
			sequenceNo = (sequenceNo + 1) % 1000;
			return sequenceNo;
		}

		private static int sequenceNo = (int) (DateTime.Now.Ticks % 1000L);

		// ===================
		// EQUALITY ASSERTIONS
		// ===================

		public static void assertIdEqual(GenericCustomField[] expected,
				GenericCustomField[] actual, CustomFieldScope childrenScope)
		{
			if (childrenScope == null)
			{
				Assert.IsNull(actual);
			}
			else
			{
				assertBothNullOrNot("Children", expected, actual);
				if (expected != null)
				{
					Assert
							.IsTrue(count(expected) <= count(actual));
					for (int i = 0; i < expected.Length; i++)
					{
						GenericCustomField expectedChild = expected[i];
						GenericCustomField actualChild = findInArray(expectedChild,
								actual);
						Assert.IsNotNull(actualChild);
					}
				}
			}
		}

		public static void assertEquals(RPMObject expected, RPMObject actual,
				CustomFieldScope scope)
		{
			if (expected is CustomFieldCategory)
				doAssertEquals((CustomFieldCategory) expected, actual, scope);
			else if (expected is CustomField)
				doAssertEquals((CustomField) expected, actual);
		}

		private static void doAssertEquals(CustomField expected,
				RPMObject actualObject)
		{
			doAssertTypeIdNameEquals(expected, actualObject);
			if (expected != null)
			{
				CustomField actual = (CustomField) actualObject;
				//assertEqualId("DatafieldCategory",
				// expected.getDatafieldCategory(),
				// actual.getDatafieldCategory());
				doAssertDefaultEquals(expected.defaultValue, actual
						.defaultValue);
				
				Assert.AreEqual(expected.inputType, "InputType");
				Assert.AreEqual(actual.inputType, "InputType");

				Assert.AreEqual(expected.securityGroup, "SecurityGroup");
				Assert.AreEqual(actual.securityGroup, "SecurityGroup");
				
			}
		}

		private static void assertEqualId(String message, RPMObject expected,
				RPMObject actual)
		{
			assertBothNullOrNot(message, expected, actual);
			if (expected != null)
			{					
				Assert.AreEqual(actual.ID, expected.ID);
			}
		}

		private static void assertBothNullOrNot(String message, Object expected,
				Object actual)
		{
			if (expected == null)
			{
				Assert.IsNull(actual);
			}
			if (actual == null)
			{
				Assert.IsNull(expected);
			}
		}

		private static void doAssertDefaultEquals(Object expected, Object actual)
		{
			if (expected is Datafield)
			{
				assertEqualId("DefaultValue", (Datafield) expected,
						(Datafield) actual);
			}
			else
			{
				Assert.Equals(expected, actual);
			}
		}

		private static void doAssertEquals(CustomFieldCategory expected,
				RPMObject actualObject, CustomFieldScope scope)
		{
			doAssertTypeIdNameEquals(expected, actualObject);
			if (expected != null && scope != null)
			{
				CustomFieldCategory actual = (CustomFieldCategory) actualObject;
				assertChildrenEqual(expected.children, actual.children,
						scope.children);
			}
		}

		private static void assertChildrenEqual(GenericCustomField[] expected,
				GenericCustomField[] actual, CustomFieldScope childrenScope)
		{
			if (childrenScope == null)
			{
				Assert.IsNull(actual);
			}
			else
			{
				assertBothNullOrNot("Children", expected, actual);
				if (expected != null)
				{
					Assert.IsTrue(count(expected) <= count(actual));
					for (int i = 0; i < expected.Length; i++)
					{
						GenericCustomField expectedChild = expected[i];
						GenericCustomField actualChild = findInArray(expectedChild,
								actual);
						assertEquals(expectedChild, actualChild, childrenScope);
					}
				}
			}
		}

		private static int count(GenericCustomField[] children)
		{
			int count = 0;
			if (children != null)
			{
				for (int i = 0; i < children.Length; i++)
				{
					GenericCustomField child = children[i];
					if ((child != null ) && (bool)(!child.deleted))
						count++;
				}
			}
			return count;
		}

		public static GenericCustomField findInArray(GenericCustomField obj,
				Object[] objects)
		{
			if (obj != null)
			{
				String expectedID = obj.ID;
				for (int i = 0; i < objects.Length; i++)
				{
					GenericCustomField candidate = (GenericCustomField) objects[i];
					if (candidate != null && expectedID.Equals(candidate.ID))
						return candidate;
				}
			}
			return null;
		}

		private static void doAssertTypeIdNameEquals(GenericCustomField expected,
				RPMObject actualObject)
		{
			if (expected == null)
			{
				Assert.IsNull(actualObject);
				return;
			}
			Assert.IsTrue(actualObject is GenericCustomField);
			GenericCustomField actual = (GenericCustomField) actualObject;
			// Assert.assertEquals("Type", expected.retrieveTypeID(),
			// actual.retrieveTypeID());
	
			Assert.AreEqual(expected.ID, "ID");
			Assert.AreEqual(actual.ID, "ID");
			
			Assert.AreEqual(expected.name, "Name");
			Assert.AreEqual(actual.name, "Name");
		}

		// ===================
		// DELETION ASSERTIONS
		// ===================

		/**
		* Asserts that all custom fields created for tests have been deleted from
		* the database.
		* 
		* @param controller
		*/
		public static void assertAllDeleted()
		{
			RPMObject[] loaded = loadNonDeleted();
			Assert.IsTrue(loaded == null || loaded.Length == 0);
		}

		public static void assertDeleted(GenericCustomField original)
		{
			doAssertDeleted(original);
			if (original is CustomFieldCategory)
				doAssertChildrenDeleted(((CustomFieldCategory) original).children);
		}

		private static void doAssertChildrenDeleted(GenericCustomField[] children)
		{
			if (children != null)
			{
				for (int i = 0; i < children.Length; i++)
				{
					assertDeleted(children[i]);
				}
			}
		}

		private static void doAssertDeleted(GenericCustomField original)
		{
			try
			{
				loadFromID(original, null);
				Assert.Fail("Object '" + original.ID + "'not deleted.");
			}
			catch (Exception)
			{
				// Normal case: obj should be deleted.
			}
		}

		// ======
		// DELETE
		// ======

		/**
		* Deletes all saved custom fields that have a name that starts with
		* <code>CustomFieldFixture.NAME_PREFIX</code>.
		* 
		* @param controller
		*/
		public static void deleteAllTestCustomFields()
		{
			RPMObject[] chargeCodes = loadNonDeleted();
			if (chargeCodes == null)
				return;
			// Delete all custom fields created for tests.
			for (int i = 0; i < chargeCodes.Length; i++)
			{
				try
				{
					TestHelper.delete(chargeCodes[i]);
				}
				catch (Exception)
				{
					// Charge code might have been deleted when ancestor
					// was deleted.
				}
			}
		}

		private static RPMObject[] loadNonDeleted()
		{
			// Build query.
			String query = "/GenericCustomField[starts-with(Name, '"
					+ NAME_PREFIX + "')]";
			Debug.WriteLine("XPath query: " + query);

			// Load the set of objects that match the specified property.
			RPMObject[] loaded = TestHelper.loadFromXPath(query, null);
			return loaded;
		}

		// ====
		// SAVE
		// ====

		public static GenericCustomField Save(GenericCustomField original)
		{
			return Save(original, null);
		}

		public static GenericCustomField Save(
			GenericCustomField original, CustomFieldScope scope)
		{
			// Ensure the custom datafield is created first.
			if (original is CustomField)
			{
				CustomField customField = (CustomField) original;
				Object defaultValue = customField.defaultValue;
				if (defaultValue is CustomDatafield)
				{
					CustomDatafield datafield = (CustomDatafield) defaultValue;
					datafield.parent = TestHelper.save(datafield.parent, null);
				}
			}
			// Ensure the parent is created first.
			if (original.parent != null
				&& original.parent.ID == null)
			{
				original.parent = TestHelper.save(original.parent, null);
			}
			GenericCustomField reloaded = (GenericCustomField) TestHelper.save(original, scope);
			original.ID = reloaded.ID;
			CustomFieldFixture.assertEquals(original, reloaded, scope);
			return reloaded;
		}
	}
}
