/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.framework;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.Properties;

import com.ibm.rpm.interfaces.Application;
import com.ibm.rpm.interfaces.ApplicationServiceLocator;
import com.ibm.rpm.interfaces.Authenticate;
import com.ibm.rpm.interfaces.AuthenticateServiceLocator;

public abstract class SampleTestController
{
    protected static Application application = null;

    protected static Authenticate authenticate = null;

    protected static String sessionID = null;

    private static Properties PROPERTY_MAP;

    public static final String PROPERTY_FILE_NAME = "test.properties";

    public SampleTestController()
    {
        try
        {
            login();

            this.runSample();

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            try
            {
                authenticate.logout(sessionID);
            }
            catch (RPMException e)
            {
                e.printStackTrace();
            }
            catch (RemoteException e)
            {
                e.printStackTrace();
            }
        }
    }

    protected static void printRPMExceptions(Result result)
    {
        if (result != null)
        {
            RPMException warnings[] = result.getWarnings();
            RPMException errors[] = result.getErrors();

            if (warnings != null)
            {
                for (int i = 0; i < warnings.length; i++)
                {
                    System.out.println("WARNING: "
                            + warnings[i].getRpmMessage());
                }
            }
            if (errors != null)
            {
                for (int i = 0; i < errors.length; i++)
                {
                    System.out.println("ERROR: " + errors[i].getRpmMessage());
                }
            }
        }
    }

    protected static void login() throws Exception
    {
        String endpointURL = getProperty("rpm.test.middleware.url");
        if (endpointURL == null)
        {
            endpointURL = "put URL here";
        }

        String userName = getProperty("rpm.test.user.name");
        if (userName == null)
        {
            userName = "put username here";
        }

        String password = getProperty("rpm.test.user.password");
        if (password == null)
        {
            password = "put password here";
        }

        System.out.println("Create Web service interfaces");
        ApplicationServiceLocator appSL = new ApplicationServiceLocator();
        AuthenticateServiceLocator authSL = new AuthenticateServiceLocator();

        authenticate = authSL.getAuthenticate(new URL(endpointURL
                + "Authenticate"));
        application = appSL
            .getApplication(new URL(endpointURL + "Application"));

        System.out.println("Login to RPM WS");
        sessionID = authenticate
            .login(userName, password, "jdbc/RPMDATASOURCE");
    }

    protected static String getProperty(String key) throws Exception
    {
        loadProperties();
        return (String) PROPERTY_MAP.get(key);
    }

    private synchronized static void loadProperties() throws Exception
    {
        if (PROPERTY_MAP == null)
        {
            PROPERTY_MAP = new Properties();
            InputStream systemResourceAsStream = ClassLoader
                .getSystemResourceAsStream(PROPERTY_FILE_NAME);

            BufferedInputStream bis = new BufferedInputStream(
                systemResourceAsStream);

            PROPERTY_MAP.load(bis);

            bis.close();
            systemResourceAsStream.close();
        }
    }

    /**
     * This method will be called by subclasses to store data in the RPM
     * repository
     */
    protected SaveResult save(RPMObject rpmObject,
            RPMObjectScope rpmObjectScope, ReloadType reloadType)
    {
        SaveResult result = null;

        try
        {
            result = application.save(sessionID, rpmObject, rpmObjectScope,
                reloadType);
            printRPMExceptions(result);
            if (result.isSuccessful())
            {
                System.out.println("Info : [Save on SessionID " + sessionID
                        + " was successful]");
            }
            else
            {
                System.err.println("ALERT : [Save on SessionID " + sessionID
                        + " was not successful]");
            }
        }
        catch (RemoteException e)
        {
            System.err.println("*** Fatal : The save failed to complete");
            e.printStackTrace();
            System.exit(1);
        }

        return result;
    }

    /**
     * This method will be called by subclasses to load an object from the RPM
     * repository by XPath
     * 
     * @param path
     * @param scope
     * @return
     */
    protected ArrayResult loadFromXpath(String path, RPMObjectScope scope)
    {
        ArrayResult result = null;

        try
        {
            result = application.loadFromXpath(sessionID, path, scope);
            printRPMExceptions(result);
            if (result.isSuccessful())
            {
                System.out.println("Info : [Load on SessionID " + sessionID
                        + " was successful]");
            }
            else
            {
                System.err.println("ALERT : [Load on SessionID " + sessionID
                        + " was not successful]");
            }
        }
        catch (RemoteException e)
        {
            System.err
                .println("*** Fatal : The load operation failed to complete");
            e.printStackTrace();
            System.exit(1);
        }

        return result;
    }

    /**
     * This method should be overriden by samples
     * 
     * @throws Exception
     */
    protected abstract void runSample() throws Exception;
}