/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import java.rmi.RemoteException;
import java.util.Calendar;

import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample modifies a Resource object's custom field then reads it back.
 * 
 * @preconditions : - the resource andreit exists in the database - At the
 *                administration level, under Custom Fields->Fields->Resources,
 *                one created a custom field called PetName
 * 
 * @desc Modify a custom field and read it back.
 */
public class ResourceCustomFieldSample extends SampleTestController
{
    private static String resourceName = "testABCDEF";

    /**
     * This field must be created in the administration view for the sample to
     * complete successfully.
     */
    private static String customFieldName = "PetName";

    /**
     * The value that will be stored in the custom field
     */
    private static String customFieldValue = "Tommy";

    /*
     * The resource must exist under the following pool for the sameple to run
     * successfully.
     */
    static String poolName = "POOL_123";

    /*
     * This resource must exist in the RPM repository for the sample to run
     * successfully.
     */
    private static final String resourcePath = "/Resource[@username='"
            + resourceName + "']";

    private void modifyCustomField()
    {
        Resource resource = null;

        /*
         * Indicate that we wish to load custom fields as well
         */
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCustomFieldAssignments(true);

        /*
         * Load the desired resource from the repository
         */
        try
        {
            resource = (Resource) application.loadFromXpath(sessionID,
                resourcePath, resourceScope).getRpmObjectList()[0];
        }
        catch (Exception e)
        {
            System.err
                .println("*** Fatal : Unable to load the resource from the database");
            e.printStackTrace();
            System.exit(1);
        }

        /*
         * Set the employee's termination date
         */
        Calendar discontinueDate = Calendar.getInstance();
        discontinueDate.roll(Calendar.YEAR, true);
        resource.setDiscontinueDate(discontinueDate);

        /*
         * Make sure that the custom field is attached to this resource and if
         * it is, change it to a new value
         */
        boolean found = false;
        CustomFieldAssignment[] customFieldAssignments = resource
            .getCustomFieldAssignments();

        for (int i = 0; i < customFieldAssignments.length; i++)
        {
            CustomFieldAssignment assignment = customFieldAssignments[i];

            if (assignment.getCustomField().getName().equals(customFieldName))
            {
                found = true;
                assignment.setValue(customFieldValue);
                break;
            }
        }

        if (!found)
        {
            System.err
                .println("*** Fatal : The resource needs to have a specific attribute.");
            System.exit(1);
        }

        try
        {
            Pool pool = (Pool) loadFromXpath("/Pool[@name='" + poolName + "']",
                null).getRpmObjectList()[0];
            resource.setParent(pool);
            application.save(sessionID, resource, resourceScope,
                ReloadType.None);
        }
        catch (RemoteException e)
        {
            e.printStackTrace();
        }
    }

    /*
     * load the value of the custom field and make sure it was really set to
     * what we have set it up to in modifyCustomField
     */
    private void verifyCustomFieldAssignment()
    {
        Resource resource = null;

        /*
         * Indicate that we wish to load the custom field assignments and load
         * the resource in question.
         */
        ResourceScope scope = new ResourceScope();
        scope.setCustomFieldAssignments(true);
        try
        {
            resource = (Resource) application.loadFromXpath(sessionID,
                "/Resource[@username='" + resourceName + "']", scope)
                .getRpmObjectList()[0];
        }
        catch (RemoteException e)
        {
            e.printStackTrace();
        }

        boolean found = false;
        CustomFieldAssignment[] customFieldAssignments = resource
            .getCustomFieldAssignments();
        /*
         * Locate the custom field we have written to previously, and see if it
         * was written to successfully.
         */
        for (int i = 0; i < customFieldAssignments.length; i++)
        {
            CustomFieldAssignment assignment = customFieldAssignments[i];
            if (assignment.getCustomField().getName().equals(customFieldName))
            {

                if (!assignment.getValue().equals(customFieldValue))
                {
                    System.err
                        .println("*** Fatal Error : The save seemes to have failed.");
                    System.err.println(customFieldName + " is "
                            + assignment.getValue() + " instead of "
                            + customFieldValue);
                }

                found = true;
                break;
            }
        }

        if (!found)
        {
            try
            {
                throw new Exception("Custom field not found");
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    }

    public static void main(String[] args)
    {
        new ResourceCustomFieldSample();
    }

    protected void runSample() throws Exception
    {
        modifyCustomField();
        verifyCustomFieldAssignment();
    }
}
