/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * This sample updates the data in task _CHILD_TASK Warnings are to be expected,
 * as per the API Documentation, since some fields are designed to be read-only
 * through the Web Services API.
 * 
 * @preconditions : There exists a Task by the name of _CHILD_TASK
 * 
 * @postconditions : The task's data is updated successfully.
 * 
 * @desc Update a specific task's details.
 * 
 */
public class WBSTaskSample extends SampleTestController
{

    public static final String taskName = "CHILD_TASK";

    public static final String proposalName = "PROPOSAL";

    protected void runSample() throws Exception
    {
        /**
         * encapsulates the returned data
         */
        SaveResult ret = null;

        /**
         * Explicit what we want loaded from the RPM Database
         */
        WorkElementScope taskScope = new WorkElementScope();
        taskScope.setScheduleDates(true);
        taskScope.setState(true);

        /**
         * Load the specific task from the database
         */
        Task task = (Task) application.loadFromXpath(sessionID,
            "/Task[name='" + taskName + "']", taskScope).getRpmObjectList()[0];

        /**
         * Update the task's estimated start and finish dates. The task's
         * expected dates can be set here as well. By default, the system sets
         * the expected dates to the estimated finish date.
         */
        Calendar estimatedStartDate = Calendar.getInstance();
        Calendar estimatedFinishDate = Calendar.getInstance();
        estimatedStartDate.set(Calendar.DAY_OF_YEAR, 300);
        estimatedFinishDate.set(Calendar.DAY_OF_YEAR, 300);
        estimatedFinishDate.set(Calendar.YEAR, estimatedFinishDate
            .get(Calendar.YEAR) + 1);

        task.setEstimatedStartDate(estimatedStartDate);
        task.setEstimatedFinishDate(estimatedFinishDate);

        /**
         * Save the task in the RPM database and reload what the database
         * returned.
         */
        ret = application.save(sessionID, task, taskScope,
            ReloadType.ReloadResult);
        printRPMExceptions(ret);
        task = (Task) ret.getRpmObject();

        /**
         * Copy all the proposed dates to planned dates
         */
        SaveResult planSave = application.copyProposedToPlan(sessionID, task,
            null);
        if (!planSave.isSuccessful())
        {
            System.err
                .println("*** Fatal : The save process failed to execute properly");
        }

        /**
         * Reload the task from the database in order to populate the actual
         * date field
         */
        task = (Task) application.loadFromXpath(sessionID,
            "/Task[name='" + taskName + "']", taskScope).getRpmObjectList()[0];

        /**
         * Populate the actual date field. The duration in minutes is mandatory
         * and must be between 0 and the planned duration in minutes
         */
        task.setActualDate(new WbsScheduleDate());
        task.getActualDate().setDurationInMinutes(new Integer(60));
        Calendar actualStartDate = Calendar.getInstance();
        actualStartDate.set(Calendar.YEAR,
            actualStartDate.get(Calendar.YEAR) + 1);
        actualStartDate.set(Calendar.DAY_OF_YEAR, 308);

        /**
         * Save the task again and reload it from the RPM database
         */
        ret = application.save(sessionID, task, taskScope,
            ReloadType.ReloadResult);
        super.printRPMExceptions(ret);
        task = (Task) ret.getRpmObject();

        /**
         * Populate the task's state field
         */
        DatafieldState state = (DatafieldState) application.loadFromXpath(
            sessionID, "/TaskState[value='In Progress']", null)
            .getRpmObjectList()[0];
        task.setState(state);

        /**
         * Save the task into the RPM database, without reading it back again
         */
        ret = application.save(sessionID, task, taskScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new WBSTaskSample();
    }
}
