package com.ibm.rpm.applicationadministration.checkpoints;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.CustomDatafield;
import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.GenericDatafield;
import com.ibm.rpm.applicationadministration.containers.State;
import com.ibm.rpm.applicationadministration.containers.ValueDatafield;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.customfield.constants.ErrorCodes;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;

/**
 * @author dominiqu
 */
public class DatafieldCheckpoint extends AbstractCheckpoint
{
    private static final Class DATAFIELD_CATEGORY_PARENTS = AdministrationModule.class;

    private static final Class DATAFIELD_PARENTS = DatafieldCategory.class;

    private static final Class CUSTOM_DATAFIELD_PARENTS = CustomDatafieldCategory.class;

    protected static DatafieldCheckpoint instance = new DatafieldCheckpoint();
    
    public static DatafieldCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if( rpmObject.getClass() == DatafieldCategory.class && rpmObject.getID() == null )
        {
            List expectedTypes = new ArrayList();
            
            expectedTypes.add(CustomDatafieldCategory.class);
            
            GenericValidator.validateCannotCreate(rpmObject, expectedTypes, msgCtx);
        }
        
        if( rpmObject.getClass() == Datafield.class && rpmObject.getID() == null )
        {
            List expectedTypes = new ArrayList();
            
            expectedTypes.add(CustomDatafield.class);
            
            GenericValidator.validateCannotCreate(rpmObject, expectedTypes, msgCtx);
        }
        
        if (rpmObject instanceof DatafieldRTF)
        {
            isValidRTF(rpmObject, msgCtx);
        }
        if (rpmObject instanceof DatafieldState)
        {
            isValidDatafieldState(rpmObject, msgCtx);
        }
        if (rpmObject instanceof State)
        {
            isValidState(rpmObject, msgCtx);
        }
        
        if (rpmObject instanceof DatafieldCategory)
        {
            isValidCategory(rpmObject, msgCtx);
        }
        else if (rpmObject instanceof ValueDatafield)
        {
            isValidDatafield(rpmObject, msgCtx);
        }
        else
        {
            GenericValidator.validateInstance(rpmObject,
                    GenericDatafield.class, msgCtx);
        }
    }
    
    /**
     * @param rpmObject
     * @param msgCtx
     */
    private void isValidRTF(RPMObject rpmObject, MessageContext msgCtx)
    {
        DatafieldRTF rtf = (DatafieldRTF) rpmObject;
        
        GenericValidator.validateNotNull(rpmObject, "appendOnly", rtf
            .getAppendOnly(), msgCtx);
    }
    
    private void isValidDatafieldState(RPMObject rpmObject, MessageContext msgCtx)
    {
        DatafieldState dfstate = (DatafieldState) rpmObject;
        
        GenericValidator.validateNotNull(rpmObject, "closedState", dfstate
            .getClosedState(), msgCtx);
    }
    
    private void isValidState(RPMObject rpmObject, MessageContext msgCtx)
    {
        State state = (State) rpmObject;
        
        GenericValidator.validateNotNull(rpmObject, "proposal", state
            .getProposal(), msgCtx);
    }

    private void isValidCategory(RPMObject rpmObject, MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, false,
                DATAFIELD_CATEGORY_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
                DatafieldCategory.class, msgCtx))
            return;

        DatafieldCategory category = (DatafieldCategory) rpmObject;
        isValidName(category, msgCtx);
    }

    private void isValidDatafield(RPMObject rpmObject, MessageContext msgCtx)
    {
        Class parentType;
        if (rpmObject instanceof CustomDatafield)
            parentType = CUSTOM_DATAFIELD_PARENTS;
        else
            parentType = DATAFIELD_PARENTS;
        super.isValidParent(rpmObject, msgCtx, false, parentType);

        if (!GenericValidator.validateInstance(rpmObject, Datafield.class,
                msgCtx))
            return;

        isValidName((Datafield) rpmObject, msgCtx);
        
        if (rpmObject instanceof ValueDatafield)
        {
            ValueDatafield datafield = (ValueDatafield) rpmObject;
            if (datafield.testValueModified())
            {
                GenericValidator.validateMaxLength(datafield, "value",
                        datafield.getValue(),
                        ValidationConstants.DATAFIELD_VALUE_MAX, msgCtx);
            }
        }
    }

    private void isValidName(GenericDatafield genDatafield,
            MessageContext msgCtx)
    {
        // There is no need to validate read-only names.
        // testNameModified will always return false if the name is read-only 
        // - DatafieldManager takes care of this.
        if (genDatafield.testNameModified())
        {
            GenericValidator.validateMaxLength(genDatafield, "name",
                    genDatafield.getName(),
                    ValidationConstants.DATAFIELD_NAME_MAX, msgCtx);
        }
    }

    public static boolean nameIsWritable(GenericDatafield genDatafield)
    {
        return !(genDatafield.getClass().equals(DatafieldCategory.class)
                || genDatafield instanceof DatafieldRTF || genDatafield instanceof DatafieldState)
                || genDatafield instanceof CustomDatafield;
    }

    public static boolean validateDatafieldInCategory(RPMObject object,
            String fieldName, Datafield datafield, DatafieldCategory category,
            MessageContext msgCtx)
    {
        boolean isInCategory = false;
        try
        {
            isInCategory = isDatafieldInCategory(datafield, category, msgCtx);
        }
        catch (Exception x)
        {
            RPMException e = new RPMException(x);
            e.assignContainer(object);
            addException(e, msgCtx);
        }

        if (!isInCategory)
        {
            String[] arguments =
            {
                    datafield.getName(), category.getName()
            };
            RPMException e = new RPMException(
                    ErrorCodes.DATAFIELD_NOT_IN_CATEGORY, arguments, object
                            .getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
        }
        return isInCategory;
    }

    public static boolean isDatafieldInCategory(Datafield datafield,
            DatafieldCategory category, MessageContext messageContext)
            throws RPMException, ParseException, SQLException
    {
        if (datafield == null)
        {
            return true;
        }
        else
        {
            SqlBuffer sql = new SqlBuffer();
            sql.appendEqualQuestionMark(DatafieldManager.NAME_PARENT_ID);
            sql.appendEqualQuestionMark(DatafieldManager.NAME_ELEMENT_ID);
            sql.appendAnd();
            sql.append(new DatafieldManager().getFilter());
            
            Object[] arguments = new Object[]
            {
                    Integer.valueOf(category.getID()),
                    Integer.valueOf(datafield.getID())
            };
            
            Integer elementId = null;
            try
            {
                elementId = (Integer) ManagerUtil.selectColumnValue(Integer.class, null,
                    null,
                    DatafieldManager.NAME_ELEMENT_ID, DatafieldManager.TABLE_NAME, sql, arguments, messageContext);
            }
            catch (RPMException e)
            {
                RPMException rpmException = new RPMException(e);
                addException(rpmException, messageContext);
            }
            
            return (elementId!=null);
        }
    }
}
