package com.ibm.rpm.client.junit.applicationadministration.testcases;

import com.ibm.rpm.applicationadministration.containers.CustomDatafield;
import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.GenericDatafield;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;

/**
 * @author legaults
 */
public class TestDatafieldCategory extends ClientTestController
{
    // ==============================
    // INITIALIZATION AND TERMINATION
    // ==============================

    protected void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        // Delete all saved generic custom fields.
        DatafieldFixture.deleteAll(this);
        // Check that all generic custom fields have been deleted.
        DatafieldFixture.assertAllDeleted(this);
        super.tearDown();
    }

    // ==========
    // LOAD TESTS
    // ==========

    public void testXPathAllDatafields() throws Exception
    {
        RPMObject[] result = loadFromXpath("/Datafield", null);
        assertNotNull(result);
        assertTrue(result.length > 2000);
    }

    public void testXPathAllCurrencies() throws Exception
    {
        RPMObject[] result = loadFromXpath("/Currency", null);
        assertNotNull(result);
        assertTrue(result.length > 0);
    }

    public void testXPathAllCustomDatafields() throws Exception
    {
        loadFromXpath("/CustomDatafield", null);
    }

    public void testLoadDatafieldCategoryFromId() throws Exception
    {
        doTestLoadDatafieldCategoryFromId("723");
    }

    public void testLoadCustomFieldCategoryFromTypeId() throws Exception
    {
        // TODO: Always comment the following code before check-in
        // because TypeID is not available from WSDL.

        // DatafieldCategory category = new DatafieldCategory();
        // category.assignTypeID(new Integer(26));
        // doTestLoadDatafieldCategory(category);
    }

    private void doTestLoadDatafieldCategoryFromId(String id) throws Exception
    {
        DatafieldCategory category = new DatafieldCategory();
        category.setID(id);
        doTestLoadDatafieldCategory(category);
    }

    private void doTestLoadDatafieldCategory(DatafieldCategory original)
            throws Exception
    {
        RPMObject loaded = this.loadFromID(original, null);
        assertNotNull(loaded);
        assertTrue(loaded instanceof DatafieldCategory);
        DatafieldCategory category = (DatafieldCategory) loaded;
        Datafield[] children = category.getChildren();
        assertNotNull(children);
        assertTrue(children.length > 0);
        for (int i = 0; i < children.length; i++)
        {
            assertSame(category, children[i].getParent());
        }
    }

    // ==============
    // CREATION TESTS
    // ==============

    public void testCreateCustomDatafieldCategory() throws Exception
    {
        CustomDatafieldCategory original = DatafieldFixture
                .createCustomDatafieldCategory(0);
        doTestCreate(original);
    }

    public void testCreateCustomDatafieldTree() throws Exception
    {
        CustomDatafieldCategory original = DatafieldFixture
                .createCustomDatafieldCategory(2);
        doTestCreate(original);
    }

    public void testCreateCustomDatafield() throws Exception
    {
        CustomDatafield original = DatafieldFixture.createCustomDatafield();
        doTestCreate(original);
    }

    private GenericDatafield doTestCreate(GenericDatafield original)
            throws Exception
    {
        // Ensure the parent is created first.
        if (original instanceof CustomDatafield)
        {
            CustomDatafieldCategory parent = (CustomDatafieldCategory) original
                    .getParent();
            if (parent != null && parent.getID() == null)
            {
                CustomDatafieldCategory reloadParent = (CustomDatafieldCategory) save(parent, null, ReloadType.SavedResult).getRpmObject();
                parent.setID(reloadParent.getID());
            }
        }
        if (original.getID() == null)
        {
            GenericDatafield saved = (GenericDatafield) save(original, null, ReloadType.SavedResult).getRpmObject();
            original.setID(saved.getID());
        }
        GenericDatafield reloaded = (GenericDatafield) loadFromID(original,
                null);
        DatafieldFixture.assertEquals(original, reloaded);
        return reloaded;
    }

    // ==============
    // UPDATE TESTS
    // ==============

    public void testUpdateCustomDatafieldCategory() throws Exception
    {
        CustomDatafieldCategory original = DatafieldFixture
                .createCustomDatafieldCategory(0);
        doTestUpdate(original);
    }

    public void testUpdateCustomDatafield() throws Exception
    {
        CustomDatafield original = DatafieldFixture.createCustomDatafield();
        doTestUpdate(original);
    }

    public void testUpdateTree() throws Exception
    {
        CustomDatafieldCategory original = DatafieldFixture
                .createCustomDatafieldCategory(3);
        doTestUpdate(original);
    }

    private void doTestUpdate(GenericDatafield original) throws Exception
    {
        // Create the object in the database.
        GenericDatafield modified = doTestCreate(original);
        // Modify the object.
        DatafieldFixture.modify(modified);
        // Save the modified object.
        save(modified, null, ReloadType.None);
        // Reload the modified object.
        RPMObject reloaded = loadFromID(modified, null);
        // Ensures all original objects have been deleted.
        DatafieldFixture.assertEquals(modified, reloaded);
    }

    // ===========
    // XPATH TESTS
    // ===========

    public void testXPathChildrenFromParentX() throws Exception
    {
        DatafieldCategory original = DatafieldFixture
                .createCustomDatafieldCategory(3);
        original = (DatafieldCategory) doTestCreate(original);

        // Build query.
        final String query = "/DatafieldCategory[Name='" + original.getName()
                + "']/Datafield";
        logger.debug("XPath query: " + query);

        // Reload the set of objects that match the specified property value
        // and then ensures that the set contains the original set.
        RPMObject[] reloadedObjects = loadFromXpath(query, null);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);
        Datafield[] originalChildren = original.getChildren();
        assertEquals(originalChildren.length, reloadedObjects.length);
        for (int i = 0; i < originalChildren.length; i++)
        {
            Datafield reloaded = (Datafield) DatafieldFixture.findInArray(
                    originalChildren[i], reloadedObjects);
            assertTrue("Original object not found in reloaded objects.",
                    reloaded != null);
            DatafieldFixture.assertEquals(originalChildren[i], reloaded);
        }
    }

    public void testXPathParentWithChildX() throws Exception
    {
        DatafieldCategory original = DatafieldFixture
                .createCustomDatafieldCategory(3);
        original = (DatafieldCategory) doTestCreate(original);

        // Build query.
        final String query = "/DatafieldCategory[Datafield[starts-with(Name, 'DatafieldFixture')]]";
        logger.debug("XPath query: " + query);

        // Reload the set of objects that match the specified property value
        // and then ensures that the set contains the original set.
        RPMObject[] reloadedObjects = loadFromXpath(
                query, null);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);
        DatafieldCategory reloaded = (DatafieldCategory) DatafieldFixture
                .findInArray(original, reloadedObjects);
        assertTrue("Original object not fount in reloaded objects.",
                reloaded != null);

        // Compare the original objects with the reloaded.
        DatafieldFixture.assertEquals(original, reloaded);
    }

    // ==============
    // DELETION TESTS
    // ==============

    public void testDeleteCustomDatafieldCategory() throws Exception
    {
        CustomDatafieldCategory original = DatafieldFixture
                .createCustomDatafieldCategory(0);
        doTestDelete(original);
    }

    public void testDeleteCustomDatafield() throws Exception
    {
        CustomDatafield original = DatafieldFixture.createCustomDatafield();
        doTestDelete(original);
    }

    public void testDeleteTree() throws Exception
    {
        CustomDatafieldCategory original = DatafieldFixture
                .createCustomDatafieldCategory(2);
        doTestDelete(original);
    }

    public void testDeleteAll() throws Exception
    {
        // Done in the tearDown method.
    }
    
    private void doTestDelete(GenericDatafield original) throws Exception
    {
        // Create the object in the database.
        GenericDatafield reloaded = doTestCreate(original);
        // Delete the object from the database.
        // Ensures the delete works even when the parent is not set.
        reloaded.setParent(null);
        delete(reloaded);
        // Ensures all original objects have been deleted.
        DatafieldFixture.assertDeleted(this, original);
    }
}
