package com.ibm.rpm.client.junit.applicationadministration.testcases;

import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.CalendarException;
import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.WorkInterval;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.types.CalendarExceptionType;
import com.ibm.rpm.applicationadministration.types.CalendarWeekDayType;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author lygagnon
 */
public class TestRPMCalendar extends ClientTestController
{
    private RPMCalendar mainCalendar = null;
    private CalendarScope mainScope = null;
    private final int NB_LEVELS = 2;
    private final int NB_CHILDREN_BY_LEVEL = 2;
    private final int NB_EXCEPTION_BY_LEVEL = 2;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        boolean isException = ((NB_EXCEPTION_BY_LEVEL > 0) ? true : false);
        mainScope = createScope(1, NB_LEVELS, isException);
    }

    public void tearDown() throws Exception
    {
        /*
         * We need to delete the parent because it was saved by the mainCalendar
         * and will not be added to the objectsToDelete arrayList.
         */
        if (mainCalendar != null && 
            mainCalendar.getParent() != null && 
            mainCalendar.getParent().getID() != null)
        {
            delete(mainCalendar.getParent());
        }
        super.tearDown();
    }

    public void testDelete() throws AssertionError, Exception
    {
        mainCalendar = RPMCalendarFixture.createFixture(null,
                NB_CHILDREN_BY_LEVEL, NB_EXCEPTION_BY_LEVEL);
        mainCalendar = saveCalendar(mainCalendar, null);
        RPMObject rpmObject = load(mainCalendar, null, true);

        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof RPMCalendar);
        RPMCalendarFixture.compare(mainCalendar, (RPMCalendar) rpmObject, null);

        delete(mainCalendar);

        RPMObject deleted = null;
        try
        {
            deleted = loadFromID(mainCalendar, null);
        } catch (Exception e)
        {
            //deleted...
        }
        assertNull(deleted);
    }

    // No scope : Insert from XPATH
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insert(null, false);
    }

    // No Scope : Insert from ID
    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(null, true);
    }

    //  No Scope : Update from XPATH
    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(mainScope, true);
    }

    public void testNonDefaultWorkingDays() throws AssertionError, Exception
    {
        mainScope = new CalendarScope();
        mainCalendar = RPMCalendarFixture.createFixture(mainScope, 0, 0);
        
        WorkInterval[] wis = new WorkInterval[5];
		wis[0] = RPMCalendarFixture.createWorkInterval("8:00", "10:00");
		wis[1] = RPMCalendarFixture.createWorkInterval("10:15", "12:15");
		wis[2] = RPMCalendarFixture.createWorkInterval("13:00", "14:30");
		wis[3] = RPMCalendarFixture.createWorkInterval("15:00", "16:00");
		wis[4] = RPMCalendarFixture.createWorkInterval("16:30", "17:00");
		
		CalendarWeekDay monday = new CalendarWeekDay();
        monday.setType(CalendarWeekDayType.NonDefaultWorking);
        monday.setIntervals(wis);
        monday.setParent(mainCalendar);
        mainCalendar.setStandardMonday(monday);
        
        mainCalendar = saveCalendar(mainCalendar, mainScope);

        RPMObject rpmObject = load(mainCalendar, mainScope, true);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof RPMCalendar);
        RPMCalendarFixture
                .compare(mainCalendar, (RPMCalendar) rpmObject, mainScope);
        RPMCalendarFixture.compareCalendarWeekDayMinutesToIntervalMinutes((RPMCalendar) rpmObject);
        
        mainScope.setExceptions(true);
        wis = new WorkInterval[2];
		wis[0] = RPMCalendarFixture.createWorkInterval("8:00", "12:00");
		wis[1] = RPMCalendarFixture.createWorkInterval("13:00", "17:00");
		
        //Monday August 8 is a non default-working day
        CalendarException exception = new CalendarException();
        java.util.Calendar nwDay = new GregorianCalendar(2005,8,8);
        exception.setDay(nwDay);
        exception.setIntervals(wis);
        exception.setParent(mainCalendar);
        exception.setType(CalendarExceptionType.NonDefaultWorking);
    	
        mainCalendar.setExceptions(new CalendarException[]{exception});
        
        mainCalendar = saveCalendar(mainCalendar, mainScope);
        rpmObject = load(mainCalendar, mainScope, true);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof RPMCalendar);
        RPMCalendarFixture
                .compare(mainCalendar, (RPMCalendar) rpmObject, mainScope);
    }

    public void testUpdateWithNonWorking()throws AssertionError, Exception{
        
        mainScope = new CalendarScope();
        mainScope.setExceptions(true);
        
        mainCalendar = RPMCalendarFixture.createFixture(mainScope, 0, 7);
        mainCalendar = saveCalendar(mainCalendar, mainScope);
        
        CalendarWeekDay nonWorking = new CalendarWeekDay();
        nonWorking.setType(CalendarWeekDayType.NonWorking);
        
        mainCalendar.setStandardMonday(nonWorking);
//        mainCalendar.setStandardTuesday(nonWorking);
//        mainCalendar.setStandardWednesday(nonWorking);
//        mainCalendar.setStandardThursday(nonWorking);
//        mainCalendar.setStandardFriday(nonWorking);
//        mainCalendar.setStandardSaturday(nonWorking);
//        mainCalendar.setStandardSunday(nonWorking);
//        
        saveMustFail(mainCalendar, mainScope);
    }
    
    private RPMCalendar saveCalendar(RPMCalendar c, CalendarScope scope)
            throws AssertionError, Exception
    {
        SingleResult result = save(c, scope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof RPMCalendar);

        c = (RPMCalendar) result.getRpmObject();
        return c;
    }

    private CalendarScope createScope(int level, int maxLevels, boolean isException)
    {
        CalendarScope scope = new CalendarScope();
        
        if (level < maxLevels)
        {
            level++;
            CalendarScope childScope = createScope(level, maxLevels, isException);
            childScope.setParent(scope);
            scope.setCalendars(childScope);
            if (isException)
            {
                scope.setExceptions(true);
            }
        }
        return scope;
    }

    private void insert(CalendarScope scope, boolean loadFromId)
            throws Exception
    {
        mainCalendar = RPMCalendarFixture.createFixture(scope,
                NB_CHILDREN_BY_LEVEL, NB_EXCEPTION_BY_LEVEL);
        mainCalendar = saveCalendar(mainCalendar, mainScope);

        RPMObject rpmObject = load(mainCalendar, scope, loadFromId);

        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof RPMCalendar);
        RPMCalendarFixture
                .compare(mainCalendar, (RPMCalendar) rpmObject, scope);

        //Update calendar structure
        RPMCalendar updatedRPMCal = (RPMCalendar) loadFromID(mainCalendar,
                scope);
        updatedRPMCal = RPMCalendarFixture.updateFixture(updatedRPMCal, scope);

        //Save the modifications
        updatedRPMCal = saveCalendar(updatedRPMCal, scope);

        RPMObject rpmObjectAfterUpdate = load(updatedRPMCal, scope, loadFromId);

        assertNotNull(rpmObjectAfterUpdate);
        assertTrue(rpmObjectAfterUpdate instanceof RPMCalendar);
        RPMCalendarFixture.compare(updatedRPMCal, (RPMCalendar) rpmObjectAfterUpdate,
                scope);
    }

    private RPMObject load(RPMCalendar calendar, CalendarScope scope,
            boolean loadFromId) throws Exception
    {
        RPMObject rpmObject = null;

        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new RPMCalendar();
            obj.setID(calendar.getID());
            rpmObject = loadFromID(obj, scope);
            assertNotNull(rpmObject);
            assertTrue(rpmObject instanceof RPMCalendar);

        }
        else
        {
            RPMObject[] rpmObjects = loadFromXpath("RPMCalendar[id='"
                    + mainCalendar.getID() + "']", scope);
            if (rpmObjects != null)
            {
                rpmObject = rpmObjects[0];
            }
        }

        if (scope == null)
        {
            logger.info("Scoping is null");
        }
        else
        {
            logger.info("Scoping" + mainScope.toString());
        }
        logger.info("Calendar " + calendar);
        logger.info("Resulting Calendar " + rpmObject);

        return rpmObject;
    }
}
