package com.ibm.rpm.client.junit.asset.testcases;

import java.sql.Date;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.DepreciationSchedule;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.DepreciationScheduleFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author ragoler
 *
 *
 * A test case for Depreciation Schedule. It test the loading, updating and deleting with
 * and without a context

 */
public class TestDepreciationSchedule extends ClientTestController
{

    private Asset mainAsset;
    private RPMObjectScope mainScope;
    private DepreciationSchedule[] depreciations;

    public static final int SIZE = 5;


    protected void setUp() throws Exception
    {
        super.setUp();

        AssetScope aScope = new AssetScope();
        aScope.setParent(new AssetScope());
        mainScope = new RPMObjectScope();
        mainScope.setParent(aScope);


        //create a new asset in the database
        mainAsset = AssetFixture.createFixture(aScope,this);

        // in order not to create automatic depreciations
        mainAsset.setDepreciationMethod(new Integer(TestAsset.DEPRECIATION_MANUAL));

        mainAsset = (Asset)save(mainAsset,aScope).getRpmObject();

        depreciations = new DepreciationSchedule[SIZE];

        //create new depreciations
        depreciations = DepreciationScheduleFixture.createFixtures(mainAsset,SIZE);

        //set the depreciations for the asset
        mainAsset.setDepreciationSchedules(depreciations);
}

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }


    /**
     * Test loading from XPath
     * Currently only loading a known ID
     */
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        saveDepreciations(null);

        RPMObject[] loadedDepreciations;

        // test loading depreciation using id
        loadedDepreciations = loadFromXpath("/DepreciationSchedule[Id='" + depreciations[0].getID()
            + "']",null);
        assertNotNull(loadedDepreciations);
        assertEquals(1,loadedDepreciations.length);
        DepreciationScheduleFixture.compare(this.depreciations[0], (DepreciationSchedule)loadedDepreciations[0],null);

        loadedDepreciations = loadFromXpath("/DepreciationSchedule[Amount=" + depreciations[0].getAmount()
                + "]",null);
        assertNotNull(loadedDepreciations);
        assertTrue(1<=loadedDepreciations.length);

    }

    /**
     * Test loading from XPath
     * Currently only loading a known ID
     */
    public void testInsertWithNoScopeLoadFromXPathGreatEqualAmount() throws AssertionError,
            Exception
    {
        saveDepreciations(null);

        RPMObject[] loadedDepreciations;

        loadedDepreciations = loadFromXpath("/DepreciationSchedule[Amount>" + new Double(depreciations[0].getAmount().doubleValue()-1.0)
                + "]",null);
        assertNotNull(loadedDepreciations);
        assertTrue(1<=loadedDepreciations.length);


    }

    /**
     * Test loading from XPath
     * Currently only loading a known ID
     */
    public void testInsertWithNoScopeLoadFromXPathGreatEqualDate() throws AssertionError,
            Exception
    {
        saveDepreciations(null);

        RPMObject[] loadedDepreciations;

        loadedDepreciations = loadFromXpath("/DepreciationSchedule[DepreciationDate='" + HelperTestUtilities.convertCalendarToString(depreciations[0].getDepreciationDate())
                + "']",null);
        assertNotNull(loadedDepreciations);
        assertTrue(1<=loadedDepreciations.length);

    }

    /**
     * Test loading from XPath
     * Using the asset
     */
    public void testLoadFromXPathIncludeAsset() throws AssertionError,
            Exception
    {
        saveDepreciations(null);

        RPMObject[] loadedDepreciations;

        // test loading depreciation using id
		loadedDepreciations = loadFromXpath("/Asset[Name='" + mainAsset.getName() + "']/DepreciationSchedule[Id='" + depreciations[0].getID() + "']",null);
        assertNotNull(loadedDepreciations);
        assertEquals(1,loadedDepreciations.length);
        DepreciationScheduleFixture.compare(this.depreciations[0], (DepreciationSchedule)loadedDepreciations[0],null);

        // test loading depreciation using id
		loadedDepreciations = loadFromXpath("/Asset[Id='" + mainAsset.getID() + "']/DepreciationSchedule",null);
        assertNotNull(loadedDepreciations);
        assertEquals(mainAsset.getDepreciationSchedules().length,loadedDepreciations.length);
        DepreciationSchedule[] newDep = (DepreciationSchedule[])(Arrays.asList(loadedDepreciations).toArray(new DepreciationSchedule[0]));
        DepreciationScheduleFixture.compare(this.depreciations, newDep);

    }

    /**
     * Test loading from XPath
     * Using the asset
     */
    public void testLoadFromXPathIncludeAssetAndFolder() throws AssertionError,
            Exception
    {
        saveDepreciations(mainScope);

        RPMObject[] loadedDepreciations;

        // test loading depreciation using id
		loadedDepreciations = loadFromXpath("/AssetFolder[Name='" + ((AssetFolder)mainAsset.getParent()).getName() + "']/Asset[Name='" + mainAsset.getName() + "']/DepreciationSchedule[Id='" + depreciations[0].getID() + "']",null);
        assertNotNull(loadedDepreciations);
        assertEquals(1,loadedDepreciations.length);
        DepreciationScheduleFixture.compare(this.depreciations[0], (DepreciationSchedule)loadedDepreciations[0],null);

        // test loading depreciation using id
		loadedDepreciations = loadFromXpath("/AssetFolder[Id='" + ((AssetFolder)mainAsset.getParent()).getID() + "']/Asset[Id='" + mainAsset.getID() + "']/DepreciationSchedule",null);
        assertNotNull(loadedDepreciations);
        assertEquals(mainAsset.getDepreciationSchedules().length,loadedDepreciations.length);
        DepreciationSchedule[] newDep = (DepreciationSchedule[])(Arrays.asList(loadedDepreciations).toArray(new DepreciationSchedule[0]));
        DepreciationScheduleFixture.compare(this.depreciations, newDep);

    }




    public void testLoadAssetFromXPathNestedDepreciationCondition() throws AssertionError,
    Exception
	{
        saveDepreciations(null);

        RPMObject[] loadedAssets;

		// test loading depreciation using id
		loadedAssets = loadFromXpath("/Asset[DepreciationSchedule[Id='" + depreciations[0].getID() +"']]",null);
		assertNotNull(loadedAssets);
		assertEquals(1,loadedAssets.length);
		AssetFixture.compare(this.mainAsset, (Asset)loadedAssets[0],null);

	}

    /**
     * Inserts a few depreciation schedules, then delete them and test that they were deleted.
     * Uses no scope
     */
    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        saveDepreciations(this.mainScope);

        List loadedDepreciations = new LinkedList();
        for (int i = 0; i < this.depreciations.length; i++)
        {
            DepreciationSchedule loadedDepreciation = loadDepreciation(this.depreciations[i],null);
            DepreciationScheduleFixture.compare(this.depreciations[i], loadedDepreciation,null);
            loadedDepreciations.add(loadedDepreciation);
        }

        //can't delete because no Asset loaded
//        for (int i = 0; i < loadedDepreciations.size(); i++)
//        {
//
//            deleteDepreciation((DepreciationSchedule)loadedDepreciations.get(i),null);
//        }


    }

    /**
     * Inserts a few depreciation schedules, then delete them and test that they were deleted.
     * Uses a scope for the parent
     */
    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        saveDepreciations(this.mainScope);

        List loadedDepreciations = new LinkedList();
        for (int i = 0; i < this.depreciations.length; i++)
        {
            DepreciationSchedule loadedDepreciation = loadDepreciation(this.depreciations[i],mainScope);
            DepreciationScheduleFixture.compare(this.depreciations[i], loadedDepreciation,mainScope);
            loadedDepreciations.add(loadedDepreciation);
        }

        for (int i = 0; i < loadedDepreciations.size(); i++)
        {

            deleteDepreciation((DepreciationSchedule)loadedDepreciations.get(i),mainScope);
        }
    }


    /**
     * Inserts a few depreciation schedules, then update them and test the update
     * Uses no scope for the parent
     */
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        saveDepreciations(null);

        //Create a new Depreciation
        DepreciationSchedule[] updatedDepreciations = DepreciationScheduleFixture.createFixtures(this.mainAsset,depreciations.length);


        for (int i = 0; i < this.depreciations.length; i++)
        {

            //By setting the ID to the depreciation ID, the updatedDepreciation represent
            //the mainDepretition with all its attribute updated.
            updatedDepreciations[i].setID(depreciations[i].getID());

            updatedDepreciations[i].setAmount(new Double(100.50));
            updatedDepreciations[i].setDepreciationDate(HelperTestUtilities.convertToCalendar(Date
                    .valueOf("2010-10-10")));
            updatedDepreciations[i] = saveDepreciation(updatedDepreciations[i],null);

            DepreciationSchedule loadedDep= loadDepreciation(updatedDepreciations[i],null);


            DepreciationScheduleFixture.compare(updatedDepreciations[i], loadedDep,null);

        }

    }

    /**
     * Inserts a few depreciation schedules, then update them and test the update
     * Uses scope for the parent
     */
    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        saveDepreciations(mainScope);

        //Create a new Depreciation
        DepreciationSchedule[] updatedDepreciations = DepreciationScheduleFixture.createFixtures(this.mainAsset,depreciations.length);


        for (int i = 0; i < this.depreciations.length; i++)
        {

            //By setting the ID to the depreciation ID, the updatedDepreciation represent
            //the mainDepretition with all its attribute updated.
            updatedDepreciations[i].setID(depreciations[i].getID());

            updatedDepreciations[i].setAmount(new Double(100.50));
            updatedDepreciations[i].setDepreciationDate(HelperTestUtilities.convertToCalendar(Date
                    .valueOf("2010-10-10")));
            updatedDepreciations[i] = saveDepreciation(updatedDepreciations[i],mainScope);

            DepreciationSchedule loadedDep= loadDepreciation(updatedDepreciations[i],mainScope);


            DepreciationScheduleFixture.compare(updatedDepreciations[i], loadedDep,mainScope);

        }
    }



    private void saveDepreciations(RPMObjectScope scope) throws Exception
    {
        for (int i = 0; i < this.depreciations.length; i++)
        {
            this.depreciations[i] = saveDepreciation(this.depreciations[i], scope);
        }
    }

    private DepreciationSchedule saveDepreciation(DepreciationSchedule depriciation,RPMObjectScope scope) throws Exception
    {
        SingleResult result;
        result = save(depriciation, scope);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof DepreciationSchedule);

        return (DepreciationSchedule)result.getRpmObject();

    }

    /**
     * @param depreciation that we are working on now
     * @throws Exception
     * @throws AssertionFailedError
     */
    private DepreciationSchedule loadDepreciation(DepreciationSchedule depreciation, RPMObjectScope scope) throws Exception
    {
        RPMObject obj = new DepreciationSchedule();
        obj.setID(depreciation.getID());
        RPMObject rpmObject = loadFromID(obj, scope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof DepreciationSchedule);

        return (DepreciationSchedule)rpmObject;
    }



    private void deleteDepreciation(DepreciationSchedule depreciation,RPMObjectScope scope) throws Exception
    {
        delete(depreciation);

        RPMObject rpmObject=null;
        try
        {
            rpmObject = loadFromID(depreciation,scope);
        }
        catch (Exception e)
        {
            logger.debug("This is a \"Good\" exception (verifies that the object were deleted)");
        }
        assertNull(rpmObject);
    }
}
