package com.ibm.rpm.client.junit.asset.testcases;

import java.sql.Date;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.MaintenanceEstimates;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.MaintenanceEstimatesFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author adler
 *
 * A test case for Maintenance Estimates. It tests the loading, updating and
 * deleting with and without a scope
 *
 */
public class TestMaintenanceEstimates extends ClientTestController
{

    private Asset mainAsset;

    private RPMObjectScope mainEsimatesScope;

    private MaintenanceEstimates[] mainEstimates;

    public static final int NUMBER_RECORDS = 5;

    public void setUp() throws Exception
    {
        super.setUp();

        //create a new asset in the database
        mainAsset = AssetFixture.createFixture(null, this);

        mainEstimates = new MaintenanceEstimates[NUMBER_RECORDS];
        mainAsset = (Asset)save(mainAsset, null).getRpmObject();

        //create new Estimares
        mainEstimates = MaintenanceEstimatesFixture.createFixture(mainAsset,
                NUMBER_RECORDS);

        //assign the Estimares for the asset
        mainAsset.setMaintenanceEstimates(mainEstimates);
        // Estimate scope
        mainEsimatesScope = new RPMObjectScope();
        // Assign Estimate scope parent as asset
        mainEsimatesScope.setParent(new AssetScope());
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    /**
     * Test loading from XPath Loading and comparing a known ID, loading
     * description & performedBy No Scope
     */
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        saveEstimates(mainEstimates,null);
        RPMObject[] loadedEstimates;

        //load Estimare object via xpath (id)
        loadedEstimates = loadFromXpath("/MaintenanceEstimates[id='"
                + mainEstimates[0].getID() + "']", null);
        assertNotNull(loadedEstimates);
        assertTrue(loadedEstimates[0] instanceof MaintenanceEstimates);
        assertEquals(1, loadedEstimates.length);
        MaintenanceEstimatesFixture.compare(mainEstimates[0],
                (MaintenanceEstimates) loadedEstimates[0], null);

        //load Estimare object via xpath (Description)
        loadedEstimates = loadFromXpath("/MaintenanceEstimates[Description='"
                + mainEstimates[1].getDescription() + "']", null);

        assertNotNull(loadedEstimates);
        assertTrue(loadedEstimates[0] instanceof MaintenanceEstimates);
        assertTrue(1 <= loadedEstimates.length);

        //load Estimare object via xpath (PerformedBy)
        loadedEstimates = loadFromXpath("/MaintenanceEstimates[PerformedBy='"
                + mainEstimates[2].getPerformedBy() + "']", null);
        assertNotNull(loadedEstimates);
        assertTrue(loadedEstimates[0] instanceof MaintenanceEstimates);
        assertTrue(1 <= loadedEstimates.length);

    } // testInsertWithNoScopeLoadFromXPath

    /**
     * Test loading from XPath Loading Greater-Equal costs No Scope
     */
    public void testInsertWithNoScopeLoadFromXPathGreatEqualCosts()
            throws AssertionError, Exception
    {
        saveEstimates(mainEstimates,null);
        RPMObject[] loadedEstimates;

        // GTE Actual Costs
        loadedEstimates = loadFromXpath("/MaintenanceEstimates[ActualCost>"
                + new Double(
                        mainEstimates[2].getActualCost().doubleValue() - 1.0)
                + "]", null);
        assertNotNull(loadedEstimates);
        assertTrue(loadedEstimates[0] instanceof MaintenanceEstimates);
        assertTrue(1 <= loadedEstimates.length);

        // GTE Estimated Costs
        loadedEstimates = loadFromXpath(
                "/MaintenanceEstimates[EstimatedCost>"
                        + new Double(mainEstimates[3].getEstimatedCost()
                                .doubleValue() - 1.0) + "]", null);
        assertNotNull(loadedEstimates);
        assertTrue(loadedEstimates[0] instanceof MaintenanceEstimates);
        assertTrue(1 <= loadedEstimates.length);

    } // testInsertWithNoScopeLoadFromXPathGreatEqualCosts

    /**
     * Test loading from XPath Loading a ScheduleDate No Scope
     */
    public void testInsertWithNoScopeLoadFromXPathEqualDate()
            throws AssertionError, Exception
    {
        saveEstimates(mainEstimates,null);
        RPMObject[] loadedEstimates;

        // Equal ScheduleDate

        loadedEstimates = loadFromXpath("/MaintenanceEstimates[ScheduleDate='"
                + HelperTestUtilities.convertCalendarToString(mainEstimates[0]
                        .getScheduleDate()) + "']", null);
        assertNotNull(loadedEstimates);
        assertTrue(1 <= loadedEstimates.length);

    }

    /**
     * Test loading from XPath Using the asset with Scope
     */
    public void testLoadFromXPathIncludeAsset() throws AssertionError,
            Exception
    {
        saveEstimates(mainEstimates, mainEsimatesScope);
        RPMObject[] loadedEstimates;

        // test loading estimate using asset and Estimates ids
        loadedEstimates = loadFromXpath("/Asset[Id='" + mainAsset.getID()
                + "']/MaintenanceEstimates[Id='" + mainEstimates[0].getID()
                + "']", mainEsimatesScope);
        assertNotNull(loadedEstimates);
        assertEquals(1, loadedEstimates.length);
        MaintenanceEstimatesFixture.compare(this.mainEstimates[0],
                (MaintenanceEstimates) loadedEstimates[0], mainEsimatesScope);

        // test loading all estimates related to an Asset id
        loadedEstimates = loadFromXpath("/Asset[Name='" + mainAsset.getName()
                + "']/MaintenanceEstimates", mainEsimatesScope);
        assertNotNull(loadedEstimates);
        assertEquals(mainAsset.getMaintenanceEstimates().length,
                loadedEstimates.length);
        MaintenanceEstimates[] newEsts = (MaintenanceEstimates[]) (Arrays
                .asList(loadedEstimates).toArray(new MaintenanceEstimates[0]));
        MaintenanceEstimatesFixture.compare(this.mainEstimates, newEsts);

    } // testLoadFromXPathIncludeAsset

    /**
     * Test loading from XPath Using the asset, nested MaintenanceEstimates and Scope
     * /Asset[MaintenanceEstimates[<condition>]]
     */
    public void testLoadAssetFromXPathNestedMaintenanceEstimates() throws AssertionError,
    Exception
	{
        // Save Maintenance Estimates
        saveEstimates(mainEstimates, null);

		RPMObject[] loadedAssets;

		// test loading nested MaintenanceEstimates using id
		loadedAssets = loadFromXpath("/Asset[MaintenanceEstimates[Id='" + mainEstimates[0].getID() +"']]",new AssetScope());
		assertNotNull(loadedAssets);
		assertEquals(1,loadedAssets.length);

	} // testLoadAssetFromXPathNestedMaintenanceEstimates

    /**
     * Inserts Maintenance Estimates, then deletes it and tests that she was
     * deleted. Uses no scope
     */
    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        saveEstimates(mainEstimates,null);

        List loadedEstimates = new LinkedList();
        for (int i = 0; i < this.mainEstimates.length; i++)
        {
            //load from DB via id
            MaintenanceEstimates loadedEstimate = loadEstimate(
                    mainEstimates[i], null);
            MaintenanceEstimatesFixture.compare(this.mainEstimates[i],
                    loadedEstimate, null);
            loadedEstimates.add(loadedEstimate);
        }

        //can't delete because no Asset loaded
        // for (int i = 0; i < loadedEstimates.size(); i++)
        //{
        //deleteEstimate((MaintenanceEstimates)loadedEstimates.get(i),null);
        //}

    } // testInsertWithNoScopeLoadFromID

    /**
     * Inserts Maintenance Estimates, then deletes it and tests that she was
     * deleted. Uses scope
     */
    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        saveEstimates(mainEstimates, mainEsimatesScope);

        List loadedEstimates = new LinkedList();
        for (int i = 0; i < this.mainEstimates.length; i++)
        {
            //load from DB via id
            MaintenanceEstimates loadedEstimate = loadEstimate(
                    mainEstimates[i], mainEsimatesScope);
            MaintenanceEstimatesFixture.compare(this.mainEstimates[i],
                    loadedEstimate, mainEsimatesScope);
            loadedEstimates.add(loadedEstimate);
        }

        // Check the delete
        for (int i = 0; i < loadedEstimates.size(); i++)
        {
            deleteEstimate((MaintenanceEstimates) loadedEstimates.get(i),
                    mainEsimatesScope);
        }
    } // testInsertWithScopeLoadFromID

    /**
     * Updates existing Maintenance Estimates, then loads it and tests if load
     * records includes the update. Uses no scope
     */
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        saveEstimates(mainEstimates, null);
        //Create a new Estimates
        MaintenanceEstimates[] updatedEstimates = MaintenanceEstimatesFixture
                .createFixture(mainAsset, mainEstimates.length);

        for (int i = 0; i < this.mainEstimates.length; i++)
        {
            //By setting the ID to the mainEstimates ID, the updatedEstimates
            // represent
            //the mainEstimates with all its attribute updated.
            updatedEstimates[i].setID(mainEstimates[i].getID());
            updatedEstimates[i].setScheduleDate(HelperTestUtilities
                    .convertToCalendar(Date.valueOf("2010-10-10")));
            updatedEstimates[i].setDescription("Effort"
                    + (new Integer(i)).toString());
            updatedEstimates[i].setActualCost(new Double(i));
            updatedEstimates[i].setEstimatedCost(new Double(i + 2));
            updatedEstimates[i].setPerformedBy("Performed By "
                    + (new Integer(i)).toString());
            //Save the modifications
            saveEstimate(updatedEstimates[i],null);
            //load from an id
            MaintenanceEstimates loadedEst = loadEstimate(updatedEstimates[i],
                    null);
            // Compare
            MaintenanceEstimatesFixture.compare(updatedEstimates[i],
                    loadedEst, null);
        }

    } // testUpdateWithNoScopeLoadFromID

    /**
     * Updates existing Maintenance Estimates, then loads it and tests if load
     * records includes the update. Uses scope for the parent
     */
    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        saveEstimates(mainEstimates, mainEsimatesScope);
        //Create a new Estimates
        MaintenanceEstimates[] updatedEstimates = MaintenanceEstimatesFixture
                .createFixture(mainAsset, mainEstimates.length);

        for (int i = 0; i < this.mainEstimates.length; i++)
        {
            //By setting the ID to the mainEstimates ID, the updatedEstimates
            // represent
            //the mainEstimates with all its attribute updated.
            updatedEstimates[i].setID(mainEstimates[i].getID());
            updatedEstimates[i].setScheduleDate(HelperTestUtilities
                    .convertToCalendar(Date.valueOf("2010-10-10")));
            updatedEstimates[i].setDescription("Effort"
                    + (new Integer(i)).toString());
            updatedEstimates[i].setActualCost(new Double(i));
            updatedEstimates[i].setEstimatedCost(new Double(i + 2));
            updatedEstimates[i].setPerformedBy("Performed By "
                    + (new Integer(i)).toString());
            //Save the modifications
            saveEstimate(updatedEstimates[i], mainEsimatesScope);
            //load from an id
            MaintenanceEstimates loadedEst = loadEstimate(updatedEstimates[i],
                    mainEsimatesScope);
            // Compare
            MaintenanceEstimatesFixture.compare(updatedEstimates[i],
                    loadedEst, mainEsimatesScope);
        }
    } // testUpdateWithScopeLoadFromID

    private void saveEstimates(MaintenanceEstimates[] estimates, RPMObjectScope scope)
            throws Exception
    {
        for (int i = 0; i < estimates.length; i++)
        {
            estimates[i] = saveEstimate(estimates[i], scope);
        }
    }

    /**
     * saveEstimate - Save one MaintenanceEstimates object (without scope)
     *
     * @param MaintenanceEstimates
     *            that we are working on now
     * @return None
     * @throws Exception
     * @throws AssertionFailedError
     *
     */
    private MaintenanceEstimates saveEstimate(MaintenanceEstimates p, RPMObjectScope scope) throws AssertionError,
            Exception
    {
        SingleResult result;
        // Save the Object
        result = save(p, scope);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof MaintenanceEstimates);

        return (MaintenanceEstimates)result.getRpmObject();
    }

    /**
     * loadEstimate - Load a MaintenanceEstimates object with scope
     *
     * @param MaintenanceEstimates
     *            the object to load
     * @param MaintenanceEstimatesScope
     *            the scope
     * @return Loaded MaintenanceEstimates
     * @throws Exception
     * @throws AssertionFailedError
     */
    private MaintenanceEstimates loadEstimate(MaintenanceEstimates estimate,
            RPMObjectScope scope) throws Exception
    {
        //load from an id
        RPMObject obj = new MaintenanceEstimates();
        obj.setID(estimate.getID());
        RPMObject rpmObject = loadFromID(obj, scope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof MaintenanceEstimates);

        return (MaintenanceEstimates) rpmObject;

    }

    /**
     * deleteEstimate - Delete a MaintenanceEstimates object with scope
     * @param MaintenanceEstimates the object to delete
     * @param MaintenanceEstimatesScope the scope
     * @return None
     * @throws Exception
     *
     */
    private void deleteEstimate(MaintenanceEstimates estimates,
            RPMObjectScope scope) throws Exception
    {
        delete(estimates);

        RPMObject rpmObject = null;
        try
        {
            rpmObject = loadFromID(estimates, scope);
        }
        catch (Exception e)
        {
            logger
                    .debug("This is a \"Good\" exception (verifies that the object were deleted)");
        }
        assertNull(rpmObject);
    }

}
