/*
 * Created on 5-Oct-2005
 */
package com.ibm.rpm.client.junit.communications.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.communications.fixtures.GenericCommunicationFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.communications.constants.ValidationConstants;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.wbs.containers.Project;

/**
 * @author dominiqu
 */
public class TestSendNotificationValidation extends GenericValidation
{
    private Notification mainNotification;

    private GenericCommunicationScope fullScope = null;

    protected void setUp() throws Exception
    {
        super.setUp();

        fullScope = GenericCommunicationFixture.createFullScope(true);

        Project attachment = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        WorkflowAttachment workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(attachment);

        mainNotification = (Notification) GenericCommunicationFixture.createFixture(
            workflowAttachment, new Notification(), fullScope, this);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithNotExistingAttachment() throws AssertionError,
            Exception
    {
        Project wrongAttachment = new Project();
        wrongAttachment.setID("NOT_EXISTING_ID");

        WorkflowAttachment workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(wrongAttachment);
        mainNotification.setWorkflowAttachment(workflowAttachment);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NOT_EXISTING_ID,
                mainNotification.getWorkflowAttachment().getClass().getName(),
                ValidationConstants.WORKFLOWATTACHMENT_ATTACHMENT_FIELD,
                mainNotification.getWorkflowAttachment().getID())
        };

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.AM_INVALID_RPMOBJECTID,
                wrongAttachment.getClass().getName(), null,
                wrongAttachment.getID())
        };

        verifySendNotificationMustFail(mainNotification, fullScope,
            expectedExceptions, expectedWarnings);
    }

    public void testFailureWithInvalidAttachmentType() throws AssertionError,
            Exception
    {
        ClientCostCenter wrongAttachment = new ClientCostCenter();
        wrongAttachment.setID("NOT_EXISTING_ID");

        WorkflowAttachment workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(wrongAttachment);
        mainNotification.setWorkflowAttachment(workflowAttachment);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE,
                mainNotification.getWorkflowAttachment().getClass().getName(),
                ValidationConstants.WORKFLOWATTACHMENT_ATTACHMENT_FIELD,
                mainNotification.getWorkflowAttachment().getID())
        };

        verifySendNotificationMustFail(mainNotification, fullScope,
            expectedExceptions, null);
    }

    public void testFailureWithInvalidResource() throws AssertionError,
            Exception
    {
        Resource notExistingResource = new Resource();
        notExistingResource.setID("NOT_EXISTING_RESOURCE");
        mainNotification.getResponses()[0].setResource(notExistingResource);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.NOT_EXISTING_ID,
                WorkflowResponse.class.getName(),
                ValidationConstants.WORKFLOWRESPONSE_RESOURCE_FIELD, null)
        };

        verifySendNotificationMustFail(mainNotification, fullScope,
            expectedExceptions, null);
    }

    public void testFailureWithDuplicateResources() throws AssertionError,
            Exception
    {
        mainNotification.getResponses()[1].setResource(mainNotification.getResponses()[0].getResource());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.communications.constants.ErrorCodes.NO_DUPLICATE_RECIPIENTS,
                Notification.class.getName(),
                ValidationConstants.GENERICCOMMUNICATION_REPLIES_FIELD, null)
        };

        verifySendNotificationMustFail(mainNotification, fullScope,
            expectedExceptions, null);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainNotification.setSubject(getTestString(ValidationConstants.GENERICCOMMUNICATION_SUBJECT_MAX));
        mainNotification.setComments(getTestString(ValidationConstants.GENERICCOMMUNICATION_COMMENTS_MAX));

        // Attempt to save the Notification object
        Notification saved = (Notification) sendNotificationMustSucceed(
            mainNotification, Notification.class, fullScope);
        mainNotification.setID(saved.getID());

        // Load the inserted Notification from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Notification[Id='"
                + mainNotification.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Notification);

        // Check that the loaded Notification is identical to the one we
        // requested to store
        GenericCommunicationFixture.compare(mainNotification,
            (Notification) rpmObjects[0], null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainNotification.setSubject(getTestString(ValidationConstants.GENERICCOMMUNICATION_SUBJECT_MAX + 1));
        mainNotification.setComments(getTestString(ValidationConstants.GENERICCOMMUNICATION_COMMENTS_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH,
                    mainNotification.getClass().getName(),
                    ValidationConstants.GENERICCOMMUNICATION_SUBJECT_FIELD,
                    mainNotification.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH,
                    mainNotification.getClass().getName(),
                    ValidationConstants.GENERICCOMMUNICATION_COMMENTS_FIELD,
                    mainNotification.getID())
        };

        verifySendNotificationMustFail(mainNotification, fullScope,
            expectedExceptions, null);
    }

    /**
     * Generic sendNotification function for sendNotification calls that must
     * succeed
     */
    protected RPMObject sendNotificationMustSucceed(Notification obj,
            Class objType, GenericCommunicationScope scope)
            throws AssertionError, Exception
    {
        assertNotNull(obj);
        assertTrue(objType.isInstance(obj));

        addObjectToDelete(obj);

        // Retrieve the recipients
        Resource[] recipients = new Resource[obj.getResponses().length];
        for (int i = 0; i < recipients.length; ++i)
        {
            if (obj.getResponses()[i] != null)
            {
                recipients[i] = obj.getResponses()[i].getResource();
            }
        }

        // Attempt to save the object
        ResourceArray resourceArray = new ResourceArray();
        resourceArray.setResourceArray(recipients);
        SingleResult result = getApplicationInterface().sendNotification(
            sessionid, scope, obj.getWorkflowAttachment().getAttachment(),
            resourceArray, obj.getSubject(), obj.getComments());

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(objType.isInstance(result.getRpmObject()));
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        obj = (Notification) result.getRpmObject();
        return obj;
    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed sendNotification
     */
    protected void verifySendNotificationMustFail(Notification obj,
            GenericCommunicationScope scope,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings) throws AssertionError,
            Exception
    {
        addObjectToDelete(obj);

        // Retrieve the recipients
        Resource[] recipients = new Resource[obj.getResponses().length];
        for (int i = 0; i < recipients.length; ++i)
        {
            if (obj.getResponses()[i] != null)
            {
                recipients[i] = obj.getResponses()[i].getResource();
            }
        }

        // Attempt to save the object
        ResourceArray resourceArray = new ResourceArray();
        resourceArray.setResourceArray(recipients);
        SingleResult saveResult = getApplicationInterface().sendNotification(
            sessionid, scope, obj.getWorkflowAttachment().getAttachment(),
            resourceArray, obj.getSubject(), obj.getComments());

        if (saveResult == null)
        {
            Assert.fail("Result is null");
        }

        // Check that we received the right exceptions
        if (saveResult.getErrors() == null)
        {
            Assert.fail(StringUtil.getShortClassName(obj.getClass())
                    + " validation failed: expected exceptions");
        }

        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, saveResult);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}
