package com.ibm.rpm.client.junit.customfield.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceModuleFixture;
import com.ibm.rpm.customfield.constants.ValidationConstants;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.resource.scope.ResourceModuleScope;

public class TestCustomFieldValidation extends GenericValidation
{
    private CustomFieldScope parentScope;
    
    private CustomFieldScope mainScope;

    private CustomFieldCategory parentCategory;
    
    private CustomField mainCustomField;
    
    private CustomField mainCustomField2;

    protected void setUp() throws Exception
    {
        super.setUp();

        //Scope
        parentScope = new CustomFieldScope();
        mainScope = new CustomFieldScope();
        parentScope.setChildren(mainScope);
        mainScope.setParent(parentScope);
        
        //Using a Module object to create the CustomFieldCategory
        ResourceModuleScope moduleScope = new ResourceModuleScope();              
        moduleScope.setCustomFieldCategories(new CustomFieldScope());        
        ResourceModule module = ResourceModuleFixture.createFixture(moduleScope, this);        
        module = (ResourceModule) save(module, moduleScope).getRpmObject();        
        parentCategory = module.getCustomFieldCategories()[0];
        
        //Create a new customField and set the parent
        mainCustomField = module.getCustomFieldCategories()[0].getChildren()[0];
        mainCustomField.setParent(parentCategory);
        
        // Create a new customField of currency type and set the parent
        mainCustomField2 = CustomFieldFixture.createCustomField(parentCategory, FieldInputType.Currency);
        parentCategory.setChildren(new CustomField [] { mainCustomField, mainCustomField2 });
    }

    protected void tearDown() throws Exception
    {
        if(parentCategory != null){
            delete(parentCategory);
        }
        
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws Exception
    {
        DatafieldCategory wrongParent = new DatafieldCategory();
        mainCustomField.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainCustomField
                    .getClass().getName(), ValidationConstants.PARENT_FIELD, 
                    mainCustomField.getID())
        };

        verifySaveMustFail(mainCustomField, mainScope, expectedExceptions);
    }

    public void testSuccessValidationMax() throws Exception
    {
        // Set field with maximum values.
        mainCustomField
                .setName(getTestString(ValidationConstants.CUSTOMFIELD_NAME_MAX));
        mainCustomField
                .setDefaultValue(getTestString(ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_TEXT_MAX));

        mainCustomField2
        	.setDefaultValue(new Double(ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_CURRENCY_MAX));

        
        doTestSuccessfulSave(mainCustomField);
    }

    public void testSuccessValidationNull() throws Exception
    {
        // Set field that accept null values.
        mainCustomField.setName(null);
        mainCustomField.setDatafieldCategory(null);
        mainCustomField.setDefaultValue(null);

        doTestSuccessfulSave(mainCustomField);
    }

    public void testFailureValidationMax() throws Exception
    {
        // Set field with maximum values + 1.
        mainCustomField
                .setName(getTestString(ValidationConstants.CUSTOMFIELD_NAME_MAX + 1));
        mainCustomField
                .setDefaultValue(getTestString(ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_TEXT_MAX + 1));
        mainCustomField2
         		.setDefaultValue(new Double(ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_CURRENCY_MAX * 10));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, mainCustomField
                        .getClass().getName(), "name", mainCustomField.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainCustomField
                        .getClass().getName(), ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD, mainCustomField
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainCustomField2
                    .getClass().getName(), ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD, mainCustomField2
                    .getID())
        };

        verifySaveMustFail(parentCategory, parentScope, expectedExceptions);
    }

    public void testFailureWithoutMandatoryFields() throws Exception
    {
        mainCustomField.setInputType(null);
        mainCustomField.setSecurityGroup(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY, mainCustomField
                        .getClass().getName(), ValidationConstants.CUSTOMFIELD_INPUTTYPE_FIELD, 
                        mainCustomField.getID())
        };

        verifySaveMustFail(mainCustomField, mainScope, expectedExceptions);
    }
    
    public void testFailureWithoutDatafieldTypeMandatoryFields() throws Exception
    {
        mainCustomField.setInputType(FieldInputType.Datafield);
        mainCustomField.setDatafieldCategory(null);
        mainCustomField.setDefaultValue(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY, mainCustomField
                    .getClass().getName(), ValidationConstants.DATAFIELDCATEGORY_FIELD, 
                    mainCustomField.getID())
        };

        verifySaveMustFail(mainCustomField, mainScope, expectedExceptions);
    }

    public void testFailureWithoutDatafieldTypeMandatoryID() throws Exception
    {
        mainCustomField.setInputType(FieldInputType.Datafield);
        mainCustomField.setDatafieldCategory(new DatafieldCategory());
        mainCustomField.setDefaultValue(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_ID, mainCustomField.getClass()
                    .getName(), ValidationConstants.DATAFIELDCATEGORY_FIELD, 
                    mainCustomField.getID())
        };

        verifySaveMustFail(mainCustomField, mainScope, expectedExceptions);
    }
    
    public void testFailureInvalidDefaultValue() throws Exception
    {
        mainCustomField.setInputType(FieldInputType.Datafield);
        DatafieldCategory datafieldc = new DatafieldCategory();
        datafieldc.setID("NOT_EXISTING_ID");
        mainCustomField.setDatafieldCategory(datafieldc);
        mainCustomField.setDefaultValue("String - Must be Datafield");

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.INVALID_TYPE, mainCustomField
                    .getClass().getName(), ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD, 
                    mainCustomField.getID()),
        };

        verifySaveMustFail(mainCustomField, mainScope, expectedExceptions);
    }
    
    public void testFailureWithoutParent() throws Exception
    {
        mainCustomField.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainCustomField.getClass()
                    .getName(), ValidationConstants.PARENT_FIELD, mainCustomField.getID()),
        };

        verifySaveMustFail(mainCustomField, mainScope, expectedExceptions);
    }

    public void testFailureWithoutParentID() throws Exception
    {
        CustomFieldCategory parent = new CustomFieldCategory();
        mainCustomField.setParent(parent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT_ID, mainCustomField
                    .getClass().getName(), ValidationConstants.PARENT_FIELD, mainCustomField.getID()),
        };

        verifySaveMustFail(mainCustomField, mainScope, expectedExceptions);
    }


    private void doTestSuccessfulSave(CustomField customField) throws Exception
    {
        // Save.
        CustomFieldCategory parentCustomField = (CustomFieldCategory) customField
                .getParent();
        CustomFieldScope parentScope = (CustomFieldScope) mainScope.getParent();
        parentCustomField = (CustomFieldCategory) saveMustSucceed(
                parentCustomField, CustomFieldCategory.class, parentScope);

        // Reload.
        CustomFieldCategory reloadedParent = (CustomFieldCategory) loadFromID(
                parentCustomField, parentScope);

        // Assert reloaded is equal to original.
        CustomFieldFixture.assertEquals(parentCustomField, reloadedParent,
                parentScope);
    }
}