/*
 * Created on 26-Oct-06
 * @author mmatwiy
 * 
 * Test cases for defect 56932. 
 * Adding a timephased financial to an existing list of timephased containers 
 * doesn't work (the new container is not saved).
 */
package com.ibm.rpm.client.junit.financial.testcases.fixes;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.util.FinancialUtil;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test56932 extends ClientTestController
{
    private WorksheetFinancial worksheetFinancial = null;

    private Project project = null;

    private WorkElementScope workElementScope = null;

    private FinancialScope finScope = null;

    public void setUp() throws Exception
    {
        super.setUp();

        finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setParent(new WorkElementScope());

        workElementScope = new WorkElementScope();
        workElementScope.setWorksheetFinancials(finScope);

        // create a parent Project and WorksheetFinancial
        project = new Project();
        project.setName(HelperTestUtilities.createName());
        save(project, workElementScope);

        worksheetFinancial = new WorksheetFinancial();
        worksheetFinancial.setParent(project);

        project.setWorksheetFinancials(new WorksheetFinancial[]
        {
            worksheetFinancial
        });

        // create and set a charge code
        worksheetFinancial.setChargeCode(ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this));

    }

    public void testAddEstimateTimePhased() throws Exception
    {
        finScope.setEstimateTimePhased(true);
        
        // create estimates for 2005
        TimePhasedFinancial estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(2005));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);
        setTimephasedValues(estimateTimePhased05);

        // create estimates for 2006
        TimePhasedFinancial estimateTimePhased06 = new TimePhasedFinancial();
        estimateTimePhased06.setParent(worksheetFinancial);
        estimateTimePhased06.setYear(new Integer(2006));
        estimateTimePhased06.setBudgetType(BudgetType.Estimate);
        setTimephasedValues(estimateTimePhased06);

        worksheetFinancial.setEstimateTimePhased(new TimePhasedFinancial[]
        {
                estimateTimePhased05, estimateTimePhased06
        });

        // save the worksheet with 2 timephased values
        worksheetFinancial = (WorksheetFinancial) save(worksheetFinancial,
            finScope).getRpmObject();

        // create timephased estimates for 2007
        TimePhasedFinancial estimateTimePhased07 = new TimePhasedFinancial();
        estimateTimePhased07.setParent(worksheetFinancial);
        estimateTimePhased07.setYear(new Integer(2007));
        estimateTimePhased07.setBudgetType(BudgetType.Estimate);
        setTimephasedValues(estimateTimePhased07);

        List existingList = new ArrayList(Arrays.asList(worksheetFinancial
            .getEstimateTimePhased()));

        // add the new timephased to the list of existing ones
        existingList.add(estimateTimePhased07);
        worksheetFinancial
            .setEstimateTimePhased((TimePhasedFinancial[]) existingList
                .toArray(new TimePhasedFinancial[0]));

        // save them all together (2 existing, 1 new)
        save(worksheetFinancial, finScope);

        // check that the new one was saved properly
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        
        FinancialUtil.sortTimePhasedByYear(reloaded.getEstimateTimePhased());
        FinancialUtil.sortTimePhasedByYear(worksheetFinancial.getEstimateTimePhased());

        FinancialFixture.compareTimePhasedArray(worksheetFinancial
            .getEstimateTimePhased(), reloaded.getEstimateTimePhased());
        assertEquals(reloaded.getEstimateTimePhased().length, 3);
    }

    public void testAddActualTimePhased() throws Exception
    {
        finScope.setActualTimePhased(true);
        
        // create actuals for 2005
        TimePhasedFinancial actualTimePhased05 = new TimePhasedFinancial();
        actualTimePhased05.setParent(worksheetFinancial);
        actualTimePhased05.setYear(new Integer(2005));
        actualTimePhased05.setBudgetType(BudgetType.Actual);
        setTimephasedValues(actualTimePhased05);

        // create actuals for 2006
        TimePhasedFinancial actualTimePhased06 = new TimePhasedFinancial();
        actualTimePhased06.setParent(worksheetFinancial);
        actualTimePhased06.setYear(new Integer(2006));
        actualTimePhased06.setBudgetType(BudgetType.Actual);
        setTimephasedValues(actualTimePhased06);

        worksheetFinancial.setActualTimePhased(new TimePhasedFinancial[]
        {
                actualTimePhased05, actualTimePhased06
        });

        // save the worksheet with 2 timephased values
        worksheetFinancial = (WorksheetFinancial) save(worksheetFinancial,
            finScope).getRpmObject();

        // create actuals for 2008
        TimePhasedFinancial actualTimePhased08 = new TimePhasedFinancial();
        actualTimePhased08.setParent(worksheetFinancial);
        actualTimePhased08.setYear(new Integer(2008));
        actualTimePhased08.setBudgetType(BudgetType.Actual);
        setTimephasedValues(actualTimePhased08);

        List existingList = new ArrayList(Arrays.asList(worksheetFinancial
            .getActualTimePhased()));

        // add the new timephased to the list of existing ones
        existingList.add(actualTimePhased08);
        worksheetFinancial
            .setActualTimePhased((TimePhasedFinancial[]) existingList
                .toArray(new TimePhasedFinancial[0]));

        // save them all together (2 existing, 1 new)
        save(worksheetFinancial, finScope);

        // check that the new one was saved properly
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        
        FinancialUtil.sortTimePhasedByYear(reloaded.getActualTimePhased());
        FinancialUtil.sortTimePhasedByYear(worksheetFinancial.getActualTimePhased());

        FinancialFixture.compareTimePhasedArray(worksheetFinancial
            .getActualTimePhased(), reloaded.getActualTimePhased());
        assertEquals(reloaded.getActualTimePhased().length, 3);
    }

    private void setTimephasedValues(TimePhasedFinancial timephased)
    {
        Double value = new Double(5);

        timephased.setMonth01Value(value);
        timephased.setMonth02Value(value);
        timephased.setMonth03Value(value);
        timephased.setMonth04Value(value);
        timephased.setMonth05Value(value);
        timephased.setMonth06Value(value);
        timephased.setMonth07Value(value);
        timephased.setMonth08Value(value);
        timephased.setMonth09Value(value);
        timephased.setMonth10Value(value);
        timephased.setMonth11Value(value);
        timephased.setMonth12Value(value);
    }
}
