package com.ibm.rpm.client.junit.financial.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.ChargeCodeState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.GenericChargeCode;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author legaults
 */
public class TestChargeCodeValidation extends GenericValidation
{
    private ChargeCodeScope mainScope;

    private ChargeCode mainChargeCode;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = ChargeCodeFixture.createSimpleScope();
        mainScope.setState(true);
        mainScope.setAssociatedChargeCode(new ChargeCodeScope());
        
        mainChargeCode = ChargeCodeFixture.createSimpleChargeCode(false);
        saveParent();
    }

    protected void tearDown() throws Exception
    {
        // Ensure the saved objects will be deleted.
        ChargeCodeFixture.deleteAllChargeCodes(this);
        ChargeCodeFixture.assertAllDeleted(this);
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws Exception
    {
        ChargeCode wrongParent = ChargeCodeFixture
                .createAndSaveSimpleChargeCode(this);
        mainChargeCode.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainChargeCode
                    .getClass().getName(), "parent", mainChargeCode.getID())
        };

        verifySaveMustFail(mainChargeCode, mainScope, expectedExceptions);
    }
    
    
    public void testDeleteAssignedCustomChargeCode() throws Exception
    {
        // Defect 59816
        // should not be able to delete a chargecode that is assigned.

        WorkElementScope wScope = new WorkElementScope();
        wScope.setWorksheetFinancials(new FinancialScope());
        wScope.getWorksheetFinancials().setChargeCode(true);

        // Creating PortFolio and Proposal
        SaveResult projectResult = null;

        Proposal wMainProposal = new Proposal();
        wMainProposal.setName(HelperTestUtilities.createName());

        // Saving Proposal according to scope
        projectResult = save(wMainProposal, wScope);
        assertNotNull(projectResult);

        assertTrue(projectResult.getRpmObject() instanceof Proposal);
        wMainProposal = (Proposal) projectResult.getRpmObject();

        wMainProposal.setAssignmentType(AssignmentType.Work);
        wMainProposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial wf = null;

        // adding non-system worksheetFinancial
        wf = new WorksheetFinancial();
        wf.setParent(wMainProposal);

        // create and set a charge code for parent
        ChargeCode CustomCC = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        wf.setChargeCode(CustomCC);

        wMainProposal.setWorksheetFinancials(new WorksheetFinancial[]
        {
            wf
        });

        // saving parent with financial
        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        Proposal parent = (Proposal) result.getRpmObject();

        assertNotNull(parent.getWorksheetFinancials()[0].getChargeCode());

        CustomCC = parent.getWorksheetFinancials()[0].getChargeCode();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_DELETE, CustomCC.getClass()
                .getName())
        };

        verifyDeleteMustFail(CustomCC, expectedExceptions, null);
    }
    
    public void testModifySystemChargeCode() throws Exception
    {
        // Defect 47407
        ChargeCodeScope ccScope = new ChargeCodeScope();

        RPMObject[] loadResult = loadFromXpath(
            "/ChargeCodeCategory[name='System Charge Codes']", ccScope);
        ChargeCodeCategory ccCategory = (ChargeCodeCategory) loadResult[0];
        ccCategory.setName("ERROR !!!");
        ccCategory.setDescription("ERROR !!!");
        ccCategory.setExternalID("ERROR !!!");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, ccCategory
                .getClass().getName())
        };

        verifySaveMustFailOneWay(ccCategory, ccScope, expectedExceptions);
    }
    
    public void testCreateSystemChargeCode() throws Exception
    {
        ChargeCodeScope ccScope = new ChargeCodeScope();
        ChargeCodeCategory ccc = new ChargeCodeCategory();
        ccc.setName("System Charge Codes");
        ccc.setID("SYSTEM_CHARGE_CODES_FOLDER      ");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, ccc.getClass()
                .getName())
        };

        verifySaveMustFailOneWay(ccc, ccScope, expectedExceptions);
    }
    
    public void testDeleteSystemChargeCode() throws Exception
    {
        ChargeCodeScope ccScope = new ChargeCodeScope();

        RPMObject[] loadResult = loadFromXpath(
            "/ChargeCodeCategory[name='System Charge Codes']", ccScope);
        ChargeCodeCategory ccCategory = (ChargeCodeCategory) loadResult[0];

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_DELETE, ccCategory
                .getClass().getName())
        };

        verifyDeleteMustFail(ccCategory, expectedExceptions, null);
    }


    public void testSuccessNestingLevelMax() throws Exception
    {
        ChargeCodeScope testScope = ChargeCodeFixture.createChargeCodeTreeScope(ValidationConstants.CHARGE_CODE_NESTINGLEVEL_MAX);
        ChargeCodeCategory testChargeCode = (ChargeCodeCategory) ChargeCodeFixture.createGenericChargeCode(ValidationConstants.CHARGE_CODE_NESTINGLEVEL_MAX, 1, null, false);
        
        testChargeCode = (ChargeCodeCategory) saveMustSucceed(testChargeCode,
            ChargeCodeCategory.class, testScope);
        
        ChargeCodeFixture.makeDeletable(this, testChargeCode);
    }
    
    public void testFailureNestingLevelSavedParentMax() throws Exception
    {
        ChargeCodeScope testScope = ChargeCodeFixture.createChargeCodeTreeScope(ValidationConstants.CHARGE_CODE_NESTINGLEVEL_MAX);
        ChargeCodeCategory testChargeCode = (ChargeCodeCategory) ChargeCodeFixture
        	.createGenericChargeCode(ValidationConstants.CHARGE_CODE_NESTINGLEVEL_MAX+1, 1, null, false);
                 
        testChargeCode = (ChargeCodeCategory) saveMustSucceed(testChargeCode,
            ChargeCodeCategory.class, testScope);
        
        ChargeCodeCategory parent = testChargeCode;
        for (int i = 0; i < ValidationConstants.CHARGE_CODE_NESTINGLEVEL_MAX-1; ++i)
            parent = (ChargeCodeCategory) parent.getChildren()[0];
            
        mainChargeCode.setParent(parent);
                
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.EXCEEDS_MAX_NESTING_LEVEL, mainChargeCode
                    .getClass().getName(), null, mainChargeCode.getID())
        };

        verifySaveMustFail(mainChargeCode, mainScope, expectedExceptions);
        
        ChargeCodeFixture.makeDeletable(this, testChargeCode);  
    }
    
    public void testFailureNestingLevelUnSavedParentMax() throws Exception
    {
        ChargeCodeScope testScope = ChargeCodeFixture.createChargeCodeTreeScope(ValidationConstants.CHARGE_CODE_NESTINGLEVEL_MAX+1);
        ChargeCodeCategory testChargeCode = (ChargeCodeCategory) ChargeCodeFixture.createGenericChargeCode(ValidationConstants.CHARGE_CODE_NESTINGLEVEL_MAX+1, 1, null, false);
                       
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.EXCEEDS_MAX_NESTING_LEVEL, mainChargeCode
                    .getClass().getName(), null, mainChargeCode.getID())
        };

        verifySaveMustFail(testChargeCode, testScope, expectedExceptions);
        
        ChargeCodeFixture.makeDeletable(this, testChargeCode);  
    }
    
    public void testSuccessValidationMax() throws Exception
    {
        // Set field with maximum values.
        mainChargeCode
                .setName(getTestString(ValidationConstants.CHARGE_CODE_NAME_MAX));
        mainChargeCode
                .setDescription(getTestString(ValidationConstants.CHARGE_CODE_DESCRIPTION_MAX));
        mainChargeCode
                .setExternalID(getTestString(ValidationConstants.CHARGE_CODE_EXTERNAL_ID_MAX));
        mainChargeCode.setFinancialCategory(FinancialCategory.Expense);

        doTestSuccessfulSave();
        ChargeCodeFixture.makeDeletable(this, mainChargeCode);
    }

    public void testSuccessValidationNull() throws Exception
    {
        // Set field with null values.
        mainChargeCode.setDescription(null);
        mainChargeCode.setExternalID(null);
        mainChargeCode.setFinancialCategory(FinancialCategory.Benefit);

        doTestSuccessfulSave();
        ChargeCodeFixture.makeDeletable(this, mainChargeCode);
    }

    public void testFailureValidation() throws Exception
    {
        // Set field with invalid values.
        mainChargeCode
                .setName(getTestString(ValidationConstants.CHARGE_CODE_NAME_MAX + 1));
        mainChargeCode
                .setDescription(getTestString(ValidationConstants.CHARGE_CODE_DESCRIPTION_MAX + 1));
        mainChargeCode
                .setExternalID(getTestString(ValidationConstants.CHARGE_CODE_EXTERNAL_ID_MAX + 1));
        mainChargeCode
                .setFinancialCategory(FinancialCategory.CapitalAverageCost);
        ChargeCodeState state = new ChargeCodeState();
        state.setID(DatafieldFixture.LANGUAGE_FRENCH);
        mainChargeCode.setState(state);

        String mainId = mainChargeCode.getID();
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH,
                        CHARGE_CODE_CLASS_NAME, "name", mainId),
                createRPMException(ErrorCodes.MAX_LENGTH,
                        CHARGE_CODE_CLASS_NAME, "description", mainId),
                createRPMException(ErrorCodes.MAX_LENGTH,
                        CHARGE_CODE_CLASS_NAME, "externalID", mainId),
                createRPMException(
                        com.ibm.rpm.financial.constants.ErrorCodes.SYSTEM_FINANCIAL_CATEGORY,
                        CHARGE_CODE_CLASS_NAME, "financialCategory", mainId),
                createRPMException(
                        com.ibm.rpm.customfield.constants.ErrorCodes.DATAFIELD_NOT_IN_CATEGORY,
                        CHARGE_CODE_CLASS_NAME, "state", mainId),
        };

        verifySaveMustFail(mainChargeCode, mainScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        mainChargeCode.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, mainChargeCode
                .getClass().getName(), "name", mainChargeCode.getID())
        };

        verifySaveMustFail(mainChargeCode, null, expectedExceptions);
    }

    public void testFailureAssociatedChargeCode() throws Exception
    {
        ChargeCodeCategory root = ChargeCodeFixture
                .createAndSaveAssociatedChargeCodes(this);
        ChargeCode cc0 = (ChargeCode) root.getChildren()[0];
        cc0.setFinancialCategory(FinancialCategory.Contingency);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_CATEGORY_MISMATCH,
                    CHARGE_CODE_CLASS_NAME, "associatedChargeCode", cc0.getID()),
        };

        ChargeCodeScope scope = ChargeCodeFixture
                .createUniversalChargeCodeScope();
        verifySaveMustFail(cc0, scope, expectedExceptions);
    }

        public void testFailureSystemChargeCode() throws Exception
        {
            RPMObject[] objects = loadFromXpath(
                    "/ChargeCodeCategory[starts-with(Name, 'System Charge Codes')]",
                    null);
            GenericChargeCode systemChargeCode = (GenericChargeCode) objects[0];
    
            RPMExceptionMock[] expectedExceptions =
            {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, systemChargeCode.getClass()
                    .getName())
            };

            verifySaveMustFailOneWay(systemChargeCode, null, expectedExceptions);

        }

    public void testFailureWithoutMandatoryFields() throws Exception
    {
        mainChargeCode.setFinancialCategory(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, CHARGE_CODE_CLASS_NAME,
                    "financialCategory", mainChargeCode.getID()),
        };

        verifySaveMustFail(mainChargeCode, mainScope, expectedExceptions);
    }

    public void testFailureWithoutParent() throws Exception
    {
        mainChargeCode.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, CHARGE_CODE_CLASS_NAME,
                    "parent", mainChargeCode.getID()),
        };

        verifySaveMustFail(mainChargeCode, mainScope, expectedExceptions);
    }

    public void testFailureWithoutParentID() throws Exception
    {
        mainChargeCode.getParent().setID(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT_ID, CHARGE_CODE_CLASS_NAME,
                    "parent", mainChargeCode.getID()),
        };

        verifySaveMustFail(mainChargeCode, mainScope, expectedExceptions);
    }

    public void testDeleteAll() throws Exception
    {
        // Done in the tearDown.
    }

    private void doTestSuccessfulSave() throws Exception
    {
        // Save.
        ChargeCode code = (ChargeCode) saveMustSucceed(mainChargeCode,
                ChargeCode.class, mainScope);

        // Reload.
        ChargeCode reloaded = (ChargeCode) loadFromID(mainChargeCode, mainScope);

        // Assert reloaded is equal to original.
        ChargeCodeFixture.assertEquals(code, reloaded, mainScope);
    }

    private void saveParent() throws Exception
    {
        ChargeCodeCategory parentCode = (ChargeCodeCategory) mainChargeCode
                .getParent();
        parentCode = (ChargeCodeCategory) saveMustSucceed(parentCode,
                ChargeCodeCategory.class, null);
    }

    private static final String CHARGE_CODE_CLASS_NAME = ChargeCode.class
            .getName();
}
