/*
 * Created on 6-Oct-2005
 */
package com.ibm.rpm.client.junit.framework.testcases;

import java.util.HashMap;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class GenericValidation extends ClientTestController
{
    protected static HashMap testStrings = new HashMap();

    protected void setUp() throws Exception
    {
        super.setUp();
    }

    public static String getTestString(int length)
    {
        Object testObj = testStrings.get(new Integer(length));
        String testString;

        if (testObj != null)
        {
            testString = testObj.toString();
        }
        else
        {
            testString = createFixedLengthStringAlpha(length);
            testStrings.put(new Integer(length), testString);
        }

        return testString;
    }
    
    public static String getTestStringNonAlpha(int length)
    {
        Object testObj = testStrings.get(new Integer(length));
        String testString;

        if (testObj != null)
        {
            testString = testObj.toString();
        }
        else
        {
            testString = createFixedLengthString(length);
            testStrings.put(new Integer(length), testString);
        }

        return testString;
    }

    // Create a fixed length string with alpha characters only
    public static String createFixedLengthStringAlpha(int length)
    {
        StringBuffer fixedLength = new StringBuffer(length);
        for (int i = 0; i < length; ++i)
            fixedLength.append((char) (65 + (i % 26)));
        return new String(fixedLength);
    }

    // Create a fixed length string with special characters
    public static String createFixedLengthString(int length)
    {
        StringBuffer fixedLength = new StringBuffer(length);

        // Indexes 0 - 4 contain extended characters: 
        int i;
        for (i = 0; i < 5 && i < length; ++i)
            fixedLength.append((char) (162 + i));

        // Indexes 5 - 9 contain French extended characters: 
        for (; i < 10 && i < length; ++i)
            fixedLength.append((char) (226 + i));

        // Indexes 10 - length contain regular characters
        for (; i < length; ++i)
            fixedLength.append((char) (32 + (i % 85)));

        return new String(fixedLength);
    }

    protected RPMExceptionMock createRPMException(int errorID, String className)
    {
        return createRPMException(errorID, className, null, null);
    }

    protected RPMExceptionMock createRPMException(int errorID,
            String className, String fieldName, String containerID)
    {
        return new RPMExceptionMock(errorID, className, fieldName, containerID);
    }

    protected RPMExceptionMock createRPMException(int errorID,
            String className, String fieldName, String secondaryFieldName,
            String containerID)
    {
        return new RPMExceptionMock(errorID, className, fieldName,
                secondaryFieldName, containerID);
    }

    /**
     * Generic save function for saves that must succeed
     */
    protected RPMObject saveMustSucceed(RPMObject obj, Class objType,
            RPMObjectScope scope) throws AssertionError, Exception
    {
        assertNotNull(obj);
        assertTrue(objType.isInstance(obj));

        SingleResult result = save(obj, scope);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(objType.isInstance(result.getRpmObject()));
        
        if( ! result.isSuccessful())
        {
            RPMException[] errors = result.getErrors();
            String errorMessage = "detected " + errors.length + " errors:";
            for (int i = 0; i < errors.length; i++)
            {
                errorMessage += "\n\t" + i + " : " + errors[i].getMessage();
            }
            fail(errorMessage);
        }
        
        if( ! result.isWarningFree())
        {
            RPMException[] errors = result.getWarnings();
            String errorMessage = "detected " + errors.length + " warnings:";
            for (int i = 0; i < errors.length; i++)
            {
                errorMessage += "\n\t" + i + " : " + errors[i].getMessage();
            }
            fail(errorMessage);
        }

        obj = result.getRpmObject();
        return obj;
    }
    
    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed save
     */
    protected void verifySaveMustFail(RPMObject obj, RPMObjectScope scope,
            RPMExceptionMock[] expectedExceptions) throws AssertionError,
            Exception
    {
        // Attempt to save the object
        SingleResult saveResult = saveMustFail(obj, scope);

        // Check that we received the right exceptions
        if (saveResult.getErrors() == null)
        {
            Assert.fail(StringUtil.getShortClassName(obj.getClass())
                    + " validation failed: expected exceptions");
        }

        String errorMessage = verifyExceptions(expectedExceptions, saveResult
                .getErrors());

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }

    /**
     * Generic function to verify that the expected exceptions and warnings are
     * received from the failed save
     */
    protected void verifySaveMustFail(RPMObject obj, RPMObjectScope scope,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings) throws AssertionError,
            Exception
    {
        // Attempt to save the object
        SingleResult saveResult = saveMustFail(obj, scope);
        
        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
                expectedWarnings, saveResult);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }

    /**
     * This method verifies that all the expected exceptions were received, 
     * yet it ignores those which weren't expected. Used as a work-around for DatafieldManager 
     * which throws Exceptions to the user of the API which are in fact destined only for the programmer.
     */
    protected void verifySaveMustFailOneWay(RPMObject obj, RPMObjectScope scope,
            RPMExceptionMock[] expectedExceptions) throws AssertionError, Exception
    {
        // Attempt to save the object
        SingleResult saveResult = saveMustFail(obj, scope);

        // Check that we received the right exceptions
        if (saveResult.getErrors() == null)
        {
            Assert.fail(StringUtil.getShortClassName(obj.getClass())
                    + " validation failed: expected exceptions");
        }

        RPMException[] thrownExceptions = saveResult.getErrors();
        
        StringBuffer loggingMessage = new StringBuffer();
        StringBuffer errorMessage = new StringBuffer();
       
        for (int i = 0; i < expectedExceptions.length; i++)
        {
            RPMExceptionMock expectedException = expectedExceptions[i];

            for (int j = 0; j < thrownExceptions.length; j++)
            {
                RPMException thrownException = thrownExceptions[j];
                if (equalExceptions(expectedException, thrownException))
                {
                    loggingMessage.append("\nReceived expected "
                            + getSeverityName(thrownException) + ": "
                            + thrownException.toString());
                    thrownExceptions[j] = null;
                    break;
                }
            }
        }
        
        loggingMessage.append(errorMessage);
        if (!StringUtil.isEmpty(loggingMessage))
        {
            logger.debug(loggingMessage);
        }
        
        if (errorMessage.length() != 0)
            Assert.fail(errorMessage.toString());
    }
    
    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed deletion
     */
    protected void verifyDeleteMustFail(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings) throws AssertionError,
            Exception
    {
        // Attempt to save the object
        Result deleteResult = deleteMustFail(obj);

        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
                expectedWarnings, deleteResult);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }

    protected String checkAndLogExceptions(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings, Result result)
    {
        String errorMessage = "";
        if (expectedExceptions != null)
        {
            if (result.getErrors() == null)
            {
                errorMessage += ("\n"
                        + StringUtil.getShortClassName(obj.getClass()) + " validation failed: expected exceptions");
            }
            else
            {
                errorMessage += verifyExceptions(expectedExceptions, result
                        .getErrors());

            }
        }
        else if (result.getErrors() != null)
        {
            for (int i = 0; i < result.getErrors().length; i++)
            {
                errorMessage += ("\n" + result.getErrors()[i].toString());
            }
        }

        // Check that we received warnings if we were expecting some
        // and that they are the correct ones
        if (expectedWarnings != null)
        {
            if (result.getWarnings() == null)
            {
                errorMessage += ("\n"
                        + StringUtil.getShortClassName(obj.getClass()) + " validation failed: expected warnings");
            }
            else
            {
                errorMessage += errorMessage = verifyExceptions(
                        expectedWarnings, result.getWarnings());
            }
        }
        else if (result.getWarnings() != null)
        {
            for (int i = 0; i < result.getWarnings().length; i++)
            {
                errorMessage += ("\n" + result.getWarnings()[i].toString());
            }
        }

        return errorMessage;

    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed loadFromID. Calls Assert.fail if the exception is not
     * received.
     */
    protected void verifyLoadFromIDMustFail(RPMObject obj,
            RPMObjectScope scope, RPMExceptionMock expectedException)
            throws AssertionError, Exception
    {
        verifyLoadFromIDMustFail(obj, scope, expectedException, true);
    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed loadFromID
     */
    protected String verifyLoadFromIDMustFail(RPMObject obj,
            RPMObjectScope scope, RPMExceptionMock expectedException,
            boolean failIfNotFound) throws AssertionError, Exception
    {
        try
        {
            loadFromID(obj, scope);
        }
        catch (RPMException e)
        {
            // Check that we received the right exception
            String errorMessage = verifyExceptions(new RPMExceptionMock[]
            {
                expectedException
            }, new RPMException[]
            {
                e
            });

            if (failIfNotFound && errorMessage.length() != 0)
                Assert.fail(errorMessage);

            return "";
        }

        if (failIfNotFound)
        {
            Assert.fail("The following container is not validated correctly:\n"
                    + "   - "
                    + StringUtil.getShortClassName(expectedException
                            .getClassName()) + "\n");
        }

        return "- "
                + StringUtil
                        .getShortClassName(expectedException.getClassName())
                + "\n";
    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed loadArrayFromID. Calls Assert.fail if the exception is not
     * received.
     */
    protected void verifyLoadArrayFromIDMustFail(RPMObject[] obj,
            RPMObjectScope scope, RPMExceptionMock expectedException)
            throws AssertionError, Exception
    {
        verifyLoadArrayFromIDMustFail(obj, scope, expectedException, true);
    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed loadArrayFromID
     */
    protected String verifyLoadArrayFromIDMustFail(RPMObject[] obj,
            RPMObjectScope scope, RPMExceptionMock expectedException,
            boolean failIfNotFound) throws AssertionError, Exception
    {
        try
        {
            loadArrayFromID(obj, scope);
        }
        catch (RPMException e)
        {
            // Check that we received the right exception
            String errorMessage = verifyExceptions(new RPMExceptionMock[]
            {
                expectedException
            }, new RPMException[]
            {
                e
            });

            if (failIfNotFound && errorMessage.length() != 0)
                Assert.fail(errorMessage);

            return "";
        }

        if (failIfNotFound)
        {
            Assert.fail("The following container is not validated correctly:\n"
                    + "   - "
                    + StringUtil.getShortClassName(expectedException
                            .getClassName()) + "\n");
        }

        return "- "
                + StringUtil
                        .getShortClassName(expectedException.getClassName())
                + "\n";
    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed loadFromXpath
     */
    protected void verifyLoadFromXpathMustFail(String Xpath,
            RPMObjectScope scope, RPMExceptionMock expectedException)
            throws AssertionError, Exception
    {
        try
        {
            loadFromXpath(Xpath, scope);
        }
        catch (RPMException e)
        {
            // Check that we received the right exception
            String errorMessage = verifyExceptions(new RPMExceptionMock[]
            {
                expectedException
            }, new RPMException[]
            {
                e
            });

            if (errorMessage.length() != 0)
                Assert.fail(errorMessage);

            return;
        }

        Assert.fail("The following container is not validated correctly:\n"
                + "   - "
                + StringUtil
                        .getShortClassName(expectedException.getClassName())
                + "\n");
    }

    /**
     * Verifies that all expected exceptions and warnings were thrown.
     * 
     * @param expectedExceptions
     * @param thrownExceptions
     * @return A string containing the expected exceptions that were not
     *         included in the thrown exceptions.
     */
    protected String verifyExceptions(RPMExceptionMock[] expectedExceptions,
            RPMException[] thrownExceptions)
    {
        StringBuffer loggingMessage = new StringBuffer();
        StringBuffer errorMessage = new StringBuffer();
        boolean found;
        for (int i = 0; i < expectedExceptions.length; i++)
        {
            found = false;

            RPMExceptionMock expectedException = expectedExceptions[i];

            for (int j = 0; j < thrownExceptions.length; j++)
            {
                RPMException thrownException = thrownExceptions[j];
                if (equalExceptions(expectedException, thrownException))
                {
                    found = true;
                    loggingMessage.append("\nReceived expected "
                            + getSeverityName(thrownException) + ": "
                            + thrownException.toString());
                    thrownExceptions[j] = null;
                    break;
                }
            }
            if (!found)
            {
                errorMessage.append("\nDid not receive expected error/warning: ");
                errorMessage.append(expectedException.toString());
            }
        }

        for (int j = 0; j < thrownExceptions.length; j++)
        {
            RPMException thrownException = thrownExceptions[j];
            if (thrownException != null)
            {
                errorMessage.append("\nReceived unexpected "
                        + getSeverityName(thrownException) + ": ");
                errorMessage.append(thrownException.toString());
            }
        }

        loggingMessage.append(errorMessage);
        if (!StringUtil.isEmpty(loggingMessage))
        {
            logger.debug(loggingMessage);
        }

        return errorMessage.toString();
    }

    private String getSeverityName(RPMException x)
    {
        String severityName = x.getSeverity().getValue();
        severityName = severityName.toLowerCase();
        return severityName;
    }

    /**
     * Compares two instances of RPMExceptions.
     * 
     * @param e1
     * @param e2
     * @return true if e1 == e2
     */
    protected boolean equalExceptions(RPMExceptionMock e1, RPMException e2)
    {
        boolean equal = true;

        equal = equal && e2 != null;

        equal = equal && e1.getErrorID() == e2.getErrorID();

        equal = equal
                && ((e1.getClassName() == null && e2.getClassName() == null) || (e1
                        .getClassName() != null && e1.getClassName().equals(
                        e2.getClassName())));

        equal = equal
                && ((e1.getFieldName() == null && e2.getFieldName() == null) || (e1
                        .getFieldName() != null && e1.getFieldName().equals(
                        e2.getFieldName())));

        equal = equal
                && ((e1.getSecondaryFieldName() == null && e2
                        .getSecondaryFieldName() == null) || (e1
                        .getSecondaryFieldName() != null && e1
                        .getSecondaryFieldName().equals(
                                e2.getSecondaryFieldName())));

        equal = equal
                && ((e1.getContainerID() == null && e2.getContainerID() == null) || (e1
                        .getContainerID() != null && e1.getContainerID()
                        .equals(e2.getContainerID())));

        return equal;
    }
    

    protected void verifyFailure(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings, Result saveResult)
            throws AssertionError, Exception
    {
        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, saveResult);

        if (errorMessage.length() != 0)
        {
            Assert.fail(errorMessage);
        }
    }
    
    protected Resource getValidResource()
    {
        return createResourceNamed("ADMIN"); 
    }
    
    protected Resource getInvalidResource()
    {
        return createResourceNamed("NOT_EXISTING_RESOURCE%457");
    }
    
    private Resource createResourceNamed(String name)
    {
        Resource res = new Resource();
        
        res.setFullName(name);
        
        return res;
    }
}