package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.scope.SecurityGroupScope;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;

/**
 * @author gosselin
 */
public class TestResource extends ClientTestController
{
    protected Resource mainResource;
    protected Pool parentPool;
    protected ResourceScope mainScope;
    protected int nbChildren = 2;
    
    public void setUp() throws Exception
    {
        super.setUp();
        mainScope = new ResourceScope();
        
        //Parent pool is needed by all the tests
        parentPool = PoolFixture.createSaveFixture(this);
    }

    public void tearDown() throws Exception
    {

        super.tearDown();
    }

    public void testUpdateWithCurrency() throws Exception
    {
		ResourceScope scope = new ResourceScope();
		scope.setRates(true);
		scope.setCurrency(true);

        Resource original = ResourceFixture.createSaveFixture(scope, this);
        
        scope.setRates(false);
		scope.setCurrency(true);
        
		Resource reloaded = (Resource)loadFromXpath(
			"/Resource[ID='" + original.getID() + "']", scope )[0];
	
		Currency newCurrency = DatafieldFixture.createCurrency(DatafieldFixture.CURRENCY_EUR);
		reloaded.setCurrency(newCurrency);

		SingleResult result = save(reloaded, scope, ReloadType.ReloadResult);
		assertNotNull(result);
		Resource updated = (Resource) result.getRpmObject();
		assertNotNull(updated);
		removeObjectToDelete(updated);
		Currency updatedCurrency = updated.getCurrency();
		
		assertEquals("Expected equal currencies; update did not succeed.", newCurrency.getID(), updatedCurrency.getID());
		
		// Ensure that rates have not been modified by the update
		scope.setRates(true);
		scope.setCurrency(false);
		Resource reloaded2 = (Resource)loadFromXpath(
			"/Resource[ID='" + original.getID() + "']", scope )[0];
		
		ResourceFixture.compare(original, reloaded2, scope);			
    }
    
    public void testUpdateWithRates() throws Exception
    {
		ResourceScope scope = new ResourceScope();
		scope.setRates(true);
		scope.setCurrency(true);

        Resource original = ResourceFixture.createSaveFixture(scope, this);
        
		scope.setCurrency(false);
        
		Resource reloaded = (Resource)loadFromXpath(
			"/Resource[ID='" + original.getID() + "']", scope )[0];

		ResourceRates rates = reloaded.getRates();
		
		rates.setRegularCostPerHour(new Double(50.0));
		rates.setSellingRatePerHour(new Double(65.0));
		rates.setSpecialCostPerHour(new Double(3.0));
		rates.setSpecialSellingRatePerHour(new Double(60.0));
		reloaded.setRates(rates);

		SingleResult result = save(reloaded, scope, ReloadType.ReloadResult);
		assertNotNull(result);
		Resource updated = (Resource) result.getRpmObject();
		assertNotNull(updated);
		removeObjectToDelete(updated);
		ResourceRates updatedRates = updated.getRates();
		
		assertEquals(new Double(50.0), updatedRates.getRegularCostPerHour());
		assertEquals(new Double(65.0), updatedRates.getSellingRatePerHour());
		assertEquals(new Double(3.0), updatedRates.getSpecialCostPerHour());
		assertEquals(new Double(60.0), updatedRates.getSpecialSellingRatePerHour());
    }
    
    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {        
        mainResource = ResourceFixture.createFixture(parentPool,null, this);

        Resource saved = saveResource(mainResource, mainScope);
        mainResource.setID(saved.getID());
        
        loadResourceFromId(mainResource, null);        
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        mainResource = ResourceFixture.createFixture(parentPool,null, this);

        Resource saved = saveResource(mainResource, null);
        mainResource.setID(saved.getID());        
        
        loadFromXPath();
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        initWithFullScope();
        
        Resource saved = saveResource(mainResource, mainScope);
        mainResource.setID(saved.getID());
        
        loadResourceFromId(mainResource, mainScope);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainResource = ResourceFixture.createFixture(parentPool,null, this);
        Resource saved = saveResource(mainResource, null);        
        mainResource.setID(saved.getID());
        
        //By setting the ID to the mainResource ID, the updatedResource represent
        //the mainResource with all its properties updated.
        Resource updatedResource = ResourceFixture.createFixture(parentPool,null, this);
        updatedResource.setID(mainResource.getID());
        // Cannot clear password
        if (updatedResource.getPassword() == null)
        {
            updatedResource.setPassword("t");
        }
        
        Resource saved2 = saveResource(updatedResource,null);
        loadResourceFromId(saved2, mainScope);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        initWithFullScope();
        
        Resource saved = saveResource(mainResource, mainScope);
        mainResource.setID(saved.getID());
        
        //Update Resource
        Resource updatedResource = ResourceFixture.updateFixture(saved, nbChildren, this);
        updatedResource.setID(mainResource.getID());
        Resource saved2 = saveResource(updatedResource, mainScope);
        
        loadResourceFromId(saved2, mainScope);
    }

    protected Resource saveResource(Resource res, ResourceScope scope)
            throws AssertionError, Exception {
        
        //Parent needs to be in scope to be saved...
        if(scope == null){
            scope = new ResourceScope();
        }
        
        if(scope.getParent() == null){
            scope.setParent(new PoolScope());
        }
        //
        
        SingleResult result = save(res, scope);

        res = (Resource) result.getRpmObject();
        return res;
    }    
    
    protected void loadFromXPath() throws AssertionError, Exception
    {
        logger.info("loadFromXPath");
        //load from an xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[fullName='"
                + mainResource.getFullName() + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

//        if (mainScope != null && mainResource != null 
//                && rpmObjects[0] != null)
//        {
//            logger.info("Scoping" + mainScope.toString());
//            logger.info("========================");
//            logger.info(mainResource.toString());
//            logger.info("========================");
//            logger.info(((Resource)rpmObjects[0]).toString());
//        }


        //TODO create a new loadFromXPath with a normal compare
        RPMObject obj = HelperTestUtilities.findRPMObjectFromList(mainResource
                .getID(), rpmObjects);
        ResourceFixture.compareWithoutCanProperties(mainResource,
                (Resource)obj, mainScope);
    }
    
    protected void loadResourceFromId(Resource objectToCompare, ResourceScope scope)
            throws AssertionError, Exception {
        
        logger.info("loadResourceFromId");

        RPMObject obj = new Resource();
        obj.setID(objectToCompare.getID());
        RPMObject rpmObject = loadFromID(obj, scope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Resource);

//        if (scope != null) {
//            logger.info("Scoping" + scope.toString());
//        } else {
//            logger.info("Scoping is null");
//        }
//        logger.info("ObjectToCompare: " + objectToCompare);
//        logger.info("LoadedObject: " + rpmObject);

        //TODO create a new loadResourceFromId with a normal compare
        ResourceFixture.compareWithoutCanProperties(objectToCompare,
                (Resource) rpmObject, scope);
    }    
    
    private void initWithFullScope() throws Exception{
    
        //Create resource scope
        mainScope.setCalendar(new CalendarScope());
        mainScope.setCompetencyAttributeAssignments(true);
        mainScope
                .setContactGroupAssignments(true);
        mainScope.setResourceCostCenterAssignments(true);
        mainScope.setCurrency(true);
        mainScope.setEducations(true);
        mainScope.setEmployeeStatus(true);
        mainScope.setEmploymentHistories(true);
        mainScope.setExperience(true);
        mainScope
                .setExecutiveAttributes(true);
        mainScope
                .setGeographicalAssignment(true);
        mainScope.setIndustryExperiences(true);
        mainScope.setLanguageProficiencies(true);
        mainScope.setLocationPreferences(true);
        mainScope
                .setOrganizationalAssignment(true);
        mainScope.setPassport(true);
        mainScope.setParent(new PoolScope());
        mainScope.setProjectHistories(true);
        mainScope.setRates(true);
        mainScope.setSkillAttributeAssignments(true);
        mainScope.setSecurityGroup(new SecurityGroupScope());
        mainScope.setExecutiveSecurityRole(new SecurityRoleScope());
        mainScope.setWorkingStatus(true);
        mainScope.setWorkLocation(true);
        mainScope.setPersonalProject(new PersonalProjectScope());
        mainScope.setVerifiedBy(true);
        mainScope.setReVerifiedBy(true);
        
        //Create resource fixture
        mainResource = ResourceFixture.createFixture(parentPool,nbChildren,mainScope, this);
      
    }
}
