package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.LanguageProficiencyFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class TestLanguageProficiencyValidation extends GenericValidation
{
    private Resource mainResource;

    private LanguageProficiency mainLanguageProficiency;

    public void setUp() throws Exception
    {
        super.setUp();
        mainResource = ResourceFixture.createSaveFixture(this);
        mainLanguageProficiency = LanguageProficiencyFixture.createFixture(mainResource);
        mainResource.setLanguageProficiencies(new LanguageProficiency[]
        {
            mainLanguageProficiency
        });
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainLanguageProficiency.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainLanguageProficiency.getClass()
                .getName(), "parent", mainLanguageProficiency.getID())
        };

        verifySaveMustFail(mainLanguageProficiency, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainLanguageProficiency.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainLanguageProficiency
                .getClass().getName(), "parent", mainLanguageProficiency.getID())
        };

        verifySaveMustFail(mainLanguageProficiency, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainLanguageProficiency.setReadLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX));
        mainLanguageProficiency.setSpeakLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX));
        mainLanguageProficiency.setWriteLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX));

        // Attempt to save the LanguageProficiency object
        mainLanguageProficiency = (LanguageProficiency) saveMustSucceed(
            mainLanguageProficiency, LanguageProficiency.class, null);

        // Load the inserted language proficiency from xpath
        RPMObject[] rpmObjects = loadFromXpath("/LanguageProficiency[id='"
            + mainLanguageProficiency.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof LanguageProficiency);

        // Check that the loaded language proficiency is identical to the one we
        // requested to store
        LanguageProficiencyFixture.compare(mainLanguageProficiency,
            (LanguageProficiency) rpmObjects[0]);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainLanguageProficiency.setReadLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN));
        mainLanguageProficiency.setSpeakLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN));
        mainLanguageProficiency.setWriteLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN));

        // Attempt to save the LanguageProficiency object
        mainLanguageProficiency = (LanguageProficiency) saveMustSucceed(
            mainLanguageProficiency, LanguageProficiency.class, null);

        // Load the inserted language proficiency from xpath
        RPMObject[] rpmObjects = loadFromXpath("/LanguageProficiency[id='"
            + mainLanguageProficiency.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof LanguageProficiency);

        // Check that the loaded employment history is identical to the one we
        // requested to store
        LanguageProficiencyFixture.compare(mainLanguageProficiency,
            (LanguageProficiency) rpmObjects[0]);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        mainLanguageProficiency.setReadLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX + 1));
        mainLanguageProficiency.setSpeakLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX + 1));
        mainLanguageProficiency.setWriteLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLanguageProficiency.getClass()
                .getName(), "readLevel", mainLanguageProficiency.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLanguageProficiency.getClass()
                .getName(), "speakLevel", mainLanguageProficiency.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLanguageProficiency.getClass()
                .getName(), "writeLevel", mainLanguageProficiency.getID())
        };

        verifySaveMustFail(mainLanguageProficiency, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set field values with min-1 and verify that they are not accepted
        mainLanguageProficiency.setReadLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN - 1));
        mainLanguageProficiency.setSpeakLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN - 1));
        mainLanguageProficiency.setWriteLevel(new Integer(
            ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLanguageProficiency.getClass()
                .getName(), "readLevel", mainLanguageProficiency.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLanguageProficiency.getClass()
                .getName(), "speakLevel", mainLanguageProficiency.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLanguageProficiency.getClass()
                .getName(), "writeLevel", mainLanguageProficiency.getID())
        };

        verifySaveMustFail(mainLanguageProficiency, null, expectedExceptions);
    }
}
