package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.LocationPreferencesFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.LocationPreferences;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author dominiqu
 */
public class TestLocationPreferencesValidation extends GenericValidation
{
    private ResourceScope mainScope;

    private Pool parentPool;

    private Resource mainResource;

    private LocationPreferences mainLocationPreferences;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ResourceScope();
        mainScope.setLocationPreferences(true);
        parentPool = PoolFixture.createSaveFixture(this);
        mainResource = ResourceFixture.createFixture(parentPool, mainScope, this);
        mainResource.setParent(parentPool);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    // Note: testFailureWithoutParent and testFailureWithInvalidParent are not
    // significant
    // here, because it is impossible to save a Passport without saving a
    // Resource.
    // savePassportandLocationPreferences sets the parent to the correct
    // resource parent
    // before calling internalSave and performing validation.

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Create LocationsPreferences with the maximum number of locations
        mainLocationPreferences = LocationPreferencesFixture.createFixture(mainResource,
            ValidationConstants.LOCATIONPREFERENCES_RELOCATELOCATIONS_MAX,
            ValidationConstants.LOCATIONPREFERENCES_TRAVELLOCATIONS_MAX);

        // Set field values with max and verify that they are accepted
        mainLocationPreferences.setAcceptableRelocateDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLERELOCATEDURATION_MAX));
        mainLocationPreferences.setAcceptableTravelDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLETRAVELDURATION_MAX));

        mainResource.setLocationPreferences(mainLocationPreferences);

        // Attempt to save the resource with LocationPreferences
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class, mainScope);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='" + mainResource.getID()
            + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);
        assertNotNull(((Resource) rpmObjects[0]).getLocationPreferences());

        // Check that the loaded location preferences is identical to the one we
        // requested to store
        LocationPreferencesFixture.compare(mainLocationPreferences,
            ((Resource) rpmObjects[0]).getLocationPreferences());
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Create LocationsPreferences with the minimum number of locations
        // (none)
        mainLocationPreferences = LocationPreferencesFixture.createFixture(mainResource,
            0, 0);

        // Set field values with min and verify that they are accepted
        mainLocationPreferences.setAcceptableRelocateDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLERELOCATEDURATION_MIN));
        mainLocationPreferences.setAcceptableTravelDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLETRAVELDURATION_MIN));

        mainResource.setLocationPreferences(mainLocationPreferences);

        // Attempt to save the resource with LocationPreferences
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class, mainScope);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='" + mainResource.getID()
            + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);
        assertNotNull(((Resource) rpmObjects[0]).getLocationPreferences());

        // Check that the loaded location preferences is identical to the one we
        // requested to store
        LocationPreferencesFixture.compare(mainLocationPreferences,
            ((Resource) rpmObjects[0]).getLocationPreferences());
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Create LocationsPreferences with maximum+1 number of locations
        mainLocationPreferences = LocationPreferencesFixture.createFixture(mainResource,
            ValidationConstants.LOCATIONPREFERENCES_RELOCATELOCATIONS_MAX + 1,
            ValidationConstants.LOCATIONPREFERENCES_TRAVELLOCATIONS_MAX + 1);

        // Set field values with max+1 and verify that they are not accepted
        mainLocationPreferences.setAcceptableRelocateDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLERELOCATEDURATION_MAX + 1));
        mainLocationPreferences.setAcceptableTravelDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLETRAVELDURATION_MAX + 1));

        mainResource.setLocationPreferences(mainLocationPreferences);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLocationPreferences.getClass()
                .getName(), "acceptableRelocateDuration", mainLocationPreferences.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLocationPreferences.getClass()
                .getName(), "acceptableTravelDuration", mainLocationPreferences.getID()),
            createRPMException(ErrorCodes.MAX_SIZE, mainLocationPreferences.getClass()
                .getName(), "relocateLocations", mainLocationPreferences.getID()),
            createRPMException(ErrorCodes.MAX_SIZE, mainLocationPreferences.getClass()
                .getName(), "travelLocations", mainLocationPreferences.getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Create LocationsPreferences with the maximum number of locations
        mainLocationPreferences = LocationPreferencesFixture.createFixture(mainResource,
            ValidationConstants.LOCATIONPREFERENCES_RELOCATELOCATIONS_MAX,
            ValidationConstants.LOCATIONPREFERENCES_TRAVELLOCATIONS_MAX);

        // Set field values with min-1 and verify that they are not accepted
        mainLocationPreferences.setAcceptableRelocateDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLERELOCATEDURATION_MIN - 1));
        mainLocationPreferences.setAcceptableTravelDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLETRAVELDURATION_MIN - 1));

        mainResource.setLocationPreferences(mainLocationPreferences);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLocationPreferences.getClass()
                .getName(), "acceptableRelocateDuration", mainLocationPreferences.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainLocationPreferences.getClass()
                .getName(), "acceptableTravelDuration", mainLocationPreferences.getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }

    public void testFailureValidationNull() throws AssertionError, Exception
    {
        // Create LocationsPreferences with the maximum number of locations
        mainLocationPreferences = LocationPreferencesFixture.createFixture(mainResource,
            ValidationConstants.LOCATIONPREFERENCES_RELOCATELOCATIONS_MAX,
            ValidationConstants.LOCATIONPREFERENCES_TRAVELLOCATIONS_MAX);

        // Set field values with min
        mainLocationPreferences.setAcceptableRelocateDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLERELOCATEDURATION_MAX));
        mainLocationPreferences.setAcceptableTravelDuration(new Integer(
            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLETRAVELDURATION_MAX));

        // Set the Boolean values to false - no children - and verify that the
        // saving fails!
        mainLocationPreferences.setWillingtoRelocate(Boolean.FALSE);
        mainLocationPreferences.setWillingtoTravel(Boolean.FALSE);

        mainResource.setLocationPreferences(mainLocationPreferences);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_BE_NULL, mainLocationPreferences.getClass()
                .getName(), "acceptableRelocateDuration", mainLocationPreferences.getID()),
            createRPMException(ErrorCodes.MUST_BE_NULL, mainLocationPreferences.getClass()
                .getName(), "acceptableTravelDuration", mainLocationPreferences.getID()),
            createRPMException(ErrorCodes.MUST_BE_NULL, mainLocationPreferences.getClass()
                .getName(), "relocateLocations", mainLocationPreferences.getID()),
            createRPMException(ErrorCodes.MUST_BE_NULL, mainLocationPreferences.getClass()
                .getName(), "travelLocations", mainLocationPreferences.getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }
}
