/*
 * Created on 1-Jun-2006
 * @author mmatwiy
 * Validation testcases for transfer under wbs methods.
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTransferUnderWBSValidation extends GenericValidation
{
    ScopeFolder scopeFolder;

    ScopeElementScope scopeElementScope;

    WorkElementScope workElementScope;

    public void setUp() throws Exception
    {
        super.setUp();

        // create a Proposal and save it; this will create a ScopeFolder
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);
        scopeFolder = (ScopeFolder) rpmObjects[0];

        scopeElementScope = new ScopeElementScope();
        scopeElementScope.setProject(new WorkElementScope());
        scopeElementScope.setChildren(new ScopeElementScope());
        scopeElementScope.setParent(new ScopeElementScope());

        workElementScope = new WorkElementScope();
        workElementScope.setChildren(new WorkElementScope());
        workElementScope.setParent(new WorkElementScope());

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testErrorNoWorkElementID() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        Risk risk = new Risk();
        risk.setParent(scopeFolder);
        risk.setName(HelperTestUtilities.createName());
        save(risk, scopeElementScope);

        risk = (Risk) loadFromID(risk, scopeElementScope);
        
        Deliverable deliverable = new Deliverable();

        // transfer the AbstractScope to the Proposal
        Result result = getApplicationInterface().moveAndTransferUnderWBS(
            getSessionID(), deliverable, risk);

        // error: no ID
        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, deliverable.getClass().getName()),
        };

        verifyFailure(deliverable, expectedExceptions, null, result);
    }

    public void testErrorNoScopeElementID() throws Exception
    {
        ChangeRequest changeRequest = new ChangeRequest();

        // attempt to move/transfer an AbstractScope with no ID
        Result result = getApplicationInterface().moveAndTransferUnderWBS(
            getSessionID(),
            WorkElementFixture.createSaveFixture(new Milestone(), null, this),
            changeRequest);

        // error: no ID
        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, changeRequest.getClass()
                .getName()),
        };

        verifyFailure(changeRequest, expectedExceptions, null, result);
    }

    public void testErrorNoScopeProjectID() throws Exception
    {
        // create a Defect with a non-saved project 
        Defect defect = new Defect();
        defect.setParent(scopeFolder);
        defect.setProject(new Project());
        defect.setName(HelperTestUtilities.createName());
        save(defect, scopeElementScope);

        // attempt to move an AbstractScope that contains a project
        // with no ID
        Result result = getApplicationInterface().moveUnderWBS(getSessionID(),
            WorkElementFixture.createSaveFixture(new Task(), null, this),
            defect);

        // error: no ID
        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, defect.getClass().getName(),
                ValidationConstants.SCOPEELEMENT_PROJECT_FIELD, defect.getID())
        };

        verifyFailure(defect, expectedExceptions, null, result);
    }

    public void testErrorCrossTransferWithoutMove() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        Action action = new Action();
        action.setParent(scopeFolder);
        action.setName(HelperTestUtilities.createName());
        save(action, scopeElementScope);

        action = (Action) loadFromID(action, scopeElementScope);
        
        // create a Task to cross transfer to
        Task task = (Task) WorkElementFixture.createSaveFixture(new Task(), null, this);

        // attempt to cross transfer
        Result result = getApplicationInterface().transferUnderWBS(
            getSessionID(),
            task,
            action);

        // error: cross transfer without move
        assertFalse(result.isSuccessful());
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.scopemanagement.constants.ErrorCodes.
                CANNOT_CROSS_TRANSFER_WITHOUT_MOVE, task.getClass().getName())
        };

        verifyFailure(action, expectedExceptions, null, result);
    }
    
    public void testErrorDeleteTransferredScopeElement() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        Action action = new Action();
        action.setParent(scopeFolder);
        action.setName(HelperTestUtilities.createName());
        save(action, scopeElementScope);

        action = (Action) loadFromID(action, scopeElementScope);

        // transfer under WBS root
        getApplicationInterface()
            .transferUnderWBS(getSessionID(), null, action);

        // attempt to delete the action without first deleting the 
        // transferred element
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.scopemanagement.constants.ErrorCodes.CANNOT_DELETE_TRANSFERRED_ELEMENTS,
                action.getClass().getName())
        };

        verifyDeleteMustFail(action, expectedExceptions, null);
    }

}