/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.framework.InternalSecuredArrayResult;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.security.containers.SecurityGroup;
import com.ibm.rpm.security.managers.SecurityMask;
import com.ibm.rpm.security.managers.SecurityUtil;
import com.ibm.rpm.security.managers.SecurityUtil.ArrayFieldDescr;
import com.ibm.rpm.security.managers.SecurityUtil.BaseFieldDescr;
import com.ibm.rpm.security.managers.SecurityUtil.FieldDescr;
import com.ibm.rpm.security.scope.SecurityGroupScope;

/**
 * @author Marc-Andr Lamothe 
 */
public abstract class BaseResourceSecurityFlagsTestLocal extends ClientTestController
{

    protected InternalSecuredArrayResult loadInternalSecuredArrayResultFromXpath(
            String xpath, RPMObjectScope scope) throws Exception
    {
        logger.info("loadFromXpath: " + xpath);
        InternalSecuredArrayResult result = (InternalSecuredArrayResult) INTERFACE_PROVIDER
            .loadFromXpath(sessionid, xpath, scope, null, null, null, null,
                null, null, null, null, null, true);
        logAndThrow(result, "loadInternalSecuredArrayResultFromXpath");
        return result;
    }

    protected String getChildName(String parentName)
    {
        return parentName + "_MODIFIED";
    }

    /**
     * Creates a new security group, saves it to the database and loads it back.
     * 
     * @param name of the new security group and the parent security group.
     * @return the new security group class.
     */
    protected SecurityGroup createSecurityGroup(String name,
            SecurityGroup parentSecurityGroup) throws Exception
    {
        SecurityGroup securityGroup = new SecurityGroup();
        securityGroup.setName(name);
        securityGroup.setParent(parentSecurityGroup);
        saveWithoutDelete(securityGroup, null);
        securityGroup = loadSecurityGroupByName(name, null);
        return securityGroup;
    }

    /**
     * Loads a security group from the database.
     * 
     * @param name of the security group to load and the scope to look in. 
     * @return the requested security group or null if it does not exist.
     */
    protected SecurityGroup loadSecurityGroupByName(final String name,
            SecurityGroupScope scope) throws Exception
    {
        RPMObject[] rpmObjects = loadFromXpath("/SecurityGroup[name='" + name
                + "']", scope);
        if (rpmObjects != null && rpmObjects.length == 1)
            return (SecurityGroup) rpmObjects[0];
        return null;
    }

    /**
     * Loads a security group from the database.
     * 
     * @param id of the security group to load and the scope to look in 
     * @return the requested security group or null if it does not exist
     */
    protected SecurityGroup loadSecurityGroupByID(final String id,
            SecurityGroupScope scope) throws Exception
    {
        RPMObject[] rpmObjects = loadFromXpath("/SecurityGroup[id='" + id
                + "']", scope);
        if (rpmObjects != null && rpmObjects.length == 1)
            return (SecurityGroup) rpmObjects[0];
        return null;
    }

    /**
     * Loads a child security group or creates one if none exists.
     *  
     * @param the parent security group and the value to initialise the flags with. 
     */
    protected void prepareSecurityGroup(SecurityGroup parentSecurityGroup,
            boolean flagValue) throws Exception
    {
        String name = getChildName(parentSecurityGroup.getName());
        SecurityGroup securityGroup = loadSecurityGroupByName(name, null);
        if (securityGroup == null)
            securityGroup = createSecurityGroup(name, parentSecurityGroup);
        setGroupFlags(securityGroup, flagValue);
        saveWithoutDelete(securityGroup, null);
    }

    /**
     * Changes the value of a flag in the security group.
     *  
     * @param the security group to modify, the field descriptor for the flag and the value to change the flag to. 
     */
    protected void setGroupFlag(SecurityGroup securityGroup,
            BaseFieldDescr desc, boolean flagValue) throws Exception
    {
        Assert.assertNotNull(desc);
        if (desc.isArray())
        {
            int length = ((ArrayFieldDescr) desc).masks.length;
            Boolean[] args = new Boolean[length];
            Arrays.fill(args, Boolean.valueOf(flagValue));
            desc.setter.invoke(securityGroup, new Object[]
            {
                args
            });
        }
        else
            desc.setter.invoke(securityGroup, new Boolean[]
            {
                new Boolean(flagValue)
            });
    }

    /**
     * Changes the value of all the flags in the security group.
     *  
     * @param the security group to modify and the value to change the flags to. 
     */
    protected void setGroupFlags(SecurityGroup securityGroup, boolean flagValue)
            throws Exception
    {
        Map mapMethods = SecurityUtil.getFieldDescrsMap(securityGroup
            .getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
            setGroupFlag(securityGroup, ((BaseFieldDescr) ((Map.Entry) it
                .next()).getValue()), flagValue);
    }

    /**
     * Verifies the value of a flag in the security group.
     *  
     * @param the security group to verify, the field descriptor for the flag and the value to compare the flag to. 
     */
    protected void verifyGroupFlag(SecurityGroup securityGroup,
            BaseFieldDescr desc, boolean flagValue) throws Exception
    {
        Assert.assertNotNull(desc);
        if (desc.isArray())
        {
            Boolean[] values = (Boolean[]) desc.getter.invoke(securityGroup,
                null);
            for (int i = 0; i < values.length; i++)
                Assert.assertEquals(securityGroup.getName() + ": " + desc.name,
                    flagValue, values[i].booleanValue());
        }
        // canViewResourceRecords flag is always set to true by SecurityUtil
        else if (((FieldDescr) desc).clearanceLevelId == 0
                && ((FieldDescr) desc).mask == SecurityMask.MASK_BIT10)
        {
            Boolean value = (Boolean) desc.getter.invoke(securityGroup, null);
            Assert.assertEquals(securityGroup.getName() + ": " + desc.name,
                true, value.booleanValue());
        }
        else
        {
            Boolean value = (Boolean) desc.getter.invoke(securityGroup, null);
            Assert.assertEquals(securityGroup.getName() + ": " + desc.name,
                flagValue, value.booleanValue());
        }
    }

    /**
     * Verifies the value of all the flags in the security group.
     *  
     * @param the security group to verify and the value to compare the flags to. 
     */
    protected void verifyGroupFlags(SecurityGroup securityGroup,
            boolean flagValue) throws Exception
    {
        //        System.out.println("TEST - Verifying "+securityGroup.getName());
        Map mapMethods = SecurityUtil.getFieldDescrsMap(securityGroup
            .getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
            verifyGroupFlag(securityGroup, ((BaseFieldDescr) ((Map.Entry) it
                .next()).getValue()), flagValue);
    }

}
