/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.UnitTestSessionUtility;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.InternalSecuredArrayResult;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.interfaces.impl.ServerFactory;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.managers.SecurityUtil;
import com.ibm.rpm.security.managers.SecurityUtil.ArrayFieldDescr;
import com.ibm.rpm.security.managers.SecurityUtil.BaseFieldDescr;
import com.ibm.rpm.security.scope.SecurityRoleScope;

/**
 * @author Marc-Andr Lamothe 
 */
public abstract class BaseRoleSecurityFlagsTestLocal extends BaseSecurityTestLocal
{

    protected InternalSecuredArrayResult loadInternalSecuredArrayResultFromXpath(
            String xpath, RPMObjectScope scope) throws Exception
    {
        logger.info("loadFromXpath: " + xpath);
        InternalSecuredArrayResult result = (InternalSecuredArrayResult) INTERFACE_PROVIDER
            .loadFromXpath(sessionid, xpath, scope, null, null, null, null,
                new Integer(96), null, null, null, null, true);
        logAndThrow(result, "loadInternalSecuredArrayResultFromXpath");
        return result;
    }

    /**
     * Creates a name for a new child object by adding _MODIFIED at the end
     * of the parent name.
     * 
     * @param parentName
     */
    protected String getChildName(String parentName)
    {
        return parentName + "_MODIFIED";
    }

    /**
     * Creates a new security role, saves it to the database and loads it back.
     * 
     * @param name
     * @param parentSecurityRole
     * @return SecurityRole
     */
    protected SecurityRole createSecurityRole(String name,
            SecurityRole parentSecurityRole) throws Exception
    {
        SecurityRole securityRole = new SecurityRole();
        securityRole.setName(name);
        securityRole.setParent(parentSecurityRole);
        saveWithoutDelete(securityRole, null);
        securityRole = loadSecurityRoleByName(name);
        return securityRole;
    }

    /**
     * Creates a new security role withotu savign it to the database.
     * 
     * @param parentRole
     * @param flagValue
     * @return SecurityRole
     */
    protected SecurityRole prepareInMemorySecurityRole(SecurityRole parentRole,
            boolean flagValue) throws Exception
    {
        SecurityRole role = (SecurityRole) parentRole.getClass().newInstance();
        role.setName(getChildName(parentRole.getName()));
        role.setParent(parentRole);
        setRoleFlags(role, flagValue);
        SecurityUtil.setClearanceLevel(SecurityUtil.getClearanceLevels(role,
            new int[SecurityUtil.SECURITY_ROLE_LEVELS_COUNT]), role);
        return role;
    }

    /**
     * Creates a security role scope.
     * 
     * @return SecurityRoleScope
     */
    protected SecurityRoleScope createSecurityRoleScope()
    {
        SecurityRoleScope scope = new SecurityRoleScope();
        scope.setParent(scope);
        return scope;
    }

    /**
     * Loads a security role from the database.
     * 
     * @param name
     * @return SecurityRole
     */
    protected SecurityRole loadSecurityRoleByName(String name) throws Exception
    {
        final RPMObject[] rpmObjects = loadFromXpath("/SecurityRole[name='"
                + name + "']", createSecurityRoleScope());
        if ((rpmObjects != null) && (rpmObjects.length > 0))
            return (SecurityRole) rpmObjects[0];
        return null;
    }

    /**
     * Loads a security role from the database.
     * 
     * @param id
     * @param scope
     * @return SecurityRole
     */
    protected SecurityRole loadSecurityRoleByID(final String id,
            RPMObjectScope scope) throws Exception
    {
        final RPMObject[] rpmObjects = loadFromXpath("/SecurityRole[id='" + id
                + "']", scope);
        if ((rpmObjects != null) && (rpmObjects.length > 0))
            return (SecurityRole) rpmObjects[0];
        return null;
    }

    /**
     * Loads a child security role or creates one if none exists.
     *  
     * @param parentSecurityRole
     * @param flagValue
     * @return SecurityRole
     */
    protected SecurityRole prepareSecurityRole(SecurityRole parentSecurityRole,
            boolean flagValue) throws Exception
    {
        if (parentSecurityRole != null)
        {
            String name = getChildName(parentSecurityRole.getName()) + "_" + HelperTestUtilities.getUserName();
            SecurityRole securityRole = loadSecurityRoleByName(name);
            if (securityRole == null)
                securityRole = createSecurityRole(name, parentSecurityRole);
            setRoleFlags(securityRole, flagValue);
            return (SecurityRole) saveWithoutDelete(securityRole,
                createSecurityRoleScope()).getRpmObject();
        }
        return null;
    }

    /**
     * Inverts the value of a flag in the security role.
     *  
     * @param securityRole
     * @param desc
     */
    protected void flipRoleFlag(SecurityRole securityRole, BaseFieldDescr desc)
            throws Exception
    {
        Assert.assertNotNull(desc);
        if (desc.isArray())
        {
            Boolean[] values = (Boolean[]) desc.getter.invoke(securityRole,
                null);
            for (int i = 0; i < values.length; i++)
                values[i] = Boolean.valueOf(!values[i].booleanValue());
            desc.setter.invoke(securityRole, new Object[]
            {
                values
            });
        }
        else
        {
            Boolean value = (Boolean) desc.getter.invoke(securityRole, null);
            desc.setter.invoke(securityRole, new Boolean[]
            {
                Boolean.valueOf(!value.booleanValue())
            });
        }
    }

    /**
     * Inverts the value of all the flags in the security role.
     *  
     * @param securityRole
     */
    protected void flipRoleFlags(SecurityRole securityRole) throws Exception
    {
        Map mapMethods = SecurityUtil
            .getFieldDescrsMap(securityRole.getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
            flipRoleFlag(securityRole,
                ((BaseFieldDescr) ((Map.Entry) it.next()).getValue()));
    }

    /**
     * Changes the value of a flag in the security role.
     *  
     * @param securityRole 
     * @param desc
     * @param flagValue
     */
    protected void setRoleFlag(SecurityRole securityRole, BaseFieldDescr desc,
            boolean flagValue) throws Exception
    {
        Assert.assertNotNull(desc);
        if (desc.isArray())
        {
            int length = ((ArrayFieldDescr) desc).masks.length;
            Boolean[] args = new Boolean[length];
            Arrays.fill(args, Boolean.valueOf(flagValue));
            desc.setter.invoke(securityRole, new Object[]
            {
                args
            });
        }
        else
            desc.setter.invoke(securityRole, new Boolean[]
            {
                new Boolean(flagValue)
            });
    }

    /**
     * Changes the value of all the flags in the security role.
     *  
     * @param securityRole 
     * @param flagValue
     */
    protected void setRoleFlags(SecurityRole securityRole, boolean flagValue)
            throws Exception
    {
        Map mapMethods = SecurityUtil
            .getFieldDescrsMap(securityRole.getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
            setRoleFlag(securityRole, ((BaseFieldDescr) ((Map.Entry) it.next())
                .getValue()), flagValue);
    }

    /**
     * Verifies the value of a flag in the security role.
     *  
     * @param securityRole 
     * @param desc 
     * @param flagValue 
     */
    protected void verifyRoleFlag(SecurityRole securityRole,
            BaseFieldDescr desc, boolean flagValue) throws Exception
    {
        Assert.assertNotNull(desc);
        if (desc.isArray())
        {
            Boolean[] values = (Boolean[]) desc.getter.invoke(securityRole,
                null);
            for (int i = 0; i < values.length; i++)
                Assert.assertEquals(securityRole.getName() + ": " + desc.name,
                    flagValue, values[i].booleanValue());
        }
        else
        {
            Boolean value = (Boolean) desc.getter.invoke(securityRole, null);
            Assert.assertEquals(securityRole.getName() + ": " + desc.name,
                flagValue, value.booleanValue());
        }
    }

    /**
     * Verifies the value of all the flags in the security role.
     *  
     * @param securityRole
     * @param flagValue  
     */
    protected void verifyRoleFlags(SecurityRole securityRole, boolean flagValue)
            throws Exception
    {
        //        System.out.println("TEST - Verifying "+securityRole.getName());
        Map mapMethods = SecurityUtil
            .getFieldDescrsMap(securityRole.getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
            verifyRoleFlag(securityRole, ((BaseFieldDescr) ((Map.Entry) it
                .next()).getValue()), flagValue);
    }
    
    /**
     * Builds an instance of the MessageContext class.
     *  
     * @param operationName
     */
    protected MessageContext buildMessageContext(String operationName)
            throws RPMException
    {
        MessageContext messageContext = new MessageContext(ServerFactory
            .getInstance());
        messageContext.setSession(UnitTestSessionUtility.INSTANCE, sessionid,
            operationName);
        return messageContext;
    }

}
