/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.role;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.client.junit.SpecificTestController;
import com.ibm.rpm.client.junit.security.testcases.framework.BaseSecurityFrameworkTestLocal;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.security.SecurityFlags;
import com.ibm.rpm.framework.util.RPMObjectScopeUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author Andrei Tanasescu
 * @author Marc-Andre Lamothe
 */
public class BaseRoleSecurityTestLocal extends BaseSecurityFrameworkTestLocal
{
    private static Log logger = LogFactory.getLog(BaseRoleSecurityTestLocal.class);
    
    static protected Resource securityTestUser;
    
    protected Map flagIDs;

    protected void setUp() throws Exception
    {
        SpecificTestController.overrideOption(
            RpmOptions.WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME, ROLE_SEC);
        super.setUp();
        /* Prepare global variables */
        securityTestUser = (Resource) loadOneFromXpath("/Resource[username='"
            + loggedAs + "']", null);       
        assertTrue("Class names must end with Local so they aren't copied over to rpm-web-unit-test",
            getClass().getName().endsWith("Local"));
    }
    
    static public Resource getSecurityTestUser()
    {
        return securityTestUser;
    }


    
    protected void flagOff(RPMObject object, String flagName) throws Exception
    {
        turnAllRelatedFlags(object, flagName, Boolean.FALSE);
    }
    
    /**
     * @param flagName
     * @throws Exception
     */
    protected void flagOn(RPMObject object, String flagName) throws Exception
    {
        turnAllRelatedFlags(object, flagName, Boolean.TRUE);
    }
    
    /**
     * This functionality exists for unit tests, but it can be externalized
     * to be useful to the API user as well. If the developers find it hard
     * to locate dependencies between flags, users most likely would find it
     * even harder. 
     * 
     * Since flags are OR'd, and some of them depend on others, we
     * automate the process of turning all related flags on or off.
     * 
     * All objects in the hierarchy must be saved for this method to work.
     * 
     * 1) dependent flags in the same object (same number)
     * 2) flag depends on superflag
     * 3) current flag depends on a parent's flag - same number
     * 4) current flag depends on parent's flag - same name
     * 
     * @param object The object for which the security role should change
     * @param flagName The name of the flag to turn on or off.
     * @param newValue What to set the flag to.
     */
    private void turnAllRelatedFlags(RPMObject object, String flagName, Boolean newValue) throws Exception
    {
        /*
         * Assumptions :
         * - we can always get to the parent of any object passed in parameters
         * - the RTA we care about is securityTestUser
         */
        /*
         * - get all the RTA's of this object as well as its parents. 
         * - get all flags related to the current flag
         * - for each security role, if it contains a related flag, turn it off.
         * - log the changes.
         */ 
        Map relatedSecurityRoles = retrieveRelatedSecurityRoles(object);
       
        Iterator securityRoleIterator = relatedSecurityRoles.keySet().iterator();
        
        /*
         * Update all security roles in the object hierarchy which
         * have an identically-named flag as "object".
         */
        String flagSetter = buildSecurityRoleSetter(flagName);
        while(securityRoleIterator.hasNext()) 
        {
            SecurityRole curRole = (SecurityRole) securityRoleIterator.next();
            introspectCallFlagSetter(newValue, flagSetter, curRole);
            logger.debug("[ Switched " + flagSetter.substring(3) + " to " + newValue + " for an object of type " 
                +  relatedSecurityRoles.get(curRole).getClass() + " bearing ID " +
                ((RPMObject)relatedSecurityRoles.get(curRole)).getID() + " ]" );
        }
        
        /*
         * Update all security roles in the hierarchy which share the same
         * identification number.
         */
        flagName = flagName.substring(0, 1).toLowerCase()+ flagName.substring(1); // lower case the first letter.
        Integer flagNumber = (Integer) SecurityFlags.getRoleSecurityFieldToIntegerMap().get(flagName);
        List relatedFlagNames = (List) getFlagIDsMap().get(flagNumber);
        if(relatedFlagNames != null)
        {
            Iterator relatedFlagsIterator = relatedFlagNames.iterator();
            while(relatedFlagsIterator.hasNext())
            {
                String relatedFlag = (String) relatedFlagsIterator.next();
                flagSetter = buildSecurityRoleSetter(relatedFlag);
                
                securityRoleIterator = relatedSecurityRoles.keySet().iterator();
                while(securityRoleIterator.hasNext()) 
                {
                    SecurityRole curRole = (SecurityRole) securityRoleIterator.next();
                    introspectCallFlagSetter(newValue, flagSetter,curRole);
                    logger.debug("[ Switched " + flagSetter.substring(3) + " to " + newValue + " for an object of type " 
                        +  relatedSecurityRoles.get(curRole).getClass() + " bearing ID " +
                        ((RPMObject)relatedSecurityRoles.get(curRole)).getID() + " ]" );
                }
            }
        }
    }
    
    protected String buildSecurityRoleSetter(String input) 
    {
        return "set" + StringUtils.capitalize(input);
    }

    private void introspectCallFlagSetter(Boolean newValue, String flagSetter, SecurityRole curRole) 
        throws IllegalAccessException, InvocationTargetException, Exception
    {
        Method objectMethods[] = curRole.getClass().getMethods();
        for(int methodI = 0; methodI < objectMethods.length; methodI++)
        {
           Method curMethod = objectMethods[methodI];
           
           if(curMethod.getName().equals(flagSetter))
           {
               curMethod.invoke(curRole, new Object[] {newValue});
               
               updateSecurityRole(curRole);
               break; // should never get the same method twice (overloading).
           }
        }
    }
    
    /**
     * Retrieve the security roles of all objects in the hierarchy.
     * 
     * @return a map in which keys are security roles and values are the objects to which they belong
     */
    Map retrieveRelatedSecurityRoles(RPMObject startObject) throws Exception
    {
        Map securityRoleMap = new HashMap();
        
        ArrayList potentialSecurityRoleContainers = retrieveParentHierarchy(startObject);
        
        /*
         * The document section uses a different name for resourceeTaskAssignments than the others.
         */
        String rtaMethodName = "getResourceTaskAssignments";
        String documentRtaMethodName = "getResourceAssignments";
        for (Iterator iter = potentialSecurityRoleContainers.iterator(); iter
            .hasNext();)
        {
            RPMObject securityRoleContainer = (RPMObject) iter.next();
            Method securityRoleMethods[] = securityRoleContainer.getClass().getMethods();
            for(int methodI = 0; methodI < securityRoleMethods.length; methodI++)
            {
               Method curMethod = securityRoleMethods[methodI];
               if(curMethod.getName().equals(rtaMethodName)) /* Scope and WBS sections */
               {
                   ResourceTaskAssignment resourceTaskAssignments[] = null;
                   try {
                       resourceTaskAssignments = (ResourceTaskAssignment []) curMethod.invoke(securityRoleContainer, null);
                       if(resourceTaskAssignments  == null)
                       {
                           break;
                       }
                   } catch (Exception e)
                   {
                       break;
                   }
                   
                   for (int taskAssignmentI = 0; taskAssignmentI < resourceTaskAssignments.length; taskAssignmentI++)
                   {
                       if(resourceTaskAssignments[taskAssignmentI].getResource().getFullName().equals(
                           securityTestUser.getFullName()))
                       {
                           securityRoleMap.put(resourceTaskAssignments[taskAssignmentI].getSecurityRole(), securityRoleContainer);
                           break;
                       }
                   }
                   
                   break;
               } 
               else if(curMethod.getName().equals(documentRtaMethodName)) /* Document section */
               {
                   ResourceRoleAssignment resourceRoleAssignments[] = null;
                   try {
                       resourceRoleAssignments = (ResourceRoleAssignment []) curMethod.invoke(securityRoleContainer, null);
                       if(resourceRoleAssignments  == null)
                       {
                           break;
                       }
                   } catch (Exception e)
                   {
                       break;
                   }
                   
                   for (int taskAssignmentI = 0; taskAssignmentI < resourceRoleAssignments.length; taskAssignmentI++)
                   {
                       if(resourceRoleAssignments[taskAssignmentI].getResource().getFullName().equals(
                           securityTestUser.getFullName()))
                       {
                           securityRoleMap.put(resourceRoleAssignments[taskAssignmentI].getSecurityRole(), securityRoleContainer);
                           break;
                       }
                   }
                   
                   break;
               }
            }
        }
      
        if(securityRoleMap.size() == 0)
        {
            throw new RuntimeException("The object passed seems to not have been saved yet. - Could not retrieve _any_ security roles.");
        }
        
        return securityRoleMap;
    }

    /**
     * Retrieve all the objects in the hierarchy, with ResourceTaskAssignment in scope. These
     * are all objects which can potentially include a SecurityRole for the user we're
     * currently logged in.
     * 
     * @param startObject Where to start the look-up
     * @return an Array of RPMObjects which have a resourceTaskAssignment accessor
     * @throws Exception
     */
    private ArrayList retrieveParentHierarchy(RPMObject startObject) throws Exception
    {
        /*
         * - get the object's scope
         * - put the parent of the object in scope
         * - load from ID the object, retrieve the parent
         * - add the parent to the list of parents
         * - if the parent is null, stop
         */
        ArrayList potentialSecurityRoleContainers = new ArrayList();

        /*
         * Monitor for when to stop going higher in the hierarchy
         */
        boolean loop = true;
        
        /*
         * If we have more than 99 parents, there is a problem somewhere. 
         * Probably an infinite loop.
         */
        int infiniteLoopSafeguard = 99;
        
        RPMObject currentRPMObject = startObject;
        do 
        {
            Class currentScopeClass = RPMObjectScopeUtil.getInstance().getScope(
                StringUtil.getShortClassName(currentRPMObject.getClass()));
            RPMObjectScope currentScopeObject = (RPMObjectScope) currentScopeClass.newInstance();
            currentScopeObject.setParent(new RPMObjectScope());
            
            /*
             * Put the ResourceTaskAssignments in scope. 
             */
            Method scopeObjectMethods[] = currentScopeClass.getMethods();
            for(int methodI = 0; methodI < scopeObjectMethods.length; methodI++)
            {
               Method curMethod = scopeObjectMethods[methodI];
               if(curMethod.getName().equals("setResourceTaskAssignments"))
               {
                   curMethod.invoke(currentScopeObject, new Object[] {new ResourceTaskAssignmentScope()});
                   break;
               } 
               else if(curMethod.getName().equals("setResourceAssignments"))
               {
                   curMethod.invoke(currentScopeObject, new Object[] {Boolean.TRUE});
                   break;
               }
            }
            
            currentRPMObject = loadFromID(currentRPMObject, currentScopeObject);
            potentialSecurityRoleContainers.add(currentRPMObject);
            if(currentRPMObject == null) 
            {
                break;
            }
            
            RPMObject parentObject = currentRPMObject.getParent();
            if(parentObject == null) 
            {
                if(currentRPMObject instanceof ScopeFolder)
                {
                    // special handling to get the parent of a scope folder
                    ScopeElementScope scopeFolderScope = new ScopeElementScope();
                    WorkElementScope projectScope = new WorkElementScope();
                    projectScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
                    scopeFolderScope.setProject(projectScope);
                    
                    ScopeFolder scopeFolder = (ScopeFolder) loadFromID(currentRPMObject, scopeFolderScope);
                    currentRPMObject = scopeFolder.getProject(); 
                    
                    potentialSecurityRoleContainers.add(currentRPMObject);
                }
                else if(currentRPMObject instanceof DocumentFolder)
                {
                    // special handling to get the parent of a document folder
                    DocumentScope scopeDocumentFolderScope = new DocumentScope();
                    WorkElementScope projectScope = new WorkElementScope();
                    projectScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
                    scopeDocumentFolderScope.setContainingProject(projectScope);
                    
                    DocumentFolder scopeFolder = (DocumentFolder) loadFromID(currentRPMObject, scopeDocumentFolderScope);
                    currentRPMObject = scopeFolder.getContainingProject();
                    
                    potentialSecurityRoleContainers.add(currentRPMObject);
                }
                else {
                    loop = false;
                }
            } else
            {
                currentRPMObject = parentObject;
            }
        } while(loop && (--infiniteLoopSafeguard > 0));
        
        if(infiniteLoopSafeguard <= 0)
        {
            throw new Exception("We have exceeded 99 parents in the object hierarchy. Prevented a _potential_ infinite loop");
        }
        
        return potentialSecurityRoleContainers;
    }
    
    /**
     * Return a map which ties an Integer from SecurityFlags.java to an ArrayList
     * of all the names of the flags which share the same Integer value.
     */
    public Map getFlagIDsMap() 
    {
        if(flagIDs == null)
        {
            flagIDs = new HashMap();
            
            Map idPerFlag = SecurityFlags.getRoleSecurityFieldToIntegerMap();
            Iterator flagNameIterator = idPerFlag.keySet().iterator();
            List relatedFlagsList = null;
            while(flagNameIterator.hasNext())
            {
                relatedFlagsList = null;
                String flagName = (String) flagNameIterator.next();
                Integer flagNumber = (Integer) idPerFlag.get(flagName);
                
                if(!flagIDs.containsKey(flagNumber))
                {
                    relatedFlagsList = new ArrayList();
                    relatedFlagsList.add(flagName);
                    flagIDs.put(flagNumber, relatedFlagsList);
                } else
                {
                    relatedFlagsList = (List) flagIDs.get(flagNumber);
                    relatedFlagsList.add(flagName);
                }
            }
        }
        
        return flagIDs;
    }
}
