package com.ibm.rpm.client.junit.timesheet.testcases;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TestSummaryTimesheetBusinessRules extends GenericValidation
{
    private SummaryTimesheet summaryTimesheet = null;

    private final int READONLY_ERROR_ID = 400046;

    private final int CANNOT_BEFORE_ERROR_ID = 406029;

    private final String FORECAST_STARTDATE = "2007-04-18";

    private String forecastFinishDateString = null;

    private SingleResult result = null;

    private Resource currentUser = null;

    public void setUp() throws Exception
    {
        super.setUp();

        // Create a TaskAssignment
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        assertNotNull(taskAssignment);

        // create a SummaryTimesheet
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        SummaryTimesheetScope summaryTimesheetScope = SummaryTimesheetFixture
            .createScope(false, false, false, true, false);
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment, weekOf, summaryTimesheetScope, false, this);
        assertNotNull(summaryTimesheet);
        assertNotNull(summaryTimesheet.getTaskAssignment());

        // get current user
        currentUser = summaryTimesheet.getResource();
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setResourceSecurityRights(true);
        currentUser = (Resource) loadFromID(currentUser, resourceScope);
        assertNotNull(currentUser);
        assertNotNull(currentUser.getResourceSecurityRights());

        // set security
        ResourceSecurityRights securityRights = currentUser
            .getResourceSecurityRights();
        securityRights.setCanUseWebServicesAPI(new Boolean(true));

        result = save(currentUser, resourceScope);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof Resource);
        currentUser = (Resource) result.getRpmObject();

        // switch user
        this.getAuthenticateInterface().changePassword(
            currentUser.getUserName(), "ibmrpm", "rpm", "jdbc/RPMDATASOURCE");
        this.performLogin(currentUser.getUserName(), "rpm");

        forecastFinishDateString = DateUtil.convertToString(TimesheetTestUtil
            .getValidWeekOf(weekOf, TimesheetTestUtil.OTHER_WEEK));
    }

    public void testForecastStartDateWithoutActuals() throws Exception
    {
        // ForecastStartDate can be modified if there is no actuals

        // setup dates
        Calendar forecastStartDate = DateUtil
            .convertToCalendarNoTime(FORECAST_STARTDATE);
        summaryTimesheet.setForecastStartDate(forecastStartDate);

        result = this.save(summaryTimesheet, null);
        validateSaveSucessful(result);
        assertNotNull(summaryTimesheet.getForecastStartDate());
        assertEquals(FORECAST_STARTDATE, DateUtil
            .convertToString(summaryTimesheet.getForecastStartDate()));
        assertNotNull(((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .getForecastStartDate());
        assertEquals(FORECAST_STARTDATE, DateUtil
            .convertToString(((TaskAssignment) summaryTimesheet
                .getTaskAssignment()).getForecastStartDate()));
    }

    public void testForecastStartDateWithActuals() throws Exception
    {
        // ForecastStartDate can't be modified and should be null when there are
        // actuals

        // setup dates
        Calendar forecastStartDate = DateUtil
            .convertToCalendarNoTime(FORECAST_STARTDATE);
        summaryTimesheet.setForecastStartDate(forecastStartDate);
        summaryTimesheet.setRegularMonday(new Double(3));
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setEETCHours(null);
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setPercentageComplete(null);

        result = this.save(summaryTimesheet, null);
        validateSaveFailure(READONLY_ERROR_ID,
            ValidationConstants.FORECAST_START_DATE_FIELD, result);

        assertNull(summaryTimesheet.getForecastStartDate());
        assertNull(((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .getForecastStartDate());
    }

    public void testForecastFinishDateWithoutActuals() throws Exception
    {
        // ForecastFinishDate can be modified when there is no actuals

        // setup dates
        Calendar forecastFinishDate = DateUtil
            .convertToCalendarNoTime(forecastFinishDateString);
        summaryTimesheet.setForecastFinishDate(forecastFinishDate);

        result = this.save(summaryTimesheet, null);
        validateSaveSucessful(result);
        assertNotNull(summaryTimesheet.getForecastFinishDate());
        assertEquals(forecastFinishDateString, DateUtil
            .convertToString(summaryTimesheet.getForecastFinishDate()));
        assertNotNull(((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .getForecastFinishDate());
        assertEquals(forecastFinishDateString, DateUtil
            .convertToString(((TaskAssignment) summaryTimesheet
                .getTaskAssignment()).getForecastFinishDate()));
        
    }

    public void testForecastFinishDateWithActualsButNotComplete()
            throws Exception
    {
        // ForecastFinishDate can be modified when there are actuals but the
        // task is not finished

        // setup dates
        Calendar forecastFinishDate = DateUtil
            .convertToCalendarNoTime(forecastFinishDateString);
        summaryTimesheet.setForecastFinishDate(forecastFinishDate);
        summaryTimesheet.setRegularMonday(new Double(3.0));
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setEETCHours(null);
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setPercentageComplete(null);

        result = this.save(summaryTimesheet, null);
        validateSaveSucessful(result);
        assertNotNull(summaryTimesheet.getForecastFinishDate());
        assertEquals(forecastFinishDateString, DateUtil
            .convertToString(summaryTimesheet.getForecastFinishDate()));
        assertNotNull(((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .getForecastFinishDate());
        assertEquals(forecastFinishDateString, DateUtil
            .convertToString(((TaskAssignment) summaryTimesheet
                .getTaskAssignment()).getForecastFinishDate()));
    }

    public void testForecastFinishDateWithActualsComplete() throws Exception
    {
        // ForecastFinishDate can't be modified and set to null when the task is
        // completed
        Calendar forecastFinishDate = DateUtil
            .convertToCalendarNoTime(forecastFinishDateString);
        summaryTimesheet.setRegularMonday(new Double(40.0));
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setEETCHours(null);
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setPercentageComplete(null);

        result = this.save(summaryTimesheet, null);
        validateSaveSucessful(result);

        assertTrue(summaryTimesheet.getTaskAssignment() instanceof TaskAssignment);
        assertTrue(((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .getComplete().booleanValue());

        // for this test, set forecastFinishDate must be done after task is
        // finished, because we calculate the EETC when we report actuals
        summaryTimesheet.setForecastFinishDate(forecastFinishDate);
        result = this.save(summaryTimesheet, null);
        validateSaveFailure(READONLY_ERROR_ID,
            ValidationConstants.FORECAST_FINISH_DATE_FIELD, result);

        assertNull(summaryTimesheet.getForecastFinishDate());
        assertNull(((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .getForecastFinishDate());
    }

    public void testForecastFinishDateBeforeWeekOfEndDate() throws Exception
    {
        String someDate = "2006-10-10";
        Calendar forecastFinishDate = DateUtil
            .convertToCalendarNoTime(someDate);
        summaryTimesheet.setRegularFriday(new Double(2.0));
        summaryTimesheet.setForecastFinishDate(forecastFinishDate);
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setEETCHours(null);
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setPercentageComplete(null);

        result = this.save(summaryTimesheet, null);
        validateSaveFailure(CANNOT_BEFORE_ERROR_ID,
            ValidationConstants.FORECAST_FINISH_DATE_FIELD, result);

        assertNull(summaryTimesheet.getForecastFinishDate());
        assertNull(((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .getForecastFinishDate());
    }

    public void testForecastFinishDateEqualsWeekOfEndDate() throws Exception
    {
        summaryTimesheet.setRegularMonday(new Double(2.0));
        int year = summaryTimesheet.getWeekOf().get(Calendar.YEAR);
        int month = summaryTimesheet.getWeekOf().get(Calendar.MONTH);
        int day = summaryTimesheet.getWeekOf().get(Calendar.DAY_OF_MONTH);

        Calendar someDate = new GregorianCalendar(year, month, day);
        someDate.add(Calendar.WEEK_OF_YEAR, 1);
        someDate.add(Calendar.DAY_OF_YEAR, -1);
        summaryTimesheet.setForecastFinishDate(someDate);
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setEETCHours(null);
        ((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .setPercentageComplete(null);

        result = this.save(summaryTimesheet, null);
        validateSaveSucessful(result);
        assertNotNull(summaryTimesheet.getForecastFinishDate());
        assertEquals(DateUtil.convertToString(someDate), DateUtil
            .convertToString(summaryTimesheet.getForecastFinishDate()));
        assertNotNull(((TaskAssignment) summaryTimesheet.getTaskAssignment())
            .getForecastFinishDate());
        assertEquals(DateUtil.convertToString(someDate), DateUtil
            .convertToString(((TaskAssignment) summaryTimesheet
                .getTaskAssignment()).getForecastFinishDate()));
    }

    private void validateSaveSucessful(SingleResult result)
    {
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();
    }

    private void validateSaveFailure(int errorCode, String fieldName,
            SingleResult result) throws Exception
    {
        RPMExceptionMock[] expectedWarnnings =
        {
            createRPMException(errorCode,
                summaryTimesheet.getClass().getName(), fieldName,
                summaryTimesheet.getID())
        };
        this.verifyFailure(summaryTimesheet, null, expectedWarnnings, result);

        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();
    }
}
