/*
 * Created on Feb 14, 2007
 * 
 * Defect 61400: WS - %c is being set when updating EETC
 * 
 * Description:
 * 1- create a TaskAssignement with EETC = 8.
 * 2- create a SummaryTimesheet for the TaskAssignement.
 * 3- set EETC on SummaryTimesheet to 16 and save.
 * 4- Notice how %c becomes 0!!
 *
 * Saving EETC should not affect %c (unless it is being set 0, I believe). 
 * %c should be calculated only if we are reporting time.
 *
 */
package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class Test61400 extends ClientTestController
{
    private SummaryTimesheet summaryTimesheet = null;

    private TaskAssignment taskAssignment = null;

    private SingleResult result = null;

    public void setUp() throws Exception
    {
        super.setUp();

        // Create a TaskAssignment
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        taskAssignment = TaskAssignmentFixture.createSaveFixture(
            taskAssignmentScope, this);
        assertNotNull(taskAssignment);

        // create a SummaryTimesheet
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment, weekOf, null, false, this);
        assertNotNull(summaryTimesheet);
        assertNotNull(summaryTimesheet.getTaskAssignment());
        assertTrue(summaryTimesheet.getTaskAssignment() instanceof TaskAssignment);
        taskAssignment = (TaskAssignment) summaryTimesheet.getTaskAssignment();
        assertNotNull(taskAssignment.getPercentageComplete());
    }
    
    public void testReportActualsInCreation() throws Exception
    {
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.OTHER_WEEK);
        summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);
        reportActuals();
    }
    
    public void testReportActualsWithEETCPercentageCInCreation() throws Exception
    {
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.OTHER_WEEK);
        taskAssignment.setEETCHours(new Double(100.0));
        taskAssignment.setPercentageComplete(new Double(20));
        summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);
        summaryTimesheet.setRegularFriday(new Double(5));
        summaryTimesheet.setForecastStartDate(weekOf);
        saveAndValidateSaveSummaryTimesheet();
        assertEquals(new Double(5), summaryTimesheet.getRegularFriday());
        assertEquals(new Double(100.0), ((TaskAssignment)summaryTimesheet.getTaskAssignment()).getEETCHours());
        assertEquals(new Double(20.0), ((TaskAssignment)summaryTimesheet.getTaskAssignment()).getPercentageComplete());
    }
    
    public void testReportActualsInCreationWithGenericTaskAssignment() throws Exception
    {
        GenericTaskAssignment genericTaskAssignment = new GenericTaskAssignment();
        genericTaskAssignment.setID(taskAssignment.getID());
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.OTHER_WEEK);
        summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(genericTaskAssignment);
        summaryTimesheet.setWeekOf(weekOf);
        reportActuals();
    }

    public void testUpdateEETCWithoutActuals() throws Exception
    {
        Double percentageC = taskAssignment.getPercentageComplete();
        Double eetc = new Double(10.0);

        changeEETC(eetc);
        assertEquals(percentageC, taskAssignment.getPercentageComplete());
    }
    
    public void testUpdatePercentageCompleteWithoutActuals() throws Exception
    {
        Double percentageC = new Double(50.0);
        taskAssignment.setPercentageComplete(percentageC);
        saveAndValidateSaveSummaryTimesheet();
        assertNotNull(taskAssignment.getPercentageComplete());
        assertEquals(new Double(0.0), taskAssignment.getPercentageComplete());
    }

    public void testUpdateEETCWithActuals() throws Exception
    {
        reportActuals();
        
        Double percentageC = taskAssignment.getPercentageComplete();
        Double oldEETC = taskAssignment.getEETCHours();

        // change EETC, the %c shouldn't change
        Double newEETC = new Double(oldEETC.doubleValue() - 10.0);
        changeEETC(newEETC);
        assertEquals(percentageC, taskAssignment.getPercentageComplete());
    }
    
    public void testUpdatePercentageCompleteWithActuals() throws Exception
    {
        reportActuals();
        
        Double percentageC = new Double(50.0);
        taskAssignment.setPercentageComplete(percentageC);
        saveAndValidateSaveSummaryTimesheet();
        assertNotNull(taskAssignment.getPercentageComplete());
        assertEquals(percentageC, taskAssignment.getPercentageComplete());
    }
    
    public void testReportActualsWithGenericTaskAssignment() throws Exception
    {
        Double actual = new Double(1.0);

        summaryTimesheet.setRegularMonday(actual);
        
        GenericTaskAssignment genericTaskAssignment = new GenericTaskAssignment();
        genericTaskAssignment.setID(taskAssignment.getID());
        summaryTimesheet.setTaskAssignment(genericTaskAssignment);

        saveAndValidateSaveSummaryTimesheet();
        assertNotNull(summaryTimesheet.getRegularMonday());
        assertEquals(actual, summaryTimesheet.getRegularMonday());
        assertNotNull(taskAssignment.getEETCHours());
        assertEquals(39.0, taskAssignment.getEETCHours().doubleValue(), 0.001);
        assertNotNull(taskAssignment.getPercentageComplete());
        assertEquals(1.0 / 40.0 * 100.0, taskAssignment.getPercentageComplete()
            .doubleValue(), 0.001);
    }

    private void changeEETC(Double eetc) throws Exception
    {
        taskAssignment.setEETCHours(eetc);
        saveAndValidateSaveSummaryTimesheet();
        assertNotNull(taskAssignment.getEETCHours());
        assertEquals(eetc, taskAssignment.getEETCHours());
        assertNotNull(taskAssignment.getPercentageComplete());
    }

    private void reportActuals() throws Exception
    {
        Double actual = new Double(1);

        summaryTimesheet.setRegularMonday(actual);

        // Always sets the EETC and %C to null when you want the manager
        // to calculate those values
        taskAssignment.setEETCHours(null);
        taskAssignment.setPercentageComplete(null);

        saveAndValidateSaveSummaryTimesheet();
        assertNotNull(summaryTimesheet.getRegularMonday());
        assertEquals(actual, summaryTimesheet.getRegularMonday());
        assertNotNull(taskAssignment.getEETCHours());
        assertEquals(39, taskAssignment.getEETCHours().doubleValue(), 0.001);
        assertNotNull(taskAssignment.getPercentageComplete());
        assertEquals(1 / 40.0 * 100, taskAssignment.getPercentageComplete()
            .doubleValue(), 0.001);
    }
    
    private void saveAndValidateSaveSummaryTimesheet() throws Exception
    {
        result = this.save(summaryTimesheet, new SummaryTimesheetScope());
        assertTrue(result.isSuccessful());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();
        assertNotNull(summaryTimesheet.getTaskAssignment());
        assertTrue(summaryTimesheet.getTaskAssignment() instanceof TaskAssignment);
        taskAssignment = (TaskAssignment) summaryTimesheet.getTaskAssignment();
    }
}
