package com.ibm.rpm.client.junit.wbs.fixtures;

import java.util.Calendar;
import java.util.GregorianCalendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.ScheduleDateFixture;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;

public class WbsScheduleDateFixture extends ScheduleDateFixture
{

    /**
     * Add a valid proposedScheduleDate with a startDate and a durationInMinutes
     * to a WorkElement of type Task or Milestone. Fields finishDate, days,
     * hours and workHours are read-only
     * 
     * ScheduleDated are read-only for projects, proposals, work organizations,
     * deliverables, work products, and summary tasks.
     */
    public static WorkElement addProposedScheduleDate(SimpleNode object,
            ClientTestController ctc) throws Exception
    {
        String errorMsg = null;

        if (object.getResourceTaskAssignments() != null)
        {
            errorMsg = "//All proposedDate fields set to read-only once resourceRoleAssignment contains a value.";
        }
        else if (object.getPlanDate() != null
                && object.getPlanDate().getStartDate() != null)
        {
            errorMsg = "All proposedDate fields set to read-only once planDate.startDate is set.";
        }

        //        else if (object.getEstimatedStartDate() == null)
        //        {
        //
        //            throw new Exception(
        //            "Cannot create a valid proposed date without knowing the
        // estimatedStartDate.");
        //        
        //        }
        else
        {

            WbsScheduleDate scheduleDate = new WbsScheduleDate();

            Calendar startDate = increaseDate(new GregorianCalendar(), 5);
            scheduleDate.setStartDate(startDate);
            scheduleDate.setDurationInMinutes(new Integer(625));

            object.setProposedDate(scheduleDate);
        }

        if (errorMsg != null)
        {

            throw new Exception(
                "WbsScheduleDateFixture.addProposedScheduleDate: " + errorMsg);
        }

        return object;
    }

    /**
     * Add a valid planScheduleDate with a startDate and a durationInMinutes to
     * a WorkElement of type Task or Milestone. Fields finishDate, days, hours
     * and workHours are read-only
     * 
     * ScheduleDated are read-only for projects, proposals, work organizations,
     * deliverables, work products, and summary tasks.
     */
    public static WorkElement addPlanScheduleDate(SimpleNode object,
            ClientTestController ctc) throws Exception
    {
        String errorMsg = null;

        if (object.getResourceTaskAssignments() != null)
        {
            errorMsg = "All planDate fields set to read-only once resourceRoleAssignment contains a value.";
        }
        else if (object.getActualDate() != null
                && object.getActualDate().getStartDate() != null)
        {
            errorMsg = "All planDate fields set to read-only once actualDate.startDate is set.";
        }
        else if (object.getProposedDate() == null
                || (object.getProposedDate() != null && (object
                    .getProposedDate().getStartDate() == null || object
                    .getProposedDate().getDurationInMinutes() == null)))
        {
            errorMsg = "The planDate fields are not editable until proposedDate.startDate and proposedDate.durationInMinutes contain values";
        }
        else
        {

            WbsScheduleDate scheduleDate = new WbsScheduleDate();

            Calendar startDate = increaseDate(object.getProposedDate()
                .getStartDate(), 5);
            scheduleDate.setStartDate(startDate);
            scheduleDate.setDurationInMinutes(new Integer(1200));

            object.setPlanDate(scheduleDate);

        }

        if (errorMsg != null)
        {
            throw new Exception("WbsScheduleDateFixture.addPlanScheduleDate: "
                    + errorMsg);
        }

        return object;
    }

    /**
     * Add a valid actualScheduleDate with a startDate and a durationInMinutes
     * to a WorkElement of type Task or Milestone. Fields finishDate, days,
     * hours and workHours are read-only
     * 
     * ScheduleDated are read-only for projects, proposals, work organizations,
     * deliverables, work products, and summary tasks.
     */
    public static WorkElement addActualScheduleDate(SimpleNode object,
            int percentageOfPlanDuration, ClientTestController ctc)
            throws Exception
    {
        String errorMsg = null;

        if (percentageOfPlanDuration < 0 || percentageOfPlanDuration > 100)
        {
            errorMsg = "percentageOfPlanDuration must be between 0 and 100.";
        }

        if (object.getResourceTaskAssignments() != null)
        {
            errorMsg = "All actualDate fields set to read-only once resourceRoleAssignment contains a value.";
        }
        else if (object.getPlanDate() == null
                || (object.getPlanDate() != null && (object.getPlanDate()
                    .getStartDate() == null
                        || object.getPlanDate().getDurationInMinutes() == null || object
                    .getPlanDate().getDurationInMinutes().intValue() == 0)))
        {
            errorMsg = "The actualDate fields are not editable until planDate.startDate and planDate.durationInMinutes contain values";
        }
        else
        {
            WbsScheduleDate scheduleDate = new WbsScheduleDate();


            Calendar startDate = increaseDate(object.getPlanDate()
                .getStartDate(), 5);
            scheduleDate.setStartDate(startDate);

            //Duration must be between 0 and 100% of the planDuration
            double totalDuration = object.getPlanDate().getDurationInMinutes()
                .intValue();
            int partialDuration = (int) Math.round(percentageOfPlanDuration
                    / 100.0 * totalDuration);

            scheduleDate.setDurationInMinutes(new Integer(partialDuration));
            object.setActualDate(scheduleDate);
        }

        if (errorMsg != null)
        {
            throw new Exception("WbsScheduleDateFixture.addPlanScheduleDate: "
                    + errorMsg);
        }

        return object;
    }

    /**
     * Add a valid forecastScheduleDate with a startDate and a durationInMinutes
     * to a WorkElement of type Task or Milestone. Fields finishDate, days,
     * hours and workHours are read-only
     * 
     * ScheduleDated are read-only for projects, proposals, work organizations,
     * deliverables, work products, and summary tasks.
     */
    public static WorkElement addForecastScheduleDate(SimpleNode object, int percentageOfPlanDuration,
            ClientTestController ctc) throws Exception
    {
        String errorMsg = null;

        if (object.getResourceTaskAssignments() != null)
        {
            errorMsg = "All forecastDate fields set to read-only once resourceRoleAssignment contains a value.";
        }

        else if (object.getActualDate() != null
                && object.getActualDate().getDurationInMinutes() != null
                && (object.getPlanDate().getDurationInMinutes().intValue() == object
                    .getActualDate().getDurationInMinutes().intValue()))
        {

            errorMsg = "The forecastDate.start and forecastDate.finish fields cannot be modified if field actualDate.finishDate contains a value. ActualDate.finishDate will have a value if actualDate.durationInMinutes = planDate.durationInMinutes";
        }
        else if (object.getPlanDate() == null
                || (object.getPlanDate() != null && (object.getPlanDate()
                    .getStartDate() == null
                        || object.getPlanDate().getDurationInMinutes() == null || object
                    .getPlanDate().getDurationInMinutes().intValue() == 0)))
        {
            errorMsg = "The forecastDate fields are not editable until planDate.startDate and planDate.durationInMinutes contain values";
        }
        else
        {
            WbsScheduleDate scheduleDate = new WbsScheduleDate();

            int forecastDuration = 0;

            //Duration must be greater than the actualDuration. Use
            // planDuration to create a duratiuon than will always be
            // greather than the actualDuration.
            
            int planDuration = object.getPlanDate().getDurationInMinutes()
                .intValue();
            forecastDuration = (percentageOfPlanDuration * planDuration)/100;

            scheduleDate.setDurationInMinutes(new Integer(forecastDuration));

            object.setForecastDate(scheduleDate);

        }

        if (errorMsg != null)
        {
            throw new Exception("WbsScheduleDateFixture.addPlanScheduleDate: "
                    + errorMsg);
        }

        return object;
    }

    public static void compareAllWbsScheduleDates(SimpleNode expected,
            SimpleNode result) throws AssertionFailedError, Exception
    {

        //All proposed dates are cleared once the plandate is entered
        if (expected.getPlanDate() == null)
        {
            //proposed
            WbsScheduleDate proposeddate1 = expected.getProposedDate();
            WbsScheduleDate proposeddate2 = result.getProposedDate();

            if (proposeddate1 != null && proposeddate2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }
            WbsScheduleDateFixture.compare(proposeddate1, proposeddate2);

        }

        //All fields are read-only once actualDate.startDate contains a
        // value
        //the planDate fields are edited when forecastDate fields are
        // entered
        if ((expected.getActualDate() == null || expected.getActualDate()
            .getStartDate() == null)
                && expected.getForecastDate() == null)
        {
            //plan
            WbsScheduleDate plandate1 = expected.getPlanDate();
            WbsScheduleDate plandate2 = result.getPlanDate();

            if (plandate1 != null && plandate2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }
            WbsScheduleDateFixture.compare(plandate1, plandate2);

        }

        //actual
        WbsScheduleDate actualdate1 = expected.getActualDate();
        WbsScheduleDate actualdate2 = result.getActualDate();
        WbsScheduleDate plandate1 = expected.getPlanDate();

        //ActualDate with a null duration will be deleted
        if (actualdate1 != null)
        {
            if (actualdate2 == null)
            {
                if (actualdate1.getDurationInMinutes() != null
                        && actualdate1.getDurationInMinutes().intValue() > 0)
                {
                    Assert.fail("RPMObject in scope is null");
                }
            }
            else
            {
                //If actualDate.durationInMinute = planDate.durationInMinutes,
                // that start date will not be updated
                if (actualdate1.getDurationInMinutes().intValue() == plandate1
                    .getDurationInMinutes().intValue())
                {
                    actualdate1.setStartDate(plandate1.getStartDate());
                }
                WbsScheduleDateFixture.compare(actualdate1, actualdate2);
            }

            //forecast
            WbsScheduleDate forecastdate1 = expected.getForecastDate();
            WbsScheduleDate forecastdate2 = result.getForecastDate();
            WbsScheduleDate plandate2 = result.getPlanDate();

            // All forecastDate values entered are copied to
            // other fields and immediately cleared.

            //If forecastDate2 is null, compare with plan date
            if (forecastdate1 != null)
                if (forecastdate2 == null)
                {
                    if (forecastdate1.getDurationInMinutes() != null
                            && forecastdate1.getDurationInMinutes().intValue() > 0)
                    {
                        Assert.fail("RPMObject in scope is null");
                    }
                }
                else
                {
                    WbsScheduleDateFixture.compare(forecastdate1, forecastdate2);
                }
        }

    }

    private static Calendar increaseDate(Calendar initialDate, int numberOfDays)
    {

        Calendar increasedDate = null;

        if (initialDate != null)
        {
            int year = initialDate.get(Calendar.YEAR);
            int month = initialDate.get(Calendar.MONTH);
            int date = initialDate.get(Calendar.DATE) + numberOfDays;

            increasedDate = new GregorianCalendar(year, month, date);
        }

        return increasedDate;
    }

}
