package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test58887 extends ClientTestController
{
    /**
     * 1. Forecast duration(d) and duration(h) fields can be updated as many
     * times as we want as long as the resulting duration is longer than the
     * actual duration. This means that the value can be both raised and
     * lowered, as long as it's superior to the actual date.
     * 
     * 2. As soon as we enter a forecast date smaller than the actual date, the
     * plan date takes on the actual date, and the forecast field becomes
     * read-only.
     * 
     * 3. We can't enter any forecast dates if there are resource task
     * assignments to the current task.
     * 
     * @throws Exception
     */
    WorkElementScope taskScope;

    Task task;

    public void setUp() throws Exception
    {
        super.setUp();
        // create a task with a parent proposal
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        save(proposal, null);

        task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(proposal);
        save(task, null);

        // copy proposal to plan on the task
        copyProposedToPlan(task, null);

        taskScope = new WorkElementScope();
        taskScope.setScheduleDates(true);

        // set task plan date to today and duration to 5 days
        WbsScheduleDate planDate = new WbsScheduleDate();
        
        // December 25th
         Calendar twentyFiveDec = Calendar.getInstance();
         twentyFiveDec.set(Calendar.MONTH, Calendar.DECEMBER);
         twentyFiveDec.set(Calendar.DAY_OF_MONTH, 25);
        
        planDate.setStartDate(twentyFiveDec);
        planDate.setDurationInMinutes(new Integer(5 * 8 * 60)); // 5 working
        // days in
        // minutes

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(Calendar.getInstance());
        actualDate.setDurationInMinutes(new Integer(8* 60)); // 1 day

        task.setPlanDate(planDate);
        task.setActualDate(actualDate);
        task.setPercentageDurationCompleted(new Double(50));
        task = (Task) save(task, taskScope).getRpmObject();
    }

    public void testLaterForecast() throws Exception
    {
        int sixDaysSixHours = 6 * 60 * 8 + 6 * 60;
        SingleResult result = setForecastDateTo(sixDaysSixHours);
        assertTrue(result.isSuccessful());
        Task reloadedTask = (Task) result.getRpmObject();

        assertTrue(reloadedTask.getPlanDate().getDurationInMinutes().intValue() == sixDaysSixHours);
    }

    public void testEarlierForecast() throws Exception
    {
        int threeDays = 1 * 8 * 60;
        Task stealth = (Task) loadFromID(task, taskScope);
        assertTrue(stealth.getActualDate().getDurationInMinutes().intValue() == task.getActualDate().getDurationInMinutes().intValue());

        SingleResult result = setForecastDateTo(threeDays);
        
        assertFalse(result.isSuccessful());
        assertNotNull(result.getErrors());
        assertTrue(result.getErrors().length == 1);
        assertTrue(result.getErrors()[0].getErrorID() == ErrorCodes.GREATER_THAN_OR_EQUAL_FIELD);
        
        Task reloadedTask = (Task) loadFromID(task, taskScope);

        assertTrue(reloadedTask.getPlanDate().getDurationInMinutes().intValue() == 
            task.getPlanDate().getDurationInMinutes().intValue());
    }

    /**
     * Increment the plandate by five hour increments, by updating the forecast date.
     * Actuals should not change, but the plan date should be updated accordingly.
     * @throws Exception
     */
    public void testIncrementAutomaticallyForecastDate() throws Exception
    {
        final int incrementAmountInHours = 5 * 60;
        int it = 5;
        final int actualDurationMinutes = task.getActualDate().getDurationInMinutes().intValue();
        
        do
        {
            int currentPlanDurationMinutes = task.getPlanDate().getDurationInMinutes().intValue();
            int desiredPlanDurationMinutes = currentPlanDurationMinutes + incrementAmountInHours;
            SingleResult saveResult = setForecastDateTo(desiredPlanDurationMinutes);
            Task localTask = (Task) saveResult.getRpmObject();
            assertTrue(localTask.getPlanDate().getDurationInMinutes().intValue() == desiredPlanDurationMinutes);
            assertTrue("Actuals have changed, without us having asked for it", localTask.getActualDate().getDurationInMinutes().intValue() == actualDurationMinutes);
            task = localTask;
        } while(it-- > 0);
    }
    
    public SingleResult setForecastDateTo(int durationInMinutes)
            throws Exception
    {
        SingleResult result = null;

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(durationInMinutes));

        // save and reload
        task.setForecastDate(forecastDate);
        result = saveMustFail(task, taskScope);

        return result;
    }
}
