/*
 * Created on Nov 16, 2005 
 * @author mmatwiy 
 * Validation of WBS container Project.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

public class TestProjectValidation extends AbstractTestWorkElementValidation
{
    private Project project;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        project = new Project();

        // create and set invalid parent
        Portfolio portfolio = new Portfolio();

        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        project
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT));

        project.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, project.getClass()
                .getName(), ValidationConstants.PARENT, project.getID())
        };

        verifySaveMustFail(project, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        project = new Project();
        project.setEstimatedStartDate(new GregorianCalendar(2005, 12, 11));
        project.setEstimatedFinishDate(new GregorianCalendar(2006, 12, 12));

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        project
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT));
        project.setProbabilityPercent(new Integer((int) Math.random() * 100));
        
        Currency currency = DatafieldFixture
            .createCurrency(DatafieldFixture.CURRENCY_USD);
        project.setCurrency(currency);

        // Attempt to save the project object
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (WorkElement) rpmObjects[0], null);
    }

    public void testInsertPublishedToTemplateNoParent() throws AssertionError,
            Exception
    {
        // once we support the type 'ToTemplate' we'll need to check
        // that the parent has an ID
        // for now - 'ToTemplate' not supported
        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        project.setPublished(ProjectPublishedType.ToTemplate);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.PUBLISHED_TO_TEMPLATE_NOT_SUPPORTED,
                project.getClass().getName(),
                ValidationConstants.PUBLISHED_FIELD, project.getID())
        };

        verifySaveMustFail(project, null, expectedExceptions);

    }

    public void testInsertSuccessModifiedEstimatedDates()
            throws AssertionError, Exception
    {
        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set both estimated dates to today's date
        project.setEstimatedStartDate(Calendar.getInstance());
        project.setEstimatedFinishDate(Calendar.getInstance());

        // Attempt to save the project object
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (WorkElement) rpmObjects[0], null);
    }

    public void testFailureInvalidEstimatedFinishDate() throws AssertionError,
            Exception
    {
        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set the estimatedFinishDate to be prior to the estimatedStartDate
        project.setEstimatedStartDate(new GregorianCalendar(2005, 12, 12));
        project.setEstimatedFinishDate(new GregorianCalendar(2005, 12, 11));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_DATES, project.getClass()
                .getName(), ValidationConstants.ESTIMATED_START_DATE_FIELD,
                project.getID())
        };

        verifySaveMustFail(project, null, expectedExceptions);
    }

    public void testFailureReadOnlyScheduleDates() throws AssertionError,
            Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
        
        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar());
        actualDate.setDurationInMinutes(new Integer(4000));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(5000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(3000));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(3000));

        project.setActualDate(actualDate);
        project.setForecastDate(forecastDate);
        project.setPlanDate(planDate);
        project.setProposedDate(proposedDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, project
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_FIELD, project
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, project
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_FIELD, project
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, project
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_FIELD, project
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, project
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_FIELD, project
                        .getID()),
        };

        verifySaveMustFail(project, scope, null, expectedWarnings);

        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        Project loadedProject = (Project) rpmObjects[0];

        // assert that the values we set were not saved in the db
        assertEquals(null, loadedProject.getActualDate());
        assertEquals(null, loadedProject.getForecastDate());
        assertEquals(null, loadedProject.getPlanDate());
        assertNotSame(project.getProposedDate(), loadedProject
            .getProposedDate());
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        project = new Project();

        // Set string values with max length + 1 and verify that they are not
        // accepted
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT + 1));
        project
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT + 1));
        project
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT + 1));
        project.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MAX + 1));
        

        // Set integers to max value + 1 and verify that they are not accepted
        project.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, project.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, project.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, project.getClass()
                    .getName(), ValidationConstants.REFERENCE_NUMBER_FIELD,
                    project.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, project.getClass()
                    .getName(), ValidationConstants.EXTERNAL_ID_FIELD, project
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, project.getClass()
                    .getName(), ValidationConstants.PRIORITY_FIELD, project.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, project.getClass()
                    .getName(), ValidationConstants.PROBABILITY_PERCENT_FIELD,
                    project.getID()),
        };

        verifySaveMustFail(project, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        project = new Project();

        // Set integers to min value -1 and verify that they are not accepted
        project.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        project.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MIN - 1));
        project.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, project.getClass()
                    .getName(), ValidationConstants.PRIORITY_FIELD, project.getID()),
                createRPMException(ErrorCodes.MIN_LENGTH, project.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, project.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, project.getClass()
                    .getName(), ValidationConstants.PROBABILITY_PERCENT_FIELD,
                    project.getID()),
        };

        verifySaveMustFail(project, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        project = new Project();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, project.getClass()
                .getName(), ValidationConstants.NAME_FIELD, project.getID())
        };

        verifySaveMustFail(project, null, expectedExceptions);
    }

    public void testInsertWithAttributePortfolio() throws AssertionError,
            Exception
    {
        project = new Project();

        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        project.setPortfolio(portfolio);

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Attempt to save the Project object
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded Project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], null);
    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {
        project = new Project();

        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        project.setParent(workOrganization);

        // Attempt to save the Project object
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded Project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], null);
    }

    public void testInsertWithParentProposalAndPublishedToTeam()
            throws AssertionError, Exception
    {
        project = new Project();

        Proposal parentProposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        project.setParent(parentProposal);
        project.setPublished(ProjectPublishedType.ToTeam);

        // Attempt to save the Project object
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded Project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], null);
    }

    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        project = new Project();

        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        project.setParent(parentProject);

        // Attempt to save the Project object
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded Project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], null);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        project = new Project();

        // Set field values with maxLength and verify that they are accepted
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        project
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT));
        project
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT));
        project.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MAX));

        // Set integers to max value and verify that they are accepted
        project.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Attempt to save the Project object
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded Project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set integers to min value and verify that they are accepted
        project.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        project.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MIN));

        // Attempt to save the Project object
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded Project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], null);
    }

    public void testFailureInvalidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);

        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        project.setState(new DeliverableState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, project.getClass()
                .getName(), ValidationConstants.STATE_FIELD, project.getID())
        };

        verifySaveMustFail(project, scope, expectedExceptions);
    }

    public void testSuccessValidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);

        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        project.setState(DatafieldFixture.createState(project));

        // Attempt to save the Project object
        project = (Project) saveMustSucceed(project, Project.class, scope);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded Project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], scope);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setTimeCodes(true);

        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // assign the timecodes to the project
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, false, this);

        project.setTimeCode1(timecode2);
        project.setTimeCode2(timecode1);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, project
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE1_FIELD, project.getID()),
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, project
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE2_FIELD, project.getID()),
        };

        verifySaveMustFail(project, scope, expectedExceptions);
    }

    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        project = new Project();

        WorkElementScope scope = new WorkElementScope();
        scope.setAssetCode(true);

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        project = (Project) saveMustSucceed(project, Project.class, scope);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the default values are now set
        Project savedProject = (Project) rpmObjects[0];
        assertNotNull(savedProject.getAssetCode());
        assertNotNull(savedProject.getEstimatedStartDate());
        assertNotNull(savedProject.getPriority());
        assertNotNull(savedProject.getAssignmentType());
        assertNotNull(savedProject.getCalendar());
        assertNotNull(savedProject.getDefaultProject());
        assertNotNull(savedProject.getPublished());
        assertNotNull(savedProject.getProbabilityPercent());

        // Save again to validate the default values
        project = (Project) saveMustSucceed(project, Project.class, scope);

        // Load the inserted project from xpath
        rpmObjects = loadFromXpath("/Project[id='" + project.getID() + "']",
            scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded Project is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], scope);
    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        project = new Project();

        WorkElementScope scope = new WorkElementScope();
        scope.setAssetCode(true);

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        project = (Project) saveMustSucceed(project, Project.class, scope);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the default values are now set
        Project savedProject = (Project) rpmObjects[0];
        assertNotNull(savedProject.getAssetCode());
        assertNotNull(savedProject.getEstimatedStartDate());
        assertNotNull(savedProject.getPriority());
        assertNotNull(savedProject.getAssignmentType());
        assertNotNull(savedProject.getCalendar());
        assertNotNull(savedProject.getDefaultProject());
        assertNotNull(savedProject.getPublished());
        assertNotNull(savedProject.getProbabilityPercent());

        // Update the default values to null
        savedProject.setAssetCode(null);
        savedProject.setEstimatedStartDate(null);
        savedProject.setPriority(null);
        savedProject.setAssignmentType(null);
        //savedProject.setCalendar(null);  not an updatable field
        savedProject.setDefaultProject(null);
        savedProject.setPublished(null);
        savedProject.setProbabilityPercent(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedProject
                    .getClass().getName(),
                    ValidationConstants.ASSET_CODE_FIELD, savedProject.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedProject
                    .getClass().getName(),
                    ValidationConstants.ESTIMATED_START_DATE_FIELD,
                    savedProject.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedProject
                    .getClass().getName(), ValidationConstants.PRIORITY_FIELD,
                    savedProject.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedProject
                    .getClass().getName(),
                    ValidationConstants.PROJECT_ASSIGNMENT_TYPE_FIELD,
                    savedProject.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedProject
                    .getClass().getName(),
                    ValidationConstants.DEFAULT_PROJECT_FIELD, savedProject
                        .getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedProject
                    .getClass().getName(), ValidationConstants.PUBLISHED_FIELD,
                    savedProject.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedProject.getClass().getName(),
                    ValidationConstants.PROBABILITY_PERCENT_FIELD,
                    savedProject.getID()),
        };

        verifySaveMustFail(savedProject, scope, expectedExceptions);

    }

    public void testFailureUpdateCalendarAndCurrency() throws AssertionError,
            Exception
    {
        project = new Project();

        // Set the mandatory fields
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        project.setCalendar(RPMCalendarFixture.createSaveDefaultCalendar(null,
            this));
        project.setCurrency(DatafieldFixture
            .createCurrency(DatafieldFixture.CURRENCY_USD));

        // Leave default values as null and save
        project = (Project) saveMustSucceed(project, Project.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + project.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        Project savedProject = (Project) rpmObjects[0];

        // Update the calendar and currency
        savedProject.setCalendar(new RPMCalendar());
        savedProject.setCurrency(DatafieldFixture
            .createCurrency(DatafieldFixture.CURRENCY_CAD));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    ErrorCodes.CANNOT_UPDATE_FIELD_ON_CONTAINER_UPDATE,
                    savedProject.getClass().getName(),
                    ValidationConstants.PROJECT_CALENDAR_FIELD, savedProject
                        .getID()),
                createRPMException(
                    ErrorCodes.CANNOT_UPDATE_FIELD_ON_CONTAINER_UPDATE,
                    savedProject.getClass().getName(),
                    ValidationConstants.PROJECT_CURRENCY_FIELD, savedProject
                        .getID())
        };

        verifySaveMustFail(savedProject, null, expectedExceptions);

    }
    
    public void testReadOnlyFields() throws Exception
    {
        project = new Project();
        project
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        
        super.testReadOnlyFields(project);
    }
}