/*
 * Created on Nov 16, 2005 
 * @author mmatwiy 
 * Validation of WBS container Proposal.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestProposalValidation extends AbstractTestWorkElementValidation
{
    private Proposal proposal;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        proposal = new Proposal();

        // create and set invalid parent
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
            this);

        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        proposal.setParent(task);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, proposal
                .getClass().getName(), ValidationConstants.PARENT, proposal
                .getID())
        };

        verifySaveMustFail(proposal, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        proposal
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT));
        proposal.setEstimatedStartDate(new GregorianCalendar(2005, 12, 11));
        proposal.setEstimatedFinishDate(new GregorianCalendar(2006, 12, 12));
        proposal.setProbabilityPercent(new Integer((int) Math.random() * 100));


        Currency currency = DatafieldFixture
            .createCurrency(DatafieldFixture.CURRENCY_USD);
        proposal.setCurrency(currency);

        // Attempt to save the object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (WorkElement) rpmObjects[0], null);
    }

    public void testInsertSuccessModifiedEstimatedDates()
            throws AssertionError, Exception
    {
        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set both estimated dates to today's date
        proposal.setEstimatedStartDate(Calendar.getInstance());
        proposal.setEstimatedFinishDate(Calendar.getInstance());

        // Attempt to save the object object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (WorkElement) rpmObjects[0], null);
    }

    public void testFailureInvalidEstimatedFinishDate() throws AssertionError,
            Exception
    {
        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set the estimatedFinishDate to be prior to the estimatedStartDate
        proposal.setEstimatedStartDate(new GregorianCalendar(2005, 12, 12));
        proposal.setEstimatedFinishDate(new GregorianCalendar(2005, 12, 11));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_DATES, proposal.getClass()
                .getName(), ValidationConstants.ESTIMATED_START_DATE_FIELD,
                proposal.getID())
        };

        verifySaveMustFail(proposal, null, expectedExceptions);
    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {
        proposal = new Proposal();

        WorkOrganization parentWorkOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        proposal.setParent(parentWorkOrganization);

        // Attempt to save the Proposal object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded Proposal is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        proposal = new Proposal();

        // Set string values with max length + 1 and verify that they are not
        // accepted
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT + 1));
        proposal
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT + 1));
        proposal
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT + 1));
        proposal.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MAX + 1));

        // Set integers to max value + 1 and verify that they are not accepted
        proposal.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, proposal.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, proposal
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, proposal.getClass()
                    .getName(), ValidationConstants.REFERENCE_NUMBER_FIELD,
                    proposal.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, proposal.getClass()
                    .getName(), ValidationConstants.EXTERNAL_ID_FIELD, proposal
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, proposal.getClass()
                    .getName(), ValidationConstants.PRIORITY_FIELD, proposal
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, proposal.getClass()
                    .getName(), ValidationConstants.PROBABILITY_PERCENT_FIELD,
                    proposal.getID()),
        };

        verifySaveMustFail(proposal, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set integers to min value -1 and verify that they are not accepted
        proposal.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        proposal.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, proposal.getClass()
                    .getName(), ValidationConstants.PRIORITY_FIELD, proposal
                    .getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, proposal.getClass()
                    .getName(), ValidationConstants.PROBABILITY_PERCENT_FIELD,
                    proposal.getID()),
        };

        verifySaveMustFail(proposal, null, expectedExceptions);
    }

    public void testFailureReadOnlyFields() throws AssertionError,
            Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
        
        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // lastModifiedDate is read-only
        proposal.setLastModifiedDate(new GregorianCalendar());

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar());
        actualDate.setDurationInMinutes(new Integer(4000));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(5000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(3000));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(3000));

        proposal.setActualDate(actualDate);
        proposal.setForecastDate(forecastDate);
        proposal.setPlanDate(planDate);
        proposal.setProposedDate(proposedDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_FIELD, proposal
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_FIELD, proposal
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_FIELD, proposal
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_FIELD, proposal
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal
                    .getClass().getName(),
                    ValidationConstants.LAST_MODIFIED_DATE_FIELD, proposal
                        .getID()),
        };

        verifySaveMustFail(proposal, scope, null, expectedWarnings);

        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        Proposal loadedProposal = (Proposal) rpmObjects[0];

        // assert that the values we set were not saved in the db
        assertEquals(null, loadedProposal.getActualDate());
        assertEquals(null, loadedProposal.getForecastDate());
        assertEquals(null, loadedProposal.getPlanDate());
        assertNotSame(proposal.getProposedDate(), loadedProposal
            .getProposedDate());
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        proposal = new Proposal();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, proposal.getClass()
                .getName(), ValidationConstants.NAME_FIELD, proposal.getID())
        };

        verifySaveMustFail(proposal, null, expectedExceptions);
    }

    public void testInsertWithAttributePortfolio() throws AssertionError,
            Exception
    {
        proposal = new Proposal();

        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        proposal.setPortfolio(portfolio);

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Attempt to save the Proposal object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded Proposal is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0], null);
    }

    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        proposal = new Proposal();

        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        proposal.setParent(parentProject);

        // Attempt to save the Proposal object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded Proposal is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0], null);
    }

    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        proposal = new Proposal();

        Proposal parentProposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        proposal.setParent(parentProposal);

        // Attempt to save the Proposal object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded Proposal is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0], null);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        proposal = new Proposal();

        // Set field values with maxLength and verify that they are accepted
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        proposal
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT));
        proposal
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT));
        proposal.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MAX));

        // Set integers to max value and verify that they are accepted
        proposal.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Attempt to save the Proposal object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded Proposal is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0], null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set integers to min value and verify that they are accepted
        proposal.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        proposal.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MIN));


        // Attempt to save the Proposal object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded Proposal is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0], null);
    }

    public void testFailureInvalidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);

        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        proposal.setState(new DeliverableState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, proposal.getClass()
                .getName(), ValidationConstants.STATE_FIELD, proposal.getID())
        };

        verifySaveMustFail(proposal, scope, expectedExceptions);
    }

    public void testSuccessValidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);

        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        proposal.setState(DatafieldFixture.createState(proposal));

        // Attempt to save the Proposal object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + proposal.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded Proposal is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0], scope);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setTimeCodes(true);

        proposal = new Proposal();

        // Set the mandatory fields
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // assign the timecodes 
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, false, this);

        proposal.setTimeCode1(timecode2);
        proposal.setTimeCode2(timecode1);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, proposal
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE1_FIELD, proposal.getID()),
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, proposal
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE2_FIELD, proposal.getID()),
        };

        verifySaveMustFail(proposal, scope, expectedExceptions);
    }
    
    public void testReadOnlyFields() throws Exception
    {
        proposal = new Proposal();
        proposal
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        
        super.testReadOnlyFields(proposal);
    }
}