/*
 * Created on Nov 30, 2005 
 * @author mmatwiy 
 * Validation tests for container SummaryTask.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.State;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestSummaryTaskValidation extends AbstractTestWorkElementValidation
{
    private SummaryTask summaryTask;

    private Project parentProject;

    public void setUp() throws Exception
    {
        super.setUp();

        summaryTask = new SummaryTask();

        parentProject = (Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this);

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
            this);

        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setParent(task);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, summaryTask
                .getClass().getName(), ValidationConstants.PARENT, summaryTask
                .getID())
        };

        verifySaveMustFail(summaryTask, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        summaryTask.setEstimatedStartDate(new GregorianCalendar(2005, 12, 11));
        summaryTask.setEstimatedFinishDate(new GregorianCalendar(2006, 12, 12));

        // Attempt to save the object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (WorkElement) rpmObjects[0],
            null);
    }

    public void testFailureInvalidEstimatedFinishDate() throws AssertionError,
            Exception
    {
        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set the estimatedFinishDate to be prior to the estimatedStartDate
        summaryTask.setEstimatedStartDate(new GregorianCalendar(2006, 01, 01));
        summaryTask.setEstimatedFinishDate(new GregorianCalendar(2005, 12, 01));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_DATES, summaryTask.getClass()
                .getName(), ValidationConstants.ESTIMATED_START_DATE_FIELD,
                summaryTask.getID())
        };

        verifySaveMustFail(summaryTask, null, expectedExceptions);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTask.setParent(parentProject);

        // Set string values with max length + 1 and verify that they are not
        // accepted
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT + 1));
        summaryTask
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_ACTIVITY_REFERENCE_NUMBER_CHARACTER_LIMIT + 1));
        summaryTask
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT + 1));

        // Set integers to max value + 1 and verify that they are not accepted
        summaryTask.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, summaryTask
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    summaryTask.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTask
                    .getClass().getName(), ValidationConstants.PRIORITY_FIELD,
                    summaryTask.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, summaryTask
                    .getClass().getName(),
                    ValidationConstants.REFERENCE_NUMBER_FIELD, summaryTask
                        .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, summaryTask
                    .getClass().getName(),
                    ValidationConstants.EXTERNAL_ID_FIELD, summaryTask.getID())

        };

        verifySaveMustFail(summaryTask, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set integers to min value -1 and verify that they are not accepted
        summaryTask.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTask.getClass()
                .getName(), ValidationConstants.PRIORITY_FIELD, summaryTask.getID()),
        };

        verifySaveMustFail(summaryTask, null, expectedExceptions);
    }

    public void testFailureReadOnlyScheduleDates() throws AssertionError,
            Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
        
        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar());
        actualDate.setDurationInMinutes(new Integer(4000));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(5000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(3000));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(3000));

        summaryTask.setActualDate(actualDate);
        summaryTask.setForecastDate(forecastDate);
        summaryTask.setPlanDate(planDate);
        summaryTask.setProposedDate(proposedDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTask
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_FIELD, summaryTask
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTask
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_FIELD,
                    summaryTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTask
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_FIELD, summaryTask
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTask
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_FIELD,
                    summaryTask.getID()),
        };

        verifySaveMustFail(summaryTask, scope, null, expectedWarnings);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);
       
        SummaryTask loadedSummaryTask = (SummaryTask) rpmObjects[0];

        // assert that the values we set were not saved in the db
        assertEquals(null, loadedSummaryTask.getActualDate());
        assertEquals(null, loadedSummaryTask.getForecastDate());
        assertEquals(null, loadedSummaryTask.getPlanDate());
        assertNotSame(summaryTask.getProposedDate(), loadedSummaryTask.getProposedDate());
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        summaryTask.setParent(parentProject);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, summaryTask.getClass()
                .getName(), ValidationConstants.NAME_FIELD, summaryTask.getID())
        };

        verifySaveMustFail(summaryTask, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, summaryTask.getClass()
                .getName(), ValidationConstants.PARENT, summaryTask.getID())
        };

        verifySaveMustFail(summaryTask, null, expectedExceptions);

    }

    public void testInsertWithParentDeliverable() throws AssertionError,
            Exception
    {
        Deliverable parentDeliverable = (Deliverable) WorkElementFixture
            .retrieveStaticFixture(new Deliverable(), this);

        // Set the mandatory fields
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setParent(parentDeliverable);

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testInsertWithParentWorkProduct() throws AssertionError,
            Exception
    {
        WorkProduct parentWorkProduct = (WorkProduct) WorkElementFixture
            .retrieveStaticFixture(new WorkProduct(), this);

        // Set the mandatory fields
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setParent(parentWorkProduct);

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {

        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);

        // Set the mandatory fields
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setParent(workOrganization);

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testInsertWithParentSummaryTask() throws AssertionError,
            Exception
    {
        SummaryTask summaryTask = (SummaryTask) WorkElementFixture
            .retrieveStaticFixture(new SummaryTask(), this);

        // Set the mandatory fields
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setParent(summaryTask);

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        Proposal parentProposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);

        // Set the mandatory fields
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setParent(parentProposal);

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setParent(parentProject);

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testInsertWithParentProjectInPortfolio() throws AssertionError,
            Exception
    {
        Portfolio parentPortfolio = PortfolioFixture.createSaveFixture(this);

        // Set the mandatory fields
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        parentProject.setParent(parentPortfolio);
        summaryTask.setParent(parentProject);

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTask.setParent(parentProject);

        // Set field values with maxLength and verify that they are accepted
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        summaryTask
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_ACTIVITY_REFERENCE_NUMBER_CHARACTER_LIMIT));
        summaryTask
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT));

        // Set integers to max value and verify that they are accepted
        summaryTask.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set integers to min value and verify that they are accepted
        summaryTask.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));

        // Attempt to save the SummaryTask object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded SummaryTask is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            null);
    }

    public void testFailureInvalidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);
        
        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setState(new State());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, summaryTask.getClass()
                .getName(), ValidationConstants.STATE_FIELD, summaryTask
                .getID())
        };

        verifySaveMustFail(summaryTask, scope, expectedExceptions);
    }

    public void testSuccessValidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);

        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        summaryTask.setState(DatafieldFixture.createState(summaryTask));

        // Attempt to save the object
        summaryTask = (SummaryTask) saveMustSucceed(summaryTask,
            SummaryTask.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[id='"
                + summaryTask.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(summaryTask, (SummaryTask) rpmObjects[0],
            scope);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setTimeCodes(true);

        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // assign the timecodes 
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, false, this);

        summaryTask.setTimeCode1(timecode2);
        summaryTask.setTimeCode2(timecode1);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, summaryTask
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE1_FIELD, summaryTask.getID()),
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, summaryTask
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE2_FIELD, summaryTask.getID()),
        };

        verifySaveMustFail(summaryTask, scope, expectedExceptions);
    }
    
    public void testReadOnlyFields() throws Exception
    {
        // Set the mandatory fields
        summaryTask.setParent(parentProject);
        summaryTask
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        
        super.testReadOnlyFields(summaryTask);
    }
}
