/*
 * Created on Mar 8, 2006
 * @author mmatwiy
 * Validation tests for container TaskAssignment with non-null actualStartDate field.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.GregorianCalendar;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTaskAssignmentWithActualStartDateValidation extends
        GenericValidation
{
    private TaskAssignmentScope taskAssignmentScope;

    private TaskAssignment loadedTaskAssignment;

    public void setUp() throws Exception
    {
        super.setUp();

        // create the grandparent
        Task parentTask = (Task) TaskAssignmentFixture
            .createProperWorkElement(this);

        // create the grandparent, parent & TaskAssignment scope
        ResourceTaskAssignmentScope rtaScope = ResourceTaskAssignmentFixture
            .createScope(true, true, false, true, false);
        taskAssignmentScope = TaskAssignmentFixture.createScope(true, false,
            false, false);
        rtaScope.setTaskAssignment(taskAssignmentScope);

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(rtaScope);
        workElementScope.setScheduleDates(true);

        // create a resource to attach to the parent
        Resource resource = TaskAssignmentFixture.createProperResource(
            parentTask, this);

        // create the parent
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();
        resourceTaskAssignment.setParent(parentTask);
        resourceTaskAssignment.setResource(resource);

        SecurityRole securityRole = new SecurityRole();
        securityRole.setID("SCTASKRESPONSIBLE");
        resourceTaskAssignment.setSecurityRole(securityRole);

        ResourceTaskAssignment[] resourceTaskAssignments =
        {
            resourceTaskAssignment
        };
        parentTask.setResourceTaskAssignments(resourceTaskAssignments);

        // create the TaskAssignment container
        TaskAssignment taskAssignment = new TaskAssignment();
        taskAssignment.setParent(resourceTaskAssignment);

        resourceTaskAssignment.setTaskAssignment(taskAssignment);
        taskAssignment.setAssignmentStartDate(new GregorianCalendar());

        save(resourceTaskAssignment, rtaScope);

        // create a timesheet with actual work hours
        SummaryTimesheet timesheet = SummaryTimesheetFixture.createFixture(
            taskAssignment, null, new SummaryTimesheetScope(), true, this);
        save(timesheet, new SummaryTimesheetScope());

        // Must reload to get the actualStartDate
        RPMObject[] rpmObjects = loadFromXpath("/TaskAssignment[id='"
                + taskAssignment.getID() + "']", taskAssignmentScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TaskAssignment);

        loadedTaskAssignment = (TaskAssignment) rpmObjects[0];
        assertNotNull(loadedTaskAssignment);
        assertNotNull(loadedTaskAssignment.getActualStartDate());

    }

    public void testConditionalReadOnly() throws AssertionError, Exception
    {
        loadedTaskAssignment.setAssignmentStartDate(new GregorianCalendar(2006,
            10, 10));
        loadedTaskAssignment.setDaysOfDelay(new Integer(
            ValidationConstants.DAYS_OF_DELAY_MAX));

        // assignmentStartDate & daysOfDelay read-only if actualStartDate has a value

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.ASSIGNMENT_START_DATE_FIELD,
                    ValidationConstants.ACTUAL_START_DATE_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.DAYS_OF_DELAY_FIELD,
                    ValidationConstants.ACTUAL_START_DATE_FIELD,
                    loadedTaskAssignment.getID())
        };

        verifySaveMustFail(loadedTaskAssignment, taskAssignmentScope, null,
            expectedWarnings);

        RPMObject[] rpmObjects = loadFromXpath("/TaskAssignment[id='"
                + loadedTaskAssignment.getID() + "']", taskAssignmentScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TaskAssignment);

        TaskAssignment loadedAssignment = (TaskAssignment) rpmObjects[0];

        //check that the values we set were not saved in the db
        assertNotSame(new GregorianCalendar(2006, 10, 10), loadedAssignment
            .getAssignmentStartDate());
        assertNotSame(new Integer(ValidationConstants.DAYS_OF_DELAY_MAX),
            loadedAssignment.getDaysOfDelay());
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // in order for actualWork, percentageComplete, EETCHours, 
        // and percentageWork to be editable, an actualStartDate
        // must exist for one of the assigned resources

        // Set values to max + 1 and verify that they are not accepted
        // actualWork, percentageComplete, and percentageWork cannot
        // be modified and saved at the same time 
        loadedTaskAssignment.setActualWork(new Double(
            ValidationConstants.ACTUAL_WORK_MAX + 1));
        loadedTaskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MAX + 1));
        loadedTaskAssignment.setPercentageWork(new Double(
            ValidationConstants.PERCENTAGE_WORK_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.ACTUAL_WORK_FIELD, loadedTaskAssignment
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.PERCENTAGE_WORK_FIELD,
                    loadedTaskAssignment.getID()),
        };

        // a warning will be generated because actualWork, percentageComplete,
        // and percentageWork are being modified and saved at the same time 
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.INDIVIDUAL_MODIFICATION_REQUIRED,
                loadedTaskAssignment.getClass().getName())

        };

        verifySaveMustFail(loadedTaskAssignment, taskAssignmentScope,
            expectedExceptions, expectedWarnings);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // in order for actualWork, percentageComplete, EETCHours, 
        // and percentageWork to be editable, an actualStartDate
        // must exist for one of the assigned resources

        // Set numeric values to min - 1 and verify that they are not accepted;
        // actualWork, percentageComplete, and percentageWork cannot
        // be modified and saved at the same time 
        loadedTaskAssignment.setActualWork(new Double(
            ValidationConstants.ACTUAL_WORK_MIN - 1));
        loadedTaskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MIN - 1));
        loadedTaskAssignment.setPercentageWork(new Double(
            ValidationConstants.PERCENTAGE_WORK_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.ACTUAL_WORK_FIELD, loadedTaskAssignment
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.PERCENTAGE_WORK_FIELD,
                    loadedTaskAssignment.getID())
        };

        // a warning will be generated because actualWork, percentageComplete,
        // and percentageWork are being modified and saved at the same time 
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.INDIVIDUAL_MODIFICATION_REQUIRED,
                loadedTaskAssignment.getClass().getName())

        };

        verifySaveMustFail(loadedTaskAssignment, taskAssignmentScope,
            expectedExceptions, expectedWarnings);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // in order for actualWork, percentageComplete, EETCHours, 
        // and percentageWork to be editable, an actualStartDate
        // must exist for one of the assigned resources

        // Set doubles to max value and verify that they are accepted
        loadedTaskAssignment.setActualWork(new Double(
            ValidationConstants.ACTUAL_WORK_MAX));
        loadedTaskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MAX));
        loadedTaskAssignment.setPercentageWork(new Double(
            ValidationConstants.PERCENTAGE_WORK_MAX));

        // the max values will be accepted and no warning will be generated
        // because percentageComplete and percentageWork are already set to 
        // their max values and therefore their modified flags won't be set

        saveMustSucceed(loadedTaskAssignment, TaskAssignment.class,
            taskAssignmentScope);

    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // in order for actualWork, percentageComplete, EETCHours, 
        // and percentageWork to be editable, an actualStartDate
        // must exist for one of the assigned resources

        // Set numerics to min value and verify that they are accepted
        loadedTaskAssignment.setActualWork(new Double(
            ValidationConstants.ACTUAL_WORK_MIN));
        loadedTaskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MIN));
        loadedTaskAssignment.setPercentageWork(new Double(
            ValidationConstants.PERCENTAGE_WORK_MIN));

        // the min values will be accepted but a warning will be generated
        // because actualWork, percentageComplete, and percentageWork cannot
        // be modified and saved at the same time 
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.INDIVIDUAL_MODIFICATION_REQUIRED,
                loadedTaskAssignment.getClass().getName())
        };

        verifySaveMustFail(loadedTaskAssignment, taskAssignmentScope, null,
            expectedWarnings);
    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        // Check that the default values are now set (saved in setup)
        assertNotNull(loadedTaskAssignment.getActualWork());
        assertNotNull(loadedTaskAssignment.getEETCHours());

        // Update the editable default values to null
        loadedTaskAssignment.setActualWork(null);
        loadedTaskAssignment.setEETCHours(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.ACTUAL_WORK_FIELD, loadedTaskAssignment
                        .getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.EETC_HOURS_FIELD, loadedTaskAssignment
                        .getID())
        };

        verifySaveMustFail(loadedTaskAssignment, taskAssignmentScope,
            expectedExceptions);

    }

    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        // Check that the default values are now set (saved in setup)
        assertNotNull(loadedTaskAssignment.getActualWork());
        assertNotNull(loadedTaskAssignment.getEETCHours());

        loadedTaskAssignment = (TaskAssignment) saveMustSucceed(
            loadedTaskAssignment, TaskAssignment.class, taskAssignmentScope);

        RPMObject[] rpmObjects = loadFromXpath("/TaskAssignment[id='"
                + loadedTaskAssignment.getID() + "']", taskAssignmentScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TaskAssignment);

        // Check that the loaded TaskAssignment is identical to the one we
        // requested to store
        TaskAssignmentFixture.compare(loadedTaskAssignment,
            (TaskAssignment) rpmObjects[0], taskAssignmentScope);

    }

    public void testFailureAcceptTaskAssignmentChanges() throws AssertionError,
            Exception
    {
        loadedTaskAssignment.setActualWork(new Double(5.0));
        loadedTaskAssignment.setEETCHours(new Double(6.0));

        (((ResourceTaskAssignment) loadedTaskAssignment.getParent())
            .getResource()).setDiscontinueDate(new GregorianCalendar(2006, 12,
            8));
        (((ResourceTaskAssignment) loadedTaskAssignment.getParent())
            .getResource()).setEffectiveStartDate(new GregorianCalendar(2006,
            12, 4));
        (((ResourceTaskAssignment) loadedTaskAssignment.getParent())
            .getResource()).getCalendar().setHoursPerDay(new Double(8));
        //resource available for 40 hours (EETCHours must be less than 40)

        // save the assignment
        loadedTaskAssignment = (TaskAssignment) saveMustSucceed(
            loadedTaskAssignment, TaskAssignment.class, taskAssignmentScope);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_APPROVE_REJECT_TASK_ASSIGNMENT,
                loadedTaskAssignment.getClass().getName(), null,
                loadedTaskAssignment.getID())
        };

        // try to approve changes, when there has been none
        SaveResult saveResult = getApplicationInterface()
            .acceptTaskAssignmentChanges(sessionid, loadedTaskAssignment,
                taskAssignmentScope, true, true);

        String errorMessage = verifyExceptions(expectedExceptions, saveResult
            .getErrors());

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }

}