/*
 * Created on 4-Oct-2006
 * @author mmatwiy
 * 
 * WorkflowProcess control and xpath test cases for a GenericDocument.
 * This class tests these xpaths, with conditions:
 * - /Document/AvailableWorkflowProcess
 * - /Document[AvailableWorkflowProcess]
 * - /Document/RunningWorkflowProcess
 * - /Document[RunningWorkflowProcess]
 */
package com.ibm.rpm.client.junit.workflow.testcases;

import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestWorkflowProcessDocument extends AbstractTestWorkflowProcess
{
    private WorkflowScope workflowScope = null;
    
    private RunningWorkflowProcessScope runningScope = null;
    
    private AvailableWorkflowProcessScope availableScope = null;

    private DocumentScope documentScope = null;

    private WorkflowProcess process = null;

    private Proposal proposal = null;

    private Document document = null;

    private static final String PROCESS_NAME = "DocumentProcess1";

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        workflowScope = new WorkflowScope();
        workflowScope.setParent(new WorkflowScope());
        workflowScope.setChildren(new WorkflowScope());
        
        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
                
        availableScope = new AvailableWorkflowProcessScope();
        availableScope.setChildren(workflowScope);
        
        documentScope = new DocumentScope();
        documentScope.setWorkflowProcessesAvailable(availableScope);
        documentScope.setWorkflowProcessesRunning(runningScope);

        proposal = (Proposal) DocumentElementFixture.createSaveParent(
            new Proposal(), this);

        RPMObject parentDocumentElement = DocumentElementFixture
            .getParentDocumentFolder(proposal);
        document = (Document) DocumentElementFixture.createFixture(
            parentDocumentElement, new Document(), 0, null, this);
        save(document, null);

        //TODO: replace this load once the API is able to create a WorkflowProcess
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        process = (WorkflowProcess) objects[0];
    }

    public void tearDown() throws Exception
    {
        // reload the Document to get the running workflows
        RPMObject[] objects = loadFromXpath("/Document[id='" + document.getID()
                + "']", documentScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof Document);
        document = (Document) objects[0];

        if (document.getWorkflowProcessesRunning() != null)
        {
            // cancel the running processes so that everything can be deleted
            for (int i = 0; i < document.getWorkflowProcessesRunning().length; i++)
            {
                getApplicationInterface().cancelWorkflow(sessionid,
                    document.getWorkflowProcessesRunning()[i]);
            }
        }

        super.tearDown();
    }

    public void testWorkflowAssociation() throws Exception
    {
        associateProcess(process, proposal, false);

        // once the process has been associated to the proposal, 
        // it should be in the list of available processes for the Document
        RPMObject[] objects = loadFromXpath("/Document[id='" + document.getID()
                + "']", documentScope);
        assertNotNull(objects);
        document = (Document) objects[0];

        assertNotNull(document.getWorkflowProcessesAvailable());
        assertNotNull(document.getWorkflowProcessesAvailable()[0]);
        assertNotNull(document.getWorkflowProcessesAvailable()[0]
            .getWorkflowProcess());
        assertFalse(document.getWorkflowProcessesAvailable()[0]
            .getDefaultFlag().booleanValue());

        unAssociateProcess(process, proposal);

        // when we unassociate it, it should be gone from the list of
        // available processes
        objects = loadFromXpath("/Document[id='" + document.getID() + "']",
            documentScope);
        assertNotNull(objects);
        document = (Document) objects[0];

        assertNull(proposal.getWorkflowProcessesAvailable());
    }

    public void testWorkflowControl() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, proposal, false);

        // start the process  
        RunningWorkflowProcess runningProcess = startProcess(process, document,
            runningScope);

        assertNotNull(runningProcess);
        assertTrue(runningProcess.getState() == WorkflowState.Open);

        // suspend the process
        RunningWorkflowProcess suspendedProcess = suspendProcess(
            runningProcess, runningScope);
        assertTrue(suspendedProcess.getState() == WorkflowState.Suspended);

        // resume the process
        RunningWorkflowProcess resumedProcess = resumeProcess(suspendedProcess,
            runningScope);
        assertTrue(resumedProcess.getState() == WorkflowState.Open);

        // cancel the process
        cancelProcess(resumedProcess);

        // reload to check that the process has been cancelled
        RPMObject[] objects = loadFromXpath("/Document[id='" + document.getID() + "']",
            documentScope);
        assertNotNull(objects);
        document = (Document) objects[0];

        assertNull(document.getWorkflowProcessesRunning());

    }

    public void testLoadDocumentRunningProcesses() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, proposal, false);

        // start the process
        RunningWorkflowProcess runningProcess = startProcess(process, document,
            runningScope);

        RPMObject[] objects = loadFromXpath(
            "/Document[@workflowProcessesRunning]", documentScope);
        assertNotNull(objects);
        
        for (int i = 0; i < objects.length; i++)
        {
            if (objects[i].getID().equals(document.getID()))
            {
                Document reloaded = (Document) objects[i];
                assertNotNull(reloaded.getWorkflowProcessesRunning());
                
                findAndCompareRunningProcess(runningProcess, reloaded
                    .getWorkflowProcessesRunning(), runningScope);
                
                break;
            }
        }
    }

    public void testLoadDocumentAvailableProcesses() throws Exception
    {
        // associate the process so that it shows up in the list of 
        // available processes
        associateProcess(process, proposal, false);

        RPMObject[] objects = loadFromXpath(
            "/Document[@workflowProcessesAvailable]", null);
        assertNotNull(objects);
        checkMatchingDocumentId(objects);
    }

    public void testLoadAvailableProcessFromDocument() throws Exception
    {
        // associate the process so that it shows up in the list of 
        // available processes
        associateProcess(process, proposal, false);

        RPMObject[] objects = loadFromXpath("/Document[id='" + document.getID()
                + "']/@workflowProcessesAvailable", availableScope);
        assertNotNull(objects);
        
        findAndCompareAvailableProcess(process, objects, workflowScope);
    }

    public void testLoadRunningProcessFromDocument() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, proposal, false);

        // start the process
        RunningWorkflowProcess runningProcess = startProcess(process, document, runningScope);

        RPMObject[] objects = loadFromXpath("/Document[id='" + document.getID()
                + "']/@workflowProcessesRunning", runningScope);
        assertNotNull(objects);

        findAndCompareRunningProcess(runningProcess, objects, runningScope);
    }

    private void checkMatchingDocumentId(RPMObject[] objects)
    {
        boolean found = false;
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Document);
            if (document.getID().equals(objects[i].getID()))
            {
                found = true;
            }
        }
        assertTrue("Document with correct ID not loaded", found);
    }
}