/*
 * Created on 4-Oct-2006
 * @author mmatwiy
 * 
 * WorkflowProcess control and xpath test cases for a WorkElement. 
 * This class tests these xpaths, with conditions:
 * - /Proposal/AvailableWorkflowProcess
 * - /Proposal[AvailableWorkflowProcess]
 * - /Proposal/RunningWorkflowProcess
 * - /Proposal[RunningWorkflowProcess]
 */
package com.ibm.rpm.client.junit.workflow.testcases;

import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestWorkflowProcessWbs extends AbstractTestWorkflowProcess
{
    private WorkflowScope workflowScope = null;
    
    private RunningWorkflowProcessScope runningScope = null;
    
    private AvailableWorkflowProcessScope availableScope = null;

    private WorkElementScope workElementScope = null;

    private WorkflowProcess process = null;

    private Proposal proposal = null;

    private static final String PROCESS_NAME = "ProjectProcess1";

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        workflowScope = new WorkflowScope();
        workflowScope.setParent(new WorkflowScope());
        workflowScope.setChildren(new WorkflowScope());
        
        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
                
        availableScope = new AvailableWorkflowProcessScope();
        availableScope.setChildren(workflowScope);

        workElementScope = new WorkElementScope();
        workElementScope.setWorkflowProcessesAvailable(availableScope);
        workElementScope.setWorkflowProcessesRunning(runningScope);

        //TODO: replace this load once the API is able to create a WorkflowProcess
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        process = (WorkflowProcess) objects[0];

        proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);
    }

    public void tearDown() throws Exception
    {
        // reload to get the running processes
        RPMObject[] objects = loadFromXpath("/Proposal[id='" + proposal.getID()
                + "']", workElementScope);
        assertNotNull(objects);
        proposal = (Proposal) objects[0];

        if (proposal.getWorkflowProcessesRunning() != null)
        {
            // cancel the running processes so that everything can be deleted
            for (int i = 0; i < proposal.getWorkflowProcessesRunning().length; i++)
            {
                cancelProcess(proposal.getWorkflowProcessesRunning()[i]);
            }
        }

        super.tearDown();
    }

    public void testWorkflowAssociation() throws Exception
    {
        associateProcess(process, proposal, true);

        // once the process has been associated to the proposal, 
        // it should be in the list of available processes
        RPMObject[] objects = loadFromXpath("/Proposal[id='" + proposal.getID()
                + "']", workElementScope);
        assertNotNull(objects);
        proposal = (Proposal) objects[0];

        assertNotNull(proposal.getWorkflowProcessesAvailable());
        assertNotNull(proposal.getWorkflowProcessesAvailable()[0]);
        assertNotNull(proposal.getWorkflowProcessesAvailable()[0]
            .getWorkflowProcess());
        assertTrue(proposal.getWorkflowProcessesAvailable()[0].getDefaultFlag()
            .booleanValue());

        unAssociateProcess(process, proposal);

        // when we unassociate it, it should be gone from the list of
        // available processes
        objects = loadFromXpath("/Proposal[id='" + proposal.getID() + "']",
            workElementScope);
        assertNotNull(objects);
        proposal = (Proposal) objects[0];

        assertNull(proposal.getWorkflowProcessesAvailable());
    }

    public void testWorkflowControl() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, proposal, false);

        // start the process  
        RunningWorkflowProcess runningProcess = startProcess(process, proposal,
            runningScope);

        assertNotNull(runningProcess);
        assertTrue(runningProcess.getState() == WorkflowState.Open);

        // suspend the process
        RunningWorkflowProcess suspendedProcess = suspendProcess(
            runningProcess, runningScope);
        assertTrue(suspendedProcess.getState() == WorkflowState.Suspended);

        // resume the process
        RunningWorkflowProcess resumedProcess = resumeProcess(suspendedProcess,
            runningScope);
        assertTrue(resumedProcess.getState() == WorkflowState.Open);

        // cancel the process
        cancelProcess(resumedProcess);

        // reload to check that the process has been cancelled
        RPMObject[] objects = loadFromXpath("/Proposal[id='" + proposal.getID() + "']",
            workElementScope);
        assertNotNull(objects);
        proposal = (Proposal) objects[0];

        assertNull(proposal.getWorkflowProcessesRunning());

    }

    public void testLoadProposalRunningProcesses() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, proposal, false);

        // start the process
        RunningWorkflowProcess runningProcess = startProcess(process, proposal,
            runningScope);

        RPMObject[] objects = loadFromXpath(
            "/Proposal[RunningWorkflowProcess]", workElementScope);
        assertNotNull(objects);
        
        for (int i = 0; i < objects.length; i++)
        {
            if (objects[i].getID().equals(proposal.getID()))
            {
                Proposal reloaded = (Proposal) objects[i];
                assertNotNull(reloaded.getWorkflowProcessesRunning());
                
                findAndCompareRunningProcess(runningProcess, reloaded
                    .getWorkflowProcessesRunning(), runningScope);
                
                break;
            }
        }
    }

    public void testLoadProposalAvailableProcesses() throws Exception
    {
        // associate the process so that it shows up in the list of 
        // available processes
        associateProcess(process, proposal, false);

        RPMObject[] objects = loadFromXpath(
            "/Proposal[AvailableWorkflowProcess]", null);
        assertNotNull(objects);
        checkMatchingProposalId(objects);
    }

    public void testLoadAvailableProcessFromProposal() throws Exception
    {
        // associate the process so that it shows up in the list of 
        // available processes
        associateProcess(process, proposal, false);

        RPMObject[] objects = loadFromXpath("/Proposal[id='" + proposal.getID()
                + "']/AvailableWorkflowProcess", availableScope);
        assertNotNull(objects);

        findAndCompareAvailableProcess(process, objects, workflowScope);
    }

    public void testLoadRunningProcessFromProposal() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, proposal, false);

        // start the process
        RunningWorkflowProcess runningProcess = startProcess(process, proposal,
            runningScope);

        RPMObject[] objects = loadFromXpath("/Proposal[id='" + proposal.getID()
                + "']/RunningWorkflowProcess", runningScope);
        assertNotNull(objects);

        findAndCompareRunningProcess(runningProcess, objects, runningScope);
    }

    private void checkMatchingProposalId(RPMObject[] objects)
    {
        boolean found = false;
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Proposal);
            if (proposal.getID().equals(objects[i].getID()))
            {
                found = true;
            }
        }
        assertTrue("Proposal with correct ID not loaded", found);
    }
}