/*
 * Created on 2-Oct-2006
 * @author mmatwiy
 * Validation test methods for suspending a workflow process. 
 */
package com.ibm.rpm.client.junit.workflow.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestSuspendWorkflowProcessValidation extends GenericValidation
{
    private WorkflowScope workflowScope = null;
    
    private RunningWorkflowProcessScope runningScope = null;
   
    private DocumentScope documentScope = null;

    private Document document = null;

    private Proposal proposal = null;

    public final static String DOCUMENT_PROCESS_NAME = "DocumentProcess1";

    public void setUp() throws Exception
    {
        super.setUp();

        workflowScope = new WorkflowScope();
        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
        
        documentScope = new DocumentScope();
        documentScope.setWorkflowProcessesRunning(runningScope);

        proposal = (Proposal) DocumentElementFixture.createSaveParent(
            new Proposal(), this);

        RPMObject parentDocumentElement = DocumentElementFixture
            .getParentDocumentFolder(proposal);
        document = (Document) DocumentElementFixture.createFixture(
            parentDocumentElement, new Document(), 0, null, this);
        save(document, null);
    }

    public void tearDown() throws Exception
    {
        // reload the Document to get the running workflows
        RPMObject[] objects = loadFromXpath("/Document[id='" + document.getID()
                + "']", documentScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof Document);
        document = (Document) objects[0];

        if (document.getWorkflowProcessesRunning() != null)
        {
            // cancel the running processes so that everything can be deleted
            for (int i = 0; i < document.getWorkflowProcessesRunning().length; i++)
            {
                getApplicationInterface().cancelWorkflow(sessionid,
                    document.getWorkflowProcessesRunning()[i]);
            }
        }

        super.tearDown();
    }

    private RunningWorkflowProcess loadRunningProcess() throws Exception
    {
        // TODO: replace this with a fixture that creates a WorkflowProcess
        // once it's possible to create a WorkflowProcess using the API,
        // then start and suspend the process
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + DOCUMENT_PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkflowProcess);
        WorkflowProcess process = (WorkflowProcess) objects[0];

        Result associateResult = getApplicationInterface().associateWorkflow(
            sessionid, process, proposal, true, false);
        assertTrue(associateResult.isSuccessful());

        Result startResult = getApplicationInterface().startWorkflow(
            getSessionID(), process, document, runningScope);
        assertTrue(startResult.isSuccessful());

        // reload the Document to get the running workflows
        objects = loadFromXpath("/Document[id='" + document.getID() + "']",
            documentScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof Document);
        document = (Document) objects[0];
        assertNotNull(document.getWorkflowProcessesRunning());

        RunningWorkflowProcess runningProcess = null;

        for (int i = 0; i < document.getWorkflowProcessesRunning().length; i++)
        {
            if (document.getWorkflowProcessesRunning()[i]
                .getWorkflowProcessName().equals(DOCUMENT_PROCESS_NAME))
            {
                runningProcess = document.getWorkflowProcessesRunning()[i];
            }
        }

        return runningProcess;
    }

    public void testFailureCancelNullRunningProcess() throws Exception
    {
        // Cancel a null runningProcess object
        Result result = getApplicationInterface().cancelWorkflow(sessionid,
            null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, "RunningWorkflowProcess")
        };

        verifyProcessFailure(null, expectedExceptions, null, result);
    }

    public void testFailureCancelUnsavedRunningProcess() throws Exception
    {
        RunningWorkflowProcess runningProcess = new RunningWorkflowProcess();

        // Cancel an unsaved runningProcess object
        Result result = getApplicationInterface().cancelWorkflow(sessionid,
            runningProcess);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, runningProcess.getClass()
                .getName())
        };

        verifyProcessFailure(runningProcess, expectedExceptions, null, result);
    }

    public void testFailureInvalidState() throws Exception
    {
        RunningWorkflowProcess runningProcess = loadRunningProcess();

        Result cancelResult = getApplicationInterface().cancelWorkflow(
            sessionid, runningProcess);
        assertTrue(cancelResult.isSuccessful());

        // Attempt to cancel a runningProcess that's already cancelled
        Result result = getApplicationInterface().cancelWorkflow(sessionid,
            runningProcess);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.workflow.constants.ErrorCodes.INVALID_STATE_TRANSITION,
                runningProcess.getClass().getName())
        };

        verifyProcessFailure(runningProcess, expectedExceptions, null, result);
    }

    private void verifyProcessFailure(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings, Result result)
            throws AssertionError, Exception
    {
        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, result);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}