/*
 * Created on 19-Oct-2006
 * @author mmatwiy
 * Validation test cases for container WorkflowRoleMapping.
 */
package com.ibm.rpm.client.junit.workflow.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.scope.ContactGroupScope;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.constants.ValidationConstants;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowRole;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.scope.WorkflowRoleMappingScope;

public class TestWorkflowRoleMappingValidation extends GenericValidation
{
    private static final String PROCESS_1_NAME = "DocumentProcess1";

    private static final String PROCESS_2_NAME = "DocumentProcess2";

    private WorkflowRoleMappingScope mappingScope = null;

    private WorkOrganization workOrganization;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mappingScope = new WorkflowRoleMappingScope();
        mappingScope.setContactGroup(new ContactGroupScope());
        mappingScope.setResource(new ResourceScope());
        mappingScope.setSecurityRole(new SecurityRoleScope());
    }

    private WorkflowRoleMapping loadEmptyWorkflowMapping() throws Exception
    {
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setWorkflowRoleMappings(mappingScope);

        //TODO: replace this load once the API is able to create a WorkflowProcess
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_1_NAME + "']", null);
        assertNotNull(objects);
        WorkflowProcess process1 = (WorkflowProcess) objects[0];

        //TODO: replace this load once the API is able to create a WorkflowProcess
        objects = loadFromXpath("/WorkflowProcess[name='" + PROCESS_2_NAME
                + "']", null);
        assertNotNull(objects);
        WorkflowProcess process2 = (WorkflowProcess) objects[0];

        // don't use the static fixture because we need the WorkOrganization
        // to be deleted at the end of the tests in order to reset everything
        workOrganization = (WorkOrganization) WorkElementFixture
            .createSaveFixture(new WorkOrganization(), null, this);

        // associate both processes to the WorkOrganization so that
        // their roles show up in the list of WorkflowRoleMappings
        getApplicationInterface().associateWorkflow(getSessionID(), process1,
            workOrganization, true, false);
        getApplicationInterface().associateWorkflow(getSessionID(), process2,
            workOrganization, true, false);

        // load the process role mappings 
        objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization loadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(loadedOrganization.getWorkflowRoleMappings());

        return loadedOrganization.getWorkflowRoleMappings()[0];
    }

    public void testInsertModifyWorkflowRoleName() throws Exception
    {
        WorkflowRoleMapping mapping = loadEmptyWorkflowMapping();
        mapping.getWorkflowRole().setName("modifyRoleName");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.workflow.constants.ErrorCodes.INVALID_WORKFLOW_ROLE_MAPPING,
                mapping.getClass().getName())
        };

        verifySaveMustFail(mapping, null, expectedExceptions);
    }
    
    public void testInsertWithNoParent() throws Exception
    {
        WorkflowRoleMapping mapping = loadEmptyWorkflowMapping();
        mapping.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mapping.getClass().getName(),
                ValidationConstants.PARENT_FIELD, mapping.getID())
        };

        verifySaveMustFail(mapping, null, expectedExceptions);
    }
    
    public void testUpdateWithNoParent() throws Exception
    {
        WorkflowRoleMapping mapping = loadEmptyWorkflowMapping();
        mapping.setResource(ResourceFixture.createSaveFixture(this));
        mapping = (WorkflowRoleMapping) save(mapping, mappingScope)
            .getRpmObject();

        mapping.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mapping.getClass()
                .getName(), null, mapping.getID())
        };

        verifySaveMustFail(mapping, null, expectedExceptions);
    }

    public void testMandatoryIDs() throws Exception
    {
        WorkflowRoleMapping mapping = new WorkflowRoleMapping();
        mapping.setWorkflowRole(new WorkflowRole());
        mapping.setParent(WorkElementFixture.retrieveStaticFixture(
            new Project(), this));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mapping.getClass().getName(),
                ValidationConstants.WORKFLOW_ROLE_FIELD, mapping.getID())
        };

        verifySaveMustFail(mapping, null, expectedExceptions);
    }

    public void testUpdateWorkflowRole() throws Exception
    {
        WorkflowRoleMapping mapping = loadEmptyWorkflowMapping();
        mapping.setResource(ResourceFixture.createSaveFixture(this));
        mapping = (WorkflowRoleMapping) save(mapping, mappingScope)
            .getRpmObject();

        mapping.getWorkflowRole().setName("new workflow role name");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, mapping.getClass()
                .getName(), ValidationConstants.WORKFLOW_ROLE_FIELD, mapping
                .getID())
        };

        verifySaveMustFail(mapping, null, expectedExceptions);
    }

    public void testUpdateWithUnsavedResource() throws Exception
    {
        WorkflowRoleMapping mapping = loadEmptyWorkflowMapping();
        mapping.setResource(new Resource());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mapping.getClass().getName(),
                ValidationConstants.RESOURCE_FIELD, mapping.getID()),
        };

        verifySaveMustFail(mapping, mappingScope, expectedExceptions);
    }

    public void testUpdateWithUnsavedContactGroup() throws Exception
    {
        WorkflowRoleMapping mapping = loadEmptyWorkflowMapping();
        mapping.setContactGroup(new ContactGroup());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mapping.getClass().getName(),
                ValidationConstants.CONTACT_GROUP_FIELD, mapping.getID()),
        };

        verifySaveMustFail(mapping, mappingScope, expectedExceptions);
    }

    public void testUpdateWithUnsavedSecurityRole() throws Exception
    {
        WorkflowRoleMapping mapping = loadEmptyWorkflowMapping();
        mapping.setSecurityRole(new ProjectSecurityRole());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mapping.getClass().getName(),
                ValidationConstants.SECURITY_ROLE_FIELD, mapping.getID()),
        };

        verifySaveMustFail(mapping, mappingScope, expectedExceptions);
    }

}