package com.ibm.rpm.customfield.checkpoints;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.ibm.rpm.applicationadministration.checkpoints.DatafieldCheckpoint;
import com.ibm.rpm.applicationadministration.containers.CustomDatafield;
import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.clientcostcenters.containers.ClientModule;
import com.ibm.rpm.customfield.constants.ValidationConstants;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.containers.GenericCustomField;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.wbs.containers.WbsModule;

public class CustomFieldCheckpoint extends AbstractCheckpoint
{
    private static final List CUSTOM_FIELD_CATEGORY_PARENTS = new ArrayList();

    private static final Class CUSTOM_FIELD_PARENTS = CustomFieldCategory.class;

    static
    {
        CUSTOM_FIELD_CATEGORY_PARENTS.add(AssetModule.class);
        CUSTOM_FIELD_CATEGORY_PARENTS.add(ClientModule.class);
        CUSTOM_FIELD_CATEGORY_PARENTS.add(DocumentModule.class);
        CUSTOM_FIELD_CATEGORY_PARENTS.add(ResourceModule.class);
        CUSTOM_FIELD_CATEGORY_PARENTS.add(ScopeManagementModule.class);
        CUSTOM_FIELD_CATEGORY_PARENTS.add(WbsModule.class);
    }

    protected static CustomFieldCheckpoint instance = new CustomFieldCheckpoint();

    public static CustomFieldCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (rpmObject instanceof CustomFieldCategory)
        {
            isValidCategory(rpmObject, msgCtx);
        }
        else if (rpmObject instanceof CustomField)
        {
            isValidCustomField(rpmObject, msgCtx);
        }
        else
        {
            GenericValidator.validateInstance(rpmObject,
                GenericCustomField.class, msgCtx);
        }        
    }    
   
    private void isValidCategory(RPMObject rpmObject, MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, false,
            CUSTOM_FIELD_CATEGORY_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            CustomFieldCategory.class, msgCtx))
            return;

        CustomFieldCategory category = (CustomFieldCategory) rpmObject;
        isValidName(category, msgCtx);
    }

    private void isValidCustomField(RPMObject rpmObject, MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, CUSTOM_FIELD_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, CustomField.class,
            msgCtx))
            return;

        CustomField customField = (CustomField) rpmObject;
        isValidName(customField, msgCtx);

        GenericValidator.validateMandatory(customField, "inputType",
            customField.getInputType(), msgCtx);
        GenericValidator.validateDefaultValueUpdate(customField,
            "securityGroup", customField.getSecurityGroup(), msgCtx);

        areValidTypeAndValue(customField, msgCtx);
    }

    private void isValidName(GenericCustomField genCustomField,
            MessageContext msgCtx)
    {
        if (genCustomField.testNameModified())
        {
            GenericValidator.validateMaxLength(genCustomField, "name",
                genCustomField.getName(),
                ValidationConstants.CUSTOMFIELD_NAME_MAX, msgCtx);
        }
    }

    private void areValidTypeAndValue(CustomField customField,
            MessageContext msgCtx)
    {
        if (customField.testNameModified()
                || customField.testDefaultValueModified()
                || customField.testDatafieldCategoryModified())
        {
            FieldInputType inputType = customField.getInputType();
            Object defaultValue = customField.getDefaultValue();
            if (FieldInputType.Checkbox.equals(inputType))
            {
                isValidDefaultValueType(customField, Boolean.class,
                    msgCtx);
            }
            else if (FieldInputType.Currency.equals(inputType))
            {
                if (isValidDefaultValueType(customField, Double.class,
                    msgCtx))
                {
                    GenericValidator.validateRange(customField, 
                        ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD,
                        (Double) customField.getDefaultValue(), 
                        ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_CURRENCY_MIN,
                        ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_CURRENCY_MAX, 
                        msgCtx);
                }                
            }
            else if (FieldInputType.Date.equals(inputType))
            {
                isValidDefaultValueType(customField, Calendar.class,
                    msgCtx);
            }
            else if (FieldInputType.Numeric.equals(inputType))
            {
                if (isValidDefaultValueType(customField, Integer.class,
                    msgCtx))
                {
                    GenericValidator.validateRange(customField, 
                        ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD,
                        (Integer) customField.getDefaultValue(), 
                        ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_INTEGER_MIN,
                        ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_INTEGER_MAX, 
                        msgCtx);
                }
            }
            else if (FieldInputType.Text.equals(inputType))
            {
                if (isValidDefaultValueType(customField, String.class,
                    msgCtx))
                {
                    GenericValidator.validateMaxLength(customField,
                        ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD, (String) defaultValue,
                        ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_TEXT_MAX, msgCtx);
                }
            }
            else if (FieldInputType.Datafield.equals(inputType))
            {
                isValidDatafield(customField, msgCtx);
            }
        }
    }

    private boolean isValidDefaultValueType(CustomField customField,
            Class expectedType, MessageContext msgCtx)
    {
        Object defaultValue = customField.getDefaultValue();
        if (defaultValue != null)
        {
            return GenericValidator.validateType(customField, expectedType,
                ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD, defaultValue, msgCtx);
        }
        
        return true;
    }

    private void isValidDatafield(CustomField customField,
            MessageContext msgCtx)
    {
        DatafieldCategory category = customField.getDatafieldCategory();
        
        // Field validation: datafieldCategory
        if (customField.getDefaultValue() == null)
        {
            // This field is mandatory when the defaultValue is null
	        GenericValidator.validateMandatory(customField, ValidationConstants.DATAFIELDCATEGORY_FIELD, 
	            category, msgCtx);
        }
        if (customField.getDatafieldCategory() != null)
        {
            // If this field is specified, it must have an ID
            GenericValidator.validateMandatoryID(customField, ValidationConstants.DATAFIELDCATEGORY_FIELD, 
                category, msgCtx);
        }
        
        // Field validation: defaultValue
        if (customField.getDefaultValue() != null)
        {
            if (!isValidDefaultValueType(customField, Datafield.class, msgCtx))
                return;
            
            Datafield datafield = (Datafield) customField.getDefaultValue();
            
            // Validate that the default value class matches the datafield
            // category class.
            if (category instanceof CustomDatafieldCategory)
            {
                GenericValidator.validateExactType(customField,
                    CustomDatafield.class, ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD, 
                    datafield, msgCtx);
            }
            else
            {
                GenericValidator.validateType(customField,
                    Datafield.class, ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD, 
                    datafield, msgCtx);
            }
            // Validate that the default value is in the specified datafield
            // category (valid domain).
            if (customField.getDatafieldCategory() != null)
            {
	            DatafieldCheckpoint.validateDatafieldInCategory(
	                customField, ValidationConstants.CUSTOMFIELD_DEFAULTVALUE_FIELD, 
	                datafield, category, msgCtx);
            }
        }
    }
}
