package com.ibm.rpm.document.transfer;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.interfaces.impl.ServerFactory;

public abstract class AbstractDocTransferServlet extends HttpServlet
{
    protected static final long serialVersionUID = 6740390299317353687L;

    protected static final int BUFFER_SIZE = 8192;

    protected static final String[] VALID_CONTEXTS = new String[]
    {
            ContextUtil.WBS_CONTEXT, ContextUtil.ASSET_CONTEXT,
            ContextUtil.SCOPE_CONTEXT, ContextUtil.CLIENT_CONTEXT,
            ContextUtil.RESOURCE_CONTEXT, ContextUtil.POOL_CONTEXT,
            ContextUtil.WBS_DOCUMENT_CONTEXT,
            ContextUtil.ASSET_DOCUMENT_CONTEXT,
            ContextUtil.SCOPE_DOCUMENT_CONTEXT,
            ContextUtil.CLIENT_DOCUMENT_CONTEXT,
            ContextUtil.RESOURCE_DOCUMENT_CONTEXT,
            ContextUtil.POOL_DOCUMENT_CONTEXT
    };

    protected static Log logger = LogFactory
        .getLog(AbstractDocTransferServlet.class);

    protected ServerFactory factory;

    public AbstractDocTransferServlet()
    {
        this(ServerFactory.getInstance());
    }

    public AbstractDocTransferServlet(ServerFactory setfactory)
    {
        factory = setfactory;
    }

    protected DataOutputStream getResponseDataOutputStream(
            HttpServletRequest req, HttpServletResponse res) throws IOException
    {
        OutputStream os = res.getOutputStream();
        try
        {
            res.setBufferSize(BUFFER_SIZE); // in case it's not implemented
        }
        catch (Exception e)
        {
        }
        os = new BufferedOutputStream(res.getOutputStream());
        return new DataOutputStream(os);
    }

    protected void doGet(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException
    {
        executeRequest(request, response);
    }

    protected void doPost(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException
    {
        executeRequest(request, response);
    }

    protected void doPut(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException
    {
        executeRequest(request, response);
    }

    protected void executeRequest(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException
    {
        try
        {
            internalExecuteRequest(request, response);
        }
        catch (Exception e)
        {

            sendResponseError(response, HttpURLConnection.HTTP_INTERNAL_ERROR,
                "Error Executing Request: " + getExceptionMessage(e));
        }
    }

    protected abstract void internalExecuteRequest(HttpServletRequest request,
            HttpServletResponse response) throws Exception;

    /**
     * check if the filename is valid
     * 
     * @param response
     *            the HTTP response that will get the error code if the check
     *            fails
     * @param filename
     *            the file name to validate
     * @return true if the file name is valid. Otherwise, the error code is
     *         placed in the HTTP response
     */
    protected boolean validateFileName(HttpServletResponse response,
            String filename)
    {
        boolean retVal = true;
        if (filename == null)
        {
            sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
                "FILE HTTP parameter not found");
            retVal = false;
        }
        else if (filename.length() <= 0)
        {
            sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
                "Invalid empty file name.");
            retVal = false;
        }
        else if (filename.length() > ValidationConstants.DOCUMENTELEMENT_NAME_MAX)
        {
            sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
                "Invalid file name length. Detected length: "
                        + filename.length() + ". Maximum length is : "
                        + ValidationConstants.DOCUMENTELEMENT_NAME_MAX);
            retVal = false;
        }
        else
        {
            File tempFile = null;
            boolean deleteTempFile = true;

            try
            {
                // attempt to create a temporary file with the matching name.
                // If the attempt fails for any reason,
                // it means the file name is invalid
                tempFile = new File(filename);
                deleteTempFile = tempFile.createNewFile();
            }
            catch (Exception e)
            {
                sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
                    "Invalid file name : " + filename);
                retVal = false;
            }
            finally
            {
                // dont delete the temp file it existed prior to the operation
                if (deleteTempFile)
                {
                    DocumentTransfer.deleteTempFile(tempFile);
                }
            }
        }

        return retVal;
    }

    protected void sendResponseError(HttpServletResponse response, int errorCode,
            String errorMessage)
    {
        try
        {
            logger.error("Sending HTTP Error Code: " + errorCode
                    + ". Message: " + errorMessage);
            response.sendError(errorCode, errorMessage);
            response.flushBuffer();
        }
        catch (Exception e)
        {
            logger.error("Failed to send error on HTTP response: " + e);
        }
    }

    protected void sendResponseSuccess(HttpServletResponse response,
            int successCode)
    {
        try
        {
            response.sendError(successCode);
            response.flushBuffer();
        }
        catch (Exception e)
        {
            logger.error("Failed to send HTTP response: " + e);
        }
    }

    protected void download(HttpServletRequest request,
            HttpServletResponse response, String action, String contextName,
            String docID, String sessionID, boolean isZipped)
            throws ServletException, IOException
    {
        DocumentTransfer documentTransfer = null;

        if (action != null)
        {
            action = action.trim();
        }

        try
        {
            documentTransfer = new DocumentTransfer(factory, sessionID,
                "document download");
            documentTransfer.validateDocumentID(contextName, docID);
            
            if (documentTransfer.prepareDownload(contextName, docID, isZipped))
            {
            	response.setContentType("application/octet-stream");
            	response.setHeader("expires", "now");

            	String filename = documentTransfer.getAttachmentName();
            	long contentLength = documentTransfer.getAttachmentSize();

            	if (contentLength == -1 || filename == null)
            	{
            		sendResponseError(response,
            				HttpURLConnection.HTTP_NO_CONTENT, "Document is empty.");
            		return;
            	}

            	response.setHeader("Content-Disposition",
            			"attachment; filename=\"" + filename + "\"");

            	response.setContentLength((int) contentLength);

            	logger.debug("Document download start. File name = " + filename
            			+ ", content length = " + contentLength + ", Action = "
            			+ action);

            	long timeStamp = System.currentTimeMillis();

            	DataOutputStream dos = getResponseDataOutputStream(request,
            			response);

            	documentTransfer.executeDownload(dos);

            	long totalTime = System.currentTimeMillis() - timeStamp;

            	logger.info("Document download completed. File name = "
            			+ filename + ", content length = " + contentLength
            			+ ", Action = " + action + ", Transfer Time = "
            			+ totalTime / 1000 + " seconds");
            }
            else
            {
                sendResponseError(response, HttpURLConnection.HTTP_NOT_FOUND,
                    "Document not found with ID=" + docID + " for context "
                            + contextName);
            }
        }
        catch (RPMException e)
        {
            sendResponseError(response, HttpURLConnection.HTTP_INTERNAL_ERROR,
                "Error while retrieving document: " + getExceptionMessage(e));
        }
        finally
        {
            if (documentTransfer != null)
            {
                documentTransfer.releaseAll();
            }
        }

    }

    protected void upload(HttpServletRequest request,
            HttpServletResponse response, InputStream inputStream,
            String action, String contextName, String docID, String sessionID,
            String filename)
    {
        DocumentTransfer documentTransfer = null;
        // Upload using the blob id only is not supported because we need to
        // know the doc id to
        // update the document with the informations related its new file
        // attachement that will
        // be stored in the blob.
        response.setHeader("expires", "now");

        try
        {
            // Patch pour websphere 6.0
            try
            {
                logger.info("testing inputstream availability");
                logger.info("available: " + inputStream.available());
            }
            catch (IOException e)
            {
                // error when reading the first byte cause problem when
                // converting to buffered input stream. No exception is thrown.
                inputStream.read();
            }

            documentTransfer = new DocumentTransfer(factory, sessionID,
            "document upload");

            documentTransfer.validateDocumentID(contextName, docID);

            // int uncompressedLength = uncompressedFileLength==null ? 0 :
            // Integer.parseInt(uncompressedFileLength);
            if (action == DocumentTransfer.ACTION_UPLOADZIP)
            {
            	// Providing null as the filename will force the
            	// AttachmentHandler to read the filename from the zip file
            	BufferedInputStream bis = new BufferedInputStream(inputStream);
            	documentTransfer
            	.uploadZipped(contextName, docID, filename, bis);
            }
            else if (action == DocumentTransfer.ACTION_UPLOADUNZIP)
            {
            	BufferedInputStream bis = new BufferedInputStream(inputStream);
            	documentTransfer.uploadUnzipped(contextName, docID, filename,
            			bis);
            }
            else
            {
            	sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
            			"Unknown action: " + action);
            	return;
            }

            // send OK only if we found no error and we got no exception
            sendResponseSuccess(response, HttpURLConnection.HTTP_OK);
        }
        catch (RPMException e)
        {
            sendResponseError(response, HttpURLConnection.HTTP_CONFLICT,
                getExceptionMessage(e));
        }
        catch (Exception e)
        {
            sendResponseError(response, HttpURLConnection.HTTP_INTERNAL_ERROR,
                getExceptionMessage(e));
        }
        finally
        {
            if (documentTransfer != null)
            {
                documentTransfer.releaseAll();
            }
        }
    }

    public static String getExceptionMessage(Exception e)
    {

        Throwable current = e;
        while (current.getMessage() == null || current.getCause() != null)
        {
            current = e.getCause();
        }

        if (current.getMessage() == null)
        {
            current = e;
        }
        return current.toString();
    }
}
