/*
 * Created on Nov 18, 2005
 */
package com.ibm.rpm.financial.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.financial.constants.ErrorCodes;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.util.ChargeCodeUtil;
import com.ibm.rpm.financial.util.FinancialCategoryUtil;
import com.ibm.rpm.financial.util.FinancialUtil;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.WorkElement;

/**
 * @author ikusakov
 */
public class FinancialCheckpoint extends AbstractCheckpoint
{
    private static final List FINANCIAL_PARENTS = new ArrayList();

    static
    {
        FINANCIAL_PARENTS.add(WorkElement.class);
        FINANCIAL_PARENTS.add(ScopeElement.class);
    }

    private static final Class WORKELEMENT_PARENTS = WorkElement.class;

    private static final Class SCOPEELEMENT_PARENTS = ScopeElement.class;

    protected static FinancialCheckpoint instance = new FinancialCheckpoint();

    public static FinancialCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        boolean createObject = (rpmObject.getID() == null);

        if (ContextUtil.WBS_CONTEXT.equals(rpmObject.getContextName()))
            super.isValidParent(rpmObject, msgCtx, createObject, true,
                WORKELEMENT_PARENTS);
        else if (ContextUtil.SCOPE_CONTEXT.equals((rpmObject.getContextName())))
            super.isValidParent(rpmObject, msgCtx, createObject, true,
                SCOPEELEMENT_PARENTS);
        else
            super.isValidParent(rpmObject, msgCtx, createObject, true,
                FINANCIAL_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            AbstractFinancial.class, msgCtx))
            return;

        AbstractFinancial financial = (AbstractFinancial) rpmObject;
        FinancialScope scope = (FinancialScope) rpmScope;

        if (createObject)
        {
            GenericValidator.validateMandatory(financial, "chargeCode",
                financial.getChargeCode(), msgCtx);
        }
        
        // wbs financial is read-only for sytem charge codes abroad
        // but for CustomChargeCodes is available for
        // Milestone,Task,ScopeTask,ScopeMilestone
        if (financial instanceof WbsFinancial)
        {
            boolean isSystemFinancial = false;

            if (financial.getChargeCode() != null
                    && financial.getChargeCode().getFinancialCategory() != null)
            {
                try
                {
                    isSystemFinancial = FinancialCategoryUtil
                        .isSystemFinancialCategory(financial.getChargeCode()
                            .getFinancialCategory());
                }
                catch (RPMException e1)
                {
                    addException(e1, msgCtx);
                }
            }

            if (isSystemFinancial
                    || ((financial.getParent() != null)
                            && !(financial.getParent() instanceof AbstractScopeTask) && !(financial
                        .getParent() instanceof SimpleNode)))
            {
                RPMException e = new RPMException(
                    ErrorCodes.FINANCIAL_WBS_READONLY, new String[]
                    {
                        "FinancialManager"
                    }, financial.getClass().getName());
                e.setFieldName("WbsFinancial");
                e.setContainerID(financial.getID());
                e.setSeverity(SeverityLevel.Warning);
                addException(e, msgCtx);
            }
        }
        
        //SubElement is READ-ONLY for: Estimate,Actual,Remaining,ETC,EAC
        if (financial instanceof WorksheetFinancial)
        {
            WorksheetFinancial wfinancial = (WorksheetFinancial) financial;

            if (wfinancial.testSubElementEstimateModified()
                    && wfinancial.getSubElementEstimate() != null)
            {
                GenericValidator.validateReadOnly(wfinancial,
                    "SubElement ESTIMATE", msgCtx);
            }

            if (wfinancial.testSubElementActualModified()
                    && wfinancial.getSubElementActual() != null)
            {
                GenericValidator.validateReadOnly(wfinancial,
                    "SubElement ACTUAL", msgCtx);
            }

            if (wfinancial.testSubElementRemainingModified()
                    && wfinancial.getSubElementRemaining() != null)
            {
                GenericValidator.validateReadOnly(wfinancial,
                    "SubElement REMAINING", msgCtx);
            }

            if (wfinancial.testSubElementETCModified()
                    && wfinancial.getSubElementETC() != null)
            {
                GenericValidator.validateReadOnly(wfinancial, "SubElement ETC",
                    msgCtx);
            }

            if (wfinancial.testSubElementEACModified()
                    && wfinancial.getSubElementEAC() != null)
            {
                GenericValidator.validateReadOnly(wfinancial, "SubElement EAC",
                    msgCtx);
            }

        }        
        

        
        if (scope != null && scope.isChargeCode() && financial.getChargeCode() != null
                && financial.getChargeCode().getID() != null
                && ChargeCodeUtil.isFTEChargeCode(financial.getChargeCode()
                    .getID()) && FinancialUtil.isFinancialLineModified(financial))
        {
            RPMException e = new RPMException(
                ErrorCodes.FINANCIAL_FTE_READONLY, new String[]
                {
                    "FinancialManager"
                }, financial.getClass().getName());
            e.setContainerID(financial.getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
        }


        // Field validation: actual
        if (financial.testActualModified())
        {
            GenericValidator.validateRange(financial, "actual", financial
                .getActual(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        // Field validation: estimate
        if (financial.testEstimateModified())
        {
            GenericValidator.validateRange(financial, "estimate", financial
                .getEstimate(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        // Field validation: ETC
        if (financial.testETCModified())
        {

            GenericValidator.validateRange(financial, "ETC",
                financial.getETC(), ValidationConstants.FINANCIAL_ETC_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }
        
        // Field validation: REMAINING
        if (financial.testRemainingModified())
        {
            GenericValidator.validateReadOnly(financial,
                "REMAINING", msgCtx);
        }
        
        // Field validation: EAC
        if (financial.testEACModified())
        {
            GenericValidator.validateReadOnly(financial,
                "EAC", msgCtx);
        }
        
        // Field validation: Probability Percent
        if (financial.testProbabilityPercentModified())
        {
            GenericValidator.validateRange(financial, "ProbabilityPercent",
                financial.getProbabilityPercent(), 0D, 100D, msgCtx);
            
            GenericValidator.validateNotNull(financial.getChargeCode(),
                "ChargeCode", ValidationConstants.CHARGE_CODE_FIELD, msgCtx);        
            
      

            if (financial instanceof WorksheetFinancial
                    && financial.getProbabilityPercent() != null)
            {
                boolean isBenefit = false;
                try
                {
                    isBenefit = FinancialCategoryUtil
                        .isBenefitFinancialCategory(financial.getChargeCode()
                            .getFinancialCategory());
                }
                catch (RPMException e)
                {
                    addException(e, msgCtx);
                }

                //Verifying that ChargeCode is Benefit if probability
                // percent is modified
                if (financial.getChargeCode() == null || !isBenefit)
                {
                    String[] errorParams =
                    {
                        StringUtil.getShortClassName(financial.getChargeCode()
                            .getClass())
                                + " "
                                + ValidationConstants.CHARGE_CODE_CATEGORIES_FIELD

                    };

                    RPMException e = new RPMException(
                        ErrorCodes.FINANCIAL_BENEFITS_CATEGORY, errorParams,
                        financial.getChargeCode().getClass().getName(),
                        ValidationConstants.PROBABILITY_PERCENT_FIELD,
                        financial.getChargeCode().getID());
                    addException(e, msgCtx);

                }
            }

        }
    }
    
    public void validateDelete(RPMObject rpmObject,
            MessageContext messageContext)
    {
        super.validateDelete(rpmObject, messageContext);

        AbstractFinancial financial = (AbstractFinancial) rpmObject;

        if (financial instanceof WorksheetFinancial
                && financial.getChargeCode() != null
                && financial.getChargeCode().getFinancialCategory() != null)
        {
            boolean isSystemChargeCode = false;
            try
            {
                isSystemChargeCode = FinancialCategoryUtil
                    .isSystemFinancialCategory(financial.getChargeCode()
                        .getFinancialCategory());
            }
            catch (RPMException e)
            {
                addException(e, messageContext);
            }

            if (financial.getID() != null && isSystemChargeCode)
            {
                GenericValidator
                    .validateCannotDelete(financial, messageContext);
            }
        }

    }
    
    public boolean validateFinancials(MessageContext msgCtx,
            AbstractFinancial financial[]) throws RPMException
    {
        AbstractFinancial hourExpense = null;
        AbstractFinancial avgExpense = null;
        AbstractFinancial laborExpense = null;

        boolean validationError = false;

        if (financial != null && financial.length >= 4)
        {
            AbstractFinancial[] financials = financial;

            hourExpense = FinancialUtil.getFinancialForChargeCode(
                FinancialUtil.EXPENSE_HOUR, financials);

            avgExpense = FinancialUtil.getFinancialForChargeCode(
                FinancialUtil.EXPENSE_AVERAGE_COST, financials);

            laborExpense = FinancialUtil.getFinancialForChargeCode(
                FinancialUtil.EXPENSE_LABOR, financials);

            if (hourExpense != null && avgExpense != null
                    && laborExpense != null)
            {
                if (FinancialUtil.isFinancialLineModified(avgExpense)
                        && !FinancialUtil.isFinancialLineModified(hourExpense)
                        && !FinancialUtil.isFinancialLineModified(laborExpense)

                        && FinancialUtil.isFinancialLineEmpty(hourExpense)
                        && FinancialUtil.isFinancialLineEmpty(laborExpense))
                {
                    String[] errorParams =
                    {
                        StringUtil.getShortClassName(avgExpense.getClass())
                                + " " + "average"
                    };
                    RPMException e = new RPMException(
                        com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_AVERAGECOST_USAGE,
                        errorParams, avgExpense.getClass().getName(),
                        "average", avgExpense.getID());
                    e.setSeverity(SeverityLevel.Error);
                    addException(e, msgCtx);

                    validationError = true;
                }
            }
        }

        return validationError;
    }

}