package com.ibm.rpm.financial.util;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.RPMException;

/**
 * @author legaults
 */
public class FinancialCategoryUtil
{
    private static Map financialCategoryToDBMap;

    private static Map dbToFinancialCategoryMap;

    private static Map financialCategoryToAssociatedMap;
    
    private static int EXPENSE_CATEGORY = 0;
    private static int CAPITAL_CATEGORY = 1;
    private static int BENEFIT_CATEGORY = 2;
    
    private static int EXPENSE_AVG = 12;
    private static int EXPENSE_FTE  = 13;
    private static int EXPENSE_HOUR = 11;
    private static int EXPENSE_LABOR = 14;
    
    private static int CAPITAL_AVG = 22;
    private static int CAPITAL_FTE  = 23;
    private static int CAPITAL_HOUR = 21;
    private static int CAPITAL_LABOR = 24;

    
    private static int BENEFIT_AVG = 32;
    private static int BENEFIT_FTE  = 33;
    private static int BENEFIT_HOUR = 31;
    private static int BENEFIT_LABOR = 34;


    static
    {
        financialCategoryToDBMap = new HashMap();
        dbToFinancialCategoryMap = new HashMap();
        financialCategoryToAssociatedMap = new HashMap();

        addToDBMap(FinancialCategory.Benefit, 				BENEFIT_CATEGORY);
        addToDBMap(FinancialCategory.BenefitAverageCost, 	BENEFIT_AVG);
        addToDBMap(FinancialCategory.BenefitFTE, 			BENEFIT_FTE);
        addToDBMap(FinancialCategory.BenefitHour, 			BENEFIT_HOUR);
        addToDBMap(FinancialCategory.BenefitLabor, 			BENEFIT_LABOR);
        addToDBMap(FinancialCategory.Capital, 				CAPITAL_CATEGORY);
        addToDBMap(FinancialCategory.CapitalAverageCost, 	CAPITAL_AVG);
        addToDBMap(FinancialCategory.CapitalFTE, 			CAPITAL_FTE);
        addToDBMap(FinancialCategory.CapitalHour, 			CAPITAL_HOUR);
        addToDBMap(FinancialCategory.CapitalLabor, 			CAPITAL_LABOR);
        addToDBMap(FinancialCategory.Contingency, 4);
        addToDBMap(FinancialCategory.Elimination, 3);
        addToDBMap(FinancialCategory.Expense, 				EXPENSE_CATEGORY);
        addToDBMap(FinancialCategory.ExpenseAverageCost, 	EXPENSE_AVG);
        addToDBMap(FinancialCategory.ExpenseFTE, 			EXPENSE_FTE);
        addToDBMap(FinancialCategory.ExpenseHour, 			EXPENSE_HOUR);
        addToDBMap(FinancialCategory.ExpenseLabor, 			EXPENSE_LABOR);
        addToDBMap(FinancialCategory.TimePhaseCalendar, 10);

        addToAssociatedMap(FinancialCategory.Benefit, new FinancialCategory[]
        {
                FinancialCategory.Capital, FinancialCategory.Expense
        });
        addToAssociatedMap(FinancialCategory.Capital, new FinancialCategory[]
        {
            FinancialCategory.Benefit
        });
        addToAssociatedMap(FinancialCategory.Contingency,
                new FinancialCategory[]
                {});
        addToAssociatedMap(FinancialCategory.Elimination,
                new FinancialCategory[]
                {
                    FinancialCategory.Expense
                });
        addToAssociatedMap(FinancialCategory.Expense, new FinancialCategory[]
        {
                FinancialCategory.Benefit, FinancialCategory.Elimination
        });
    }

    private static void addToDBMap(FinancialCategory fc, int dbCode)
    {
        Integer codeObject = new Integer(dbCode);
        financialCategoryToDBMap.put(fc, codeObject);
        dbToFinancialCategoryMap.put(codeObject, fc);
    }

    private static void addToAssociatedMap(FinancialCategory fc,
            FinancialCategory[] validAssociation)
    {
        financialCategoryToAssociatedMap.put(fc, validAssociation);
    }

    /**
     * Returns the TMT_CHARGECODE.CAPITAL_EXPENSE value that matches the
     * specified fc FinancialCategory.
     * 
     * @param fc
     *            Financial category to convert.
     * @return Matching DB code.
     * @throws RPMException
     */
    public static Integer toDBCode(FinancialCategory fc) throws RPMException
    {
        Integer value = (Integer) financialCategoryToDBMap.get(fc);
        if (value == null)
            throw new RPMException("Unknown FinancialCategory " + fc);
        return value;
    }

    /**
     * Returns the FinancialCategory value that matches the specified database
     * code, as found in TMT_CHARGECODE.CAPITAL_EXPENSE.
     * 
     * @param dbCode
     *            Database code to convert.
     * @return Matching FinancialCategory.
     * @throws RPMException
     */
    public static FinancialCategory toFinancialCategory(int dbCode)
            throws RPMException
    {
        Integer key = new Integer(dbCode);
        FinancialCategory value = (FinancialCategory) dbToFinancialCategoryMap
                .get(key);
        if (value == null)
            throw new RPMException("Unknown FinancialCategory code " + dbCode);
        return value;
    }

    /**
     * Returns <code>true</code> when <code>fc</code> is a system, read-only
     * financial category, used only in predefined, system charge codes.
     */
    public static boolean isSystemFinancialCategory(FinancialCategory fc)
            throws RPMException
    {
        return isSystemFinancialCategory(toDBCode(fc).intValue());
    }

    /**
     * Returns <code>true</code> when <code>dbCode</code> is a system,
     * read-only financial category database code, used only in predefined,
     * system charge codes.
     */
    public static boolean isSystemFinancialCategory(int dbCode)
    {
        return dbCode >= 10;
    }
    
    /**
     * Returns <code>true</code> when <code>fc</code> is a benefit, read-only
     * financial category, used only in predefined, system charge codes.
     */
    public static boolean isBenefitFinancialCategory(FinancialCategory fc)
            throws RPMException
    {
        return isBenefitFinancialCategory(toDBCode(fc).intValue());
    }

    /**
     * Returns <code>true</code> when <code>dbCode</code> is a benefit,
     * read-only financial category database code, used only in predefined,
     * system charge codes.
     */
    public static boolean isBenefitFinancialCategory(int dbCode)
    {
        return (dbCode >= 31 && dbCode <=34);
    }
    
    /**
     * Returns <code>true</code> when <code>fc</code> is an expense, read-only
     * financial category, used only in predefined, system charge codes.
     */
    public static boolean isExpenseFinancialCategory(FinancialCategory fc)
            throws RPMException
    {
        return isExpenseFinancialCategory(toDBCode(fc).intValue());
    }

    /**
     * Returns <code>true</code> when <code>dbCode</code> is an expense,
     * read-only financial category database code, used only in predefined,
     * system charge codes.
     */
    public static boolean isExpenseFinancialCategory(int dbCode)
    {
        return (dbCode >= 11 && dbCode <=14);
    }
    
    /**
     * Returns <code>true</code> when <code>fc</code> is a capital, read-only
     * financial category, used only in predefined, system charge codes.
     */
    public static boolean isCapitalFinancialCategory(FinancialCategory fc)
            throws RPMException
    {
        return isCapitalFinancialCategory(toDBCode(fc).intValue());
    }

    /**
     * Returns <code>true</code> when <code>dbCode</code> is a capital,
     * read-only financial category database code, used only in predefined,
     * system charge codes.
     */
    public static boolean isCapitalFinancialCategory(int dbCode)
    {
        return (dbCode >= 21 && dbCode <=24);
    }



    /**
     * Returns true if a charge code with
     * <code>financialCategory</code> <code>targetFC</code> could be
     * associated with a charge code
     * <code>financialCategory</code> <code>originFC</code>.
     */
    public static boolean isValidAssociatedFinancialCategory(
            FinancialCategory originFC, FinancialCategory targetFC)
    {
        FinancialCategory[] validTargets = (FinancialCategory[]) financialCategoryToAssociatedMap
                .get(originFC);
        for (int i = 0; i < validTargets.length; i++)
        {
            if (validTargets[i].equals(targetFC))
            {
                return true;
            }
        }
        return false;
    }
    
    
    /**
     * returns true if there is an expense financial category in the array
     */
    public static boolean containsExpenseCategory(WorksheetFinancial[] wf)
            throws RPMException
    {
        FinancialCategory fc = null;
        boolean expenseChargeCodePresent = false;

        if (wf != null)
        {
            for (int i = 0; i < wf.length; i++)
            {
                if (wf[i].getChargeCode() != null
                        && wf[i].getChargeCode().getFinancialCategory() != null)
                {
                    fc = null;
                    fc = wf[i].getChargeCode().getFinancialCategory();

                    if (isExpenseFinancialCategory(fc))
                    {
                        expenseChargeCodePresent = true;
                        break;
                    }
                }
            }
        }

        return expenseChargeCodePresent;
    }
    
    /**
     * returns true if all four chargeCodes are in the expense financial category array
     */
    public static boolean isValidSystemExpenseChargeCodeCategory(
            AbstractFinancial[] wfArray) throws RPMException
    {
        FinancialCategory fc = null;
        boolean expenseChargeCodePresent = false;
        Integer value = null;

        boolean avgPresent = false;
        boolean hourPresent = false;
        boolean elPresent = false;
        boolean ftePresent = false;

        if (wfArray != null && wfArray.length >= 3)
        {
            for (int i = 0; i < wfArray.length; i++)
            {
                if (wfArray[i].getChargeCode() != null
                        && wfArray[i].getChargeCode().getFinancialCategory() != null)
                {
                    fc = null;
                    fc = wfArray[i].getChargeCode().getFinancialCategory();
                    value = (Integer) financialCategoryToDBMap.get(fc);

                    if (value != null)
                    {
                        if (value.intValue() == EXPENSE_AVG)
                        {
                            avgPresent = true;
                        }
                        else if (value.intValue() == EXPENSE_FTE)
                        {
                            ftePresent = true;
                        }
                        else if (value.intValue() == EXPENSE_HOUR)
                        {
                            hourPresent = true;
                        }
                        else if (value.intValue() == EXPENSE_LABOR)
                        {
                            elPresent = true;
                        }
                    }
                }
            }
        }

        expenseChargeCodePresent = avgPresent && hourPresent && elPresent
                && ftePresent;

        return expenseChargeCodePresent;
    }

    /**
     * returns true if all four chargeCodes are in the capital financial category array
     */
    public static boolean isValidSystemCapitalChargeCodeCategory(
            WorksheetFinancial[] wfArray) throws RPMException
    {
        FinancialCategory fc = null;
        boolean capitalChargeCodePresent = false;
        Integer value = null;

        boolean avgPresent = false;
        boolean hourPresent = false;
        boolean elPresent = false;
        boolean ftePresent = false;

        if (wfArray != null && wfArray.length >= 3)
        {
            for (int i = 0; i < wfArray.length; i++)
            {
                if (wfArray[i].getChargeCode() != null
                        && wfArray[i].getChargeCode().getFinancialCategory() != null)
                {
                    fc = null;
                    fc = wfArray[i].getChargeCode().getFinancialCategory();
                    value = (Integer) financialCategoryToDBMap.get(fc);

                    if (value != null)
                    {
                        if (value.intValue() == CAPITAL_AVG)
                        {
                            avgPresent = true;
                        }
                        else if (value.intValue() == CAPITAL_FTE)
                        {
                            ftePresent = true;
                        }
                        else if (value.intValue() == CAPITAL_HOUR)
                        {
                            hourPresent = true;
                        }
                        else if (value.intValue() == CAPITAL_LABOR)
                        {
                            elPresent = true;
                        }
                    }
                }
            }
        }

        capitalChargeCodePresent = avgPresent && hourPresent && elPresent
                && ftePresent;

        return capitalChargeCodePresent;
    }

    /**
     * returns true if all four chargeCodes are in the benefit financial category array
     */
    public static boolean isValidSystemBenefitChargeCodeCategory(
            WorksheetFinancial[] wfArray) throws RPMException
    {
        FinancialCategory fc = null;
        boolean benefitChargeCodePresent = false;
        Integer value = null;

        boolean avgPresent = false;
        boolean hourPresent = false;
        boolean elPresent = false;
        boolean ftePresent = false;

        if (wfArray != null && wfArray.length >= 3)
        {
            for (int i = 0; i < wfArray.length; i++)
            {
                if (wfArray[i].getChargeCode() != null
                        && wfArray[i].getChargeCode().getFinancialCategory() != null)
                {
                    fc = null;
                    fc = wfArray[i].getChargeCode().getFinancialCategory();
                    value = (Integer) financialCategoryToDBMap.get(fc);

                    if (value != null)
                    {
                        if (value.intValue() == BENEFIT_AVG)
                        {
                            avgPresent = true;
                        }
                        else if (value.intValue() == BENEFIT_FTE)
                        {
                            ftePresent = true;
                        }
                        else if (value.intValue() == BENEFIT_HOUR)
                        {
                            hourPresent = true;
                        }
                        else if (value.intValue() == BENEFIT_LABOR)
                        {
                            elPresent = true;
                        }
                    }
                }
            }
        }

        benefitChargeCodePresent = avgPresent && hourPresent && elPresent
                && ftePresent;

        return benefitChargeCodePresent;
    }

}
