package com.ibm.rpm.framework;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.framework.checkpoints.ForceCheckinCheckpoint;
import com.ibm.rpm.framework.checkpoints.ManualCheckinCheckoutCheckpoint;
import com.ibm.rpm.framework.checkpoints.ScopeCheckpoint;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.CheckoutSerializer;
import com.ibm.rpm.framework.util.CheckpointCaller;
import com.ibm.rpm.framework.util.CollectionUtil;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.LevelMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.PreparedStatementHelper;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.framework.util.ValidateSerializer;
import com.ibm.rpm.framework.util.stats.StatsUtil;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.Join;
import com.ibm.rpm.metadata.model.JoinConditions;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.xpathparser.Condition;
import com.ibm.rpm.xpathparser.ContainerCondition;
import com.ibm.rpm.xpathparser.ContainerFieldMap;
import com.ibm.rpm.xpathparser.FieldCondition;
import com.ibm.rpm.xpathparser.ReflectOpCodes;
import com.ibm.rpm.xpathparser.Table;
import com.ibm.rpm.xpathparser.XPathContainer;
import com.ibm.rpm.xpathparser.XPathUtil;

public abstract class AbstractRPMObjectManager implements RPMObjectManager
{
    private static Log logger = LogFactory.getLog(RPMObjectManager.class);

    public static String STRING_PK_SEPARATOR = new String("+++");

    private static String INVALID_JOIN_CONDITION = "invalid_join_condition";

    private static String TYPE_ID_COLUMN_NAME = "TYPE_ID";

    // =============
    // CONTAINER MAP
    // =============

    /**
     * <code>containerMap</code> is a type id to container class map and a
     * container class to type ids map. It has to be initialized in each
     * sub-class. In the following example, <code>Fruit</code> is generic and
     * does not have its "TYPE_ID". But its mapping will automatically include
     * the type ids of all its derived classes
     * <code>{Apple.TYPE_ID, Pear.TYPE_ID}</code>.<code>
     * protected static ContainerMap = new ContainerMap(FruitManager.class);
     * static 
     * {
     *     containerMap.add(Fruit.class);
     *     containerMap.add(Apple.class);
     *     containerMap.add(Pear.class);
     * }
     * </code>
     */

    protected boolean useDBStatementsForLOBS = false;

    /**
     * Default constructor needed by all the managers. Always use
     * RPMManagerFactory.getInstance().getRPMObjectManager(RPMObject.class.getName());
     * to get a Manager Instance.
     */
    public AbstractRPMObjectManager()
    {
        super();
    }

    protected class RPMObjectComparator implements Comparator
    {
        public RPMObjectComparator()
        {
            super();
        }

        public int compare(Object o1, Object o2)
        {

            String key1 = null;
            String key2 = null;

            if (o1 != null)
            {
                key1 = getPrimaryKeyString((RPMObject) o1);
            }
            if (o2 != null)
            {
                key2 = getPrimaryKeyString((RPMObject) o2);
            }

            if (key1 == null && key2 == null)
            {
                return 0;
            }
            if (key1 == null && key2 != null)
            {
                return -1;
            }
            if (key1 != null && key2 == null)
            {
                return 1;
            }
            return key1.compareTo(key2);
        }

    }

    protected class RPMStringComparator implements Comparator
    {
        public RPMStringComparator()
        {
            super();
        }

        public int compare(Object o1, Object o2)
        {

            String key1 = null;
            String key2 = null;

            if (o1 != null)
            {
                key1 = getPrimaryKeyString((RPMObject) o1);
            }
            if (o2 != null)
            {
                // o2 is always a String representing the primaryKey
                key2 = (String) o2;
            }

            if (key1 == null && key2 == null)
            {
                return 0;
            }
            if (key1 == null && key2 != null)
            {
                return -1;
            }
            if (key1 != null && key2 == null)
            {
                return 1;
            }
            return key1.compareTo(key2);
        }

    }

    /**
     * This method creates a new container and adds it to the cache if
     * the caller wants to.
     * 
     * @param primaryKey
     * @param typeId
     * @param messageContext
     * @return
     * @throws RPMException
     */
    protected final RPMObject createContainer(Object[] primaryKey,
            ResultSet resultSet, MessageContext messageContext,
            String contextName, RPMObjectScope scope, boolean addToCache) throws RPMException,
            SQLException
    {
        RPMObject object = null;

        object = createContainer(resultSet, messageContext);
        // set contextName
        object.setContextName(contextName);
        object = setPrimaryKeyFields(primaryKey, object);
        
        if (addToCache)
        {
            messageContext.getCache().addContainerInCache(this, primaryKey,
                object, scope);
        }
        return object;
    }

    /**
     * This method should be overriden the object should not be taken from the
     * cache
     * 
     * @param primaryKey
     * @param messageContext
     * @return
     */
    public boolean containerExistInCache(Object[] primaryKey,
            MessageContext messageContext)
    {
        return messageContext.getCache().containerExistInCache(this, primaryKey);
    }
    
    public boolean containerExistInCacheWithScope(Object[] primaryKey,
            MessageContext messageContext, RPMObjectScope rpmObjectScope)
    {
        return messageContext.getCache().containerExistInCacheWithScope(this, primaryKey, rpmObjectScope);
    }

    public void addScopeInCache(RPMObject rpmObject, RPMObjectScope rpmObjectScope, MessageContext messageContext)
    {
    	messageContext.getCache().addScopeInCache(rpmObject, rpmObjectScope);
    }
    /**
     * @param messageContext
     * @return RPMObject object representing the current user.
     */
    public static RPMObject getUser(MessageContext messageContext)
    {
        return messageContext.getUser();
    }

    /**
     * @param procedureName
     * @param args
     * @return the element_id of the object that was insert/update/delete
     * @throws SQLException
     * @throws RPMException
     */
    protected final void executeDeleteProcedure(RPMObject object,
            MessageContext context, String procedureName, Object[] args)
            throws SQLException, RPMException
    {
        try
        {
            executeProcedure(context, procedureName, args);
        }
        catch (RPMException e)
        {
            if (e.getErrorID() == ErrorCodes.RECORD_NOT_FOUND)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(object.getClass()),
                        object.getID()
                };
                e = new RPMException(ErrorCodes.AM_INVALID_RPMOBJECTID,
                    errorParams, e);
                SeverityLevel severity = context.isRootObject(object)
                        ? SeverityLevel.SevereError : SeverityLevel.Warning;
                e.setSeverity(severity);
            }
            context.addException(e, object);
        }
    }

    /**
     * @param procedureName
     * @param args
     * @return the element_id of the object that was insert/update/delete
     * @throws SQLException
     * @throws RPMException
     */
    protected final String executeProcedure(MessageContext context,
            String procedureName, Object[] args) throws SQLException,
            RPMException
    {
        return ManagerUtil.executeProcedure(this.getClass(), context,
            procedureName, args);
    }

    /**
     * @return Name of the table associated with the AbstractRPMObjectManager
     */
    public abstract String getTableName();

    // Managers attached to multiple tables (ex : attributes) should override
    // this method
    public String getTableName(RPMObject rpmObject)
    {
        return getTableName();
    }

    public String getTableName(String contextName)
    {
        return getTableName();
    }

    /**
     * Use to calculatethe level in an XPath Query Manager for which the
     * relationship to the parent is defined in another table should override
     * this method (eg. Resource relation to a Pool is defined in table
     * TMT_POOLS while the manager's table is RESOURCES)
     * 
     * @param contextName
     * @return
     */
    public String getTreeTableName(String contextName)
    {
        return getTableName(contextName);
    }

    /**
     * @return an array of type string containing all the fields of the table
     *         associated with the AbstractRPMObjectManager.
     */
    protected abstract String[] getFieldsNames();

    /**
     * @return an array of type string containing all the fields which are not
     *         supported by the xpath framework of the table associated with the
     *         AbstractRPMObjectManager
     */
    protected String[] getUnsupportedFieldsNames()
    {
        return null;
    }

    /**
     * @return string that contains the comma separated fields of the table
     *         associated with the AbstractRPMObjectManager.
     */
    protected abstract String getAllFields();

    /**
     * @return single primary key of the table associated with the
     *         AbstractRPMObjectManager Override if your primary key has only
     *         one field
     */
    public String getPrimaryKey() throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(this.getClass())
        };
        RPMException e = new RPMException(ErrorCodes.AM_NI_GETPRIMARY_KEY,
            errorParams);
        throw e.log();
    }

    /**
     * @return an array of fields that represents the primary key of the table
     *         associated with the AbstractRPMObjectManager Override if your
     *         primary key contains more than one field
     */
    public String[] getPrimaryKeyArray(RPMObject rpmObject) throws RPMException
    {
        return new String[]
        {
            getPrimaryKey()
        };
    }

    /**
     * @param rpmObject
     * @return an array of objects that represents the primary key of the table
     *         associated with the AbstractRPMObjectManager Override if your
     *         primary contains more than one field. IMPORTANT: Always put the
     *         ID property in the primary key. The primary key array should
     *         contains a null to indicate that the object is new.
     */
    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        return new Object[]
        {
            rpmObject.getID()
        };
    }

    public String getPrimaryKeyString(RPMObject rpmObject)
    {
        Object[] args = getPrimaryKeyValues(rpmObject);

        return getPrimaryKeyString(args);
    }

    private String getPrimaryKeyString(Object[] primaryKeyArray)
    {
        StringBuffer primaryKeyString = new StringBuffer();
        for (int i = 0; i < primaryKeyArray.length; i++)
        {
            if (primaryKeyArray[i] == null)
            {
                return null;
            }
            primaryKeyString.append(primaryKeyArray[i].toString().trim());
        }
        return primaryKeyString.toString();
    }

    public String getCheckInCheckOutPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        Object[] args = getManagerInstance(rpmObject.getClass()).getPrimaryKeyValues(
            rpmObject);

        if (args.length == 0)
        {
            return args.toString();
        }
        else if (args.length == 1)
        {
            return args[0].toString();
        }

        StringBuffer primaryKeyString = new StringBuffer();
        for (int i = 0; i < args.length; i++)
        {
            if (args[i] == null)
            {
                return null;
            }
            primaryKeyString.append(args[i].toString().trim());
            primaryKeyString.append(STRING_PK_SEPARATOR);
        }
        return primaryKeyString.toString();
    }

    public Object[] getPKValuesFromString(String pkString)
    {
        StringTokenizer tokenizer = new StringTokenizer(pkString,
            STRING_PK_SEPARATOR, false);
        Object[] objects = new Object[tokenizer.countTokens()];
        int index = 0;
        while (tokenizer.hasMoreTokens())
        {
            objects[index] = tokenizer.nextToken();
            ++index;
        }
        return objects;
    }

    /**
     * @param primaryKey
     * @param rpmObject
     *            sets the fields that are part of the primaryKey Override if
     *            your primary contains more than one field
     */
    protected RPMObject setPrimaryKeyFields(Object[] primaryKey,
            RPMObject rpmObject) throws RPMException
    {
        rpmObject.setID((String) primaryKey[0]);
        return rpmObject;
    }

    /**
     * Gets the primary key array from the resultSet. Override if some of the
     * primary key column are can't be read as String.
     * 
     * @param resultSet
     * @return
     * @throws RPMException,
     *             SQLException
     */
    protected Object[] getPrimaryKey(ResultSet resultSet) throws RPMException,
            SQLException
    {

        String[] primaryKeyArray = getPrimaryKeyArray(null);
        String[] primaryKey = new String[primaryKeyArray.length];
        for (int i = 0; i < primaryKeyArray.length; i++)
        {
            // This line removes the table name
            primaryKeyArray[i] = primaryKeyArray[i].substring(primaryKeyArray[i].indexOf('.') + 1);
            primaryKey[i] = resultSet.getString(primaryKeyArray[i]);

            // remove time for dates (fix for Oracle)
            if (primaryKey[i] != null)
            {
                primaryKey[i] = primaryKey[i].trim();
                int index = primaryKey[i].indexOf(" 00:00:00.0");
                if (index > 0)
                {
                    primaryKey[i] = primaryKey[i].substring(0, index);
                }
            }
        }

        return primaryKey;

    }

    /**
     * Override this method when basic filtering is required (most of the
     * cases).
     * 
     * @see RPMObjectManager#getFilter(String)
     * @see #getFinalCondition(XPathContainer, String)
     */
    public String getFilter(String contextName)
    {
        // Default implementation is to call deprecated implementation.
        return getFilter();
    }

    /**
     * Deprecated. Call or override getFilter(String contextName) instead. NEVER
     * call this method from generic code. TODO SLegault: refactor all calls
     * from getFilter() to getFilter(String) and then delete getFilter().
     */
    protected String getFilter()
    {
        return null;
    }

    protected Integer getContainerType(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {

        return getContainerType(resultSet);
    }

    /**
     * If the manager's table can store more than one type of objects (ex:
     * tree), the typeId is read in the resultSet. Otherwise it is default type
     * or 0; Please override
     * <code>createContainer(ResultSet, MessageContext)</code> instead of
     * <code>createContainer(int)</code> and
     * <code>getContainerType(ResultSet)</code>.
     * 
     * @param resultSet
     * @return
     * @throws RPMException,
     *             SQLException
     */
    // @deprecated Use <code>createContainer(ResultSet, MessageContext)</code>
    // instead.
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        ExceptionUtil.handleNotImplementedJoinCondition(this, manager,
            ExceptionUtil.FOREIGN_KEY);

        return null;
    }

    public ArrayList getJoinConditions(Stack tableStack,
            RPMObjectManager manager, XPathContainer container,
            LevelMap levels, String contextName, boolean isSubTable)
            throws RPMException
    {

        ArrayList joinConditions = null;

        if (!tableStack.isEmpty())
        {
            joinConditions = new ArrayList();

            Table table = (Table) tableStack.pop();

            JoinCondition join = null;

            // remove level logic
            int level = 1;
            if (levels.containsKey(getTreeTableName(contextName)))
            {
                level = levels.getNextTableLevel(getTreeTableName(contextName));
            }
            else
            {
                levels.initTable(getTreeTableName(contextName));
            }

            if (manager != null)
            {
                // use the join info specified in the metadata if it exists
                join = getMetadataJoinCondition(container, contextName,
                    isSubTable);
            }

            if (join == null)
            {
                // logging xpath generation step
                if (logger.isDebugEnabled())
                {
                    StringBuffer sb = new StringBuffer("Calling ");
                    sb.append(StringUtil.getShortClassName(this.getClass()));
                    sb.append(".");
                    if (isSubTable)
                    {
                        sb.append("getSubTableJoinCondition()");
                    }
                    else
                    {
                        sb.append("getJoinCondition()");
                    }
                    sb.append("(manager = ");
                    if (manager == null)
                    {
                        sb.append("null");
                    }
                    else
                    {
                        sb.append(StringUtil.getShortClassName(manager.getClass()));
                    }
                    sb.append(")");

                    logger.debug(sb.toString());
                }

                if (isSubTable)
                {
                    join = getSubTableJoinCondition(manager, container,
                        contextName);
                }
                else
                {
                    join = getJoinCondition(manager, container, contextName);
                }
            }

            if (join != null && join.getCondition() != null
                    && join.getCondition().equals(INVALID_JOIN_CONDITION))
            {
                // the metadata join for this context does not exist,
                // add the invalid join to the list
                joinConditions.add(join);
            }
            else if (join == null)
            {
                // Process will not continue if a null joinCondition is found
                join = addInvalidJoinCondition();
                joinConditions.add(join);
            }
            else
            {
                joinConditions.add(join);
                // logging xpath generation step
                logger.debug("SQL: " + join.toString());

                // Conditions
                ArrayList subJoinConditions = join.getConditions();
                ArrayList conditions = table.getConditions();
                if (conditions != null)
                {
                    for (Iterator iter = conditions.iterator(); iter.hasNext();)
                    {
                        Condition condition = (Condition) iter.next();
                        if (condition instanceof FieldCondition)
                        {
                            // For FieldConditions, add one condition
                            FieldCondition fCondition = (FieldCondition) condition;
                            JoinCondition subCondition = buildFieldCondition(
                                fCondition, table, contextName);
                            if (subCondition != null)
                            {
                                subCondition.setOperator(condition.getOperator());
                                subJoinConditions.add(subCondition);
                            }
                            StringBuffer sb = new StringBuffer("XPath: ");
                            sb.append(fCondition.toString());
                            logger.debug(sb.toString());
                            logger.debug("SQL: " + subCondition.toString());
                        }
                        else
                        {
                            // For ContainerConditions, add an array of
                            // conditions
                            ContainerCondition cCondition = (ContainerCondition) condition;
                            Stack subTableStack = cCondition.getSubTables();

                            StringBuffer sb = new StringBuffer("XPath: ");
                            sb.append(StringUtil.getShortClassName(cCondition.getClass()));
                            sb.append(": Operator ");
                            sb.append(cCondition.getOperator());
                            logger.debug(sb.toString());

                            Table subTable = (Table) subTableStack.peek();

                            RPMObjectManager subTableManager = RPMManagerFactory.getInstance().getRPMObjectManager(
                                subTable.getObjectName(), false);

                            if (subTableManager == null)
                            {
                                ExceptionUtil.createAndHandleRPMException(
                                    this.getClass(),
                                    ErrorCodes.XPATH_INVALID_CONTAINER_OR_FIELD,
                                    new String[]
                                    {
                                        subTable.getObjectName()
                                    });
                            }

                            // Fill xpathContainer for sub object;
                            XPathContainer subContainer = buildXPathContainer(
                                subTable, table);
                            ;
                            // TODO remove after refactoring
                            subContainer.setLevel(level);

                            ArrayList subTableJoins = subTableManager.getJoinConditions(
                                subTableStack, this, subContainer, levels,
                                contextName, true);

                            if (subTableJoins != null
                                    && !subTableJoins.isEmpty())
                            {
                                // Add operator to the first condition
                                JoinCondition firstCondition = (JoinCondition) subTableJoins.get(0);
                                if (firstCondition != null)
                                {
                                    firstCondition.setOperator(condition.getOperator());
                                }
                                subJoinConditions.add(subTableJoins);
                            }

                        }
                    }

                    join.setConditions(subJoinConditions);
                }

                // getJoinConditions for other tables in the stack
                if (!tableStack.empty())
                {

                    Table nextTable = (Table) tableStack.peek();

                    RPMObjectManager nextManager = RPMManagerFactory.getInstance().getRPMObjectManager(
                        nextTable.getObjectName(), false);

                    if (nextManager == null)
                    {
                        ExceptionUtil.createAndHandleRPMException(
                            this.getClass(),
                            ErrorCodes.XPATH_INVALID_CONTAINER_OR_FIELD,
                            new String[]
                            {
                                nextTable.getObjectName()
                            });
                    }

                    // Fill xpathContainer for next object;
                    XPathContainer nextContainer = buildXPathContainer(
                        nextTable, table);
                    ;
                    // TODO remove after refactoring
                    nextContainer.setLevel(level);

                    ArrayList nextJoins = nextManager.getJoinConditions(
                        tableStack, this, nextContainer, levels, contextName,
                        false);
                    if (nextJoins != null && !nextJoins.isEmpty())
                    {
                        joinConditions.addAll(nextJoins);
                    }
                }
            }
        }

        return joinConditions;
    }

    private JoinCondition addInvalidJoinCondition()
    {
        JoinCondition join = new JoinCondition();
        join.setCondition(INVALID_JOIN_CONDITION);

        // logging xpath generation step
        logger.debug(join.toString());

        return join;
    }

    private JoinCondition getMetadataJoinCondition(XPathContainer container,
            String contextName, boolean isSubTable) throws RPMException
    {
        MetadataStore store = MetadataStore.getInstance();
        Container metadataContainer = null;
        String type = null;
        String fieldName = null;

        if (isSubTable)
        {
            // for a subjoin, use the previous object and the previous field
            // name
            type = container.getPreviousType();
            metadataContainer = store.getContainerByName(type);
            fieldName = container.getPreviousFieldName();
        }
        else
        {
            // for a join, use the current object and the current field name
            type = container.getType().toString();
            metadataContainer = store.getContainerByJavaClass(container.getType());
            fieldName = container.getFieldName();
        }

        StringBuffer sb = new StringBuffer(
            "Attempting to retrieve metadata join information for container "
                    + StringUtil.getShortClassName(type) + " and field "
                    + fieldName);
        logger.debug(sb.toString());

        JoinCondition join = null;

        if (metadataContainer != null && fieldName != null)
        {
            Field field = metadataContainer.getField(fieldName);

            JoinConditions joinConditions = null;

            if (field != null && field.getJoinConditions() != null
                    && !field.getJoinConditions().isEmpty())
            {
                joinConditions = findJoinConditionsByContext(field, contextName);

                if (joinConditions != null)
                {
                    // retrieve the foreign manager
                    RPMObjectManager foreignManager = null;
                    if (isSubTable)
                    {
                        foreignManager = RPMManagerFactory.getInstance().getRPMObjectManager(
                            container.getPreviousType());
                    }
                    else
                    {
                        foreignManager = RPMManagerFactory.getInstance().getRPMObjectManager(
                            field.getTypeNoArray());
                    }

                    join = new JoinCondition();
                    Iterator iterator = null;

                    if (isSubTable)
                    {
                        // if we're dealing with a subtable join, we need to
                        // invert the
                        // order of the joins specified in the metadata
                        List subJoins = new ArrayList();

                        for (int i = joinConditions.getJoins().size() - 1; i >= 0; i--)
                        {
                            subJoins.add(joinConditions.getJoins().get(i));
                        }
                        iterator = subJoins.iterator();

                    }
                    else
                    {
                        iterator = joinConditions.getJoins().iterator();
                    }

                    while (iterator.hasNext() && join != null)
                    {
                        // get the next join specified in the metadata
                        Join metadataJoin = (Join) iterator.next();

                        String localTableName = null;
                        String foreignTableName = null;

                        // use the table name specified in the metadata if it
                        // exists
                        if (isSubTable)
                        {
                            localTableName = metadataJoin.getForeignTableName() != null
                                    ? metadataJoin.getForeignTableName()
                                    : this.getTableName(contextName);

                            foreignTableName = metadataJoin.getLocalTableName() != null
                                    ? metadataJoin.getLocalTableName()
                                    : foreignManager.getTableName(contextName);
                        }
                        else
                        {
                            localTableName = metadataJoin.getLocalTableName() != null
                                    ? metadataJoin.getLocalTableName()
                                    : this.getTableName(contextName);

                            foreignTableName = metadataJoin.getForeignTableName() != null
                                    ? metadataJoin.getForeignTableName()
                                    : foreignManager.getTableName(contextName);
                        }

                        // append the join condition(s) specified in the
                        // metadata
                        if (localTableName != null && foreignTableName != null)
                        {
                            String finalCondition = null;
                            if (!iterator.hasNext())
                            {
                                // get the final condition only if this is the
                                // last subselect
                                getFinalCondition(container, contextName);
                            }

                            if (isSubTable)
                            {
                                join.appendSubSelect(
                                    foreignTableName + "."
                                            + metadataJoin.getLocalColumnName(),
                                    localTableName
                                            + "."
                                            + metadataJoin.getForeignColumnName(),
                                    localTableName, finalCondition);
                            }
                            else
                            {
                                join.appendSubSelect(
                                    foreignTableName
                                            + "."
                                            + metadataJoin.getForeignColumnName(),
                                    localTableName + "."
                                            + metadataJoin.getLocalColumnName(),
                                    localTableName, finalCondition);
                            }

                            if (iterator.hasNext())
                            {
                                // adding another join so append a where
                                join.appendWhere();
                            }
                        }
                        else
                        {
                            join = null;
                        }
                    }
                }
                else
                {
                    // if joinConditions is null but metadata join conditions
                    // exist,
                    // then the join is not valid for this specific context -
                    // processing will continue for other contexts
                    join = addInvalidJoinCondition();
                }
            }
        }
        return join;
    }

    private JoinConditions findJoinConditionsByContext(Field field,
            String contextName)
    {
        // find the metadata JoinConditions object with the correct context
        Iterator iterator = field.getJoinConditions().iterator();
        JoinConditions joinConditions = null;
        JoinConditions nullContextCondition = null;

        boolean found = false;

        while (iterator.hasNext())
        {
            joinConditions = (JoinConditions) iterator.next();

            if (joinConditions.getContextName() == null)
            {
                nullContextCondition = joinConditions;

                if (contextName == null)
                {
                    found = true;
                    break;
                }
            }
            else if (joinConditions.getContextName().equals(contextName))
            {
                found = true;
                break;
            }
        }
        if (!found)
        {
            // use the null context joinCondition if it exists
            joinConditions = nullContextCondition;
        }

        return joinConditions;
    }

    /**
     * Returns the condition to be added at the end of the join condition. This
     * default implementation is the combination of the basic filter and the
     * container filter.
     */
    protected String getFinalCondition(XPathContainer container,
            String contextName)
    {
        // Add the basic filtering condition
        SqlBuffer finalCondition = new SqlBuffer(getFilter(contextName));

        // Add the typeId condition if it exists
        finalCondition = getContainerFilter(finalCondition, container,
            contextName);

        return finalCondition.toString();
    }

    /**
     * Returns the SQL condition that will select only the records that match
     * the given container type. Remember that this container might be generic
     * thus matching many subtypes.
     */
    protected SqlBuffer getContainerFilter(SqlBuffer finalCondition,
            XPathContainer container, String contextName)
    {
        String typeIdColumn = getTableName(contextName) + "."
                + TYPE_ID_COLUMN_NAME;
        Collection typeIds = null;

        // if the manager has a containerMap, get the typeIds using it
        ContainerMap containerMap = getContainerMap();
        if (containerMap != null)
        {
            typeIds = containerMap.getTypeIds(container.getType());
        }
        // if not, get the manager's set of containers and retrieve
        // the typeIds using it
        else
        {
            HashSet containers = getContainers();
            if (containers != null)
            {
                typeIds = new ArrayList();

                RPMObject[] objs = new RPMObject[containers.size()];
                containers.toArray(objs);
                for (int i = 0; i < objs.length; i++)
                {
                    typeIds.add(objs[i].retrieveTypeID());
                }
            }
        }

        if (getTableName(contextName) != null && typeIds != null)
        {
            finalCondition.appendIn(typeIdColumn, typeIds);
        }

        return finalCondition;
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition join = getJoinCondition(manager,
            null, container.getLevel(),
            container.getShortTypeName(), contextName);

        if (manager == null && join == null)
        {
            // if a join for a null manager is not specified in
            // getJoinCondition,
            // use the final condition; this allows us to override the
            // null manager join in getJoinCondition - for new managers we can
            // simply
            // override getFinalCondition
            StringBuffer sb = new StringBuffer("Calling "
                    + StringUtil.getShortClassName(this.getClass()) + "."
                    + "getFinalCondition for manager == null with context: "
                    + contextName);
            logger.debug(sb.toString());

            join = new JoinCondition(getTableName(contextName),
                getFinalCondition(container, contextName));
        }

        return join;
    }

    /**
     * OBSOLETE - Implement getJoinCondition(RPMObjectManager manager,
     * XPathContainer container, String contextName) instead.
     * 
     * TODO: Remove this method.
     * 
     * Convert XPath relation of type "/parent/child" into an SQL Statement
     * 
     * If the manager is null, it returns a joinCondition object that contains
     * the condition received as a parameter.
     * 
     * @param manager
     * @param condition
     *            Condition applicable to the joined table
     * @param level
     *            Use in case of a self join to set the minimum level of a child
     *            object
     * @param containerName
     * @param contextName
     * 
     * @return
     * @throws RPMException
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // a missing join is valid for manager == null or if the join
        // is specified in the metadata (metadata is checked prior to this)
        if (manager != null)
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return null;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        return getSubTableJoinCondition(manager, null,
            container.getLevel(), container.getShortTypeName(), contextName);
    }

    /**
     * OBSOLETE - Implement getSubTableJoinCondition(RPMObjectManager manager,
     * XPathContainer container, String contextName) instead.
     * 
     * TODO: Remove this method.
     * 
     * Convert XPath relation of type "/parent[child]" into an SQL Statement
     * 
     * @param manager
     * @param condition
     * @param level
     * @param containerName
     * @param contextName
     * @return
     * @throws RPMException
     */
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.SUB_TABLE_JOIN);

        return null;
    }

    /**
     * @return HashMap that contains the relations between the RPMObject's
     *         properties and the name of the database's fields
     */
    protected HashMap getPropertiesMap() throws RPMException
    {
        ExceptionUtil.handleUnsupportedContext(this, ((String) null),
            ExceptionUtil.GET_PROPERTIES);
        return null;
    }

    /*
     * Managers should return the ContainerMap if they have one.
     */
    public ContainerMap getContainerMap()
    {
        return null;
    }

    /*
     * Managers should return the set of containers if they don't have a
     * ContainerMap
     */
    public HashSet getContainers()
    {
        return null;
    }

    /**
     * Overide this method if you have more than one map. (see
     * WbsScheduleDateManager)
     * 
     * @return HashMap that contains the relations between the RPMObject's
     *         properties and the name of the database's fields
     */
    protected HashMap getPropertiesMap(String fieldName, String containerName)
            throws RPMException
    {
        return getPropertiesMap();
    }

    /**
     * Overide this method if you a manager handles multiple similar tables.
     * (see DocumentManager)
     * 
     * @return the qualified name of the database field corresponding to the
     *         propertyName
     */
    protected Object getQualifiedColumnName(HashMap propertiesMap,
            String propertyName, String contextName)
    {
        return propertiesMap.get(propertyName.toUpperCase());
    }

    /**
     * @param propertyName
     * @return the name of the database field corresponding to the propertyName
     * @throws RPMException
     */
    protected final String getColumnName(String propertyName,
            String containerName, String contextName) throws RPMException
    {
        String columnName = null;

        HashMap propertiesMap = getPropertiesMap(propertyName, containerName);

        Object o = getQualifiedColumnName(propertiesMap, propertyName,
            contextName);

        if (o != null && !propertyName.equalsIgnoreCase("deleted")
                && !propertyName.equalsIgnoreCase("parent"))
        {
            columnName = (String) o;
        }
        else
        {
            ExceptionUtil.handleUnsupportedFieldInXpath(containerName,
                propertyName);
        }

        return columnName;
    }

    /**
     * Creates the container that matches the given <code>resultSet</code>.
     * Default implementation calls <code>getContainerType(ResultSet)</code>
     * and <code>createContainer(int)</code> for backward compatibility.
     * Please override this method and implement it by merging your old
     * implementations of <code>getContainerType(ResultSet)</code> and
     * <code>createContainer(int)</code>. Then delete these two deprecated
     * methods.
     * 
     * @param resultSet
     *            Table row.
     * @param messageContext
     *            Message context.
     * @return A new empty container that will be filled in decodeRow.
     */
    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        Integer type = getContainerType(resultSet);
        return createContainer(type.intValue());
    }

    /**
     * Please override <code>createContainer(ResultSet, MessageContext)</code>
     * instead of <code>createContainer(int)</code> and
     * <code>getContainerType(ResultSet)</code>.
     * 
     * @param typeId
     * @return
     * @throws RPMException
     */
    // @deprecated Use <code>createContainer(ResultSet, MessageContext)</code>
    // instead.
    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return null;
    }

    protected final JoinCondition buildFieldCondition(
            FieldCondition fieldCondition, Table table, String contextName)
            throws RPMException
    {

        JoinCondition joinCondition = null;
        Class fieldType = null;

        StringBuffer sb = new StringBuffer();
        sb.append("(");

        if (fieldCondition != null)
        {
            joinCondition = new JoinCondition();
            sb.append(getColumnName(fieldCondition.getProperty(),
                table.getObjectName(), contextName));

            // convert enumerated type to database value
            fieldType = ContainerFieldMap.getInstance().getFieldType(
                table.getObjectType(), fieldCondition.getProperty());

            if (EnumeratedTypeUtil.getInstance().isEnumeratedType(
                table.getObjectType(), fieldType))
            {
                if (fieldCondition.getOperation() == ReflectOpCodes.OP_NOTEQUALS)
                {
                    sb.append(" NOT");
                }
                sb.append(" IN ");

                ArrayList enumValues = EnumeratedTypeUtil.getInstance().getValues(
                    table.getObjectType(),
                    fieldType,
                    StringUtil.removeQuotationMarks(fieldCondition.getValue().toString()));

                if (enumValues == null)
                {
                    String[] params = new String[]
                    {
                            table.getObjectName(),
                            fieldCondition.getProperty(),
                            StringUtil.getShortClassName(fieldType)
                    };
                    throw new RPMException(
                        ErrorCodes.XPATH_UNSUPPORTED_ENUMERATED_TYPE, params);
                }

                sb.append("(");
                for (int i = 0; i < enumValues.size(); ++i)
                {
                    if (i != 0)
                    {
                        sb.append(",");
                    }
                    sb.append("'");
                    sb.append(enumValues.get(i).toString());
                    sb.append("'");
                }
                sb.append(")");
            }
            else
            {
                sb.append(fieldCondition.getStringOperator());

                if (fieldType != null
                        && Boolean.class.isAssignableFrom(fieldType))
                {
                    sb.append("'");
                    sb.append(StringUtil.removeQuotationMarks(fieldCondition.getValue().toString()));
                    sb.append("'");
                }
                else
                {
                    sb.append(StringUtil.replaceApostrophesWithQuotes(fieldCondition.getValue()));
                }
            }
        }

        sb.append(")");

        joinCondition.setCondition(sb.toString());

        return joinCondition;
    }

    protected final XPathContainer buildXPathContainer(Table currentTable,
            Table previousTable) throws RPMException
    {

        XPathContainer container = new XPathContainer();

        container.setType(currentTable.getObjectType());
        container.setShortTypeName(currentTable.getObjectName());
        container.setFieldName(currentTable.getFieldName());
        container.setTypeId(XPathUtil.getinstance().retrieveTypeId(currentTable));

        if (previousTable != null)
        {
            container.setPreviousType(previousTable.getObjectName());
            container.setPreviousFieldName(previousTable.getFieldName());
        }

        return container;
    }

    private final boolean findInSelectPattern(String input)
    {
        boolean found = false;
        Pattern pattern = Pattern.compile("(?i)select.{1,}from(?!.*where)");

        // remove the contents in quotes (ie. names of elements) so that
        // the pattern matcher doesn't get messed up by names that
        // include words found in the pattern (select, where, from)
        String strippedInput = StringUtil.removeQuotedContent(input.toString());
        if (pattern.matcher(strippedInput).find())
        {
            found = true;
        }

        return found;
    }

    protected final String buildQuery(ArrayList joinConditions,
            boolean subCondition, boolean nextIsWhere, ArrayList tables)
            throws RPMException
    {
        StringBuffer query = new StringBuffer();
        StringBuffer where = new StringBuffer();

        Object joinObject;
        String tableName;
        boolean invalidQuery = false;
        for (int j = 0; j < joinConditions.size(); j++)
        {
            joinObject = joinConditions.get(j);

            /*
             * Object in the ArrayList can be a joinConditoin or an ArrayList of
             * JoinCondition
             */
            if (joinObject instanceof JoinCondition)
            {

                JoinCondition condition = (JoinCondition) joinObject;

                if (!condition.getCondition().equals(INVALID_JOIN_CONDITION))
                {
                    tableName = condition.getTableName();
                    if (tableName != null && tableName.length() > 0)
                    {
                        if (!tables.contains(tableName))
                        {
                            tables.add(tableName);
                        }
                    }

                    if (condition.getCondition() != null
                            && condition.getCondition().length() > 0)
                    {

                        if (condition.getOperator() != null)
                        {
                            where.append(condition.getOperator());
                        }
                        else
                        {

                            if (nextIsWhere)
                            {
                                where.append(" WHERE ");
                            }
                            else
                            {
                                if (findInSelectPattern(where.toString()))
                                {
                                    where.append(" WHERE ");
                                }
                                else
                                {
                                    // parenthesis needed for 'or' operation to
                                    // make sure that the base condition is
                                    // never excluded
                                    where.append(" AND ( ");
                                }
                            }
                        }
                        where.append(condition.getCondition());

                        if (findInSelectPattern(where.toString()))
                        {
                            nextIsWhere = true;
                        }
                        else
                        {
                            nextIsWhere = false;
                        }

                    }

                    if (condition.getConditions() != null
                            && !condition.getConditions().isEmpty())
                    {
                        String subQuery = buildQuery(condition.getConditions(),
                            true, nextIsWhere, tables);
                        if (subQuery != null && subQuery.length() > 0)
                        {
                            where.append(subQuery);
                        }
                        else
                        {
                            invalidQuery = true;
                            // reset query
                            query = new StringBuffer();
                            break;
                        }

                        // reevaluate the need for a where after the sub
                        // condition have been appended to the query
                        if (findInSelectPattern(where.toString()))
                        {
                            nextIsWhere = true;
                        }
                        else
                        {
                            nextIsWhere = false;
                        }
                    }

                }
                else
                {
                    invalidQuery = true;
                    // reset query
                    query = new StringBuffer();
                    break;
                }

            }
            else
            {
                ArrayList subTableJoins = ((ArrayList) joinObject);
                if (subTableJoins != null && !subTableJoins.isEmpty())
                {
                    String subQuery = buildQuery(subTableJoins, true,
                        nextIsWhere, tables);
                    if (subQuery != null && subQuery.length() > 0)
                    {
                        where.append(subQuery);
                    }
                    else
                    {
                        invalidQuery = true;
                        // reset query
                        query = new StringBuffer();
                        break;
                    }
                }
            }
        }

        if (!invalidQuery)
        {
            if (subCondition)
            {
                query = where;
            }
            else
            {
                query.append("SELECT ");
                query.append(getAllFields());
                query.append(" FROM ");
                for (int i = 0; i < tables.size(); i++)
                {
                    if (i > 0)
                    {
                        query.append(",");
                    }
                    query.append((String) tables.get(i));
                }
                query.append(" ");
                if (where != null && where.length() > 0)
                {
                    query.append(where.toString());
                }
            }

            // Add missing parenthesis
            int left = StringUtil.countOccurence(query.toString(), "\\(");
            int right = StringUtil.countOccurence(query.toString(), "\\)");

            // excludes parenthesis between quotes
            int excludedLeft = StringUtil.countOccurenceBetweenQuotes(
                query.toString(), "\\(");
            int excludedRight = StringUtil.countOccurenceBetweenQuotes(
                query.toString(), "\\)");

            int missing = left - excludedLeft - (right - excludedRight);
            for (int i = 0; i < missing; i++)
            {
                query.append(')');
            }

        }

        return query.toString();
    }

    public ArrayList load(Stack tableStack, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (messageContext == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.AM_INVALID_MC);
        }

        ArrayList containers = null;
        ArrayList joinConditions = null;
        String[] contextNames = null;
        String query = null;

        if (!tableStack.isEmpty())
        {
            Table table = (Table) tableStack.peek();

            // To support an internal use of load from xpath
            if (table.getObjectType() == null)
            {
                XPathUtil.getinstance().validateAndPrepareTableStack(
                    tableStack, tableStack.size() - 1);
            }

            ScopeCheckpoint.getInstance().isValid(
                ((Table) tableStack.peek()).getObjectName(), rpmScope);

            containers = new ArrayList();

            contextNames = ContextUtil.getInstance().getContextNames(
                table.getObjectName());

            /*
             * 60302 : don't query contexts if they're out of scope for objects a level below 
             * the current level in the table stack
             * 
             * if parent is umbrella and the current obj a child context
             *      or if current is umbrella and parent a child context
             */
            int tableSize = tableStack.size();
            if(tableSize >= 2) {
                Table parentTable = (Table) tableStack.elementAt(tableSize - 2);
                
                String[] parentContextNames = ContextUtil.getInstance().getContextNames(
                    parentTable.getObjectName());
                
                contextNames = ContextUtil.determineValidContexts(contextNames, parentContextNames);
            }
            
            ArrayList tableNames = new ArrayList();

            ArrayList tempContainers = null;

            for (int i = 0; i < contextNames.length; i++)
            {
                tempContainers = null;

                joinConditions = new ArrayList();

                Stack tempStack  = Table.cloneTableStack(tableStack);

                // Fill xpathContainer for current object;
                XPathContainer container = buildXPathContainer(table, null);

                logger.debug("***** Generating sql statement for context: "
                        + contextNames[i] + " *****");

                joinConditions = getJoinConditions(tempStack, null, container,
                    new LevelMap(), contextNames[i], false);

                query = buildQuery(joinConditions, false, true, new ArrayList());

                // Note: avoid executing a query twice on the same table
                String tableName = ((JoinCondition) joinConditions.get(0)).getTableName();

                if (query != null && query.length() > 0
                        && !tableNames.contains(tableName))
                {
                    tableNames.add(tableName);

                    logger.debug("****** Executing generated sql statement ******");

                    tempContainers = loadBySelect(null, null, rpmScope,
                        messageContext, query, null, contextNames[i], false);

                    if (tempContainers != null)
                    {
                        containers.addAll(tempContainers);
                    }
                }
                else
                {
                    if (contextNames.length == 1)
                    {
                        String[] errorParams =
                        {
                            "Cannot generate an sql for this context: \""
                                    + contextNames[i] + "\"."
                        };
                        RPMException ex = new RPMException(
                            ErrorCodes.INTERNAL_ERROR, errorParams);
                        ex.setSeverity(SeverityLevel.Error);
                        messageContext.addExceptionNoThrow(ex);
                    }
                    else
                    {
                        logger.debug("A query was already executed on table related to context: \""
                                + contextNames[i] + "\".");
                    }
                }

            }
            containers = CollectionUtil.nullify(containers);
        }

        return containers;
    }

    /**
     * Design note: This method should be called only from
     * <code>ManagerCaller</code>.
     */
    public RPMObject load(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ScopeCheckpoint.getInstance().isValid(rpmObject, rpmScope);

        messageContext.getCache().resetLoadedContainers();

        if (messageContext == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.AM_INVALID_MC);
        }

        isValidPrimaryKey(rpmObject, messageContext);

        return loadByPrimaryKey(rpmObject, rpmScope, messageContext, false);
    }

    /**
     * This method is used to refresh the fields of a container that are not
     * part of the scope (calls decode row only).
     * 
     * @param rpmObject
     * @param messageContext
     * @return
     */
    private RPMObject reloadWithoutScope(MessageContext context,
            RPMObject rpmObject) throws RPMException
    {
        if (rpmObject == null)
        {
            return null;
        }
        Connection c = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        try
        {
            if (getPrimaryKeyString(rpmObject) == null)
            {
                ExceptionUtil.handleRPMException(this,
                    ErrorCodes.SAVED_OBJECT_NOT_RELOADED, rpmObject, null, null);
            }

            c = Manager.getConnection(context);

            // Builds the SQL statement.
            SqlBuffer statement = new SqlBuffer();
            statement.appendSelect();
            statement.append(getAllFields());
            statement.appendFrom(getTableName(rpmObject));
            statement.appendWhere();
            String[] keyColumns = getPrimaryKeyArray(rpmObject);
            Object[] keyValues = getPrimaryKeyValues(rpmObject);
            // TODO SLegault: What if keyColumns.length != keyValues.length?
            for (int i = 0; i < keyColumns.length; i++)
            {
                statement.appendEqualQuestionMark(keyColumns[i]);
            }
            statement.appendCondition(getFilter(rpmObject.getContextName()));

            String statementText = statement.toString();
            statementText = Manager.makeStatementUncommitedRead(statementText,
                c);
            logger.debug(statementText);

            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(context,
                c);
            ps = preparedStatementHelper.createPreparedStatement(statementText,
                keyValues);

            rs = ManagerUtil.executeSqlQuery(context, ps, statementText,
                keyValues);
            if (rs.next())
            {
                context.getStatsOperation().incrementLoadedObjectCount();
                rpmObject = decodeRow(rs, rpmObject, false);
            }
        }
        catch (Exception e)
        {
            context.addException(e);
        }
        finally
        {
            Manager.close(context, rs);
            Manager.releaseConnection(context, c);
        }

        return rpmObject;
    }

    public RPMObject loadByPrimaryKey(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        if (rpmObject == null)
        {
            return null;
        }

        if (!isDelta)
        {
            RPMObject cachedContainer = getContainerInCacheWithScope(rpmObject,
                rpmScope, messageContext);

            if (cachedContainer != null)
            {
                return cachedContainer;
            }
        }

        String[] possibleContextNames = ContextUtil.getInstance().getContextNames(
            rpmObject.getClass().getName());

        // note: needed for some cases where the parent context must
        // match the object context.(e.g. financial)
        RPMObject parent = rpmObject.getParent();
        if (possibleContextNames.length > 1 && parent != null
                && parent.getContextName() != null)
        {
            rpmObject.setContextName(parent.getContextName());
        }

        // If context isn't specified, try all of the tables
        if (rpmObject.getContextName() == null)
        {

            for (int i = possibleContextNames.length - 1; i >= 0; --i)
            {
                rpmObject.setContextName(possibleContextNames[i]);

                RPMObject rpmObjectWithContext = null;
                try
                {
                    rpmObjectWithContext = loadByPrimaryKeyWithSpecifiedContext(
                        rpmObject, rpmScope, messageContext, isDelta);
                }
                catch (RPMException e)
                {
                    // Only throw the exception in the following cases :
                    // If the errorId is different from the invalid RPMObjectID,
                    // to allow trying multiple contexts
                    // If it's the last possible context, this means that
                    // indeed, the object does not exist in all the contexts
                    if (e.getErrorID() != ErrorCodes.AM_INVALID_RPMOBJECTID
                            || i == 0)
                    {
                        throw e;
                    }
                }

                if (rpmObjectWithContext != null)
                {
                    return rpmObjectWithContext;
                }
            }
        }
        else
        {
            return loadByPrimaryKeyWithSpecifiedContext(rpmObject, rpmScope,
                messageContext, isDelta);
        }

        return null;
    }

    public RPMObject getContainerInCacheWithScope(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext)
    {
        RPMObjectCache cache = messageContext.getCache();
        Object[] primaryKeyValues = getPrimaryKeyValues(rpmObject);
        RPMObject objectInCache = cache.getContainerInCache(this,
            primaryKeyValues);

        // first, the object must be in the cache
        if (objectInCache != null)
        {
            // second, the scope must either be null or match a scope that was
            // previously
            // applied to the container.
            if (rpmScope == null
                    || cache.isContainerInCacheWithScope(objectInCache,
                        rpmScope))
            {
                cache.addCacheHit();
                return objectInCache;
            }
        }

        cache.addCacheMiss();
        return null;
    }

    private RPMObject loadByPrimaryKeyWithSpecifiedContext(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        RPMObject outputObject = null;
        try
        {
            isValidPrimaryKey(rpmObject, messageContext);
            String primaryKeyString = getPrimaryKeyString(rpmObject);
            if (rpmObject == null || primaryKeyString == null)
            {
                return null;
            }

            // Builds the SQL statement.
            SqlBuffer statement = new SqlBuffer();
            statement.appendSelect();
            statement.append(getAllFields());
            statement.appendFrom(getTableName(rpmObject));
            statement.appendWhere();
            String[] keyColumns = getPrimaryKeyArray(rpmObject);
            Object[] keyValues = getPrimaryKeyValues(rpmObject);
            // TODO: What if keyColumns.length != keyValues.length?
            for (int i = 0; i < keyColumns.length; i++)
            {
                statement.appendEqualQuestionMark(keyColumns[i]);
            }
            /* This change is done to fix defect RATLC00257017
               This requires that the AttributeAssignment should be loaded with the
               Attributes for the Attributes that are soft deleted */
            if(!(this instanceof AttributeManager))
            {
                statement.appendCondition(getFilter(rpmObject.getContextName()));
            }

            // Reset the contextName.
            String contextName = null;
            if (rpmObject != null)
            {
                contextName = rpmObject.getContextName();
                // Get the context from the map if the type is not
                // multiple-context
                if (contextName == null)
                {
                    String[] contextNames = ContextUtil.getInstance().getContextNames(
                        rpmObject.getClass().getName());
                    if (contextNames.length == 1)
                    {
                        rpmObject.setContextName(contextNames[0]);
                        contextName = contextNames[0];
                    }
                }
            }

            RPMObject inputObject = isDelta ? rpmObject : null;
            RPMObject[] inputObjects = new RPMObject[]
            {
                inputObject
            };

            ArrayList outputObjects = loadBySelect(null, inputObjects,
                rpmScope, messageContext, statement.toString(), keyValues,
                contextName, isDelta);

            // TODO: What if outputObjects.size() > 1?
            if (!CollectionUtil.isEmpty(outputObjects))
            {
                outputObject = (RPMObject) outputObjects.get(0);
            }
            else
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(rpmObject.getClass()),
                        primaryKeyString
                };

                RPMException e = new RPMException(
                    ErrorCodes.AM_INVALID_RPMOBJECTID, errorParams,
                    rpmObject.getClass().getName(), null, rpmObject.getID());
                SeverityLevel severity = messageContext.isRootObject(rpmObject)
                        ? SeverityLevel.SevereError : SeverityLevel.Warning;
                e.setSeverity(severity);
                messageContext.addException(e);
            }
        }
        catch (Exception x)
        {
            messageContext.addException(x, rpmObject);
        }
        return outputObject;
    }

    /**
     * Method used to load related objects (ex: Children from the same table,
     * Resource in a Pool etc.). The method
     * getForeignKeyCondition(AbstractRPMObjectManager, String) will be used to
     * determine the join condition between the parent and the children. If the
     * foreign key condition includes ?, the parameter args should contains the
     * values that will replace the ? in the PreparedStatement. The parameter
     * extraWhere is usefull if you need to add an extra condition (Ex: TYPE_ID = ?)
     * 
     * @param rpmScope
     * @param messageContext
     * @param manager
     * @param args
     * @param extraWhere
     * @param contextName
     * 
     * @return
     */
    public final ArrayList loadByForeignKey(RPMObject[] rpmObjects,
            RPMObjectScope rpmScope, MessageContext messageContext,
            RPMObjectManager manager, Object args[], String extraWhere,
            String contextName, boolean isDelta) throws RPMException,
            SQLException, ParseException
    {
        return loadByForeignKeyWithSpecifiedParent(null, rpmObjects, rpmScope,
            messageContext, manager, args, extraWhere, contextName, isDelta);
    }

    public final ArrayList loadByForeignKeyWithSpecifiedParent(
            RPMObject parent, RPMObject[] rpmObjects, RPMObjectScope rpmScope,
            MessageContext messageContext, RPMObjectManager manager,
            Object args[], String extraWhere, String contextName,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ArrayList result = null;
        try
        {
            JoinCondition condition = getForeignKeyCondition(manager,
                extraWhere, contextName);

            // Append filter. Filter is appended after all parenthesis are
            // closed to
            // be sure that the condition will be applied to the loaded objects.
            condition.closeAllParenthesis();
            condition.appendCondition(getFilter(contextName));

            // Reset the contextName.
            if (parent != null && parent.getContextName() == null)
            {
                // Get the context from the map if the type is not
                // multiple-context
                if (contextName == null)
                {
                    String[] contextNames = ContextUtil.getInstance().getContextNames(
                        parent.getClass().getName());
                    if (contextNames.length == 1)
                    {
                        parent.setContextName(contextNames[0]);
                        contextName = contextNames[0];
                    }
                }
                else
                {
                    parent.setContextName(contextName);
                }
            }

            String tableName = condition.getTableName();
            String allFields = getAllFields();
            result = loadByWhere(parent, rpmObjects, rpmScope, messageContext,
                condition.toString(), args, tableName, allFields, contextName,
                isDelta);
        }
        catch (Exception x)
        {
            messageContext.addException(x);
        }
        return result;
    }

    /**
     * Constructs the select statement and calls the other loadBySelect to
     * execute the query and construct the objects
     * 
     * @param rpmScope
     * @param messageContext
     * @param where
     *            Where clause with ?
     * @param args
     *            Arguments that will replace the ? in the statement
     * @param tables
     * @param fieldList
     * 
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    protected final ArrayList loadByWhere(RPMObject parent,
            RPMObject[] rpmObjects, RPMObjectScope rpmScope,
            MessageContext messageContext, String where, Object[] args,
            String tables, String fieldList, String contextName, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(fieldList);
        stmt.appendFrom();
        stmt.append(tables);
        stmt.appendWhere(where);

        return loadBySelect(parent, rpmObjects, rpmScope, messageContext,
            stmt.toString(), args, contextName, isDelta);
    }

    protected Object loadWithSecurity(MessageContext msgContext, Connection c,
            String selectStmtStr, Object[] args,
            String contextName) throws SQLException, RPMException
    {
        PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(msgContext,
            c);
        PreparedStatement ps = preparedStatementHelper.createPreparedStatement(
            selectStmtStr, args, null, useDBStatementsForLOBS);
        ResultSet rs = ManagerUtil.executeSqlQuery(msgContext, ps,
            selectStmtStr, args);
        return rs;
    }

    protected void decodeSecurityRoleIds(MessageContext msgContext,
            ResultSet rs, String contextName) throws RPMException, SQLException
    {
        // By default, do nothing
        // Managers implementing security should override this method (ex:
        // WBSManager, ScopeManager)
    }


    /**
     * Loads an RPMObject list using the given <code>selectStatement</code>.
     * 
     * @param rpmScope
     * @param messageContext
     * @param selectStatement
     *            selectStmt clause with ?
     * @param args
     *            Arguments that will replace the ? in the statement
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    protected ArrayList loadBySelect(RPMObject parent, RPMObject[] rpmObjects,
            RPMObjectScope rpmScope, MessageContext messageContext,
            String selectStatement, Object[] args, String contextName,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        Connection c = null;
        ResultSet rs1 = null;
        ArrayList result = null;

        // stats
        long selectStartTime = 0;
        boolean registerSelectStats = StatsUtil.isRegisterSelectDuration(messageContext);
        if (registerSelectStats)
        {
            selectStartTime = System.currentTimeMillis();
        }

        try
        {
            c = Manager.getConnection(messageContext);
            selectStatement = Manager.makeStatementUncommitedRead(
                selectStatement, c);
           
            if (messageContext.isWebServicesSecurityEnabled())
            {
                rs1 = (ResultSet)loadWithSecurity(messageContext, c,
                    selectStatement, args, contextName);             
            }
            else
            {
                PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(messageContext,
                    c);
                PreparedStatement ps = preparedStatementHelper.createPreparedStatement(
                    selectStatement, args, null, useDBStatementsForLOBS);
                rs1 = ManagerUtil.executeSqlQuery(messageContext,ps,
                    selectStatement, args);
            }

            if (ArrayUtil.isEmpty(rpmObjects))
            {
                rpmObjects = new RPMObject[]
                {
                    null
                };
            }            
            
            result = loadByResultSet(parent, rpmObjects, c, rs1, rpmScope,
                messageContext, contextName, isDelta);
            
        }
        finally
        {
            Manager.close(messageContext, rs1);
            Manager.releaseConnection(messageContext, c);
        }

        if (registerSelectStats)
        {
            long selectEndTime = System.currentTimeMillis();
            StatsUtil.setSelectDuration(messageContext, selectEndTime
                    - selectStartTime);
        }

        return result;
    }

    private ArrayList loadByResultSet(RPMObject parent, RPMObject[] rpmObjects,
            Connection connection, ResultSet resultSet,
            RPMObjectScope rpmScope, MessageContext messageContext,
            String contextName, boolean isDelta) throws RPMException,
            SQLException
    {
        ArrayList result = null;
        ArrayList reloadedObjects = null;
        FieldValueMap relatedFields = null;
        try
        {
            result = new ArrayList();
            reloadedObjects = new ArrayList();

            if (rpmObjects != null)
            {
                Arrays.sort(rpmObjects, new RPMObjectComparator());
            }

            relatedFields = new FieldValueMap();

            // Load the object itself, without its related objects.
            int index = -1;
           
            while (resultSet.next())
            {
                try
                {
                    // Validate the primary key.
                    Object[] primaryKey = getPrimaryKey(resultSet);
                    for (int i = 0; i < primaryKey.length; i++)
                    {
                        if (primaryKey[i] == null)
                        {
                            String[] errorParams =
                            {
                                StringUtil.getShortClassName(this.getClass())
                            };
                            RPMException e = new RPMException(
                                ErrorCodes.AM_INVALID_PK, errorParams);
                            e.setSeverity(SeverityLevel.SevereError);
                            messageContext.addException(e);
                        }
                    }

                    // Get or create the container.
                    boolean existInCacheWithScope = containerExistInCacheWithScope(primaryKey,
                            messageContext, rpmScope);
                    
                    RPMObject object = matchContainer(rpmObjects, primaryKey,
                        resultSet, isDelta, messageContext, contextName, rpmScope);
                    
                    // Call decodeRow and decodeRelatedFields.
                    if (object != null && !object.getDeleted().booleanValue()
                    		&& !existInCacheWithScope)
                    {
                        index++;
                        if (!isDelta)
                        {
                            object.setParent(parent);
                        }

                        if (isDelta)
                        {
                            messageContext.getStatsOperation()
                                .incrementLoadedObjectCount();
                        }
                        else
                        {
                            messageContext.getStatsOperation()
                                .incrementLoadedObjectCount();
                        }

                        if (messageContext.isWebServicesSecurityEnabled())
                        {
                            // decode securityRole Ids
                            decodeSecurityRoleIds(messageContext, resultSet,
                                contextName);
                            if (isDelta)
                            {
                                // Restore null fields due to security
                                // if isDelta is true the it is a save operation,
                                // which means that the rpmObject
                                // passed to use here as a parameter is what the
                                // client passed to the web-services to save,
                                // which has the null filtered values -due to
                                // security- that need to be restored.
                                // if isDelta is false, or, if security is not even
                                // enabled, then there is nothing to do
                                prepareForDecodeRow(object, primaryKey, resultSet,
                                    messageContext, contextName, rpmScope);
                            }
                        }

                        object = decodeRow(resultSet, object, isDelta);
                        object.setContextName(contextName);
                        logger.debug("Loading: " + object.getClass().getName()
                                + " Id: " + object.getID());
                        relatedFields = decodeRelatedFields(resultSet,
                            relatedFields, index);

                        reloadedObjects.add(object);
                    }

                    // Add the object to the result.
                    if (object != null)
                    {
                        result.add(object);
                    }
                }
                catch (Exception x)
                {
                    messageContext.addException(x);
                }
            }            
        }
        finally
        {
            Manager.close(messageContext, resultSet);
            Manager.releaseConnection(messageContext, connection);
        }

        // Load the related objects.
        if (!reloadedObjects.isEmpty())
        {
            for (int i = 0; i < reloadedObjects.size(); i++)
            {
                try
                {
                    ScopeCheckpoint.getInstance().isValid(
                        (RPMObject) reloadedObjects.get(i), rpmScope);
                    loadRelatedObjects((RPMObject) reloadedObjects.get(i),
                        rpmScope, messageContext, relatedFields, i, isDelta);
                }
                catch (Exception x)
                {
                    messageContext.addException(x);
                }
            }
        }
        
        // Add newly created objects.
        for (int i = 0; i < rpmObjects.length; i++)
        {
            RPMObject currentObject = rpmObjects[i];
            if (currentObject != null
                    && getPrimaryKeyString(currentObject) == null)
            {
                result.add(currentObject);
            }
        }

        result = CollectionUtil.nullify(result);
        return result;
    }   
    

    /**
     * This method will load the objects related to the loaded RPMObject(parent,
     * children etc.) Override if the object can have a scope or if you need to
     * load children (ex: Assignments).
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        return rpmObject;
    }

    protected RPMObjectManager getManagerInstance(String className)
            throws RPMException
    {
        return RPMManagerFactory.getInstance().getRPMObjectManager(className);
    }

    protected RPMObjectManager getManagerInstance(Class containerClass)
            throws RPMException
    {
        return RPMManagerFactory.getInstance().getRPMObjectManager(
            containerClass);
    }

    /**
     * Transforms a ResultSet iterating on the table associated with the Manager
     * on an RPMObject pObject.
     * 
     * @param rs
     *            the ResultSet to be transformed
     * @param pObject
     *            object to be updated with the ResultSet
     * @param isDelta
     *            true if we're updating an object
     * @return pObject resulting RPMObject pObject
     * @throws SQLException
     * @throws ParseException
     * @throws RPMException
     */
    protected abstract RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException;

    /**
     * Override this method if you need fields in he current result set to load
     * the related objects (ex: parentId) Key of the value in the map will be
     * the index of the object concatenated with the name of the column
     * 
     * @param resultSet
     * @param relatedFields
     * @param index
     * @return
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        return relatedFields;
    }

    public final boolean internalValidateObjectsToCheckout(
            List objectsToCheckout, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (objectsToCheckout == null)
        {
            return true;
        }

        boolean isValid = true;
        for (Iterator iterator = objectsToCheckout.iterator(); iterator.hasNext();)
        {
            RPMObject objectToCheckout = (RPMObject) iterator.next();
            if (objectToCheckout != null)
            {
                if (objectToCheckout.getID() != null)
                {
                    RPMObjectManager manager = getManagerInstance(objectToCheckout.getClass());

                    String rpmObjectCKID = manager.getCheckInCheckOutPrimaryKey(
                        objectToCheckout, messageContext);
                    boolean checkedOutFromSession = messageContext.getFactory().getSessionCheckoutDAO().isCheckedOut(
                        rpmObjectCKID, messageContext);
                    boolean checkedOutFromRPM = manager.isCheckedOut(
                        messageContext, objectToCheckout);

                    isValid &= ManualCheckinCheckoutCheckpoint.isValidUpdateCheckout(
                        objectToCheckout, checkedOutFromSession,
                        checkedOutFromRPM, messageContext);

                    if (!isValid && checkedOutFromSession)
                    {
                        messageContext.getFactory().getSessionCheckoutDAO().removeSessionCheckoutsForRPMObject(
                            rpmObjectCKID, messageContext);
                    }
                }
            }
        }
        return isValid;
    }

    public final boolean internalValidateSave(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException
    {
        boolean rpmObjectValid = true;

        ValidateSerializer validateSerializer = new ValidateSerializer(
            rpmObject, rpmScope);
        messageContext.addObjectsToValidate(validateSerializer);

        for (Iterator iterator = validateSerializer.iterator(); iterator.hasNext();)
        {
            RPMObject objectToValidate = (RPMObject) iterator.next();

            if (objectToValidate != null)
            {
                boolean isValid = false;

                if (objectToValidate.getDeleted() != null
                        && objectToValidate.getDeleted().booleanValue())
                {
                    isValid = CheckpointCaller.isValidDelete(objectToValidate,
                        messageContext)
                            && CheckpointCaller.canDelete(rpmObject,
                                messageContext);
                    isValid &= ManualCheckinCheckoutCheckpoint
                        .isValidDeleteCheckout(objectToValidate, messageContext);
                }
                else
                {
                    if (objectToValidate.getID() == null)
                    {
                        // Design note: need to be called before isValid.
                        objectToValidate.setModified();
                    }

                    RPMObjectScope objectScope = validateSerializer.getScope(objectToValidate);

                    isValid = CheckpointCaller.isValidSave(objectToValidate,
                        objectScope, messageContext)
                            && CheckpointCaller.canSave(objectToValidate,
                                objectScope, messageContext);
                }

                if (!isValid)
                {
                    rpmObjectValid = false;
                }
            }
        }

        // reset objetsToValidate
        messageContext.addObjectsToValidate(null);
        return rpmObjectValid;
    }

    final public RPMObject save(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (messageContext == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.AM_INVALID_MC);
        }
        if (rpmObject == null)
        {
            return null;
        }

        RPMObject reloaded = null;
        // Verify that the scope matches the object
        if (ScopeCheckpoint.getInstance().isValid(rpmObject, rpmScope,
            messageContext))
        {
            // Preload
            if (rpmObject.getID() != null)
            {
                // stats
                long preloadStartTime = System.currentTimeMillis();

                messageContext.getCache().resetLoadedContainers();
                rpmObject = loadByPrimaryKey(rpmObject, rpmScope,
                    messageContext, true);

                long preloadEndTime = System.currentTimeMillis();
                StatsUtil.setPreloadDuration(messageContext, preloadEndTime
                        - preloadStartTime);

                // reset the cache before calling validation
                messageContext.getCache().resetLoadedContainers();
            }

            // Note: If rpmscope is of type RPMObjectScope, the object is saved
            // with a
            // null scope but is reloaded with the defined scope
            RPMObjectScope saveScope = null;
            if (rpmScope != null && rpmScope.getClass() != RPMObjectScope.class)
            {
                saveScope = rpmScope;
            }

            if (rpmObject != null
                    && internalValidateSave(rpmObject, saveScope,
                        messageContext))
            {
                try
                {
                    // pre-save validation
                    CheckoutSerializer ckSerializer = new CheckoutSerializer(
                        rpmObject, rpmScope);
                    if (!internalValidateObjectsToCheckout(
                        ckSerializer.getArrayList(), messageContext))
                    {
                        return null;
                    }

                    if (messageContext.isAutomaticCheckout())
                    {
                        // automatic checkout of all related objects to the save
                        messageContext.addObjectsToCheckout(ckSerializer.getArrayList());
                        internalCheckoutAll(messageContext);
                    }

                    reloaded = internalSave(rpmObject, saveScope,
                        messageContext);
                }
                catch (Exception e)
                {
                    throw RpmResult.translateException(e, rpmObject);
                }
                finally
                {

                    // automatic checkin of all objects
                    if (messageContext.isAutomaticCheckout())
                    {
                        internalCheckinAll(messageContext);
                        messageContext.addObjectsToCheckout(null);
                    }
                }

                // Reload graph (default behavior) if the save was successful
                if (reloaded != null && getPrimaryKeyString(reloaded) != null)
                {
                    ReloadType reloadType = messageContext.getReloadType();
                    if (reloadType == null
                            || reloadType == ReloadType.ReloadResult)
                    {
                        // Stats
                        long reloadStartTime = System.currentTimeMillis();

                        reloaded = load(rpmObject, rpmScope, messageContext);

                        long reloadEndTime = System.currentTimeMillis();
                        StatsUtil.setReloadDuration(messageContext,
                            reloadEndTime - reloadStartTime);
                    }
                }
            }
            messageContext.getCache().resetLoadedContainers();
        }
        return reloaded;
    }

    /**
     * Deletes, creates, saves and/or reloads the specified
     * <code>rpmObject</code>, as required. Deletes when
     * <code>rpmObject.getDeleted().booleanValue()</code> is true. Returns
     * null when <code>rpmObject</code> is not valid. Creates when
     * <code>rpmObject.getID() == null</code>. Saves <code>rpmObject</code>.
     * Reloads <code>rpmObject</code> if required by the ReloadType specified
     * in the <code>messageContext</code>.
     */
    public RPMObject internalSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (rpmObject != null)
        {
            if (rpmObject.getDeleted() != null
                    && rpmObject.getDeleted().booleanValue())
            {
                doDelete(rpmObject, messageContext);
                rpmObject = null;
            }
            else
            {
                rpmObject = doSave(rpmObject, rpmScope, messageContext);
            }
        }

        return rpmObject;
    }

    /**
     * Override this method to disable check-out/in on object creation. Design
     * note: check out/in might be required for some objects because checkin
     * implicitely propagates (i.e. rolls up) values to other objects. (This is
     * Steve's best guess up to now.) The other reason is to lock the object 
     * until the creation is completed.
     */
    protected boolean isCheckingOutOnCreate()
    {
        // Default is true.
        return true;
    }
    
    private RPMObject doSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (logger.isDebugEnabled())
        {
            logger.debug("\n========= About to save an object of type " + rpmObject.getClass().getName() + 
                " =========");
            Method[] objectMethods = rpmObject.getClass().getMethods();
            for(int i = 0; i < objectMethods.length; i++)
            {
                Method curMethod = objectMethods[i];
                String curMethodName = curMethod.getName();
                
                if(curMethodName.startsWith("is") || curMethodName.startsWith("get"))
                {
                    Object result = null;
                    try {
                        result = curMethod.invoke(rpmObject, null);
                    } catch(Exception e) { /* not handled. */ }
                    
                    if(result != null)
                    {
                        logger.debug(curMethodName + " : " + result);
                    }
                }
            }
            logger.debug("==========================================\n\n");
        }
        
        if (rpmObject.getID() == null)
        {
            // Design note: need to be called before isValid.
            rpmObject.setModified();
        }

        boolean isNewRpmObject = false;
        try
        {
            // CREATE, if required.
            if (rpmObject.getID() == null)
            {
                isNewRpmObject = true;
                // Create the object
                createObject(rpmObject, messageContext);

                // After creation we also need to check out in the session DB.
                if (ContainerCheckoutModeType.getContainerCheckoutModeType(
                    rpmObject).equals(ContainerCheckoutModeType.Self)
                    && isCheckingOutOnCreate())
                {
                    if (!isCheckedOut(messageContext, rpmObject))
                    {
                        checkoutObject(rpmObject, messageContext);
                    }

                    // SLegault: why is the following done even id object
                    // was already checked out. Please document.
                    String rpmObjectCKID = getCheckInCheckOutPrimaryKey(
                        rpmObject, messageContext);
                    messageContext.getFactory().getSessionCheckoutDAO().storeSessionCheckout(
                        rpmObjectCKID, rpmObject.getClass().getName(), false,
                        messageContext);

                    // SLegault: why is the following done even id object
                    // was already checked out. Please document.
                    if (messageContext.getObjectsToCheckout() != null)
                    {
                        messageContext.getObjectsToCheckout().add(rpmObject);
                    }
                }
            }
            
            if (isNewRpmObject)
            {
                if (!CheckpointCaller.canUpdateOnCreate(rpmObject, rpmScope,
                    messageContext))
                {
                    //we won't delete the object since the original code doesn't
                    //do that on exception (look at the catch clause below), so,
                    //we'll just throw an exception and let normal exception
                    //handling logic proceed
                    RPMException ex = SecurityControllerUtil
                        .extractSecurityException(messageContext);
                    if (ex == null)
                    {
                        //This case of (ex ==null) should never happen
                        ex = new RPMException("General Security Exception");
                    }
                    throw ex;
                }
            }

            // UPDATE
            rpmObject = updateObject(rpmObject, rpmScope, messageContext);

            // RELOAD CONTAINER
            ReloadType reloadType = messageContext.getReloadType();
            if (reloadType == ReloadType.SavedResult)
            {
                rpmObject = reloadWithoutScope(messageContext, rpmObject);
            }

            rpmObject.resetIsModified();
        }
        catch (RPMException e)
        {
            messageContext.addException(e);
            if (rpmObject.getID() != null
                    && isCheckedOut(messageContext, rpmObject))
            {
                undoCheckoutObject(rpmObject, messageContext);
                String rpmObjectCKID = getCheckInCheckOutPrimaryKey(rpmObject,
                    messageContext);
                messageContext.getFactory().getSessionCheckoutDAO().removeSessionCheckoutsForRPMObject(
                    rpmObjectCKID, messageContext);
            }
        }

        if (isNewRpmObject && !messageContext.isAutomaticCheckout()
                && isCheckingOutOnCreate()
                && isCheckedOut(messageContext, rpmObject))
        {
            internalCheckin(rpmObject, messageContext);
        }

        return rpmObject;
    }

    /**
     * Updates the children. Calls <code>internalSave</code> for all children
     * then rebuilds and returns the updated children array. Also, this method
     * ensures the parent-child relation is bidirectional for all original and
     * updated children. Restriction: all children must use the same manager.
     * 
     * Note: if you need a context to be propagated to children, call
     * propagateContextName() BEFORE calling this method
     * 
     * @param parent
     *            Parent of the children to be updated. May be null.
     * @param children
     *            Children to be updated. When null, children are obviously not
     *            updated.
     * @param childrenScope
     *            Scope of the children to be updated. When null, children are
     *            not updated.
     * @param messageContext
     *            Message context for the current <code>save</code>.
     * @param childrenBaseClass
     *            Common base class used to create the children array. Should be
     *            null except when children are not all of the same class (like
     *            a charge code category, for instance).
     * @return Updated children array. Deleted and null children are removed.
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateChildren(
     *      com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObject[],
     *      boolean, com.ibm.rpm.framework.MessageContext, Class)
     */
    protected RPMObject[] updateChildren(RPMObject parent,
            RPMObject[] children, RPMObjectScope childrenScope,
            MessageContext messageContext, Class childrenBaseClass)
            throws RPMException, SQLException, ParseException
    {
        if (childrenScope != null)
        {
            children = doUpdateChildren(parent, children, childrenScope,
                messageContext, childrenBaseClass);
        }
        return children;
    }

    /**
     * Updates the children. Calls <code>internalSave</code> with null scope
     * for all children then rebuilds and returns the updated children array.
     * Also, this method ensures the parent-child relation is bidirectional for
     * all original and updated children, except when the parent is a Module.
     * Restriction: all children must use the same manager.
     * 
     * @param parent
     *            Parent of the children to be updated. May be null.
     * @param children
     *            Children to be updated. When null, children are obviously not
     *            updated.
     * @param childrenAreInScope
     *            Scope of the children to be updated. When false, children are
     *            not updated.
     * @param messageContext
     *            Message context for the current <code>save</code>.
     * @param childrenBaseClass
     *            Common base class used to create the children array. Should be
     *            null except when children are not all of the same class (like
     *            a charge code category, for instance).
     * @return Updated children array. Deleted and null children are removed.
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateChildren(
     *      com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObject[],
     *      com.ibm.rpm.framework.RPMObjectScope,
     *      com.ibm.rpm.framework.MessageContext, Class)
     */
    protected RPMObject[] updateChildren(RPMObject parent,
            RPMObject[] children, boolean childrenAreInScope,
            MessageContext messageContext, Class childrenBaseClass)
            throws RPMException, SQLException, ParseException
    {
        if (childrenAreInScope)
        {
            children = doUpdateChildren(parent, children, null, messageContext,
                childrenBaseClass);
        }
        return children;
    }

    /**
     * Updates the children. Calls <code>internalSave</code> for all children
     * then rebuilds and returns the updated children array. Also, this method
     * ensures the parent-child relation is bidirectional for all original and
     * updated children. Restriction: all children must use the same manager.
     * 
     * @param parent
     *            Parent of the children to be updated. May be null.
     * @param children
     *            Children to be updated. When null, children are obviously not
     *            updated.
     * @param childrenScope
     *            Scope of the children to be updated.
     * @param messageContext
     *            Message context for the current <code>save</code>.
     * @param childrenBaseClass
     *            Common base class used to create the children array. Should be
     *            null except when children are not all of the same class.
     * @return Updated children array. Deleted and null children are removed.
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateChildren(
     *      com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObject[],
     *      com.ibm.rpm.framework.RPMObjectScope,
     *      com.ibm.rpm.framework.MessageContext, Class)
     */
    protected RPMObject[] doUpdateChildren(RPMObject parent,
            RPMObject[] children, RPMObjectScope childrenScope,
            MessageContext messageContext, Class childrenBaseClass)
            throws RPMException, SQLException, ParseException
    {
        // DESIGN NOTE: children manager is selected from the type of the
        // first non-null child.
        if (children != null)
        {
            // Update all children.
            RPMManagerFactory managerFactory = RPMManagerFactory.getInstance();
            RPMObjectManager childrenManager = null;
            ArrayList updatedChildrenList = new ArrayList();
            for (int i = 0; i < children.length; i++)
            {
                RPMObject originalChild = children[i];
                if (originalChild != null)
                {
                    // Select the right manager for children, if required.
                    if (childrenManager == null)
                    {
                        childrenManager = managerFactory.getRPMObjectManager(originalChild);
                    }
                    originalChild.setParent(parent);
                    RPMObject updatedChild = childrenManager.internalSave(
                        originalChild, childrenScope, messageContext);
                    if (updatedChild != null)
                    {
                        updatedChild.setParent(parent);
                        updatedChildrenList.add(updatedChild);
                    }
                }
            }

            // Convert the children list to a children array.
            int updatedChildrenCount = updatedChildrenList.size();
            if (updatedChildrenCount > 0)
            {
                // Get the children base class from the first child, if
                // required.
                if (childrenBaseClass == null)
                {
                    childrenBaseClass = updatedChildrenList.get(0).getClass();
                }
                children = (RPMObject[]) Array.newInstance(childrenBaseClass,
                    updatedChildrenCount);
                updatedChildrenList.toArray(children);
            }
        }
        return children;
    }

    /**
     * Updates the child. Calls <code>internalSave</code> for the child. Also,
     * this method ensures the parent-child relation is bidirectional for the
     * original and updated child.
     * 
     * @param parent
     *            Parent of the child to be updated.
     * @param child
     *            Child to be updated. When null, child is obviously not
     *            updated.
     * @param childrenScope
     *            Scope of the child to be updated. When null, child is not
     *            updated.
     * @param messageContext
     *            Message context for the current <code>save</code>.
     * @return Updated child. Null if deleted.
     */
    protected RPMObject updateChild(RPMObject parent, RPMObject child,
            RPMObjectScope childScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (childScope != null)
        {
            child = doUpdateChild(parent, child, childScope, messageContext);
        }
        return child;
    }

    /**
     * Updates the child. Calls <code>internalSave</code> with null scope for
     * the child. Also, this method ensures the parent-child relation is
     * bidirectional for the original and updated child.
     * 
     * @param parent
     *            Parent of the child to be updated.
     * @param child
     *            Child to be updated. When null, child is obviously not
     *            updated.
     * @param childIsInScope
     *            Scope of the child to be updated. When null, child is not
     *            updated.
     * @param messageContext
     *            Message context for the current <code>save</code>.
     * @return Updated child. Null if deleted.
     */
    protected RPMObject updateChild(RPMObject parent, RPMObject child,
            boolean childIsInScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (childIsInScope)
        {
            child = doUpdateChild(parent, child, null, messageContext);
        }
        return child;
    }

    private RPMObject doUpdateChild(RPMObject parent, RPMObject child,
            RPMObjectScope childScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (child != null)
        {
            RPMObjectManager childManager = RPMManagerFactory.getInstance().getRPMObjectManager(
                child);
            child.setParent(parent);
            child = childManager.internalSave(child, childScope, messageContext);
            if (child != null)
            {
                child.setParent(parent);
            }
        }
        return child;
    }

    /**
     * Propagates the context name of the parent to the children, where
     * 
     * @param parent
     *            Parent object.
     * @param children
     *            Children to which the parent's context name should be
     *            propagated.
     */
    protected void propagateContextName(RPMObject parent, RPMObject[] children)
    {
        if (children != null)
        {
            String parentContextName = parent.getContextName();
            for (int i = 0; i < children.length; i++)
            {
                RPMObject child = children[i];
                if (child != null)
                {
                    if (child.getContextName() == null)
                    {
                        children[i].setContextName(parentContextName);
                    }
                }
            }
        }
    }

    protected void setParent(ArrayList children, RPMObject parent)
    {
        if (children != null && !children.isEmpty())
        {
            for (int i = 0; i < children.size(); i++)
            {
                ((RPMObject) children.get(i)).setParent(parent);
            }
        }
    }

    public abstract RPMObject updateObject(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws SQLException, RPMException, ParseException;

    public abstract void createObject(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException;

    public final void delete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (CheckpointCaller.isValidDelete(rpmObject, messageContext)
                && CheckpointCaller.canDelete(rpmObject, messageContext)
                && ManualCheckinCheckoutCheckpoint.isValidDeleteCheckout(
                    rpmObject, messageContext))
        {
            try
            {
                doDelete(rpmObject, messageContext);
            }
            catch (RPMException e)
            {
                if (e.getErrorID() == ErrorCodes.RECORD_NOT_FOUND
                        || e.getErrorID() == ErrorCodes.SQL_ERROR_ROW_NOT_FOUND)
                {
                    ExceptionUtil.handleInvalidId(this,
                        StringUtil.getShortClassName(rpmObject.getClass()),
                        rpmObject.getID());
                }
                else
                {
                    throw e;
                }
            }
        }
    }

    protected abstract void doDelete(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException;

    protected final RPMObject matchContainer(RPMObject[] rpmObjects,
            Object[] primaryKey, ResultSet resultSet, boolean isDelta,
            MessageContext messageContext, String contextName,
            RPMObjectScope scope) throws RPMException, SQLException
    {
        RPMObject object = null;

        if (primaryKey == null || primaryKey[0] == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(this.getClass())
            };
            RPMException e = new RPMException(ErrorCodes.AM_INVALID_NULL_PK,
                errorParams);
            e.setSeverity(SeverityLevel.SevereError);
            messageContext.addException(e);
        }

        RPMObjectCache cache = messageContext.getCache();
        // Concatenation of the primarykey fields
        if (containerExistInCacheWithScope(primaryKey,
                messageContext, scope))
        {
        	object = cache.getContainerInCache(this, primaryKey);
        }
        else
        {
            if (isDelta)
            {
                String primaryKeyString = getPrimaryKeyString(primaryKey);
                int index = -1;
                if (rpmObjects != null)
                {
                    index = Arrays.binarySearch(rpmObjects, primaryKeyString,
                        new RPMStringComparator());
                }
                if (index >= 0)
                {
                    object = rpmObjects[index];
                    cache.addContainerInCache(this, primaryKey, object, scope);
                }
            }
            else
            {
                object = createContainer(primaryKey, resultSet,
                    messageContext, contextName, scope, true);//add it to cache
            }
        }

        return object;
    }

    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        // Default is to do nothing.
    }

    private void internalCheckinAll(MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (messageContext.getObjectsToCheckout() != null)
        {
            Iterator iter = messageContext.getObjectsToCheckout().iterator();
            while (iter.hasNext())
            {
                RPMObject rpmObject = (RPMObject) iter.next();
                internalCheckin(rpmObject, messageContext);
            }
        }
    }

    public void internalCheckin(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, SQLException
    {
        if (ContainerCheckoutModeType.getContainerCheckoutModeType(rpmObject).equals(
            ContainerCheckoutModeType.Self))
        {
            RPMObjectManager manager = getManagerInstance(rpmObject.getClass());

            if (rpmObject.getID() != null
                    && !rpmObject.getDeleted().booleanValue())
            {
                String rpmObjectCKID = manager.getCheckInCheckOutPrimaryKey(rpmObject,
                    messageContext);
                boolean checkedOutFromSession = messageContext.getFactory().getSessionCheckoutDAO().isCheckedOut(
                    rpmObjectCKID, messageContext);

                boolean checkedOutFromRPM = manager.isCheckedOut(
                    messageContext, rpmObject);

                if (ManualCheckinCheckoutCheckpoint.isValidInternalCheckin(
                    rpmObject, checkedOutFromSession, checkedOutFromRPM,
                    messageContext))
                {
                    manager.checkinObject(rpmObject, messageContext);
                    messageContext.getFactory().getSessionCheckoutDAO().removeSessionCheckoutsForRPMObject(
                        rpmObjectCKID, messageContext);
                }
            }
        }
    }

    /**
     * Checkin the rpmObject without validating if it has been checked out by
     * the delphi client or the ws-api
     * 
     * @param rpmObject
     * @param messageContext
     * @throws RPMException
     * @throws SQLException
     */
    public void forceCheckin(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (ContainerCheckoutModeType.getContainerCheckoutModeType(rpmObject).equals(
            ContainerCheckoutModeType.Self))
        {
            RPMObjectManager manager = getManagerInstance(rpmObject.getClass());
            if (rpmObject.getID() != null
                    && !rpmObject.getDeleted().booleanValue()
                    && manager.isCheckedOut(messageContext, rpmObject))
            {
                manager.checkinObject(rpmObject, messageContext);
                String rpmObjectCKID = manager.getCheckInCheckOutPrimaryKey(rpmObject,
                    messageContext);
                messageContext.getFactory().getSessionCheckoutDAO().removeSessionCheckoutsForRPMObject(
                    rpmObjectCKID, messageContext);
            }
        }
    }

    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        // Default is to do nothing.
    }

    public void forceCheckinOnSessionsCheckOut(String targetSessionID,
            MessageContext messageContext) throws RPMException, SQLException
    {
        if (ForceCheckinCheckpoint.isForceCheckinValidForSession(
            "forceCheckinOnSessionsCheckOut", targetSessionID, messageContext))
        {
            ArrayList rpmObjects = messageContext.getFactory().getSessionCheckoutDAO().retrieveSessionCheckedOuts(
                targetSessionID, messageContext);
            Iterator iterator = rpmObjects.iterator();

            while (iterator.hasNext())
            {
                RPMObject rpmObject = (RPMObject) iterator.next();
                forceCheckin(rpmObject, messageContext);
            }
        }
    }

    public void forceCheckinOnUsersCheckOut(Resource user,
            MessageContext messageContext) throws RPMException, SQLException
    {
        if (ForceCheckinCheckpoint.isForceCheckinValidForUser(
            "forceCheckinOnUsersCheckOut", user, messageContext))
        {
            ArrayList rpmObjects = messageContext.getFactory().getSessionCheckoutDAO().retrieveUserCheckedOut(
                user, messageContext);
            Iterator iterator = rpmObjects.iterator();

            while (iterator.hasNext())
            {
                RPMObject rpmObject = (RPMObject) iterator.next();
                forceCheckin(rpmObject, messageContext);
            }
        }
    }

    public void forceCheckinOnRPMObjectCheckOut(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, SQLException
    {
        RPMObjectManager manager = getManagerInstance(rpmObject.getClass());
        manager.isCheckedOutBy(messageContext, rpmObject);
        String rpmObjectCKID = manager.getCheckInCheckOutPrimaryKey(rpmObject, messageContext);
        if (ForceCheckinCheckpoint.isForceCheckinValidForRPMObject(
            "forceCheckinOnRPMObjecCheckOut", rpmObjectCKID, messageContext))
        {
            forceCheckin(rpmObject, messageContext);
        }
    }

    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        // Default is to do nothing.

    }

    public boolean isCheckedOutBy(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException, SQLException
    {
        if (!ContainerCheckoutModeType.getContainerCheckoutModeType(rpmObject).equals(
            ContainerCheckoutModeType.Self)
                || rpmObject.getID() == null)
        {
            return false;
        }

        RPMObjectManager manager = getManagerInstance(rpmObject.getClass());
        if (!manager.isCheckedOut(messageContext, rpmObject))
        {
            // resetting values
            messageContext.setCheckOutRecDateTime(null);
            messageContext.setCheckOutRecUser(null);
            return false;
        }
        return true;
    }

    public boolean isCheckedOut(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException, SQLException
    {
        // TODO Implement isCheckedOut
        return false;
    }

    private void internalCheckoutAll(MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (messageContext.getObjectsToCheckout() != null)
        {
            Iterator iter = messageContext.getObjectsToCheckout().iterator();
            while (iter.hasNext())
            {
                RPMObject rpmObject = (RPMObject) iter.next();
                internalCheckout(rpmObject, false, messageContext);
            }
        }
    }

    public void internalCheckout(RPMObject rpmObject, boolean isTimeless,
            MessageContext messageContext) throws RPMException, SQLException
    {
        if (ContainerCheckoutModeType.getContainerCheckoutModeType(rpmObject).equals(
            ContainerCheckoutModeType.Self)
                && rpmObject.getID() != null)
        {
            RPMObjectManager manager = getManagerInstance(rpmObject.getClass());

            String rpmObjectCKID = manager.getCheckInCheckOutPrimaryKey(rpmObject, messageContext);
            if (messageContext.isAutomaticCheckout())
            {
                if (!manager.isCheckedOut(messageContext, rpmObject)
                        && ManualCheckinCheckoutCheckpoint.canCheckOut(
                            messageContext, rpmObject, isTimeless))
                {
                    manager.checkoutObject(rpmObject, messageContext);
                    messageContext.getFactory().getSessionCheckoutDAO()
                        .storeSessionCheckout(rpmObjectCKID,
                            rpmObject.getClass().getName(), isTimeless,
                            messageContext);
                }
                else
                {
                    // if is checkedOut from SessionCheckoutDAO, it means that
                    // in a previous session the rpmObject was checked out
                    // with timeless flag on.
                    if (!messageContext.getFactory().getSessionCheckoutDAO().isCheckedOut(
                        rpmObjectCKID, messageContext))
                    {
                        ExceptionUtil.handleRPMException(
                            this,
                            ErrorCodes.RPMOBJECT_CHECKED_OUT,
                            new String[]
                            {
                                    StringUtil.getShortClassName(rpmObject.getClass()),
                                    rpmObject.getID(),
                                    messageContext.getCheckOutRecUser().getFullName(),
                                    "checkout"
                            });
                    }
                }
            }
            else
            {
                boolean checkedOutFromSession = messageContext.getFactory().getSessionCheckoutDAO().isCheckedOut(
                    rpmObjectCKID, messageContext);

                boolean checkedOutFromRPM = manager.isCheckedOut(
                    messageContext, rpmObject);

                if (ManualCheckinCheckoutCheckpoint.isValidInternalCheckout(
                    rpmObject, checkedOutFromSession, checkedOutFromRPM,
                    messageContext)
                        && ManualCheckinCheckoutCheckpoint.canCheckOut(
                            messageContext, rpmObject, isTimeless))
                {
                    manager.checkoutObject(rpmObject, messageContext);
                    if (!checkedOutFromSession)
                    {
                        messageContext.getFactory().getSessionCheckoutDAO()
                            .storeSessionCheckout(rpmObjectCKID,
                                rpmObject.getClass().getName(), isTimeless,
                                messageContext);
                    }
                }
            }
        }
    }

    public void isValidPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        boolean valid = false;

        if (getPrimaryKeyString(rpmObject) != null)
        {
            valid = true;
        }
        else
        {
            String[] primaryKeyArray = getPrimaryKeyArray(rpmObject);
            if (primaryKeyArray == null || primaryKeyArray.length != 1)
            {
                // This method must be overriden by the manager
                String[] errorParams =
                {
                    StringUtil.getShortClassName(this.getClass())
                };
                RPMException e = new RPMException(
                    ErrorCodes.OVERRIDE_ISVALIDPK, errorParams);
                e.setSeverity(SeverityLevel.SevereError);
                messageContext.addException(e);
            }
        }

        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject, "ID");
        }
    }   

    public void fillFieldsWithDefaultValues(RPMObject objectToFill,
            Container classInformation) throws RPMException
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();

            if (fieldInfo.getDefaultValue() != null
                    && (!fieldInfo.canBeNullValue()))
            {
                // get the current value
                Method methodToInvoke = fieldInfo.getGetterMethod();
                Object currValue = null;
                try
                {
                    currValue = methodToInvoke.invoke(objectToFill,
                        new Object[0]);
                }
                catch (Exception e)
                {
                    throw new RPMException("Failed to get field: "
                            + fieldInfo.getName() + " value: ", e);
                }

                if (currValue == null)
                {
                    fillField(objectToFill, fieldInfo,
                        fieldInfo.getTypedDefaultValue(),
                        fieldInfo.getSetterMethod());
                }
            }
        }
    }

    public void fillField(RPMObject objectToFill, Field fieldInfo,
            Object value, Method methodToCall) throws RPMException
    {
        Object[] args = new Object[1];
        args[0] = value;

        try
        {
            methodToCall.invoke(objectToFill, args);
        }
        catch (Exception e)
        {
            throw new RPMException("Failed to assign field: "
                    + fieldInfo.getName() + ", with value: " + value, e);
        }
    }
    
    
    //This function restores rpmObject's fields that were filtered out by security 
    //to their original value before saving so that the delta operation will not
    //set field_Xxx_isMoified to true when comparing rpmObject's fields values with
    //those we loaded from the DB
    //XXX IMPORTANT: MUST ONLY BE CALLED IF isDelta is true
    private void prepareForDecodeRow(RPMObject rpmObject, Object[] primaryKey,
            ResultSet resultSet,
            MessageContext messageContext, String contextName,
            RPMObjectScope rpmScope) 
    {
        RestoreRelevantFilteredFieldsParameter param = new RestoreRelevantFilteredFieldsParameter(
            this, rpmObject, primaryKey, resultSet, messageContext,
            contextName, rpmScope);        
        CheckpointCaller.restoreRelevantFilteredFields(param);
    }
}