package com.ibm.rpm.framework;

import java.lang.reflect.Field;
import java.util.AbstractMap;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.util.ExceptionUtil;

/**
 * Contains a type id to container class map and a container class to type ids
 * map.
 * 
 * @author legaults
 */
public class ContainerMap
{
    private static Log logger = LogFactory.getLog(ContainerMap.class);

    /**
     * Class to type id map. Key is the <code>Class</code>. Value is the
     * <code>Set</code> of <code>Integer</code> type ids.
     */
    private AbstractMap classToTypeIds;

    /**
     * Type id to class name map. Key is the Integer type id. Value is the fully
     * qualified, mixed case class name.
     */
    private AbstractMap typeIdToClass;

    private Class managerClass;

    // ==============
    // INITIALIZATION
    // ==============

    public ContainerMap(Class managerClass)
    {
        if (managerClass == null)
        {
            logger
                    .error("ContainerMap(Class managerClass). managerClass is null.");
        }
        typeIdToClass = new HashMap();
        classToTypeIds = new HashMap();
        this.managerClass = managerClass;
    }

    public ContainerMap(Class managerClass, Class[] containerClasses)
    {
        this(managerClass);
        add(containerClasses);
    }

    /**
     * Adds the given <code>containerClass</code> to: (1) the
     * container-class-to-manager-class mapping, (2) the
     * type-id-to-container-class mapping, (3) the container-class-to-type-id
     * mapping. This <code>containerClass.TYPE_ID</code> will be extracted
     * automatically if it exists. <code>containerClass</code> typeIds set
     * will automatically include the TYPE_ID of all its derived classes, if
     * any.
     * 
     * @param containerClass
     *            Class to be added to the mappings.
     */
    public void add(Class containerClass)
    {
        doAdd(containerClass, false);
    }

    public void add(Class[] containerClasses)
    {
        for (int i = 0; i < containerClasses.length; i ++)
        {
            doAdd(containerClasses[i], false);
        }
    }

    /**
     * This method has the same behavior as add(Class containerClass) except
     * that it is optimized to NOT ADD THE TYPE_ID FILTER in the get*Condition.
     * Use it only if you know what you are doing.
     * <p>
     * Adds the given <code>containerClass</code> to: (1) the
     * container-class-to-manager-class mapping, (2) the
     * type-id-to-container-class mapping. This
     * <code>containerClass.TYPE_ID</code> will be extracted automatically if
     * it exists. <code>containerClass</code> typeIds set will be empty. This
     * means that there will be no type id filtering for XPath queries using
     * this container.
     * 
     * @param containerClass
     *            Class to be added to the mappings.
     */
    public void addRoot(Class containerClass)
    {
        doAdd(containerClass, true);
    }

    private void doAdd(Class containerClass, boolean isRoot)
    {
        // Container-class-to-manager-class.
        RPMManagerFactory.getInstance().addContainerToMap(containerClass,
                managerClass);

        // Type-id-to-container-class.
        Integer thisTypeId = getThisTypeId(containerClass);
        if (thisTypeId != null)
        {
            addTypeIdToClass(thisTypeId, containerClass);
        }

        // Container-class-to-type-id.
        updateOtherClassToTypeIds(containerClass, thisTypeId);
        // Skip when generic (no TYPE_ID) and top of its class hierarchy
        // (superclass is RPMObject).
        if (!isRoot)
        {
            addClassToTypeIds(containerClass, thisTypeId);
        }
    }

    /**
     * Adds <code>extraTypeIds</code> for the given
     * <code>containerClass</code>. All these extra type ids map to the same
     * container class. This is a very rarely used method. It was required for
     * WbsManager and ScopeManager.
     * 
     * @param containerClass
     *            Container class.
     * @param extraTypeIds
     *            Type identifiers.
     */
    public void addExtraTypeIds(Class containerClass, int[] extraTypeIds)
    {
        addExtraTypeIds(containerClass, intArrayToSet(extraTypeIds));
    }

    /**
     * Adds <code>extraTypeIds</code> for the given
     * <code>containerClass</code>. All these extra type ids map to the same
     * container class. This is a very rarely used method. It was required for
     * WbsManager and ScopeManager.
     * 
     * @param containerClass
     *            Container class.
     * @param extraTypeIds
     *            Type identifiers.
     */
    public void addExtraTypeIds(Class containerClass, Collection extraTypeIds)
    {
        if (extraTypeIds != null)
        {
            addTypeIdsToClass(extraTypeIds, containerClass);
            addClassToTypeIds(containerClass, extraTypeIds);
            updateOtherClassToTypeIds(containerClass, extraTypeIds);
        }
    }

    private void addClassToTypeIds(Class containerClass, Collection typeIds)
    {
        Set typeIdSet = (Set) classToTypeIds.get(containerClass);
        if (typeIdSet == null)
        {
            if (typeIds instanceof Set)
            {
                typeIdSet = (Set) typeIds;
            }
            else
            {
                typeIdSet = new HashSet(typeIds);
            }
        }
        else
        {
            typeIdSet.addAll(typeIds);
        }
        classToTypeIds.put(containerClass, typeIdSet);
    }

    private void addTypeIdsToClass(Collection typeIds, Class containerClass)
    {
        Iterator i = typeIds.iterator();
        while (i.hasNext())
        {
            Integer typeId = (Integer) i.next();
            addTypeIdToClass(typeId, containerClass);
        }
    }

    private void addTypeIdToClass(Integer typeId, Class containerClass)
    {
        Class otherClass = (Class) typeIdToClass.get(typeId);
        if (otherClass != null)
        {
            logger.error("Attempting to map type id " + typeId + " to class "
                    + containerClass + ". It is already mapped to class "
                    + otherClass + '.');
        }
        typeIdToClass.put(typeId, containerClass);
    }

    /**
     * Returns the value of the <code>containerClass.TYPE_ID</code>.
     * 
     * @param containerClass
     *            Container <code>Class</code>.
     * @return Type id. Nullable.
     */
    private Integer getThisTypeId(Class containerClass)
    {
        Integer typeId = null;
        try
        {
            Field typeIdField = containerClass.getDeclaredField("TYPE_ID");
            typeId = (Integer) typeIdField.get(containerClass);
        }
        catch (Exception e)
        {
            // Ignore all exceptions.
        }
        return typeId;
    }

    /**
     * Updates other classes mappings, if required.
     * 
     * @param newClass
     *            Newly added class.
     * @param newTypeId
     *            Type id of newClass. Nullable.
     */
    private void updateOtherClassToTypeIds(Class newClass, Integer newTypeId)
    {
        if (newTypeId != null)
        {
            Collection newTypeIds = new HashSet();
            newTypeIds.add(newTypeId);
            updateOtherClassToTypeIds(newClass, newTypeIds);
        }
    }

    /**
     * Updates other classes mappings, if required.
     * 
     * @param newClass
     *            Class that matches the extra type ids.
     * @param newTypeId
     *            Type ids of newClass. Nullable.
     */
    private void updateOtherClassToTypeIds(Class newClass, Collection newTypeIds)
    {
        if (newTypeIds != null)
        {
            Iterator i = classToTypeIds.keySet().iterator();
            while (i.hasNext())
            {
                Class otherClass = (Class) i.next();
                // When otherClass is superclass of newClass.
                if (otherClass.isAssignableFrom(newClass))
                {
                    getTypeIds(otherClass).addAll(newTypeIds);
                }
            }
        }
    }

    /**
     * Adds the <code>newClass</code> mapping, including all its derived
     * classes type ids.
     * 
     * @param newClass
     *            Newly added class.
     * @param newTypeId
     *            Type id of newClass. Nullable.
     */
    private void addClassToTypeIds(Class newClass, Integer newTypeId)
    {
        // Sets this new class' mapping, if required.
        Set newTypeIds = new HashSet();
        if (newTypeId != null)
        {
            newTypeIds.add(newTypeId);
        }
        Iterator i = classToTypeIds.keySet().iterator();
        while (i.hasNext())
        {
            Class otherClass = (Class) i.next();
            // When newClass is superclass of otherClass.
            if (newClass.isAssignableFrom(otherClass))
            {
                newTypeIds.addAll(getTypeIds(otherClass));
            }
        }
        addClassToTypeIds(newClass, newTypeIds);
    }

    private static Set intArrayToSet(int[] typeIds)
    {
        Set typeIdCollection = null;
        if (typeIds != null)
        {
            typeIdCollection = new HashSet();
            for (int i = 0; i < typeIds.length; i++)
            {
                typeIdCollection.add(new Integer(typeIds[i]));
            }
        }
        return typeIdCollection;
    }

    // =======
    // MAPPING
    // =======

    /**
     * Returns the set of type identifiers that are assignment compatible with
     * the given container. The result is typically used to filter the right
     * type subset in a SELECT statement.
     * 
     * @param containerClass
     *            Class of the container.
     * @return Set of type identifiers.
     * @see com.ibm.rpm.framework.util.SqlBuffer#appendIn(String, Collection)
     */
    public Set getTypeIds(Class containerClass)
    {
        return (Set) classToTypeIds.get(containerClass);
    }

    public Integer getTypeId(Class containerClass)
    {
        Set typeIds = getTypeIds(containerClass);
        if (typeIds == null || typeIds.size() != 1)
        {
            throw new RuntimeException("The container '" + containerClass.getName()
                    + "' does not match only one type id.");
        }
        return (Integer) typeIds.iterator().next();
    }

    /**
     * Returns the container class that matches the given type id. The result is
     * typically used to create the correct container instance.
     * 
     * @param typeId
     *            Type identifier.
     * @return Matching container class.
     */
    public Class getContainerClass(Integer typeId)
    {
        return (Class) typeIdToClass.get(typeId);
    }

    /**
     * Creates the container that matches the given <code>typeId<code>.
     * @param typeId Type identifier.
     * @return A new instance of the matching container.
     */
    public RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;
        try
        {
            Integer typeIdObject = new Integer(typeId);
            Class containerClass = getContainerClass(typeIdObject);
            object = (RPMObject) containerClass.newInstance();
            object.assignTypeID(typeIdObject);
        }
        catch (Exception x)
        {
            ExceptionUtil.handleUnsupportedTypeId(managerClass, typeId);
        }
        return object;
    }
}
