package com.ibm.rpm.framework;

import java.util.ArrayList;

import com.ibm.rpm.framework.util.SqlBuffer;

/**
 * Encapsulates an SQL "WHERE" condition. Note that new constructors and
 * SqlBuffer behavior were added 2006-01-23.
 * 
 * @author Boris, Simon Legault
 */
public class JoinCondition extends SqlBuffer
{
    private String tableName = null;
    
    private String operator = null;

    private ArrayList conditions = new ArrayList();

    /**
     * Creates an empty <code>JoinCondition</code>.
     */
    public JoinCondition()
    {
        super();
    }

    /**
     * Creates an empty <code>JoinCondition</code> with the given
     * <code>tableName</code>.
     * 
     * @param tableName
     *            Name of the database table.
     */
    public JoinCondition(String tableName)
    {
        super();
        setTableName(tableName);
    }

    /**
     * Creates a <code>JoinCondition</code> with the given
     * <code>tableName</code> and <code>initialCondition</code>.
     * 
     * @param tableName
     *            Name of the database table. May be <code>null</code>.
     * @param initialCondition
     *            Initial condition.
     */
    public JoinCondition(String tableName, String initialCondition)
    {
        super(initialCondition);
        setTableName(tableName);
    }

    /**
     * Creates a <code>JoinCondition</code> with the given
     * <code>tableName</code> and <code>initialCondition</code>.
     * 
     * @param tableName
     *            Name of the database table. May be <code>null</code>.
     * @param initialCondition
     *            Initial condition.
     */
    public JoinCondition(String tableName, SqlBuffer initialCondition)
    {
        super(initialCondition);
        setTableName(tableName);
    }

    /**
     * Creates a <code>JoinCondition</code> and appends a sub-select
     * statement.
     * <code>[" AND " +] foreignField + "IN (SELECT " + localColumnName + " FROM " + tableName [+ " WHERE " + condition]</code>.
     * <code>" AND "</code> is appended only if required.
     * <code>" WHERE " + condition</code> is appended only if condition is not
     * empty. This is only a shortcut for a very common case.
     * 
     * @see com.ibm.rpm.framework.util.SqlBuffer#appendSubSelect(String, String,
     *      String, String)
     */
    public static JoinCondition createSubSelect(String foreignField,
            String localColumnName, String myTableName, String condition)
    {
        JoinCondition instance = new JoinCondition();
        instance.appendSubSelect(foreignField, localColumnName, myTableName,
                condition);
        return instance;
    }

    /**
     * Sets this JoinCondition with the given <code>newCondition</code>. Keep
     * this method for backward compatibility.
     * 
     * @param newCondition
     * @see com.ibm.rpm.framework.util.SqlBuffer#SqlBuffer(String)
     */
    public void setCondition(String newCondition)
    {
        empty();
        append(newCondition);
    }

    /**
     * Returns the textual portion of this JoinCondition. Keep this method for
     * backward compatibility. Use toString() instead.
     * 
     * @see com.ibm.rpm.framework.util.SqlBuffer#toString()
     */
    public String getCondition()
    {
        // Simon Legault, 2006-01-20.
        // If my new extension breaks everything, uncomment the
        // following lines for complete backward compatibility.
        // If not, eventually delete this comment. Thanks.
        //if (StringUtil.isEmpty(buffer))
        //    return null;
        return toString();
    }

    public void setTableName(String table)
    {
        tableName = table;
    }

    public String getTableName()
    {
        return tableName;
    }

    // TODO: add a merge method that merges two JoinCondition instances.
    /**
     * @return Returns the operator.
     */
    public String getOperator()
    {
        return operator;
    }
    /**
     * @param operator The operator to set.
     */
    public void setOperator(String operator)
    {
        this.operator = operator;
    }
    /**
     * @return Returns the conditions.
     */
    public ArrayList getConditions()
    {
        return conditions;
    }
    /**
     * @param conditions The conditions to set.
     */
    public void setConditions(ArrayList conditions)
    {
        this.conditions = conditions;
    }
}
