package com.ibm.rpm.framework;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.security.ContainerSecurityDescriptor;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.MetadataRulesUtility;
import com.ibm.rpm.framework.util.ValidateSerializer;
import com.ibm.rpm.framework.util.stats.Operation;
import com.ibm.rpm.interfaces.SessionUtility;
import com.ibm.rpm.interfaces.impl.ServerFactory;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author Genevieve
 */
public class MessageContext extends RpmResult
{
    // Design note: Merging MessageContext and RpmResult allows
    // the API to result partial results and all exceptions
    // to date, even when the operation is interrupted.
    // The final decision of what is returned can be taken
    // when RpmResult is converted to Result*.

    // Design note: MessageContext could delegate to RpmResult
    // rather than derive from it. If ever required.

    private static final String CURRENT_USER = "CURRENT_USER";

    private static final String RELOAD_TYPE = "RELOAD_TYPE";

    private static final String DATASOURCE = "DATASOURCE";

    private static final String NB_CALL_RETRY = "NB_CALL_RETRY";

    private static final String STATS = "STATS";

    private static final String STATS_SELECT_DURATION = "SELECT_DURATION";

    private static final String VALIDATE = "VALIDATE";

    private static final String CHECKOUTS = "CHECKOUTS";

    private Map messageContextObjects = new HashMap();

    private static Log logger = LogFactory.getLog(MessageContext.class);

    private static Map registeredMessageContexts = Collections
        .synchronizedMap(new WeakHashMap());

    private String currentSpCall = null;

    private long currentSpTime = 0;

    private RPMObject template = null;

    private Resource checkOutRecUser = new Resource();

    private Calendar checkOutRecDateTime = null;

    private boolean automaticCheckout = true;

    private String sessionID = null;

    private String workflowId;

    private LoadParams loadParams;

    private ServerFactory factory;

    private RPMObjectCache objectCache;

    private Connection connection;

    private HashMap hmStatementsByConnections = new HashMap();

    public MessageContext(ServerFactory factoryToUse)
    {
        super();
        factory = factoryToUse;

        // generate a null pointer exception if this is wrong
        factory.getClass();
    }

    public MessageContext(ServerFactory factoryToUse, String sessionId,
            String operationName) throws RPMException
    {
        this(factoryToUse);
        setSession(getSessionUtility(), sessionId, operationName);
    }

    public ServerFactory getFactory()
    {
        return factory;
    }

    private SessionUtility getSessionUtility()
    {
        return factory.getSession();
    }

    public MessageContext(ServerFactory factoryToUse, String sessionId,
            boolean fillContext, String operationName) throws RPMException
    {
        this(factoryToUse);
        if (fillContext)
        {
            setSession(getSessionUtility(), sessionId, operationName);
        }
        else
        {
            this.sessionID = sessionId;
        }
    }

    public void setSession(SessionUtility sessionUtility, String sessionID,
            String operationName) throws RPMException
    {
        registerMessageContext();
        sessionUtility.fillContext(this, sessionID, operationName);
        this.sessionID = sessionID;
    }

    public String getSession()
    {
        return sessionID;
    }

    private Map getMap()
    {
        if (messageContextObjects == null)
        {
            messageContextObjects = new HashMap();
        }
        return messageContextObjects;
    }

    private void put(String key, Object object)
    {
        getMap().put(key, object);
    }

    private Object get(String key)
    {
        return getMap().get(key);
    }

    //CURRENT USER

    public void addUser(RPMObject resource) throws RPMException
    {

        //Cannot refer directly to the class because it is outside the
        // framework package
        String type = "com.ibm.rpm.resource.containers.Resource";

        if (resource != null && resource.getClass().getName().equals(type))
        {
            put(CURRENT_USER, resource);
        }
        else
        {
            String[] params = new String[]
            {
                    this.getClass().getName(), type
            };
            ExceptionUtil.createAndHandleRPMException(this.getClass(),
                ErrorCodes.INVALID_USER_TYPE, params);
        }
    }

    public RPMObject getUser()
    {
        return (RPMObject) get(CURRENT_USER);
    }

    //DATASOURCE

    public void addDatasource(DataSource datasource)
    {
        put(DATASOURCE, datasource);
    }

    public DataSource getDatasource()
    {
        return (DataSource) get(DATASOURCE);
    }

    //RELOAD TYPE

    public void addReloadType(ReloadType reloadType)
    {
        // Sets the default reloadType.
        if (reloadType == null)
        {
            reloadType = ReloadType.ReloadResult;
        }
        put(RELOAD_TYPE, reloadType);
    }

    public ReloadType getReloadType()
    {
        return (ReloadType) get(RELOAD_TYPE);
    }

    //CACHE

    public RPMObjectCache getCache()
    {
        if (objectCache == null)
        {
            objectCache = new RPMObjectCache();
        }
        return objectCache;
    }

    // TODO: Make private?
    public void registerMessageContext()
    {
        registeredMessageContexts.put(this, null);
    }

    //SP Logging

    public void addCurrentSPCall(String spCall)
    {

        currentSpCall = spCall;
        currentSpTime = System.currentTimeMillis();
    }

    public void removeCurrentSPCall()
    {

        currentSpCall = null;
        currentSpTime = 0;
    }

    public static void dumpStoredProcedureStack()
    {
        StringBuffer log = new StringBuffer(
            "\n*** begin dumpStoredProcedureStack ***\n");

        long currentTime = System.currentTimeMillis();

        int size = registeredMessageContexts.size();
        MessageContext[] contexts = (MessageContext[]) registeredMessageContexts
            .keySet().toArray(new MessageContext[size]);

        if (contexts != null)
        {
            for (int i = 0; i < contexts.length; i++)
            {
                MessageContext context = contexts[i];
                if (context != null)
                {
                    String sp = context.currentSpCall;
                    if (sp != null)
                    {
                        long waitingTime = currentTime - context.currentSpTime;
                        log.append(" waiting time: " + waitingTime + " ms - "
                                + sp + "\n");
                    }
                }
            }
        }
        log.append(" *** ends dumpStoredProcedureStack *** \n");
        logger.error(log);
    }

    //STATS

    public void addStatsOperation(Operation operation)
    {
        put(STATS, operation);
    }

    public Operation getStatsOperation()
    {
        Operation stats = (Operation) get(STATS);
        if (stats == null)
        {
            stats = new Operation();
            addStatsOperation(new Operation());
        }
        return stats;
    }

    public void addSelectDurationFlag()
    {
        put(STATS_SELECT_DURATION, STATS_SELECT_DURATION);
    }

    public String getSelectDurationFlag()
    {
        return (String) get(STATS_SELECT_DURATION);
    }

    //ROLLBACK
    public void addNbCallRetry(Integer value)
    {
        put(NB_CALL_RETRY, value);
    }

    public Integer getNbCallRetry()
    {
        return (Integer) get(NB_CALL_RETRY);
    }

    // =====================
    // ROOT OBJECT AND SCOPE
    // =====================

    private RPMObject rootObject;

    private RPMObjectScope rootScope;

    public boolean isRootObject(RPMObject otherObject)
    {
        return ManagerUtil.areSame(rootObject, otherObject);
    }

    public RPMObject getRootObject()
    {
        return rootObject;
    }

    public void setRootObject(RPMObject rootObject)
    {
        this.rootObject = rootObject;
    }

    public boolean isRootScope(RPMObjectScope otherScope)
    {
        return ManagerUtil.areSame(rootScope, otherScope);
    }

    public RPMObjectScope getRootScope()
    {
        return rootScope;
    }

    public void setRootScope(RPMObjectScope rootScope)
    {
        this.rootScope = rootScope;
    }

    //Validation

    void addObjectsToValidate(ValidateSerializer validateSerializer)
    {
        put(VALIDATE, validateSerializer);
    }

    public ValidateSerializer getObjectsToValidate()
    {
        return (ValidateSerializer) get(VALIDATE);
    }

    public void addObjectsToCheckout(List checkouts)
    {
        put(CHECKOUTS, checkouts);
    }

    public List getObjectsToCheckout()
    {
        return (List) get(CHECKOUTS);
    }

    public void setTemplate(RPMObject template)
    {
        this.template = template;
    }

    public RPMObject getTemplate()
    {
        return template;
    }

    public void setCheckOutRecUser(Resource checkOutRecUser)
    {
        this.checkOutRecUser = checkOutRecUser;
    }

    public Resource getCheckOutRecUser()
    {
        return checkOutRecUser;
    }

    public boolean isAutomaticCheckout()
    {
        return automaticCheckout;
    }

    public void setAutomaticCheckout(boolean automaticCheckout)
    {
        this.automaticCheckout = automaticCheckout;
    }

    public Calendar getCheckOutRecDateTime()
    {
        return checkOutRecDateTime;
    }

    public void setCheckOutRecDateTime(Calendar checkOutRecDateTime)
    {
        this.checkOutRecDateTime = checkOutRecDateTime;
    }

    public String getWorkflowId()
    {
        return workflowId;
    }

    public void setWorkflowId(String workflowId)
    {
        this.workflowId = workflowId;
    }

    /**
     * @return Returns the loadParams.
     */
    public LoadParams getLoadParams()
    {
        return loadParams;
    }

    /**
     * @param loadParams The loadParams to set.
     */
    public void setLoadParams(LoadParams loadParams)
    {
        this.loadParams = loadParams;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.RpmResult#toInternalSecuredArrayResult()
     */
    public InternalSecuredArrayResult toInternalSecuredArrayResult()
    {
        InternalSecuredArrayResult internalSecuredArrayResult = super
            .toInternalSecuredArrayResult();

        if (getLoadParams() != null)
        {
            if (getLoadParams().getRowCount() != null)
            {
                internalSecuredArrayResult.setRowCount(getLoadParams()
                    .getRowCount().intValue());
            }
        }

        try
        {
            //security
            ISecurityController.INSTANCE
                .fillContainerSecurityDescriptors(internalSecuredArrayResult
                    .getContainerSecurityDescriptorsMap(), this);

            //read-only restrictions
            MetadataRulesUtility
                .applyReadOnlyRestrictions(internalSecuredArrayResult
                    .getContainerSecurityDescriptorsMap());
        }
        catch (RPMException ex)
        {
            RPMException rpmEx = new RPMException(ErrorCodes.INTERNAL_ERROR,
                ContainerSecurityDescriptor.class.getName());
            rpmEx.setSeverity(SeverityLevel.Warning);
            addExceptionNoThrow(rpmEx);
        }

        return internalSecuredArrayResult;
    }

    /**
     * @return the connection
     */
    public Connection getConnection()
    {
        return connection;
    }

    /**
     * @param connection the connection to set
     */
    public void setConnection(Connection connection)
    {
        this.connection = connection;
    }

    public HashMap getAllStatementsByConnection(Connection connection)
    {
        HashMap allStatements = null;
        if (connection != null)
        {
            allStatements = (HashMap) hmStatementsByConnections.get(connection);

        }
        return allStatements;
    }

    private Map outstandingResultSets = new HashMap();

    private Map reservedStatement = new HashMap();

    
    public void reservePreparedStatement(ResultSet rs, PreparedStatement stm)
    {
        if (isConnectionCachingEnabled(this))
        {
            Object s = null;
            try
            {
                s = rs.getStatement();
            }
            catch (SQLException ex)
            {
                //ignore
            }
            if (s == null)
            {
                s = stm;
            }
            outstandingResultSets.put(rs, s);
            reservedStatement.put(s, rs);
        }
    }

    public void unReservePreparedStatement(ResultSet rs)
    {        
        if (isConnectionCachingEnabled(this))
        {
            Object statement = outstandingResultSets.remove(rs);
            reservedStatement.remove(statement);          
        }
    }

    public PreparedStatement getStatementFromCache(Connection connection,
            String sql)
    {
        if (isConnectionCachingEnabled(this))
        {
            if (connection != null && sql != null)
            {
                HashMap hmStatements = (HashMap) hmStatementsByConnections
                    .get(connection);
                if (hmStatements != null)
                {

                    Set set = (Set) hmStatements.get(sql);
                    if (set != null)
                    {
                        Iterator it = set.iterator();
                        while (it.hasNext())
                        {
                            PreparedStatement stm = (PreparedStatement) it
                                .next();
                            if (!reservedStatement.containsKey(stm))
                            {
                                return stm;
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    public void addStatementToCache(Connection connection, String sql,
            PreparedStatement ps)
    {
        if (isConnectionCachingEnabled(this))
        {
            if (connection != null && sql != null && ps != null)
            {
                if (!hmStatementsByConnections.containsKey(connection))
                {
                    hmStatementsByConnections.put(connection, new HashMap());
                }
                HashMap hmStatements = (HashMap) hmStatementsByConnections
                    .get(connection);

                Set set = (Set) hmStatements.get(sql);
                if (set == null)
                {
                    set = new HashSet();
                    hmStatements.put(sql, set);
                }
                set.add(ps);
            }
        }
    }

    public void clearStatementsCache(Connection con)
    {
        HashMap hmStatements = (HashMap) hmStatementsByConnections.remove(con);
        if (hmStatements != null)
        {
            Iterator it = hmStatements.values().iterator();
            while (it.hasNext())
            {
                Set set = (Set) it.next();
                Iterator it_2 = set.iterator();
                while (it_2.hasNext())
                {
                    outstandingResultSets.remove(reservedStatement.remove(it_2
                        .next()));
                }
            }
        }
    }   

    public boolean isSingleConnectionPerContext()
    {
        RpmOptions options = getFactory().getCurrentOptions();
        return options.isSingleConnectionPerContext();
    }

    public static boolean isConnectionCachingEnabled(
            MessageContext messageContext)
    {
        return messageContext != null
                && messageContext.isSingleConnectionPerContext();
    }

    public boolean isWebServicesSecurityEnabled()
    {
        return getFactory().getCurrentOptions().isWebServicesSecurityEnabled();
    }

    private RPMSecurityCache rpmSecurityCache;

    public RPMSecurityCache getRPMSecurityCache()
    {
        if (rpmSecurityCache == null)
        {
            rpmSecurityCache = new RPMSecurityCache(this);
        }
        return rpmSecurityCache;
    }
}
