package com.ibm.rpm.framework;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Stack;

import com.ibm.rpm.framework.util.LevelMap;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.xpathparser.XPathContainer;

public interface RPMObjectManager
{

    /**
     * @param tableStack
     * @param rpmScope
     * @param messageContext
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    public ArrayList load(Stack tableStack, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException;

    /**
     * @param rpmObject
     * @param rpmScope
     * @param messageContext
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    public RPMObject load(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException;

    public RPMObject loadByPrimaryKey(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException;

    /**
     * Method used to load related objects (ex: Children from the same table,
     * Resource in a Pool etc.). The method
     * getForeignKeyCondition(AbstractRPMObjectManager, String) will be used to
     * determine the join condition between the parent and the children. If the
     * foreign key condition includes ?, the parameter args should contains the
     * values that will replace the ? in the PreparedStatement. The parameter
     * extraWhere is usefull if you need to add an extra condition (Ex: TYPE_ID = ?)
     * 
     * @param rpmScope
     * @param messageContext
     * @param manager
     * @param args
     * @param extraWhere
     * @param contextName
     * 
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    public ArrayList loadByForeignKey(RPMObject[] rpmObjects,
            RPMObjectScope rpmScope, MessageContext messageContext,
            RPMObjectManager manager, Object args[], String extraWhere,
            String contextName, boolean isDelta) throws RPMException,
            SQLException, ParseException;

    /**
     * @param rpmObject
     * @param messageContext
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    public void delete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException;

    /**
     * @param rpmObject
     * @param rpmScope
     * @param messageContext
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    public RPMObject save(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException;;

    /**
     * Deletes, creates, saves and/or reload the specified
     * <code>rpmObject</code>, as required. Deletes when
     * <code>rpmObject.getDeleted().booleanValue()</code> is true. Returns
     * null when <code>rpmObject</code> is not valid. Creates when
     * <code>rpmObject.getID() == null</code>. Saves <code>rpmObject</code>.
     * Reloads <code>rpmObject</code> if required by the ReloadType specified
     * in the <code>messageContext</code>.
     */
    public RPMObject internalSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException;

    /**
     * @param rpmObject
     * @param messageContext
     * @throws RPMException
     */
    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException;

    /**
     * @param rpmObject
     * @param messageContext
     * @throws RPMException
     * @throws SQLException
     */
    public void internalCheckin(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, SQLException;

    /**
     * @param rpmObject
     * @param messageContext
     * @throws RPMException
     */
    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException;

    /**
     * @param rpmObject
     * @param messageContext
     * @throws RPMException
     * @throws SQLException
     */
    public void internalCheckout(RPMObject rpmObject, boolean isTimeless,
            MessageContext messageContext) throws RPMException, SQLException;

    /**
     * @param rpmObject
     * @param messageContext
     * @throws RPMException
     */
    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException;

    public void forceCheckinOnSessionsCheckOut(String targetSessionID,
            MessageContext messageContext) throws RPMException, SQLException;

    public void forceCheckinOnUsersCheckOut(Resource user,
            MessageContext messageContext) throws RPMException, SQLException;

    public void forceCheckinOnRPMObjectCheckOut(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, SQLException;

    /**
     * @param messageContext
     * @param rpmObject
     * @throws RPMException
     * @throws SQLException
     */
    public boolean isCheckedOut(MessageContext context, RPMObject rpmObject)
            throws RPMException, SQLException;
    
    public boolean isCheckedOutBy(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException, SQLException;

    /**
     * @return Name of the table associated with the AbstractRPMObjectManager
     */
    public abstract String getTableName();

    public abstract String getTableName(RPMObject rpmObject);

    public abstract String getTableName(String contextName);

    /**
     * @return single primary key of the table associated with the
     *         AbstractRPMObjectManager Override if your primary key has only
     *         one field
     * @throws RPMException
     */
    public String getPrimaryKey() throws RPMException;

    /**
     * @return an array of fields that represents the primary key of the table
     *         associated with the AbstractRPMObjectManager Override if your
     *         primary key contains more than one field
     */
    public String[] getPrimaryKeyArray(RPMObject rpmObject) throws RPMException;

    /**
     * Returns a concatenation of the primaryKey values to create a unique
     * primaryKey String. Returns null if at least one of the primaryKey values
     * is null.
     * 
     * @param object
     * @return
     */
    public String getPrimaryKeyString(RPMObject rpmObject);

    /**
     * Returns the checkout id.
     * 
     * @param rpmObject
     * @return
     */
    public String getCheckInCheckOutPrimaryKey(RPMObject rpmObject, MessageContext messageContext) throws RPMException;

    /**
     * Returns an array of objects retrieved from the concatenated primary key
     * 
     * @param pkString
     * @return
     */
    public Object[] getPKValuesFromString(String pkString);

    /**
     * @param rpmObject
     * @return an array of objects that represents the primary key of the table
     *         associated with the AbstractRPMObjectManager Override if your
     *         primary contains more than one field. IMPORTANT: Always put the
     *         ID property in the primary key. The primary key array should
     *         contains a null to indicate that the object is new.
     */
    public Object[] getPrimaryKeyValues(RPMObject rpmObject);

    /**
     * @param tableStack
     * @param manager
     * @param xPathContainer
     * @param levels
     * @param contexName
     * @param isSubTable
     * @return
     * @throws RPMException
     */
    public ArrayList getJoinConditions(Stack tableStack,
            RPMObjectManager manager, XPathContainer xPathContainer,
            LevelMap levels, String contextName, boolean isSubTable)
            throws RPMException;

    /**
     * 
     * @param manager
     * @param container
     * @param contextName
     * @return
     * @throws RPMException
     */
    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException;
    
    /**
     * 
     * @param manager
     * @param container
     * @param contextName
     * @return
     * @throws RPMException
     */
    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException;
    
    
    /**
     * Override this method if needed.
     * 
     * (eg. Some tables contains a root object that you will want to exclude
     * from your selects. In this case, getFilter() will return "NAME_ELEMENT_ID
     * IS NOT NULL")
     * 
     * (eg. Soft deletion is used in some tables, in this case, getFilter() will
     * return NAME_REC_STATUS != 'D'.)
     * 
     * (eg. Some tables can have more than one TYPE_ID in the same table but
     * your manager only handles one TYPE_ID. In this case, getFilter() will
     * return "TYPE_ID = <Container>.TYPE_ID")
     * 
     * @return a condition that will be appended to all the select statements
     *         executed in loadByPrimaryKey, loadByForeignKey. NOTE: You must
     *         append this filter to the joinConditions returned by the
     *         getJoinCondition and getSubTableJoinCondition when it applies.
     */
    public String getFilter(String contextName);

    /**
     * Method used to validate that RPMObjects contain the necessary primary
     * keys before attempting to load them. AbstractRPMObjectManager provides a
     * default implementation. All managers that override the
     * getPrimaryKeyStringArray() function must also implement this function.
     * 
     * @param rpmObject
     * @param messageContext
     * 
     * @throws RPMException
     */
    public abstract void isValidPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException;
}
