package com.ibm.rpm.framework;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;

import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.ArrayUtil;

/**
 * Result used internally. Translated to *Result at the Application interface
 * level.
 */
public class RpmResult
{
    private ArrayList errors;

    private ArrayList warnings;

    private ArrayList objects;

    public RpmResult()
    {
        super();
        objects = new ArrayList();
        errors = new ArrayList();
        warnings = new ArrayList();
    }

    // =================
    // RESULTING OBJECTS
    // =================

    public RPMObject getFirstObject()
    {
        RPMObject firstObject = null;
        if (objects.size() > 0)
        {
            firstObject = (RPMObject) objects.get(0);
        }
        return firstObject;
    }

    public ArrayList getObjects()
    {
        return objects;
    }

    public void addObjects(ArrayList newObjects)
    {
        if (newObjects != null)
        {
            this.objects.addAll(newObjects);
        }
    }

    /**
     * Adds the given object to this result's object list. Object reference will
     * be added even if null.
     * 
     * @param object
     *            Object to be added. Nullable.
     */
    public void addObject(RPMObject object)
    {
        this.objects.add(object);
    }

    // ==================
    // EXCEPTION HANDLING
    // ==================

    public ArrayList getErrors()
    {
        return errors;
    }

    public boolean isSuccessful()
    {
        return errors.size() == 0;
    }

    public boolean hasWarnings()
    {
        return warnings.size() > 0;
    }

    public void addException(Throwable exception) throws RPMException
    {
        addException(exception, null);
    }

    /**
     * To be used in rare cases in the validation process and the ManagerCaller.
     */
    public void addExceptionNoThrow(Throwable exception)
    {
        doAddException(exception, null);
    }

    public void addException(Throwable exception, RPMObject rpmObject)
            throws RPMException
    {
        RPMException x = translateException(exception, rpmObject);
        if (isSevere(x))
        {
            // rethrow the error without putting it in the message context.
            // a catch statement in the call stack should put this
            // error in the message context
            throw x;
        }
        else
        {
            // minor error, don't throw back, just add to the message context.
            doAddException(x, rpmObject);
        }
    }

    private RPMException doAddException(Throwable exception, RPMObject rpmObject)
    {
        // Design note: when the same exception is caught and rethrown, it
        // can be caught again. In this case, we do not want to add and log the
        // same exception again.
        RPMException x = null;
        if (exception != null && !contains(exception))
        {
            x = translateException(exception, rpmObject);
            if (isWarning(x))
            {
                warnings.add(x);
            }
            else
            {
                errors.add(x);
            }
            // Logs the exception right away, in context.
            x.log();
        }
        return x;
    }

    private boolean contains(Throwable newException)
    {
        boolean result = false;
        if ((newException instanceof RPMException)
                && isSevere((RPMException) newException))
        {
            Iterator i = errors.iterator();
            while (i.hasNext())
            {
                RPMException oldException = (RPMException) i.next();
                if (oldException == newException)
                {
                    result = true;
                    break;
                }
            }
        }
        return result;
    }

    public static RPMException translateException(Throwable exception,
            RPMObject rpmObject)
    {
        RPMException rpmException;

        if (exception instanceof RPMException)
        {
            rpmException = (RPMException) exception;
        }
        else if (exception instanceof SQLException)
        {
            SQLException sqlException = (SQLException) exception;
            int errorCode = sqlException.getErrorCode();
            String errorMessage = getMessageFromSqlException(sqlException);

            rpmException = new RPMException(ErrorCodes.INTERNAL_ERROR,
                new String[]
                {
                    errorMessage
                }, sqlException);
            if (errorCode == 100)
            {
                rpmException.setSeverity(SeverityLevel.Warning);
            }
        }
        else
        {
            rpmException = new RPMException(exception);
        }

        if (rpmObject != null)
        {
            rpmException.setClassName(rpmObject.getClass().getName());
            rpmException.setContainerID(rpmObject.getID());
        }

        return rpmException;
    }

    public static String getMessageFromSqlException(SQLException sqlException)
    {
        int errorCode = sqlException.getErrorCode();
        String errorMessage = null;

        try
        {
            errorMessage = RPMException.ecp
                .getErrorDescriptionFromBundle(Integer.toString(errorCode));
        }
        catch (Exception e)
        {
            // ignore errors
        }

        if (errorMessage == null)
        {
            errorMessage = sqlException.toString();
            SQLException nextSqlException = sqlException.getNextException();
            SQLException nextSqlException2;
            while (nextSqlException != null && nextSqlException != sqlException)
            {
                errorMessage += "\nNext SQL Error: "
                        + nextSqlException.toString();
                nextSqlException2 = nextSqlException.getNextException();
                if (nextSqlException2 == nextSqlException)
                {
                    nextSqlException = null;
                }
                else
                {
                    nextSqlException = nextSqlException2;
                }
            }

            Throwable causeException = sqlException.getCause();
            Throwable causeException2;

            while (causeException != null && causeException != sqlException)
            {
                errorMessage += "\nPrevious Cause: "
                        + causeException.toString();
                causeException2 = causeException.getCause();
                if (causeException2 == causeException)
                {
                    causeException = null;
                }
                else
                {
                    causeException = causeException2;
                }
            }
        }
        return errorMessage;
    }

    private boolean isSevere(RPMException x)
    {
        SeverityLevel severity = x.getSeverity();

        return (x != null) && !SeverityLevel.Warning.equals(severity)
                && !SeverityLevel.Information.equals(severity)
                && !SeverityLevel.Error.equals(severity);
    }

    private boolean isWarning(RPMException x)
    {
        return (x != null) && SeverityLevel.Warning.equals(x.getSeverity());
    }

    // ==========
    // CONVERTION
    // ==========

    public Result toResult()
    {
        return toResult(Result.class);
    }

    public SingleResult toSingleResult()
    {
        return (SingleResult) toResult(SingleResult.class);
    }

    public ArrayResult toArrayResult()
    {
        return (ArrayResult) toResult(ArrayResult.class);
    }

    public InternalSecuredArrayResult toInternalSecuredArrayResult()
    {
        return (InternalSecuredArrayResult) toResult(InternalSecuredArrayResult.class);
    }

    public AlreadyCheckedOutResult toAlreadyCheckedOutResult(
            AlreadyCheckedOutResult acr)
    {
        return (AlreadyCheckedOutResult) toResult(acr);
    }

    private Result toResult(Result otherResult)
    {
        if (otherResult != null)
        {
            // Set exceptions.
            otherResult.setSuccessful(isSuccessful());
            otherResult.setWarningFree(!hasWarnings());
            otherResult.setErrors((RPMException[]) ArrayUtil.listToArray(
                RPMException.class, errors));
            otherResult.setWarnings((RPMException[]) ArrayUtil.listToArray(
                RPMException.class, warnings));

            // Set object(s).
            if (otherResult instanceof ArrayResult)
            {
                ((ArrayResult) otherResult)
                    .setRpmObjectList((RPMObject[]) ArrayUtil.listToArray(
                        RPMObject.class, objects));
            }
            else if (otherResult instanceof SingleResult)
            {
                ((SingleResult) otherResult).setRpmObject(getFirstObject());
            }
        }
        return otherResult;
    }

    private Result toResult(Class otherResultClass)
    {
        // Create the other result.
        Result otherResult = null;
        try
        {
            otherResult = (Result) otherResultClass.newInstance();
        }
        catch (Exception x)
        {
            addExceptionNoThrow(x);
        }

        return toResult(otherResult);
    }
}
