/*
 * Created on Jul 20, 2006
 */
package com.ibm.rpm.framework;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.PreparedStatementHelper;
import com.ibm.rpm.interfaces.SessionUtility;
import com.ibm.rpm.interfaces.impl.ServerFactory;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author gosselin
 */
public class SessionCheckoutsDAO
{
    
    private static Log logger = LogFactory.getLog(SessionCheckoutsDAO.class);
    
    private static String TIMELESS_CHECKOUT = "TIMELESS_CHECKOUT";

    private static String WEBAPI_SESSION_CHECKOUTS_TABLE = "WEBAPI_SESSION_CHECKOUTS";
    
    private static String RPMOBJECT_CK_ID = "RPMOBJECT_CK_ID";
    
    private static String RESOURCE_ID = "RESOURCE_ID";
    
    private static String SESSION_ID = "SESSION_ID";
    
    private static String RPMOBJECT_CLASS = "RPMOBJECT_CLASS";
    
    private ServerFactory factory;
    
    public SessionCheckoutsDAO(ServerFactory setfactory)
    {
        factory = setfactory;
    }


    /**
     * Applies an 'OR' on the results of both isCheckedOutFromCurrentSession and
     * isTimelessCheckoutFromUser
     * 
     * @param rpmObjectCKID
     * @param user
     * @param messageContext
     * @return
     * @throws RPMException
     */
    public boolean isCheckedOut(String rpmObjectCKID,
            MessageContext messageContext) throws RPMException
    {
        boolean isCheckedOut = isCheckedOutFromCurrentSession(rpmObjectCKID,
            messageContext);

        if (messageContext.getUser() != null)
        {
            isCheckedOut = isCheckedOut
                    || isTimelessCheckoutFromUser(rpmObjectCKID, messageContext);
        }

        return isCheckedOut;
    }
    
    /**
     * Determine if the targetSessionID, if stored in the 
     * WEBAPI_SESSION_CHECKOUTs_TABLE, belongs to the owner.
     * @param targetSessionID
     * @param owner
     * @param messageContext
     * @return
     * @throws RPMException
     */
    public boolean isSessionOwner(String targetSessionID, Resource owner,
            MessageContext messageContext) throws RPMException
    {
        boolean isSessionOwner = false;
        Connection c = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;

        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "SELECT " + RESOURCE_ID + " FROM "
                    + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE " + SESSION_ID
                    + " = ?";

            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
                messageContext, c);

            Object[] arguments = new Object[]
            {
                targetSessionID
            };

            stmt = preparedStatementHelper.createPreparedStatement(selectStmt,
                arguments);

            rs = ManagerUtil.executeSqlQuery(messageContext, stmt, selectStmt,
                arguments);
            
            String resourceID = null;
            if (rs != null && rs.next())
            {
                resourceID = rs.getString(1);
                isSessionOwner = owner.getID().equals(resourceID);
            }     
            else
            {
                isSessionOwner = true; //There are no checkouts for this session
            }
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return isSessionOwner;
    }
    
    /**
     * Determine if the checkout on the RPMObject identified by the rpmObjectCKID
     * belongs to the owner.
     * @param rpmObjectCKID
     * @param owner
     * @param messageContext
     * @return
     * @throws RPMException
     */
    public boolean isCheckoutOwner(String rpmObjectCKID, Resource owner,
            MessageContext messageContext) throws RPMException
    {
        boolean isCheckoutOwner = false;
        Connection c = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;

        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "SELECT " + RESOURCE_ID + " FROM "
                    + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE "
                    + RPMOBJECT_CK_ID + " = ?";

            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
                messageContext, c);

            Object[] arguments = new Object[]
            {
                rpmObjectCKID
            };

            stmt = preparedStatementHelper.createPreparedStatement(selectStmt,
                arguments);

            rs = ManagerUtil.executeSqlQuery(messageContext, stmt, selectStmt,
                arguments);
            
            String resourceID = null;
            if (rs != null && rs.next())
            {
                resourceID = rs.getString(1);
                isCheckoutOwner = owner.getID().equals(resourceID);
            }           
            else
            {
                isCheckoutOwner = true; //There are no checkouts
            }
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return isCheckoutOwner;
    }
    

    /**
     * Look if the rpmObject has a timeless checkout from the user.
     * 
     * @param user
     * @param messageContext
     * @return
     * @throws RPMException
     */
    private boolean isTimelessCheckoutFromUser(String rpmObjectCKID,
            MessageContext messageContext) throws RPMException
    {
        boolean isCheckedOut = false;
        Connection c = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;

        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "SELECT COUNT(" + RPMOBJECT_CK_ID
                    + ") FROM " + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE "
                    + RPMOBJECT_CK_ID + " = ? AND " + RESOURCE_ID + " = ? AND "
                    + TIMELESS_CHECKOUT + " = ?";

            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
                messageContext, c);

            Object[] arguments = new Object[]
            {
                    rpmObjectCKID, messageContext.getUser().getID(),
                    new Integer(1)
            };

            stmt = preparedStatementHelper.createPreparedStatement(selectStmt,
                arguments);

            rs = ManagerUtil.executeSqlQuery(messageContext, stmt, selectStmt,
                arguments);
            
            if (rs != null && rs.next())
            {
                isCheckedOut = rs.getInt(1) > 0;
            }
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return isCheckedOut;
    }

    /**
     * Look if the rpmObject is checked out by the session from the
     * messageContext
     * 
     * @param rpmObject
     * @param messageContext
     * @return true if the rpmObject is checked out from the sessionID in
     *         messageContext
     * @throws RPMException
     */
    private boolean isCheckedOutFromCurrentSession(String rpmObjectCKID,
            MessageContext messageContext) throws RPMException
    {
        boolean isCheckedOut = false;
        Connection c = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;

        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "SELECT COUNT(" + RPMOBJECT_CK_ID + ")"
                    + " FROM " + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE "
                    + RPMOBJECT_CK_ID + " = ?" + " AND " + SESSION_ID + " = ?";

            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
                messageContext, c);

            Object[] arguments = new Object[]
            {
                    rpmObjectCKID, messageContext.getSession()
            };

            stmt = preparedStatementHelper.createPreparedStatement(selectStmt,
                arguments);

            rs = ManagerUtil.executeSqlQuery(messageContext, stmt, selectStmt,
                arguments);
            
            if (rs != null && rs.next())
            {
                isCheckedOut = rs.getInt(1) > 0;
            }
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return isCheckedOut;
    }

    public void storeSessionCheckout(String rpmObjectCKID,
            String rpmObjectClass, boolean isTimeless,
            MessageContext messageContext) throws RPMException
    {
        Connection c = null;
        ResultSet rs = null;
        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "INSERT INTO " + WEBAPI_SESSION_CHECKOUTS_TABLE
                    + " (" + RPMOBJECT_CK_ID + "," + SESSION_ID + ","
                    + RESOURCE_ID + "," + RPMOBJECT_CLASS + ","
                    + TIMELESS_CHECKOUT + ")" + " VALUES (?, ?, ?, ?, ?)";

            Object[] arguments = new Object[]
            {
                    rpmObjectCKID, messageContext.getSession().toString(),
                    messageContext.getUser().getID(), rpmObjectClass,
                    new Boolean(isTimeless)
            };

            ManagerUtil.executeSqlUpdate(messageContext, c, selectStmt,
                arguments);
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
    }

    /**
     * Check in all checked out objects associated with the sessionID, unless
     * these checkouts are flagged as timeless.
     * 
     * @param sessionID
     * @throws RPMException
     */
    public void undoSessionCheckouts(MessageContext messageContext, String sessionID) throws RPMException
    {
        ArrayList rpmObjects = retrieveCheckedOutRPMObject(sessionID,
            messageContext);

        if (rpmObjects != null && rpmObjects.size() > 0)
        {
            for (int i = 0; i < rpmObjects.size(); i++)
            {
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(rpmObjects.get(i).getClass());
                try
                {
                    manager.checkinObject((RPMObject) rpmObjects.get(i),
                        messageContext);
                }
                catch (RPMException e)
                {
                    // nothing to do, we want to process with the other checked
                    // out RPMObjects
                }
            }
        }

        removeSessionCheckoutsForSession(sessionID, messageContext);
    }

    /**
     * 
     * @param sessionID
     * @param messageContext
     * @return an ArrayList of RPMObject
     * @throws RPMException
     */
    private ArrayList retrieveCheckedOutRPMObject(String sessionID,
            MessageContext messageContext) throws RPMException
    {
        Connection c = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;
        ArrayList objects = null;
        Long session = new Long(sessionID);
        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "SELECT " + RPMOBJECT_CK_ID + ","
                    + RPMOBJECT_CLASS + " FROM "
                    + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE " + SESSION_ID
                    + "= ?" + " AND " + TIMELESS_CHECKOUT + " = ?";
            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
                messageContext, c);
            Object[] arguments = new Object[]
            {
                    session, new Integer(0)
            };
            stmt = preparedStatementHelper.createPreparedStatement(selectStmt,
                arguments);

            rs = ManagerUtil.executeSqlQuery(messageContext, stmt, selectStmt,
                arguments);
            objects = new ArrayList();
            while (rs.next())
            {
                String rpmObjectClass = rs.getString(RPMOBJECT_CLASS);
                RPMObject rpmObject = (RPMObject) Class.forName(rpmObjectClass)
                    .newInstance();
                String rpmObjectCKID = rs.getString(RPMOBJECT_CK_ID);
                String rpmObjectID = restoreRPMObjectID(rpmObject,
                    rpmObjectCKID);
                rpmObject.setID(rpmObjectID);
                objects.add(rpmObject);
            }
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return objects;
    }

    /**
     * Retrieves a list of check-in/check-out primary keys from a partial
     * primary key in the provided session.
     * 
     * Each list item contains an array of primary keys for a single object that
     * is currently checked out in the session.
     * 
     * @param sessionID
     *            The ID of the session for which you want to retrieve the
     *            check-in/check-out primary key.
     * @param substringCK_ID
     *            The partial primary key to search for.
     * @param messageContext
     * @return 
     * @throws RPMException
     */
    public ArrayList retrieveCK_IDFromPartialCK_ID(String sessionID,
            String substringCK_ID, MessageContext messageContext)
            throws RPMException
    {
        ArrayList checkedOutRPMObjectsInSession = new ArrayList();
        Connection c = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;
        Long session = new Long(sessionID);
        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "SELECT " + RPMOBJECT_CK_ID + ","
                    + RPMOBJECT_CLASS + " FROM "
                    + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE " + SESSION_ID
                    + "= ?" + " AND " + TIMELESS_CHECKOUT + " = ?" + " AND "
                    + RPMOBJECT_CK_ID + " LIKE '%" + substringCK_ID + "%'";
            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
                messageContext, c);
            Object[] arguments = new Object[]
            {
                    session, new Integer(0)
            };
            stmt = preparedStatementHelper.createPreparedStatement(selectStmt,
                arguments);

            rs = ManagerUtil.executeSqlQuery(messageContext, stmt, selectStmt,
                arguments);
            if (rs.next())
            {
                String rpmObjectCKID = rs.getString(RPMOBJECT_CK_ID);
                RPMObjectManager manager = RPMManagerFactory.getInstance().getRPMObjectManager(
                    rs.getString(RPMOBJECT_CLASS));
                checkedOutRPMObjectsInSession.add(manager.getPKValuesFromString(rpmObjectCKID));
            }
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return checkedOutRPMObjectsInSession;
    }

    public ArrayList retrieveUserCheckedOut(Resource user,
            MessageContext messageContext) throws RPMException
    {
        Connection c = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;
        ArrayList objects = null;
        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "SELECT " + RPMOBJECT_CK_ID + ","
                    + RPMOBJECT_CLASS + " FROM "
                    + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE " + RESOURCE_ID
                    + "= ?";

            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
                messageContext, c);

            Object[] arguments = new Object[]
            {
                    user.getID()
            };
            
            stmt = preparedStatementHelper.createPreparedStatement(selectStmt,
                arguments);

            rs = ManagerUtil.executeSqlQuery(messageContext, stmt, selectStmt,
                arguments);
            objects = new ArrayList();
            
            while (rs.next())
            {
                String rpmObjectClass = rs.getString(RPMOBJECT_CLASS);
                RPMObject rpmObject = (RPMObject) Class.forName(rpmObjectClass)
                    .newInstance();
                String rpmObjectCKID = rs.getString(RPMOBJECT_CK_ID);
                String rpmObjectID = restoreRPMObjectID(rpmObject,
                    rpmObjectCKID);
                rpmObject.setID(rpmObjectID);
                objects.add(rpmObject);
            }

        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return objects;
    }

    public ArrayList retrieveSessionCheckedOuts(String ckSessionID,
            MessageContext messageContext) throws RPMException
    {
        Connection c = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;
        ArrayList objects = null;
        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "SELECT " + RPMOBJECT_CK_ID + ","
                    + RPMOBJECT_CLASS + " FROM "
                    + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE " + SESSION_ID
                    + " = ?";

            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
                messageContext, c);

            Object[] arguments = new Object[]
            {
                ckSessionID
            };

            stmt = preparedStatementHelper.createPreparedStatement(selectStmt,
                arguments);

            rs = ManagerUtil.executeSqlQuery(messageContext, stmt, selectStmt,
                arguments);

            objects = new ArrayList();
            while (rs.next())
            {
                String rpmObjectClass = rs.getString(RPMOBJECT_CLASS);
                RPMObject rpmObject = (RPMObject) Class.forName(rpmObjectClass)
                    .newInstance();
                String rpmObjectCKID = rs.getString(RPMOBJECT_CK_ID);
                String rpmObjectID = restoreRPMObjectID(rpmObject,
                    rpmObjectCKID);
                rpmObject.setID(rpmObjectID);
                objects.add(rpmObject);
            }
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return objects;
    }

    public void removeSessionCheckoutsForRPMObject(String rpmObjectCKID,
            MessageContext messageContext) throws RPMException
    {
        String condition = RPMOBJECT_CK_ID + "= '" + rpmObjectCKID + "'";
        deleteSessionCheckouts(condition, messageContext);
    }

//TODO clean up if necessary    
//    private void removeSessionCheckoutsForUser(MessageContext messageContext)
//            throws RPMException
//    {
//        Resource resource = (Resource) messageContext.getUser();
//        String condition = RESOURCE_ID + "= '" + resource.getID() + "'";
//        deleteSessionCheckouts(condition, messageContext);
//    }

    /**
     * delete session related checkouts, without removing timeless checkouts
     * 
     * @param sessionID
     * @param messageContext
     * @throws RPMException
     */
    private void removeSessionCheckoutsForSession(String sessionID,
            MessageContext messageContext) throws RPMException
    {
        String condition = SESSION_ID + "= " + sessionID
                + " AND " + TIMELESS_CHECKOUT + "= 0";
        deleteSessionCheckouts(condition, messageContext);
    }

    /**
     * 
     * @param condition
     * @throws RPMException
     */
    private void deleteSessionCheckouts(String condition,
            MessageContext messageContext) throws RPMException
    {
        Connection c = null;
        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStmt = "DELETE FROM "
                    + WEBAPI_SESSION_CHECKOUTS_TABLE + " WHERE " + condition;

            ManagerUtil.executeSqlUpdate(messageContext, c, selectStmt, null);
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(this, e);
        }
        finally
        {
            Manager.releaseConnection(messageContext, c);
        }
    }

    private String restoreRPMObjectID(RPMObject rpmObject, String rpmObjectCKID)
            throws RPMException
    {
        RPMObjectManager manager = RPMManagerFactory.getInstance()
            .getRPMObjectManager(rpmObject.getClass());
        Object[] primaryKeys = manager.getPKValuesFromString(rpmObjectCKID);
        String retVal = null;
        if( primaryKeys == null || primaryKeys.length == 0 )
        {
            // failed to load primary key
            logger.warn("Failed to load primary on object of type : "
                    + rpmObject.getClass() + " using primary key string : "
                    + rpmObjectCKID);
        }
        else
        {
            retVal = (String) primaryKeys[0];
        }
        return retVal;
    }
    
    private DataSource getRPMDataSource() throws RPMException
    {
        SessionUtility sessionUtility = factory.getSession();
        return sessionUtility.getDataSource(SessionUtility.RPM_DATASOURCE);
    }
    
    private void setDatasourceIfNecessary(MessageContext messageContext)throws RPMException
    {
        if(messageContext.getDatasource()==null)
        {
            DataSource ds = getRPMDataSource();
            messageContext.addDatasource(ds);
        }
    }
}
