/*
 * Created on Jul 5, 2006
 */
package com.ibm.rpm.framework.checkpoints;

import java.sql.SQLException;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author gosselin
 */
public class ManualCheckinCheckoutCheckpoint
{
    public static boolean isValidDeleteCheckout(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, SQLException
    {
        RPMObjectManager manager = RPMManagerFactory.getInstance()
            .getRPMObjectManager(rpmObject);

        if (!messageContext.isAutomaticCheckout()
                && manager.isCheckedOut(messageContext, rpmObject))
        {
            RPMException e = new RPMException(ErrorCodes.RPMOBJECT_CHECKED_OUT,
                new String[]
                {
                        StringUtil.getShortClassName(rpmObject.getClass()),
                        rpmObject.getID(),
                        messageContext.getCheckOutRecUser().getID(),
                        "delete"
                });
            e.setSeverity(SeverityLevel.Error);
            messageContext.addExceptionNoThrow(e);
        }
        return messageContext.isSuccessful();
    }

    public static boolean isValidUpdateCheckout(RPMObject rpmObject,
            boolean checkedOutFromSession, boolean checkedOutFromRPM,
            MessageContext messageContext) throws RPMException, SQLException
    {
        boolean isValid = false;

        if (checkedOutFromSession && !checkedOutFromRPM)
        {
            // RPMObject is still checked out in the session but checked in
            // the rpm database, most probably the same user checked out from
            // API and checked in from rpm client
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getID(),
                    messageContext.getCheckOutRecUser().getID()
            };
            RPMException e = new RPMException(ErrorCodes.WAS_CHECKED_IN,
                errorParams);
            e.setSeverity(SeverityLevel.Error);
            messageContext.addException(e, rpmObject);
            isValid = false;

            /*
             * Remove the RPM WS session checkout on this object since it was
             * checked in by some other source.
             */
            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(rpmObject.getClass());
            
            String rpmObjectCKID = manager.getCheckInCheckOutPrimaryKey(rpmObject, messageContext);
            
            messageContext.getFactory().getSessionCheckoutDAO()
                .removeSessionCheckoutsForRPMObject(rpmObjectCKID,
                    messageContext);
        }
        else if (!checkedOutFromSession && checkedOutFromRPM)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getID(),
                    messageContext.getCheckOutRecUser().getID(),
                    "checkout"
            };
            RPMException e = new RPMException(ErrorCodes.RPMOBJECT_CHECKED_OUT,
                errorParams);
            e.setSeverity(SeverityLevel.Error);
            messageContext.addExceptionNoThrow(e);
            isValid = false;
        }
        else if(!checkedOutFromSession && !checkedOutFromRPM)       
        {
            if (messageContext.isAutomaticCheckout())
            {
                isValid = true;
            }
            else
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(rpmObject.getClass()),
                        "checkOut"
                };
                RPMException e = new RPMException(
                    ErrorCodes.MANUAL_CHECKOUT_REQUIRED, errorParams);
                e.setSeverity(SeverityLevel.Error);
                messageContext.addExceptionNoThrow(e);
                isValid = false;
            }            
        }
        else if (checkedOutFromSession && checkedOutFromRPM)
        {
            isValid = true;
        }
        
        return isValid;
    }

    public static boolean isValidInternalCheckout(RPMObject rpmObject,
            boolean checkedOutFromSession, boolean checkedOutFromRPM,
            MessageContext messageContext) throws RPMException
    {
        if (checkedOutFromSession && checkedOutFromRPM)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getID()
            };
            RPMException e = new RPMException(
                ErrorCodes.ALREADY_CHECKED_OUT_WITH_SESSION, errorParams);
            e.setSeverity(SeverityLevel.Warning);
            messageContext.addExceptionNoThrow(e);
            return false;
        }
        else if (checkedOutFromSession && !checkedOutFromRPM)
        {
            // RPMObject is still checked out in the session but checked in
            // the rpm database, most probably the same user checked out from
            // API and checked in from rpm client
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getID(),
                    messageContext.getCheckOutRecUser().getID()
            };
            RPMException e = new RPMException(ErrorCodes.WAS_CHECKED_IN,
                errorParams);
            e.setSeverity(SeverityLevel.Warning);
            messageContext.addException(e, rpmObject);
            return true;
        }
        else if (!checkedOutFromSession && checkedOutFromRPM)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getID(),
                    messageContext.getCheckOutRecUser().getID(),"checkout"
            };
            RPMException e = new RPMException(ErrorCodes.RPMOBJECT_CHECKED_OUT,
                errorParams);
            e.setSeverity(SeverityLevel.Error);
            messageContext.addExceptionNoThrow(e);
            return false;
        }
        else //!checkedOutFromSession && !checkedOutFromRPM
        {
            return true;
        }
    }
    
    public static boolean isValidInternalCheckin(RPMObject rpmObject,
            boolean checkedOutFromSession, boolean checkedOutFromRPM,
            MessageContext messageContext) throws RPMException
    {
        if (!checkedOutFromSession && checkedOutFromRPM)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getID(),
                    messageContext.getCheckOutRecUser().getID(),
                    "checkin"
            };
            RPMException e = new RPMException(
                ErrorCodes.CHECKED_OUT_FROM_DELPHI_CLIENT, errorParams);
            e.setSeverity(SeverityLevel.Error);
            messageContext.addExceptionNoThrow(e);
            return false;
        }
        else if (checkedOutFromSession && !checkedOutFromRPM)
        {
            //RPMObject is still checked out in the session but checked in
            //the rpm database, most probably the same user checked out from 
            // API and checked in from rpm client
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getID(),
                    messageContext.getCheckOutRecUser().getID()
            };
            RPMException e = new RPMException(ErrorCodes.WAS_CHECKED_IN,
                errorParams);
            e.setSeverity(SeverityLevel.Warning);
            messageContext.addException(e, rpmObject);
            //return true because we need to remove the checkout from the 
            //SESSIONCHECKOUT table
            return true;
        }
        else if (!checkedOutFromSession && !checkedOutFromRPM)
        {
            //no action required
            return false;
        }
        else //checkedOutFromSession && checkedOutFromRPM
        {
            return true;
        }
    }
    
    public static final boolean canCheckOut(MessageContext messageContext,
            RPMObject rpmObject, boolean isTimeless)
    {
        try
        {
            return (ISecurityController.INSTANCE.canCheckOut(messageContext,
                rpmObject, isTimeless).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            messageContext.addExceptionNoThrow(SecurityValidationResult
                .makeException(e));
            return false;
        }
    }   
}
