/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.RestoreRelevantFilteredFieldsParameter;
import com.ibm.rpm.framework.RpmLog;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;

/**
 * @author Yasser Lulu
 * 
 */
final class SecurityControllerDispatcher implements ISecurityController
{
    private static final ISecurityController NULL_CONTROLLER_IMPL = new BaseSecurityController();

    private static final String DEFAULT_CONTROLLER_IMPL = "com.ibm.rpm.framework.security.controller.impl.rpm.RPMSecurityControllerImpl";

    private ISecurityController securityControllerImpl = null;
    
    public void fillContainerSecurityDescriptors(
            Map containerSecurityDescriptorsMap, MessageContext messageContext)
            throws RPMException
    {
        if (messageContext == null || (containerSecurityDescriptorsMap == null))
        {
            throw new RPMException(
                "Invalid parameters for SecurityController::fillContainerSecurityDescriptors(...)")
                .log();
        }

        List rpmObjects = messageContext.getObjects();
        if ((rpmObjects == null) || rpmObjects.isEmpty())
        {
            return; //nothing to do
        }

        getSecurityControllerImpl(messageContext)
            .fillContainerSecurityDescriptors(containerSecurityDescriptorsMap,
                messageContext);
    }

    private ISecurityController getSecurityControllerImpl(
            MessageContext messageContext) throws RPMException
    {
        //If there is an error, then ignore security completely since there is no point!!
        if (!messageContext.isSuccessful())
        {
            return NULL_CONTROLLER_IMPL;
        }

        if (!messageContext.isWebServicesSecurityEnabled())
        {
            //security not installed/used, then use a NULL security controller
            RpmLog.getLog().warn("Security is not enforced");
            return NULL_CONTROLLER_IMPL;
        }
        else
        {
            if(securityControllerImpl==null)
            {
                try
                {
                    String securityControllerClassName = messageContext
                        .getFactory().getCurrentOptions()
                        .getWebServiceSecurityControllerClassName();

                    if (securityControllerClassName == null
                            || securityControllerClassName.trim().length() == 0)
                    {
                        securityControllerImpl = (ISecurityController) Class
                            .forName(DEFAULT_CONTROLLER_IMPL).newInstance();
                    }
                    else
                    {
                        securityControllerImpl = (ISecurityController) Class
                            .forName(securityControllerClassName.trim())
                            .newInstance();
                    }
                }
                catch (Exception ex)
                {
                    throw new RPMException(
                        "Security was enabled but an invalid security controller is specified.",
                        ex).log();
                }                
            }
        }
        
        return securityControllerImpl;
    }

    private SecurityValidationResult handleResult(
            SecurityValidationResult result, MessageContext messageContext)
    {
        if (result.type == SecurityValidationResult.FALSE)
        {
            messageContext.addExceptionNoThrow(result.securityException);
        }
        return result;
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext).canRead(
            rpmObject, fieldName, messageContext), messageContext);
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext).canWrite(
            rpmObject, fieldName, messageContext), messageContext);
    }

    //DOES NOT ADD EXCEPTION TO MESSAGE_CONTEXT, THEREFORE IT DOESN'T CALL handleResult(...)
    public SecurityValidationResult canView(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        return getSecurityControllerImpl(messageContext).canView(rpmObject,
            messageContext);
    }

    public SecurityValidationResult canAcceptTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean acceptEETC,
            boolean acceptForecastDates) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canAcceptTaskAssignmentChanges(messageContext, taskAssignment,
                scope, acceptEETC, acceptForecastDates), messageContext);
    }

    public SecurityValidationResult canAssignScoreCard(
            MessageContext messageContext, RPMObject objectToAssign,
            RPMObjectScope objectToAssignScope, Scorecard scoreCardToAssign,
            ReloadType reloadType) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canAssignScoreCard(messageContext, objectToAssign,
                objectToAssignScope, scoreCardToAssign, reloadType),
            messageContext);
    }

    public SecurityValidationResult canAssociateWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            GenericProject genericProject, boolean associate, boolean isDefault)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canAssociateWorkflow(messageContext, process, genericProject,
                associate, isDefault), messageContext);
    }

    public SecurityValidationResult canCancelWorkflow(
            MessageContext messageContext, RunningWorkflowProcess runningProcess)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canCancelWorkflow(messageContext, runningProcess), messageContext);
    }

    public SecurityValidationResult canCheckOut(MessageContext messageContext,
            RPMObject rpmObject, boolean isTimeless) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canCheckOut(messageContext, rpmObject, isTimeless), messageContext);
    }

    public SecurityValidationResult canConvertContainer(
            MessageContext messageContext, RPMObject inputContainer,
            RPMObject outputContainer, RPMObjectScope rpmScope)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canConvertContainer(messageContext, inputContainer,
                outputContainer, rpmScope), messageContext);
    }

    public SecurityValidationResult canCopyAssignmentProposedToPlan(
            MessageContext messageContext,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canCopyAssignmentProposedToPlan(messageContext,
                resourceTaskAssignment, scope), messageContext);
    }

    public SecurityValidationResult canCopyProposedToPlan(
            MessageContext messageContext, WorkElement workElement,
            WorkElementScope workElementScope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canCopyProposedToPlan(messageContext, workElement,
                workElementScope), messageContext);
    }

    public SecurityValidationResult canCreateObjectFromTemplate(
            MessageContext messageContext, RPMObject inputObject,
            RPMObject template, RPMObjectScope scope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canCreateObjectFromTemplate(messageContext, inputObject, template,
                scope), messageContext);
    }

    public SecurityValidationResult canDelete(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canDelete(messageContext, rpmObject), messageContext);
    }

    public SecurityValidationResult canMoveAndTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canMoveAndTransferUnderWBS(messageContext, workElement,
                abstractScope), messageContext);
    }

    public SecurityValidationResult canMoveUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canMoveUnderWBS(messageContext, workElement, abstractScope),
            messageContext);
    }

    public SecurityValidationResult canPublishScorecard(
            MessageContext messageContext, AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canPublishScorecard(messageContext, assignedScorecard,
                scorecardScope), messageContext);
    }

    public SecurityValidationResult canRejectTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean rejectEETC,
            boolean rejectForecastDates) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canRejectTaskAssignmentChanges(messageContext, taskAssignment,
                scope, rejectEETC, rejectForecastDates), messageContext);
    }

    public SecurityValidationResult canReplaceProfileWithResource(
            MessageContext messageContext, ResourceTaskAssignment rta,
            ResourceTaskAssignmentScope rtaScope, Resource resource)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canReplaceProfileWithResource(messageContext, rta, rtaScope,
                resource), messageContext);
    }

    public SecurityValidationResult canReplaceResourceWithProfile(
            MessageContext messageContext, ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope) throws RPMException
    {
        return handleResult(
            getSecurityControllerImpl(messageContext)
                .canReplaceResourceWithProfile(messageContext, assignment,
                    rtaScope), messageContext);
    }

    public SecurityValidationResult canResumeWorkflow(
            MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canResumeWorkflow(messageContext, runningProcess, scope),
            messageContext);
    }

    public SecurityValidationResult canRollupProjectEV(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canRollupProjectEV(messageContext, rpmObject, scope),
            messageContext);
    }

    public SecurityValidationResult canUpdate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canUpdate(messageContext, rpmObject, rpmObjectScope),
            messageContext);
    }

    public SecurityValidationResult canCreate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canCreate(messageContext, rpmObject, rpmObjectScope),
            messageContext);
    }

    public SecurityValidationResult canSaveProjectBaseline(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canSaveProjectBaseline(messageContext, rpmObject, scope),
            messageContext);
    }

    public SecurityValidationResult canStartWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            RPMObject object, RunningWorkflowProcessScope scope)
            throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canStartWorkflow(messageContext, process, object, scope),
            messageContext);
    }

    public SecurityValidationResult canSuspendWorkflow(
            MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canSuspendWorkflow(messageContext, runningProcess, scope),
            messageContext);
    }

    public SecurityValidationResult canTransferUnderScope(
            MessageContext messageContext, AbstractAggregateScope scopeElement,
            WorkElement workElement) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canTransferUnderScope(messageContext, scopeElement, workElement),
            messageContext);
    }

    public SecurityValidationResult canTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canTransferUnderWBS(messageContext, workElement, abstractScope),
            messageContext);
    }

    public SecurityValidationResult canLockDocument(
			MessageContext messageContext, GenericDocument documentToLock)
			throws RPMException 
	{
    	return handleResult(getSecurityControllerImpl(messageContext)
                .canLockDocument(messageContext, documentToLock),
                messageContext);
	}
    
    public SecurityValidationResult canUnlockDocument(
			MessageContext messageContext, GenericDocument documentToUnlock)
			throws RPMException 
	{
    	return handleResult(getSecurityControllerImpl(messageContext)
                .canUnlockDocument(messageContext, documentToUnlock),
                messageContext);
	}
    
    public ArrayList applyFiltering(MessageContext messageContext,
            ArrayList loadedObjects) throws RPMException
    {
        return getSecurityControllerImpl(messageContext).applyFiltering(
            messageContext, loadedObjects);
    }

    //DOES NOT ADD EXCEPTION TO MESSAGE_CONTEXT, THEREFORE IT DOESN'T CALL handleResult(...)
    public SecurityValidationResult applyFieldFiltering(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        return getSecurityControllerImpl(messageContext).applyFieldFiltering(
            rpmObject, messageContext);
    }

    public SecurityValidationResult restoreRelevantFilteredFields(
            RestoreRelevantFilteredFieldsParameter param) throws RPMException
    {
        return getSecurityControllerImpl(param.getMessageContext())
            .restoreRelevantFilteredFields(param);
    }

    public SecurityValidationResult canUpdateOnCreate(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope rpmScope) throws RPMException
    {
        return handleResult(getSecurityControllerImpl(messageContext)
            .canUpdateOnCreate(messageContext, rpmObject, rpmScope),
            messageContext);
    }
}
