/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.rpm;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.RestoreRelevantFilteredFieldsParameter;
import com.ibm.rpm.framework.security.SecurityDescriptorUtil;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.resource.ResourceBasedSecurityControllerImpl;
import com.ibm.rpm.framework.security.controller.impl.role.RoleBasedSecurityControllerImpl;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;

/**
 * @author Yasser Lulu
 *
 */

public class RPMSecurityControllerImpl implements ISecurityController
{

    private RoleBasedSecurityControllerImpl roleSecurityController;

    private ResourceBasedSecurityControllerImpl resourceSecurityController;

    public RPMSecurityControllerImpl()
    {
        roleSecurityController = new RoleBasedSecurityControllerImpl();
        resourceSecurityController = new ResourceBasedSecurityControllerImpl();
    }

    /**
     * @return the resourceSecurityController
     */
    private ResourceBasedSecurityControllerImpl getResourceSecurityController()
    {
        return resourceSecurityController;
    }

    /**
     * @return the roleSecurityController
     */
    private RoleBasedSecurityControllerImpl getRoleSecurityController()
    {
        return roleSecurityController;
    }

    public void fillContainerSecurityDescriptors(
            Map containerSecurityDescriptorsMap, MessageContext messageContext)
            throws RPMException
    {
        Set flatRPMObjectInfos = SecurityControllerUtil
            .getFilteringResultFlatRPMObjectInfos(messageContext, this);

        if (flatRPMObjectInfos == null)
        {
            flatRPMObjectInfos = SecurityControllerUtil
                .getFlatRPMObjectInfos(messageContext);
        }

        Map roleDescrsMap = new HashMap();
        getRoleSecurityController().doFillContainerSecurityDescriptors(
            roleDescrsMap, flatRPMObjectInfos, messageContext);

        Map resourceDescrsMap = new HashMap();
        getResourceSecurityController().doFillContainerSecurityDescriptors(
            resourceDescrsMap, flatRPMObjectInfos, messageContext);

        SecurityDescriptorUtil.combineSecurityDescriptorsMaps(
            containerSecurityDescriptorsMap, roleDescrsMap, resourceDescrsMap,
            false);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController().canRead(
            rpmObject, fieldName, messageContext);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canRead(rpmObject, fieldName, messageContext);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canView(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController().canView(
            rpmObject, messageContext);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canView(rpmObject, messageContext);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController().canWrite(
            rpmObject, fieldName, messageContext);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canWrite(rpmObject, fieldName, messageContext);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canAcceptTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean acceptEETC,
            boolean acceptForecastDates) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canAcceptTaskAssignmentChanges(messageContext, taskAssignment,
                scope, acceptEETC, acceptForecastDates);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canAcceptTaskAssignmentChanges(messageContext, taskAssignment,
                scope, acceptEETC, acceptForecastDates);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;

    }

    public SecurityValidationResult canAssignScoreCard(
            MessageContext messageContext, RPMObject objectToAssign,
            RPMObjectScope objectToAssignScope, Scorecard scoreCardToAssign,
            ReloadType reloadType) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canAssignScoreCard(messageContext, objectToAssign,
                objectToAssignScope, scoreCardToAssign, reloadType);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canAssignScoreCard(messageContext, objectToAssign,
                objectToAssignScope, scoreCardToAssign, reloadType);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;

    }

    public SecurityValidationResult canAssociateWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            GenericProject genericProject, boolean associate, boolean isDefault)
            throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canAssociateWorkflow(messageContext, process, genericProject,
                associate, isDefault);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canAssociateWorkflow(messageContext, process, genericProject,
                associate, isDefault);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canCancelWorkflow(
            MessageContext messageContext, RunningWorkflowProcess runningProcess)
            throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canCancelWorkflow(messageContext, runningProcess);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canCancelWorkflow(messageContext, runningProcess);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canCheckOut(MessageContext messageContext,
            RPMObject rpmObject, boolean isTimeless) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canCheckOut(messageContext, rpmObject, isTimeless);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canCheckOut(messageContext, rpmObject, isTimeless);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canConvertContainer(
            MessageContext messageContext, RPMObject inputContainer,
            RPMObject outputContainer, RPMObjectScope rpmScope)
            throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canConvertContainer(messageContext, inputContainer,
                outputContainer, rpmScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canConvertContainer(messageContext, inputContainer,
                outputContainer, rpmScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canCopyAssignmentProposedToPlan(
            MessageContext messageContext,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canCopyAssignmentProposedToPlan(messageContext,
                resourceTaskAssignment, scope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canCopyAssignmentProposedToPlan(messageContext,
                resourceTaskAssignment, scope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canCopyProposedToPlan(
            MessageContext messageContext, WorkElement workElement,
            WorkElementScope workElementScope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canCopyProposedToPlan(messageContext, workElement,
                workElementScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canCopyProposedToPlan(messageContext, workElement,
                workElementScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canCreateObjectFromTemplate(
            MessageContext messageContext, RPMObject inputObject,
            RPMObject template, RPMObjectScope scope) throws RPMException
    {
        SecurityValidationResult result = getResourceSecurityController()
            .canCreateObjectFromTemplate(messageContext, inputObject, template,
                scope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getRoleSecurityController()
            .canCreateObjectFromTemplate(messageContext, inputObject, template,
                scope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canCreate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        SecurityValidationResult result = getResourceSecurityController()
            .canCreate(messageContext, rpmObject, rpmObjectScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getRoleSecurityController()
            .canCreate(messageContext, rpmObject, rpmObjectScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canDelete(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canDelete(messageContext, rpmObject);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canDelete(messageContext, rpmObject);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canMoveAndTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canMoveAndTransferUnderWBS(messageContext, workElement,
                abstractScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canMoveAndTransferUnderWBS(messageContext, workElement,
                abstractScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canMoveUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canMoveUnderWBS(messageContext, workElement, abstractScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canMoveUnderWBS(messageContext, workElement, abstractScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canPublishScorecard(
            MessageContext messageContext, AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canPublishScorecard(messageContext, assignedScorecard,
                scorecardScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canPublishScorecard(messageContext, assignedScorecard,
                scorecardScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canRejectTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean rejectEETC,
            boolean rejectForecastDates) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canRejectTaskAssignmentChanges(messageContext, taskAssignment,
                scope, rejectEETC, rejectForecastDates);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canRejectTaskAssignmentChanges(messageContext, taskAssignment,
                scope, rejectEETC, rejectForecastDates);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canReplaceProfileWithResource(
            MessageContext messageContext, ResourceTaskAssignment rta,
            ResourceTaskAssignmentScope rtaScope, Resource resource)
            throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canReplaceProfileWithResource(messageContext, rta, rtaScope,
                resource);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canReplaceProfileWithResource(messageContext, rta, rtaScope,
                resource);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canReplaceResourceWithProfile(
            MessageContext messageContext, ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canReplaceResourceWithProfile(messageContext, assignment, rtaScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canReplaceResourceWithProfile(messageContext, assignment, rtaScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canResumeWorkflow(
            MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canResumeWorkflow(messageContext, runningProcess, scope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canResumeWorkflow(messageContext, runningProcess, scope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canRollupProjectEV(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canRollupProjectEV(messageContext, rpmObject, scope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canRollupProjectEV(messageContext, rpmObject, scope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canUpdate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canUpdate(messageContext, rpmObject, rpmObjectScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canUpdate(messageContext, rpmObject, rpmObjectScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canUpdateOnCreate(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope rpmObjectScope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canUpdateOnCreate(messageContext, rpmObject, rpmObjectScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canUpdateOnCreate(messageContext, rpmObject, rpmObjectScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canSaveProjectBaseline(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canSaveProjectBaseline(messageContext, rpmObject, scope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canSaveProjectBaseline(messageContext, rpmObject, scope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canStartWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            RPMObject object, RunningWorkflowProcessScope scope)
            throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canStartWorkflow(messageContext, process, object, scope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canStartWorkflow(messageContext, process, object, scope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canSuspendWorkflow(
            MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canSuspendWorkflow(messageContext, runningProcess, scope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canSuspendWorkflow(messageContext, runningProcess, scope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canTransferUnderScope(
            MessageContext messageContext, AbstractAggregateScope scopeElement,
            WorkElement workElement) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canTransferUnderScope(messageContext, scopeElement, workElement);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canTransferUnderScope(messageContext, scopeElement, workElement);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }

    public SecurityValidationResult canTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .canTransferUnderWBS(messageContext, workElement, abstractScope);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canTransferUnderWBS(messageContext, workElement, abstractScope);
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }
    
    public SecurityValidationResult canLockDocument(
			MessageContext messageContext, GenericDocument documentToLock)
			throws RPMException 
	{
		SecurityValidationResult result = getRoleSecurityController()
				.canLockDocument(messageContext, documentToLock);
		if (result.type == SecurityValidationResult.FALSE) {
			return result;
		}
		SecurityValidationResult result_2 = getResourceSecurityController()
				.canLockDocument(messageContext, documentToLock);
		if (result_2.type != SecurityValidationResult.UNDEFINED) {
			return result_2;
		}
		return result;
	}
    
    public SecurityValidationResult canUnlockDocument(
			MessageContext messageContext, GenericDocument documentToUnlock)
			throws RPMException 
	{
		SecurityValidationResult result = getRoleSecurityController()
				.canUnlockDocument(messageContext, documentToUnlock);
		if (result.type == SecurityValidationResult.FALSE) {
			return result;
		}
		SecurityValidationResult result_2 = getResourceSecurityController()
				.canUnlockDocument(messageContext, documentToUnlock);
		if (result_2.type != SecurityValidationResult.UNDEFINED) {
			return result_2;
		}
		return result;
	}

    public ArrayList applyFiltering(MessageContext messageContext,
            ArrayList loadedObjects) throws RPMException
    {
        ArrayList filteringResult = SecurityControllerUtil.applyFiltering(
            messageContext, loadedObjects, this);
        if (filteringResult != null)
        {
            return filteringResult;
        }
        return loadedObjects;
    }

    public SecurityValidationResult applyFieldFiltering(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController().canView(
            rpmObject, messageContext);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        SecurityValidationResult result_2 = getResourceSecurityController()
            .canView(rpmObject, messageContext);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result_2;
        }

        getRoleSecurityController().applyFieldFiltering(rpmObject,
            messageContext, true);

        getResourceSecurityController().applyFieldFiltering(rpmObject,
            messageContext, true);

        return (result_2.type == SecurityValidationResult.TRUE) ? result_2
                : result;
    }

    public SecurityValidationResult restoreRelevantFilteredFields(
            RestoreRelevantFilteredFieldsParameter param) throws RPMException
    {
        SecurityValidationResult result = getRoleSecurityController()
            .restoreRelevantFilteredFields(param);
        SecurityValidationResult result_2 = getResourceSecurityController()
            .restoreRelevantFilteredFields(param);
        if (result.type == SecurityValidationResult.FALSE)
        {
            return result;
        }
        if (result_2.type != SecurityValidationResult.UNDEFINED)
        {
            return result_2;
        }
        return result;
    }
}
