package com.ibm.rpm.framework.util;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.RPMObject;

/**
 * Maps the container class to a checkpoint class.
 */
public class ContainerToCheckpointMap
{
    private static Log logger = LogFactory
            .getLog(ContainerToCheckpointMap.class);

    /**
     * Map the container class to a checkpoint instance.
     */
    private static Map containerToCheckpointMap = new HashMap();

    /**
     * Adds an entry to the map.
     * 
     * @param containerClass
     *            Class of the container.
     * @param checkpointClass
     *            Class of the checkpoint. Nullable.
     */
    public static void add(Class containerClass, Class checkpointClass)
    {
        if (checkpointClass != null)
        {
            // Check that containerClass is not already in the map.
            if (containerToCheckpointMap.containsKey(containerClass))
            {
                logger.error(containerClass.getName()
                                + " must be added only once to the ContainerToCheckpointMap.");
            }

            // Instantiates the checkpoint.
            AbstractCheckpoint checkpoint = null;
            try
            {
                checkpoint = (AbstractCheckpoint) checkpointClass.newInstance();
            }
            catch (Exception x)
            {
                logger.error("Cannot instantiate checkpoint "
                        + checkpointClass.getName(), x);
            }

            // Adds the instance to the map.
            if (checkpoint != null)
            {
                containerToCheckpointMap.put(containerClass, checkpoint);
            }
        }
    }

    /**
     * Returns the checkpoint matching the given <code>container</code>.
     * 
     * @param container
     *            Container to be validated.
     * @return Matching checkpoint.
     */
    public static AbstractCheckpoint getCheckpoint(RPMObject container)
    {
        return getCheckpoint(container.getClass());
    }

    /**
     * Returns the checkpoint best matching the given
     * <code>containerClass</code>. Returns the non-null checkpoint of the
     * closest superclass when the <code>containerClass</code> has no specific
     * checkpoint. Returns null if no checkpoint is found.
     * 
     * @param containerClass
     *            Class of the container to be validated.
     * @return Matching checkpoint.
     */
    public static AbstractCheckpoint getCheckpoint(Class containerClass)
    {
        AbstractCheckpoint result = null;

        for (;;)
        {
            result = (AbstractCheckpoint) containerToCheckpointMap
                    .get(containerClass);
            if (result != null)
                break;
            containerClass = containerClass.getSuperclass();
            if (containerClass == null)
                break;
        }

        return result;
    }
}
