package com.ibm.rpm.framework.util;

import java.lang.reflect.Field;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.sql.DataSource;

import com.ibm.rpm.framework.LoadParams;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RpmLog;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.stats.StatsUtil;

/**
 * This class is a collection of methods used to communicate with database. Even
 * though JDBC is supposed to abstract everything, it't doesn't always do
 * everything we need. This class will perform the necessary work to operate on
 * DB2 or Oracle database.
 */
public abstract class Manager
{
//For debug purpose only    
//    public static int openConnections = 0;
//    public static int openStatements = 0;
//    public static int openResultSets = 0;
    
    private static final String CALL_START_STRING = "{call ";
    public static final String DB2_UNCOMMITED_READ_SUFFIX = " WITH UR";

    private static RpmLog logger = RpmLog.getLog();

    private static int oracleCursorCode = 0;

    static {
    	try {
			Class clazz = Class.forName("oracle.jdbc.OracleTypes");
			Field cursor = clazz.getField("CURSOR");
			oracleCursorCode = cursor.getInt(null);
			logger.info("Oracle type CURSOR - Oracle driver loaded successfully");
		} catch (Exception e) {
			// Oracle driver not in the class path.
			// Normal if we are using another database
		}
    }

    public static String makeStatementUncommitedRead(String statement,
            Connection c) throws SQLException
    {
        String retVal = statement;
        if (!isOracle(c))
        {
            retVal += DB2_UNCOMMITED_READ_SUFFIX;
        }
        return retVal;
    }

    public static void makeStatementUncommitedRead(StringBuffer statement,
            Connection c) throws SQLException
    {
        if (!isOracle(c))
        {
            statement.append(DB2_UNCOMMITED_READ_SUFFIX);
        }
    }

    public static void commitConnection(Connection c)
    {
        try
        {
            if (c != null && !c.isClosed())
            {
                c.commit();
            }
        }
        catch (Exception x)
        {
              logger.critical("Could not commit the connection: " + x.toString(), x);
        }
    }
    
    public static void rollbackConnection(Connection c)
    {
        try
        {
            if (c != null && !c.isClosed())
            {
                logger.debug("Rolling back the connection");
                c.rollback();
            }
        }
        catch (Exception x)
        {
            logger.critical("Could not rollback the connection: " + x.toString(), x);
        }
    }
    
    public static void closeConnection(MessageContext messageContext, Connection c, boolean isForceConnectionClose)
    {
        if((messageContext!=null) && (isForceConnectionClose || MessageContext.isConnectionCachingEnabled(messageContext)))
        {
            //close all the cached statements for the current connection
            HashMap allStatementsByConnection = messageContext.getAllStatementsByConnection(c);
            if(allStatementsByConnection!=null)
            {
                Iterator iterValues = allStatementsByConnection.values().iterator();
                while(iterValues.hasNext())
                {                    
                    Set set = (Set) iterValues.next();
                    Iterator it_2 = set.iterator();
                    while (it_2.hasNext())
                    {
                        close((Statement)it_2
                            .next());
                    }                   
                }                
            }

            
            //remove the cached statements
            messageContext.clearStatementsCache(c);
            
            //remove the cached connection
            messageContext.setConnection(null);
        }
        
        try
        {
            if (c != null && !c.isClosed())
            {
//TODO clean up later
//openConnections--;
//System.out.println(" CLOSED A CONNECTION. TOTAL = "+openConnections);
                c.close();
            }
        }
        catch (Exception x)
        {
            logger.critical("Could not close the connection: " + x.toString(),
                x);
        }
    }
    
    
    public static void commitAndCloseConnection(MessageContext messageContext)
    {
        Connection c = (messageContext!=null)?messageContext.getConnection():null;
        commitAndCloseConnection(messageContext,c);
    }
    
    // Should only be called when we want to force a connection close
    // for example, at the end of ManagerCaller methods
    public static void commitAndCloseConnection(MessageContext messageContext, Connection c)
    {
        releaseConnection(messageContext, c, true);
    }
    
    public static void releaseConnection(MessageContext messageContext, Connection c)
    {
        releaseConnection(messageContext, c, false);
    }

    private static void releaseConnection(MessageContext messageContext, Connection c, boolean isForceConnectionClose)
    {
        
        //XXX commitConnection(c);
        
        // Close the connection in one of the following cases :
        // 1- boolean isForceConnectionClose = true => called by
        // commitAndCloseConnection
        // 2- messageContext = null => not in a potential
        // singleConnectionPerContext situation
        // 3- messageContext.isSingleConnectionPerContext is false => not in
        // singleConnectionPerContext mode       
        if (isForceConnectionClose || !MessageContext.isConnectionCachingEnabled(messageContext))
        {
            commitConnection(c);
            closeConnection(messageContext, c, isForceConnectionClose);
        }
    }
    
    /**
     * Initiates a database transaction. <br>
     * When working within a transaction, you should invoke this method first.
     * The connection is returned just in case you need to set the isolation
     * level.
     *
     * @return a non-auto commit connection with the default transaction
     *         isolation level
     */
    public static Connection beginTransaction(Connection c)
    {
        // Connection c = getConnection();
        // c.setAutoCommit(false);
        // trans_conn.set(c);
        return c;
    }

    /**
     * Releases connection used for the transaction and performs a commit or
     * rollback.
     *
     * @param commit
     *            tells whether this connection should be committed true for
     *            commit(), false for rollback()
     */
    public static void endTransaction(boolean commit, Connection c)
            throws SQLException
    {
        // Connection c = (Connection)trans_conn.get();
        if (c == null)
        {
            return;
        }

        try
        {
            if (commit)
            {
                c.commit();
            }
            else
            {
                c.rollback();
            }
        }
        finally
        {
            //c.setAutoCommit(true);
            // trans_conn.set(null);
            closeConnection(null,c, false);
        }
    }

    public static void executeSQL(Connection c, String sql) throws SQLException
    {
        Statement stmt = null;
        try
        {
            stmt = c.createStatement();
            stmt.execute(sql);
        }
        finally
        {
            close(stmt);
        }
    }

    public static String createStoredProcedureCallString(Connection c,
            String name, Object[] args) throws SQLException
    {
        boolean replaceArguments = true;
        return createStoredProcedureCallString(c, name, args, false, replaceArguments);
    }
    
    public static String createStoredProcedureCallString(Connection c,
            String name, Object[] args, boolean isMultiResultSet, boolean replaceArguments) throws SQLException
    {
        boolean isOracle = isOracle(c);
        
        String callstr = null;
        StringBuffer argstr = new StringBuffer();

        argstr.append('(');
        for (int i = 0; i < args.length; i++)
        {
            Object argument = args[i];

            if(isOracle || replaceArguments)
            {
                //In Oracle we replace all '?' by their values
                //For now, only SP_*_PAGES will use '?'
                appendArgument(argstr, argument);                
            }
            else
            {
                //In DB2, we use ? and set the parameter later
                argstr.append(" ? ");
            }

            if (i != args.length - 1)
            {
                argstr.append(',');
            }
        }
        
        //if isOracle and isMultiResultSet we need to add additional ? parameters
        if(isOracle && isMultiResultSet)
        {
            argstr.append(",?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?");
        }
        
        argstr.append(')');
       
        callstr = formCallString(name, argstr.toString(), isOracle);

        return callstr;
    }

    //Called only by loadWithSecurity methods
    public static Object executeStoredProcedure(Connection c, String procedureName,
            Object[] args, int[] spParamsTypes, boolean logArguments, MessageContext messageContext, boolean isMultiResultSet, boolean useDBStatementsForLOBS) throws SQLException,
            RPMException
    {
        //for SP_*_PAGES, if caching is enabled, we DO NOT replace the arguments and we use ? and we set the params later
        boolean isConnectionCachingEnabled = MessageContext.isConnectionCachingEnabled(messageContext);
        boolean replaceArguments = !isConnectionCachingEnabled;
        String callstr = createStoredProcedureCallString(c, procedureName, args, isMultiResultSet, replaceArguments);
        if(replaceArguments)
        {
            args = null;
        }
        return executeStoredProcedure(c, procedureName, callstr, args, spParamsTypes, logArguments, messageContext, isMultiResultSet, useDBStatementsForLOBS);
    }
    
    //used only by loadWithSecurity methods
    private static Object executeStoredProcedure(Connection c,
            String procedureName, String callstr, Object[] args, int[] spParamsTypes, boolean logArguments,
            MessageContext messageContext, boolean isMultiResultSet, boolean useDBStatementsForLOBS) throws SQLException, RPMException
    {
        CallableStatement stmt = null;
        String msgCallString = callstr;
        long spStartTime = 0;

        try{

	        if(!logArguments)
		    {
		            msgCallString = procedureName;
		    }
	        
		    messageContext.addCurrentSPCall(msgCallString);

		    boolean isOracle = isOracle(c);
            
	        if (isOracle)
	        {
                //For now, no caching is supported with oracle
                //All '?' are explicitly replaced by their values
                stmt = c.prepareCall(callstr);
	            stmt.registerOutParameter(1, oracleCursorCode); 
	            if(isMultiResultSet)
	            {
	                for(int i=1;i<=20;i++)
	                {
	                    stmt.registerOutParameter(i+1, oracleCursorCode);
	                }
	            }
	        }
            else
            {
                PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(messageContext,
                    c, true);
                stmt = (CallableStatement)preparedStatementHelper.createPreparedStatement(
                    callstr, args, spParamsTypes, useDBStatementsForLOBS);
            }
	
	        //STATS
            spStartTime = System.currentTimeMillis();

	        if (stmt.execute() || isOracle)
	        {
                ResultSet rs = getResultSetFromLoadWithSecuritySP(stmt, messageContext, isOracle);
                messageContext.reservePreparedStatement(rs,stmt);
                return rs;
	        }
	        else
	        {
	            String[] errorParams =
	            {
	                "Manager",
	                msgCallString
	            };
	            RPMException e = new RPMException(ErrorCodes.AM_NO_RESULT_SET, errorParams, "com.ibm.rpm.framework.util.Manager");
	            throw e;
	        }
        }
        catch (SQLException e)
        {
            //TODO dump only when error code = 911
            //MessageContext.dumpStoredProcedureStack();
            throw e;

        }
        catch (RPMException e)
        {
            //TODO dump only when error code = 911
            //MessageContext.dumpStoredProcedureStack();
            throw e;
        }
        finally
        {
            //STATS
            long spEndTime = System.currentTimeMillis();
            long spDuration = spEndTime-spStartTime;
            StatsUtil.updateSPDuration(messageContext, spDuration);
            logger.debug(msgCallString + " execution time : " + spDuration + " (ms)");
            messageContext.removeCurrentSPCall();
        }

//        return stmt;
    }
    
    
    private static ResultSet getResultSetFromLoadWithSecuritySP(
            CallableStatement callableStatement, MessageContext messageContext,
            boolean isOracle) throws SQLException, RPMException
    {
        ResultSet oracleRsError = null;
        ResultSet rs0 = null;// contains ERROR or ROW_COUNT
        ResultSet rs1 = null;
        try
        {
            //in the case of multi resultsets, the behavior is much different
            //between oracle and db2
            if (isOracle)
            {
                //First resultSet contains the error (always sent in oracle)
                oracleRsError = Manager.getResultSetFromOracleStatement(
                    callableStatement, 1);
                Manager
                    .checkForErrorsAndThrowMessageForOracleMultiResultSet(oracleRsError);

                // First resultSet contains the Row_Count
                rs0 = Manager.getResultSetFromOracleStatement(
                    callableStatement, 2);
                if (rs0 != null)
                {
                    Manager.checkForErrorsAndThrowMessage(rs0, true);

                    if (rs0.next())
                    {
                        int rowCount = rs0.getInt(1);
                        LoadParams loadParams = messageContext.getLoadParams();
                        // we only set the row count once which is for the root
                        // objects
                        if (loadParams != null && !loadParams.isAlreadySet())
                        {
                            loadParams.setRowCount(new Integer(rowCount));
                            loadParams.setAlreadySet(true);
                        }
                    }
                }

                // Second resultSet contains data
                rs1 = Manager.getResultSetFromOracleStatement(
                    callableStatement, 3);
                if (rs1 != null)
                {
                    Manager.checkForErrorsAndThrowMessage(rs1, false);
                }
            }
            else
            //db2
            {
                // First resultSet contains the Row_Count
                rs0 = Manager.getResultSetFromStatement(callableStatement,
                    isOracle);
                Manager.checkForErrorsAndThrowMessage(rs0, true);
                if (rs0 != null)
                {
                    if (rs0.next())
                    {
                        int rowCount = rs0.getInt(1);
                        LoadParams loadParams = messageContext.getLoadParams();
                        // we only set the row count once which is for the root
                        // objects
                        if (loadParams != null && !loadParams.isAlreadySet())
                        {
                            loadParams.setRowCount(new Integer(rowCount));
                            loadParams.setAlreadySet(true);
                        }
                    }
                }

                // Second resultSet contains data
                if (callableStatement.getMoreResults())
                {
                    rs1 = callableStatement.getResultSet();
                    Manager.checkForErrorsAndThrowMessage(rs1, false);
                }
            }
        }
        finally
        {
            try
            {
                if (oracleRsError != null)
                {
                    oracleRsError.close();
                }
                if (rs0 != null)
                {
                    rs0.close();
                }
            }
            catch (Exception ex)
            {

            }
        }

        return rs1;

    }
    
   
    
    public static ResultSet executeStoredProcedure(Connection c, String name,
            Object[] args, boolean logArguments, MessageContext messageContext) throws SQLException,
            RPMException
    {
        return executeStoredProcedure(c, name, args, logArguments, messageContext, false);
    }
   
    public static ResultSet executeStoredProcedure(Connection c, String name,
            Object[] args, boolean logArguments, MessageContext messageContext,
            boolean useDBStatementsForLOBS) throws SQLException, RPMException
    {
        String callstr = createStoredProcedureCallString(c, name, args);
        //Here we always replace '?' by the argument values, so we reset args
        args = null;
        return executeStoredProcedure(c, name, callstr, args, logArguments,
            messageContext, useDBStatementsForLOBS);
    }
    
    public static ResultSet executeStoredProcedure(Connection c,
            String procedureName, String callstr, Object[] args, boolean logArguments,
            MessageContext messageContext, boolean useDBStatementsForLOBS) throws SQLException, RPMException
    {
        ResultSet rs = null;
        CallableStatement stmt = null;
        String msgCallString = callstr;
        long spStartTime = 0;

        try{

	        if(!logArguments)
		    {
		            msgCallString = procedureName;
		    }
	        
		    messageContext.addCurrentSPCall(msgCallString);

		    boolean isOracle = isOracle(c);
	
            if (isOracle)
            {
                //For now, no caching is supported with oracle
                //All '?' are explicitly replaced by their values
                stmt = c.prepareCall(callstr);
                stmt.registerOutParameter(1, oracleCursorCode); 
            }
            else
            {
                PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(messageContext,
                    c, true);
                stmt = (CallableStatement)preparedStatementHelper.createPreparedStatement(
                    callstr, args, null, useDBStatementsForLOBS);
            }
            
	        //STATS
            spStartTime = System.currentTimeMillis();

	        if (stmt.execute() || isOracle)
	        {
	        	rs = getResultSetFromStatement(stmt, isOracle);
                messageContext.reservePreparedStatement(rs,stmt);
	        }
	        else
	        {
	            String[] errorParams =
	            {
	                "Manager",
	                msgCallString
	            };
	            RPMException e = new RPMException(ErrorCodes.AM_NO_RESULT_SET, errorParams, "com.ibm.rpm.framework.util.Manager");
	            throw e;
	        }
        }
        catch (SQLException e)
        {
            //TODO dump only when error code = 911
            //MessageContext.dumpStoredProcedureStack();
            throw e;

        }
        catch (RPMException e)
        {
            //TODO dump only when error code = 911
            //MessageContext.dumpStoredProcedureStack();
            throw e;
        }
        finally
        {
            //STATS
            long spEndTime = System.currentTimeMillis();
            long spDuration = spEndTime-spStartTime;
            StatsUtil.updateSPDuration(messageContext, spDuration);
            logger.debug(msgCallString + " execution time : " + spDuration + " (ms)");
            messageContext.removeCurrentSPCall();
        }

        return rs;
    }
    
    private static boolean ORACLE_CHECK_ERROR = false;

    public static boolean isOracle(Connection c)
    {
        boolean returnValue = false;
        try
        {
            if( c == null )
            {
                logOracleCheckError("null connection");
                return false;
            }
            
            DatabaseMetaData metaData = c.getMetaData();
            if( metaData == null )
            {
                logOracleCheckError("null connection.getMetaData()");
                return false;
            }
            
            String databaseProductName = metaData.getDatabaseProductName();
            if( databaseProductName == null )
            {
                logOracleCheckError("null connection.getMetaData().getDatabaseProductName()");
                return false;
            }
            
            returnValue = databaseProductName.startsWith("Oracle");
        }
        catch (Exception e)
        {
            logger.error("Unexpected error while verifying the database product name.", e);
        }
        return returnValue;
    }

    private static void logOracleCheckError(String message)
    {
        if( ! ORACLE_CHECK_ERROR )
        {
            // only log this error once.
            ORACLE_CHECK_ERROR = true;
            logger.warn("Unexpected error while verifying the database product name:" + message);
        }
    }

    public static boolean isOracle(MessageContext msgCtx) throws SQLException
    {
        boolean result = false;
        Connection c = null;
        try
        {
            c = Manager.getConnection(msgCtx);
            result = isOracle(c);
        }
        finally
        {
            Manager.releaseConnection(msgCtx, c);
        }
        return result;
    }

    private static String formCallString(String name, String argstr,
            boolean isOracle)
    {
        if (isOracle)
        {
            return formOracleCallString(name, argstr);
        }
        return formCallString(name, argstr);
    }

    private static String formCallString(String name, String argstr)
    {
        String callstr;
        callstr = CALL_START_STRING + name + argstr + "}";
        return callstr;
    }

    private static String formOracleCallString(String name, String argstr)
    {
        String callstr = "{? = call " + name + argstr + "}";

        return callstr;
    }

    private static void appendArgument(StringBuffer argstr, Object argument)
    {
        if (argument instanceof String)
        {
            argstr.append('\'');
            argstr.append(((String) argument).replaceAll("'", "''"));
            argstr.append('\'');
        }
        else if (argument instanceof Date)
        {
            argstr.append('\'');
            argstr.append(argument);
            argstr.append('\'');
        }
        else if (argument instanceof Boolean)
        {
            if (((Boolean) argument).booleanValue())
            {
                argstr.append('1');
            }
            else
            {
                argstr.append('0');
            }
        }
        else
        {
            if (argument == null)
            {
                argstr.append("NULL");
            }
            else
            {
                argstr.append(argument);
            }
        }
    }

    public static ResultSet getResultSetFromStatement(
            CallableStatement statement, boolean isOracle) throws RPMException,
            SQLException
    {
        if (isOracle)
        {
            return getResultSetFromOracleStatement(statement);
        }        
        return statement.getResultSet();
    }

    public static ResultSet getResultSetFromOracleStatement(
            CallableStatement statement) throws RPMException
    {
        return getResultSetFromOracleStatement(statement,1);
    }
    
    public static ResultSet getResultSetFromOracleStatement(
            CallableStatement statement, int index) throws RPMException
    {
        ResultSet rs = null;
        try
        {
        	rs = (ResultSet)statement.getObject(index);
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(Manager.class, e);
        }

        return rs;
    }
    
    ////////////////////////////////////////////////////
    // Utils method
    ////////////////////////////////////////////////////

    /**
     * Closes the passed Statement.
     */
    private static void close(Statement s)
    {
        try
        {
            if (s != null)
            {
                s.close();
//TODO clean up later
//openStatements--;
//System.out.println(" CLOSED A STATEMENT. TOTAL = "+openStatements);
            }
        }
        catch (Exception x)
        {
            //log the error
            logger.critical("Could not close statement!: " + x.toString(), x);
        }
    }

    /**
     * Closes the passed ResultSet and calls close(Statement s)
     */
    public static void close(MessageContext messageContext, ResultSet rs)
    {
        Statement statement = null;
        try
        {
            if (rs != null)
            {
                statement = rs.getStatement();
                messageContext.unReservePreparedStatement(rs);
                rs.close();
//TODO clean up later
//openResultSets--;
//System.out.println(" CLOSED A RESULTSET. TOTAL = "+openResultSets);
            }
        }
        catch (Exception x)
        {
            //Do not log anything here
        }
        finally
        {
            if(!MessageContext.isConnectionCachingEnabled(messageContext))
            {
                close(statement);                
            }
        }
    }

    /**
     * Retrieves an int value from the passed result set as an Integer object.
     */
    public static Integer getInteger(ResultSet rs, int pos) throws SQLException
    {
        int i = rs.getInt(pos);
        return rs.wasNull() ? (Integer) null : new Integer(i);
    }

    /**
     * Retrieves an int value from the passed result set as an Integer object.
     */
    public static Calendar getCalendar(ResultSet rs, int pos)
            throws SQLException
    {
        Date date = rs.getDate(pos);
        
        return rs.wasNull() ? (Calendar) null : DateUtil.convertToCalendar(date);
    }

    /**
     * Retrieves an int value from the passed result set as an Integer object.
     */
    public static Calendar getTimestamp(ResultSet rs, int pos)
            throws SQLException
    {
        Timestamp time = rs.getTimestamp(pos);
        return rs.wasNull() ? (Calendar) null : DateUtil.convertToCalendar(time);
    }

    /**
     * Set an Integer object to the passed prepared statement as an int or as
     * null.
     */
    public static void setInteger(PreparedStatement ps, int pos, Integer i)
            throws SQLException
    {
        if (i == null)
        {
            ps.setNull(pos, Types.INTEGER);
        }
        else
        {
            ps.setInt(pos, i.intValue());
        }
    }

    /**
     * Retrieves a float value from the passed result set as a Float object.
     */
    public static Float getFloat(ResultSet rs, int pos) throws SQLException
    {
        float f = rs.getFloat(pos);
        return rs.wasNull() ? (Float) null : new Float(f);
    }

    /**
     * Set a Float object to the passed prepared statement as a float or as
     * null.
     */
    public static void setFloat(PreparedStatement ps, int pos, Float f)
            throws SQLException
    {
        if (f == null)
        {
            ps.setNull(pos, Types.FLOAT);
        }
        else
        {
            ps.setFloat(pos, f.floatValue());
        }
    }

    /**
     * Retrieves a double value from the passed result set as a Double object.
     */
    public static Double getDouble(ResultSet rs, int pos) throws SQLException
    {
        double d = rs.getDouble(pos);
        return rs.wasNull() ? (Double) null : new Double(d);
    }

    /**
     * Set a Double object to the passed prepared statement as a double or as
     * null.
     */
    public static void setDouble(PreparedStatement ps, int pos, Double d)
            throws SQLException
    {
        if (d == null)
        {
            ps.setNull(pos, Types.DOUBLE);
        }
        else
        {
            ps.setDouble(pos, d.doubleValue());
        }
    }

    /**
     * @param rs
     * @throws SQLException
     */
    public static void checkForErrorsAndThrowMessage(ResultSet rs, boolean checkForTotalRowNum) throws SQLException, RPMException
    {
        // We use metadata to determine if the SP returned an ERROR column
        // We only move the cursor if there is an error so that we don't have
        // to reposition it back before the first row before calling
        // loadByResultSet
        ResultSetMetaData rsmd = rs.getMetaData();
        String columnName = rsmd.getColumnName(1);
        if ("ERROR".equalsIgnoreCase(columnName))
        {
            if (rs.next())
            {
                Object object = rs.getObject(2);
                String message = null;
                if(object!=null)
                {
                    message = object.toString();
                }
                else
                {
                    message = "Error getting message";
                }
                RPMException e = new RPMException(message);
                throw e;
            }
        }
        else if (checkForTotalRowNum
                && !"TOTAL_ROW_NUMBER".equalsIgnoreCase(columnName))
        {
            RPMException e = new RPMException("Error getting resultset");
            throw e;
        }
    }
    
    //in oracle multi resultset context, we are not getting a message column
    //instead we have an ID column
    public static void checkForErrorsAndThrowMessageForOracleMultiResultSet(ResultSet rs) throws SQLException, RPMException
    {
        ResultSetMetaData rsmd = rs.getMetaData();
		String columnName = rsmd.getColumnName(1);
		if ("ERROR".equalsIgnoreCase(columnName))
		{
			if (rs.next()) {
		        int error = rs.getInt("ERROR");
		        if(error!=0)
		        {
                    Object object = rs.getObject(2);
                    String ID = null;
                    if(object!=null)
                    {
                        ID = object.toString();
                    }
                    else
                    {
                        ID = "Error getting ID";
                    }
					RPMException e = new RPMException(ID);
					throw e;
		        }
			}
		}
    }
    
    /**
     * @param rs
     * @throws SQLException
     */
    public static int checkForError(ResultSet rs) throws SQLException
    {
        int error = rs.getInt("ERROR");
        return rs.wasNull() ? 0 : error;
    }

    /**
     * Retrieves a long value from the passed result set as a Long object.
     */
    public static Long getLong(ResultSet rs, int pos) throws SQLException
    {
        long l = rs.getLong(pos);
        return rs.wasNull() ? (Long) null : new Long(l);
    }

    /**
     * Set a Long object to the passed prepared statement as a long or as null.
     */
    public static void setLong(PreparedStatement ps, int pos, Long l)
            throws SQLException
    {
        if (l == null)
        {
            ps.setNull(pos, Types.BIGINT);
        }
        else
        {
            ps.setLong(pos, l.longValue());
        }
    }

    /**
     * Retrieves a boolean value from the passed result set as a Boolean object.
     * Converts several representation to true or false for Oracle.
     */
    public static Boolean getBoolean(ResultSet rs, int pos) throws SQLException
    {
        String test = rs.getString(pos);
        boolean b = test != null
                && (test.equals("1") || test.equalsIgnoreCase("yes")
                        || test.equalsIgnoreCase("y")
                        || test.equalsIgnoreCase("true") 
                        || test.equalsIgnoreCase("t"));
        return rs.wasNull() ? (Boolean) null : new Boolean(b);
    }

    /**
     * Set a Boolean object to the passed prepared statement as a boolean or as
     * null.
     */
    public static void setBoolean(PreparedStatement ps, int pos, Boolean b)
            throws SQLException
    {
        if (b == null)
        {
            ps.setNull(pos, Types.BOOLEAN);
        }
        else
        {
            ps.setBoolean(pos, b.booleanValue());
        }
    }

    /**
     * Gets a connection.
     *
     * @throws SQLException
     */
    public static Connection getConnection(MessageContext messageContext)
            throws SQLException
    {
        Connection connection = null;
        
        if(MessageContext.isConnectionCachingEnabled(messageContext))
        {
            connection = messageContext.getConnection();
        }

        if(connection==null || connection.isClosed())
        {
            DataSource dataSource = messageContext.getDatasource();

            try
            {
                //stats
                long createConnectionStartTime = System.currentTimeMillis();

                connection = getConnectionFromDatasource(dataSource);
                
                //set the context connection
                if(MessageContext.isConnectionCachingEnabled(messageContext))
                {
                    messageContext.setConnection(connection);
                }
                
                long createConnectionEndTime = System.currentTimeMillis();
                StatsUtil.updateCreateConnectionDuration(messageContext, createConnectionEndTime-createConnectionStartTime);
            }
            catch (SQLException e)
            {
                int error = e.getErrorCode();
                if (ManagerUtil.canRetryDBCall(error, messageContext))
                {
                    logger
                        .debug("Retrying Datasource connection request on DB error: "
                                + error);
                    return getConnection(messageContext);
                }
                throw e;
            }            
        }

        return connection;
    }

    public static Connection getConnectionFromDatasource(DataSource dataSource)
            throws SQLException
    {
        Connection connection = dataSource.getConnection();
        //for DB2 v9 support
        connection.setAutoCommit(false);
        
        if (isOracle(connection))
        {
            executeSQL(connection, "ALTER SESSION SET NLS_TERRITORY = AMERICA");
            executeSQL(connection,
                "ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD'");
            executeSQL(connection,
                "ALTER SESSION SET NLS_TIMESTAMP_FORMAT = 'YYYY-MM-DD hh24:mi:ss.ff6'");
            executeSQL(connection,
                "ALTER SESSION SET NLS_TIMESTAMP_TZ_FORMAT = 'YYYY-MM-DD hh24:mi:ss.ff6'");
            executeSQL(connection,
                "ALTER SESSION SET NLS_NUMERIC_CHARACTERS = '.,'");
            executeSQL(connection, "ALTER SESSION SET NLS_SORT = BINARY");
        }

//TODO clean up later
//openConnections++;
//System.out.println(" OPENED A NEW CONNECTION. TOTAL = "+openConnections);
        
        return connection;
    }

}
