package com.ibm.rpm.interfaces.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.util.AboutProperties;
import com.ibm.rpm.framework.util.SystemInfo;

/**
 * @author legaults
 */
public class DatabaseCompatibility
{
    private static final String FAILED_TO_FETCH_DATABASE_VERSION = "Failed to fetch database version.";

    private static final String FAILED_TO_FETCH_API_VERSION = "Failed to fetch API version.";

    public static final int UNKNOWN = 0;

    public static final int INCOMPATIBLE = 1;

    public static final int PARTIALLY_COMPATIBLE = 2;

    public static final int COMPATIBLE = 3;

    private static final String UNKNOWN_STR = "'unknown'";

    private static final char DOT = '.';

    private int databaseCompatibilityLevel = UNKNOWN;

    private String apiVersion = UNKNOWN_STR;

    private String databaseVersion = UNKNOWN_STR;
    
    private String message;

    private static Log logger = LogFactory.getLog(DatabaseCompatibility.class);
    
    public DatabaseCompatibility(MessageContext context)
    {
        message = null;
        boolean alreadyFailed = false;

        try
        {
            apiVersion = AboutProperties.getInstance().getString(
            	"com.ibm.rpm.ws.build.version", context);
        }
        catch (Exception e1)
        {
            alreadyFailed = true;
            databaseCompatibilityLevel = INCOMPATIBLE;
            logger.error(FAILED_TO_FETCH_API_VERSION
                    + e1.getMessage(), e1);
            message = FAILED_TO_FETCH_API_VERSION + " " + e1.getMessage();
        }
        
        try
        {
            databaseVersion = SystemInfo.getDatabaseVersion(context);
        }
        catch (Exception e)
        {
            alreadyFailed = true;
            databaseCompatibilityLevel = INCOMPATIBLE;
            logger.error(FAILED_TO_FETCH_DATABASE_VERSION
                    + e.getMessage(), e);
            message = FAILED_TO_FETCH_DATABASE_VERSION + " " + e.getMessage();
        }

        if( alreadyFailed )
        {
            // nothing to do here
        }
        else if( !context.getFactory().getSession().isSessionTableFunctionnal())
        {
            databaseCompatibilityLevel = INCOMPATIBLE;
            message = "Internal error: Session Table incompatible.";
        }
        else if( databaseVersion == null )
        {
            databaseCompatibilityLevel = INCOMPATIBLE;
            message = FAILED_TO_FETCH_DATABASE_VERSION + " SP_GET_VERSION returned null.";
        }
        else if( apiVersion == null )
        {
            databaseCompatibilityLevel = UNKNOWN;
            message = FAILED_TO_FETCH_API_VERSION + " API version returned null.";
        }
        else if (apiVersion.startsWith(databaseVersion))
        {
            databaseCompatibilityLevel = COMPATIBLE;
        }
        else
        {
            // Check the major and minor version numbers.
            String databaseMajorMinor = getMajorAndMinor(databaseVersion);
            String apiMajorMinor = getMajorAndMinor(apiVersion);

            if (databaseMajorMinor != null
                    && databaseMajorMinor.equals(apiMajorMinor))
            {
                databaseCompatibilityLevel = PARTIALLY_COMPATIBLE;
                message = "Minor version mismatch between API and Database.";
            }
            else
            {
                databaseCompatibilityLevel = INCOMPATIBLE;
                message = "Major version mismatch between API and Database.";
            }
        }

        if (databaseVersion == null)
        {
            databaseVersion = UNKNOWN_STR;
        }
        if (apiVersion == null)
        {
            apiVersion = UNKNOWN_STR;
        }

    }

    private String getMajorAndMinor(String version)
    {
        String result = null;
        int firstDot = version.indexOf(DOT);
        if (firstDot >= 0)
        {
            int secondDot = version.indexOf(DOT, firstDot + 1);
            if (secondDot >= 0)
            {
                result = version.substring(0, secondDot);
            }
        }
        return result;
    }

    /**
     * Returns a code that states the compatibility of the WS API with the
     * database. Can be: UNKNOWN; INCOMPATIBLE; PARTIALLY_COMPATIBLE;
     * COMPATIBLE.
     */
    public int getLevel()
    {
        return databaseCompatibilityLevel;
    }

    /**
     * Returns a message that states the compatibility of the WS API with the
     * database.
     */
    public String getMessage()
    {
        StringBuffer msgBuffer = new StringBuffer();
        msgBuffer.append("Web Services API ");
        msgBuffer.append(apiVersion);
        msgBuffer.append(" and Database ");
        msgBuffer.append(databaseVersion);
        
        if (databaseCompatibilityLevel == COMPATIBLE)
        {
            msgBuffer.append(" are compatible.");
        }
        else if (databaseCompatibilityLevel == PARTIALLY_COMPATIBLE)
        {
            msgBuffer.append(" are PARTIALLY COMPATIBLE.");
        }
        else if (databaseCompatibilityLevel == INCOMPATIBLE)
        {
            msgBuffer.append(" are INCOMPATIBLE.");
        }
        else
        {
            msgBuffer.append(" compatibility is UNKNOWN.");
        }
        
        if( message != null )
        {
            msgBuffer.append(" ");
            msgBuffer.append(message);
        }
        
        
        return msgBuffer.toString();
    }

    /**
     * Logs getMessage() with the right severity level.
     */
    public void log()
    {
        String message = getMessage();
        if (databaseCompatibilityLevel == COMPATIBLE)
        {
            logger.info(message);
        }
        else if (databaseCompatibilityLevel == PARTIALLY_COMPATIBLE)
        {
            logger.warn(message);
        }
        else
        {
            logger.error(message);
        }
    }
}
