/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.util.FinancialUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Impact;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.OrderOfMagnitudeCostBenefits;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

/**
 * @author dominiqu
 */
public class AbstractScopeCheckpoint extends AbstractAggregateScopeCheckpoint
{
    private static final List ABSTRACTSCOPE_PARENTS = new ArrayList();

    static
    {
        ABSTRACTSCOPE_PARENTS.add(ScopeFolder.class);
        ABSTRACTSCOPE_PARENTS.add(Action.class);
        ABSTRACTSCOPE_PARENTS.add(Issue.class);
        ABSTRACTSCOPE_PARENTS.add(ChangeRequest.class);
        ABSTRACTSCOPE_PARENTS.add(Defect.class);
        ABSTRACTSCOPE_PARENTS.add(Requirement.class);
        ABSTRACTSCOPE_PARENTS.add(Risk.class);
        ABSTRACTSCOPE_PARENTS.add(ServiceRequest.class);
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, msgCtx, true, ABSTRACTSCOPE_PARENTS);

        // Initialize
        AbstractScope mainAbstractScope = (AbstractScope) rpmObject;
        ScopeElementScope scope = (ScopeElementScope) rpmScope;

        // Field validation: approvedByName
        if (mainAbstractScope.testApprovedByNameModified())
        {
        	if(mainAbstractScope.getApprovedByName() != null)
        	{
        	 GenericValidator.validateMaxLength(mainAbstractScope,
                     ValidationConstants.APPROVED_BY_NAME_FIELD, mainAbstractScope
                         .getApprovedByName(), ValidationConstants.APPROVED_BY_NAME_MAX, msgCtx);
        	}
        }

        // Field validation: externalReferenceNumber
        if (mainAbstractScope.testExternalReferenceNumberModified())
        {
            GenericValidator
                .validateMaxLength(
                    mainAbstractScope,
                    ValidationConstants.ABSTRACTSCOPE_EXTERNAL_REFERENCE_NUMBER_FIELD,
                    mainAbstractScope.getExternalReferenceNumber(),
                    ValidationConstants.EXTERNALREFERENCENUMBER_MAX, msgCtx);
        }
        

        // Container validation: impact
        if (scope != null && scope.isImpact())
        {
            Impact impact = mainAbstractScope.getImpact();
            if( impact == null )
            {
                if ( mainAbstractScope.getID() != null )
                {
                    // field can't be reset to null after object creation
                    GenericValidator.validateNotNull(mainAbstractScope, "impact", impact, msgCtx);
                }
            }
            else
            {
                ImpactCheckpoint.getInstance().validateSave(
                    impact, msgCtx);
            }
        }

        //ClientCostCenter Validation: An unsaved ClientCostCenter cant be assigned.
        if(scope != null && scope.isClientCostCenter())
        {
        	ClientCostCenter ccc = mainAbstractScope.getClientCostCenter();
        	if(ccc != null)
        	{
        		if(ccc.getID() == null)
        		{
        			RPMException e = new RPMException(
                          ErrorCodes.CANNOT_ASSIGN_UNSAVED_CLIENT_COST_CENTER,
                          null, mainAbstractScope.getClass().getName(),
                          ValidationConstants.CLIENT_COST_CENTER_FIELD,
                          mainAbstractScope.getID());
                      addException(e, msgCtx);
        		}
        	}
        }
        
        //Asset Validation: An unsaved Asset cant be assigned
        if(scope != null && scope.isAsset())
        {
        	Asset asset = mainAbstractScope.getAsset();
        	if(asset != null)
        	{
        		if(asset.getID() == null)
        		{
        			RPMException e = new RPMException(
                            ErrorCodes.CANNOT_ASSIGN_UNSAVED_ASSET,
                            null, mainAbstractScope.getClass().getName(),
                            ValidationConstants.ASSET_FIELD,
                            mainAbstractScope.getID());
                        addException(e, msgCtx);
        		}
        	}
        }
        
        // Container validation: orderOfMagnitudeCostBenefits
        if (scope != null && scope.isOrderOfMagnitudeCostBenefits())
        {
            OrderOfMagnitudeCostBenefits orderOfMagnitudeCostBenefits = mainAbstractScope.getOrderOfMagnitudeCostBenefits();
            if( orderOfMagnitudeCostBenefits == null )
            {
                if ( mainAbstractScope.getID() != null )
                {
                    // field can't be reset to null after object creation
                    GenericValidator.validateNotNull(mainAbstractScope, "orderOfMagnitudeCostBenefits", orderOfMagnitudeCostBenefits, msgCtx);
                }
            }
            else
            {
                OrderOfMagnitudeCostBenefitsCheckpoint.getInstance().validateSave(
                    orderOfMagnitudeCostBenefits, msgCtx);
            }
        }

        // Field validation: rank
        if (mainAbstractScope.testRankModified())
        {
            GenericValidator.validateRange(mainAbstractScope,
                ValidationConstants.ABSTRACTSCOPE_RANK_FIELD, mainAbstractScope
                    .getRank(), ValidationConstants.RANK_MIN,
                ValidationConstants.RANK_MAX, msgCtx);
        }

        // Field validation: task
        if (mainAbstractScope.getTask() != null
                && mainAbstractScope.getTask().booleanValue())
        {
            // children field must be null when converting to a Task,
            // or when saving an AbstractScopeTask
            GenericValidator.validateMustBeNull(mainAbstractScope,
                ValidationConstants.CHILDREN_FIELD, mainAbstractScope
                    .getChildren(), " the task flag is set", msgCtx);


            //Should not be able to save a ScopeElement that has been set to Task
            //with Financials Value.
            if (scope != null && scope.getWorksheetFinancials() != null)
            {
                if (mainAbstractScope.getID() == null
                        && mainAbstractScope.getWorksheetFinancials() != null) // CREATE
                {
                    GenericValidator.validateCannotSave(mainAbstractScope,
                        msgCtx);
                }
                else
                // UPDATE
                {
                    if(mainAbstractScope.getWorksheetFinancials() != null)
                    {
                        WorksheetFinancial[] financials = mainAbstractScope.getWorksheetFinancials();
                        
                        for(int i=0; i<financials.length; i++ )
                        {
                            if(FinancialUtil.isFinancialLineModified(financials[i]))
                            {
                                GenericValidator.validateCannotSave(mainAbstractScope,
                                    msgCtx);
                            }
                        }
                    }
                }
            }
                    

        }

        // Field validation: defaultScope 
        // A task cannot be set to default
        if (mainAbstractScope.testDefaultScopeModified())
        { 
            GenericValidator.validateBooleanDependancy(mainAbstractScope,
                "task", mainAbstractScope.getTask(), true, "defaultScope",
                mainAbstractScope.getDefaultScope(), false, msgCtx);
        }
        
        // Field validation: task
        // A default AbstractScope object cannot be converted to a task
        if (mainAbstractScope.testTaskModified())
        { 
            GenericValidator.validateBooleanDependancy(mainAbstractScope,
                "defaultScope", mainAbstractScope.getDefaultScope(), true,
                "task", mainAbstractScope.getTask(), false, msgCtx);
        }
    }
    
    
    public void validateAbstractScopeCreation(AbstractScope inputObject,
            MessageContext msgCtx)
    {
        // validate template is in fact a template
        if (((AbstractScope) msgCtx.getTemplate()).getPublished() == null
                || !(PublishedType.ToTemplate.getValue()
                    .equals(((AbstractScope) msgCtx.getTemplate())
                        .getPublished().getValue())))
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(inputObject.getClass()) + " "
                        + ValidationConstants.TEMPLATE_FIELD

            };
            // invalid project template
            RPMException e = new RPMException(
                ErrorCodes.INVALID_OBJECT_TEMPLATE_TYPE, errorParams,
                inputObject.getClass().getName(),
                ValidationConstants.TEMPLATE_FIELD, inputObject.getID());
            addException(e, msgCtx);
        }
        else
        {
            validateAbstractScopeReadOnlyFields(inputObject, msgCtx);

        }
    }
 
    
    /**
	 * @param inputObject
	 * @param msgCtx
	 */
	protected void validateAbstractScopeReadOnlyFields(AbstractScope inputObject, MessageContext msgCtx) {		
		
		//Action specific fields validation
		if(inputObject instanceof Action)
		{
			Action action = (Action)inputObject;
			
			//Field validation: Approved Cost Benefits
			if (action.testApprovedCostBenefitsModified())
			{
			    GenericValidator.validateReadOnly(action,
			        ValidationConstants.APPROVED_COST_BENEFITS_FIELD, msgCtx);
			}
		}
		
		
		//Issue specific fields validation
		if(inputObject instanceof Issue)
		{
			Issue issue = (Issue)inputObject;
			
			//Field validation: Number Times Reassigned
			if (issue.testNumberTimesReassignedModified())
			{
			    GenericValidator.validateReadOnly(issue,
			        ValidationConstants.NUMBER_TIMES_REASSIGNED_FIELD, msgCtx);
			}
			
			//Field validation: Occured Date Time
			if (issue.testOccuredDateTimeModified())
			{
			    GenericValidator.validateReadOnly(issue,
			        ValidationConstants.OCCURED_DATE_TIME_FIELD, msgCtx);
			}

		}
		
		//Requirement specific fields validation
		if(inputObject instanceof Requirement)
		{
			Requirement requirement = (Requirement)inputObject;
			
			//Field validation: Approved Cost Benefits
			if (requirement.testApprovedCostBenefitsModified())
			{
			    GenericValidator.validateReadOnly(requirement,
			        ValidationConstants.APPROVED_COST_BENEFITS_FIELD, msgCtx);
			}
		}
		
		
		//Risk specific fields validation
		if(inputObject instanceof Risk)
		{
			Risk risk = (Risk)inputObject;
			
			//Field validation: Occured Date Time
			if (risk.testOccuredDateTimeModified())
			{
			    GenericValidator.validateReadOnly(risk,
			        ValidationConstants.OCCURED_DATE_TIME_FIELD, msgCtx);
			}
			
			//Field validation: Risk Matrix
			if (risk.testRiskMatrixModified())
			{
			    GenericValidator.validateReadOnly(risk,
			        ValidationConstants.RISK_MATRIX_FIELD, msgCtx);
			}

		}
		
		
		
		//-----Common AbstractScope fiels validation
		
		//Field validation: Approved By Date
		if (inputObject.testApprovedByDateModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.APPROVED_BY_DATE_FIELD, msgCtx);
		}
		
		
		//Field validation: Approved By Name
		if (inputObject.testApprovedByNameModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.APPROVED_BY_NAME_FIELD, msgCtx);
		}
		
		
		//Field validation: Approved Date
		if (inputObject.testApprovedDateModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.APPROVED_DATE_FIELD, msgCtx);
		}
		
		//Field validation: Asset Code
		if (inputObject.testAssetCodeModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.ASSET_CODE_FIELD, msgCtx);
		}
		
		//Field validation: Asset
		if (inputObject.testAssetModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.ASSET_FIELD, msgCtx);
		}
		
		//Field validation: Assigned Score Card
		if (inputObject.testAssignedScorecardModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.ASSIGNED_SCORECARD_FIELD, msgCtx);
		}
		
		//Field validation: Client Cost Center
		if (inputObject.testClientCostCenterModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.CLIENT_COST_CENTER_FIELD, msgCtx);
		}
		
		//Field validation: Default Scope
		if (inputObject.testDefaultScopeModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.DEFAULT_SCOPE_FIELD, msgCtx);
		}
		
		//Field validation: Document Folder
		if (inputObject.testDocumentFolderModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.DOCUMENT_FOLDER_FIELD, msgCtx);
		}
		
		//Field validation: Duplicate Number
		if (inputObject.testDuplicateNumberModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.DUPLICATE_NUMBER_FIELD, msgCtx);
		}
		
		//Field validation: External Reference Number
		if (inputObject.testExternalReferenceNumberModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.EXTERNAL_REFERENCE_NUMBER_FIELD, msgCtx);
		}
		
		//Field validation: Impact
		if (inputObject.testImpactModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.IMPACT_FIELD, msgCtx);
		}
		
		//Field validation: Order Of Magnitude Cost Benefits
		if (inputObject.testOrderOfMagnitudeCostBenefitsModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.ORDER_OF_MAGNITUDE_COSTBENEFITS_FIELD, msgCtx);
		}
		
		//Field validation: Published
		if (inputObject.testPublishedModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.PUBLISHED_FIELD, msgCtx);
		}
		
		//Field validation: Rank
		if (inputObject.testRankModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.RANK_FIELD, msgCtx);
		}
		
		//Field validation: State
		if (inputObject.testStateModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.STATE_FIELD, msgCtx);
		}
		
		//Field validation: Target Date
		if (inputObject.testTargetDateModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.TARGET_DATE_FIELD, msgCtx);
		}

	}

}