/*
 * Created on 1-Jun-2006
 * @author mmatwiy
 * Checkpoint validation for Scope Management application methods.
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.checkpoints.ScopeCheckpoint;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsUtil;

public class ScopeMethodCheckpoint extends AbstractCheckpoint
{
    protected static ScopeMethodCheckpoint instance = new ScopeMethodCheckpoint();

    public static ScopeMethodCheckpoint getInstance()
    {
        return instance;
    }

    public void validateTransferAbstractScopeID(AbstractScope abstractScope,
            MessageContext msgCtx)
    {
        GenericValidator.validateMandatoryID("AbstractScope", abstractScope,
            msgCtx);
    }

    public void validateScopeObjectMove(ScopeElement source,
            ScopeElement target, MessageContext messageContext)
    {
        GenericValidator.validateMandatoryID("ScopeElement", source,
            messageContext);
        GenericValidator.validateMandatoryID("ScopeElement", target,
            messageContext);

        // determine if the scope is moved within the same project
        // or move to another project.
        try
        {
            String sourceProjectID;
            if (source.getProject() != null
                    && source.getProject().getID() != null)
            {
                sourceProjectID = source.getProject().getID();
            }
            else
            {
                SqlBuffer condition = new SqlBuffer();
                condition.appendEqualQuestionMark(ScopeManager.NAME_ELEMENT_ID);
                
                Object[] arguments = new Object[]
                {
                    source.getID()
                };
                
                sourceProjectID = (String) ManagerUtil.selectColumnValue(
                    Object.class, null, null, ScopeManager.NAME_PROJECT_ID,
                    ScopeManager.TABLE_NAME, condition, arguments, messageContext);
            }

            String targetProjectID;
            if (target.getProject() != null
                    && target.getProject().getID() != null)
            {
                targetProjectID = target.getID();
            }
            else
            {
                SqlBuffer condition = new SqlBuffer();
                condition.appendEqualQuestionMark(ScopeManager.NAME_ELEMENT_ID);
                
                Object[] arguments = new Object[]
                {
                    target.getID()
                };
                
                targetProjectID = (String) ManagerUtil.selectColumnValue(
                    Object.class, null, null, ScopeManager.NAME_PROJECT_ID,
                    ScopeManager.TABLE_NAME, condition, arguments, messageContext);
            }

            if (!targetProjectID.equals(sourceProjectID))
            {
                // Move outside the current project
                addException(new RPMException(
                    ErrorCodes.CANNOT_MOVE_FROM_PROJECT_THROUGH_PARENT),
                    messageContext);
            }
        }
        catch (Exception e)
        {
            addException(new RPMException(e), messageContext);
        }
    }

    public void validateTransferAndMoveIDs(WorkElement workElement,
            AbstractScope abstractScope, MessageContext msgCtx)
    {
        // the workelement must have an ID
        GenericValidator
            .validateMandatoryID("WorkElement", workElement, msgCtx);

        // the AbstractScope's project must have an ID
        GenericValidator.validateMandatoryID(abstractScope,
            ValidationConstants.SCOPEELEMENT_PROJECT_FIELD, abstractScope
                .getProject(), msgCtx);
    }

    public void validateTransferUnderWbs(WorkElement workElement,
            AbstractScope abstractScope, MessageContext msgCtx)
            throws RPMException
    {
        String projectId = WbsUtil.getWorkElementProjectID(workElement, msgCtx);

        if (abstractScope != null && abstractScope.getProject() != null)
        {
            if (!(projectId.equals(abstractScope.getProject().getID())))
            {
                RPMException e = new RPMException(
                    ErrorCodes.CANNOT_CROSS_TRANSFER_WITHOUT_MOVE, workElement
                        .getClass().getName());
                addException(e, msgCtx);
            }
        }
    }

    public final boolean canConvertScopeElement(
            AbstractScope inputScopeElement, RPMObject outputScopeElement,
            RPMObjectScope rpmScope, MessageContext msgCtx)
    {
        try
        {
            return (ISecurityController.INSTANCE.canConvertContainer(msgCtx,
                inputScopeElement, outputScopeElement, rpmScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e), msgCtx);
            return false;
        }
    }

   

    public void validateConvertScopeElement(AbstractScope inputScopeElement,
            RPMObject outputScopeElement, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // the inputScopeElement must be previously saved
        GenericValidator.validateMandatoryID("AbstractScope",
            inputScopeElement, msgCtx);

        // make sure the scope is of the correct type
        ScopeCheckpoint.getInstance().isValid(inputScopeElement, rpmScope,
            msgCtx);

        // An AbstractScope can be converted to an AbstractScope.
        // To convert an AbstractScope to an AbstractScopeTask, the user
        // must set the task flag and save the container.
        if (!(outputScopeElement instanceof AbstractScope))
        {
            GenericValidator.validateConversionType(inputScopeElement,
                outputScopeElement, msgCtx);
        }
        else
        {
            AbstractScope outputAbstractScope = (AbstractScope) outputScopeElement;

            if (inputScopeElement.getTask() != null
                    && inputScopeElement.getTask().booleanValue())
            {
                // can't convert from an [AbstractScope]Task to a non-task AbstractScope
                if (outputAbstractScope.getTask() == null
                        || !(outputAbstractScope.getTask().booleanValue()))
                {
                    String[] errorParams =
                    {
                            StringUtil.getShortClassName(inputScopeElement
                                .getClass().getName())
                                    + " Task",
                            StringUtil.getShortClassName(outputScopeElement
                                .getClass().getName()),
                    };
                    RPMException e = new RPMException(
                        com.ibm.rpm.framework.constants.ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                        errorParams, inputScopeElement.getClass().getName());
                    addException(e, msgCtx);
                }
            }
            else if (outputAbstractScope.getTask() != null
                    && outputAbstractScope.getTask().booleanValue())
            {
                // can't convert from a non-task AbstractScope to an [AbstractScope]Task
                if (inputScopeElement.getTask() == null
                        || !(inputScopeElement.getTask().booleanValue()))
                {
                    String[] errorParams =
                    {
                            StringUtil.getShortClassName(inputScopeElement
                                .getClass().getName()),
                            StringUtil.getShortClassName(outputScopeElement
                                .getClass().getName())
                                    + " Task",
                    };
                    RPMException e = new RPMException(
                        com.ibm.rpm.framework.constants.ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                        errorParams, inputScopeElement.getClass().getName());
                    addException(e, msgCtx);
                }
            }

        }
    }

    /**
     * @param inputObject
     * @param msgCtx
     * @return
     */
    public boolean isTemplateAndInputAvailable(RPMObject inputObject,
            MessageContext msgCtx)
    {
        return GenericValidator.validateNotNull(inputObject,
            ValidationConstants.TEMPLATE_INPUT_FIELD, inputObject, msgCtx)
                && GenericValidator.validateNotNull(inputObject,
                    ValidationConstants.TEMPLATE_FIELD, msgCtx.getTemplate(),
                    msgCtx);
    }

    /**
     * @param inputObject
     * @param msgCtx
     * @param expectedClassList
     */
    public void validateTemplateAndInput(RPMObject inputObject,
            MessageContext msgCtx)
    {
        Class[] expectedClassList =
        {
                Risk.class, Requirement.class, Issue.class, Defect.class,
                ChangeRequest.class, Action.class, ServiceRequest.class
        };

        //-----validations for all cases

        //ID is Manadatory for template
        GenericValidator.validateMandatoryID(inputObject,
            ValidationConstants.TEMPLATE_FIELD, msgCtx.getTemplate(), msgCtx);

        //ID must be null for inputObject
        GenericValidator.validateMustBeNull(inputObject,
            ValidationConstants.ID_FIELD, inputObject.getID(),
            "creating an object with template", msgCtx);

        //Parent should be present...
        GenericValidator.validateMandatory(inputObject,
            ValidationConstants.PARENT_FIELD, inputObject.getParent(), msgCtx);

        //Validate that input is same instance as template
        GenericValidator.validateExactType(inputObject, inputObject.getClass(),
            ValidationConstants.TEMPLATE_INPUT_FIELD, msgCtx.getTemplate(),
            msgCtx);

        //Validate that input is instance of acceptable class
        GenericValidator.validateType(inputObject, expectedClassList,
            ValidationConstants.TEMPLATE_INPUT_FIELD, inputObject, msgCtx);

        //Validate that template is instance of acceptable class
        GenericValidator.validateType(msgCtx.getTemplate(), expectedClassList,
            ValidationConstants.TEMPLATE_FIELD, msgCtx.getTemplate(), msgCtx);
    }

    public final boolean canMoveAndTransferUnderWBS(WorkElement workElement,
            AbstractScope abstractScope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canMoveAndTransferUnderWBS(
                messageContext, workElement, abstractScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }

    public final boolean canMoveUnderWBS(WorkElement workElement,
            AbstractScope abstractScope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canMoveUnderWBS(
                messageContext, workElement, abstractScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    public final boolean canTransferUnderWBS(WorkElement workElement,
            AbstractScope abstractScope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canTransferUnderWBS(
                messageContext, workElement, abstractScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    public final boolean canCreateFromTemplate(RPMObject rpmObject,
            RPMObject template, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canCreateObjectFromTemplate(
                messageContext, rpmObject, template, rpmScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }


}