/**
 * @author ragoler
 *
 */
package com.ibm.rpm.scorecard.checkpoints;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.RpmResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.AggregateScope;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AbstractScorecardElement;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class ScorecardMethodCheckpoint
{
    protected static ScorecardMethodCheckpoint instance = new ScorecardMethodCheckpoint();

    public static ScorecardMethodCheckpoint getInstance()
    {
        return instance;
    }

    public final boolean canAssignScorecard(RPMObject objectToAssign,
            RPMObjectScope objectToAssignScope, Scorecard scoreCardToAssign,
            ReloadType reloadType, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canAssignScoreCard(
                messageContext, objectToAssign, objectToAssignScope,
                scoreCardToAssign, reloadType).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            messageContext.addExceptionNoThrow(SecurityValidationResult
                .makeException(e));
            return false;
        }
    }
   

    public boolean validateScorecardAssignment(RPMObject objectToAssign,
            RPMObjectScope objectToAssignScope, Scorecard scoreCardToAssign,
            ReloadType reloadType, MessageContext msgCtx)
    {

        boolean valid = true;
        
        valid &= isValidObject(objectToAssign, objectToAssignScope, msgCtx);

        valid &= isValidScorecard(scoreCardToAssign, objectToAssignScope,
            msgCtx);

        valid &= isValidContext(scoreCardToAssign, objectToAssign, msgCtx);

        valid &= isValidNumberOfAssignedScorecards(objectToAssign, msgCtx);

        valid &= isValidReloadType(objectToAssign, reloadType, msgCtx);

        valid &= isTransferredScopeElement(objectToAssign, msgCtx);

        valid &= isTransferredScopeElement(objectToAssign, msgCtx);

        return valid;
    }
    
    public boolean validateResourceScorecardAssignment(Resource resource, 
            ResourceScope resourceScope, 
            Scorecard scorecard, ReloadType reloadType, 
            GenericProject project, MessageContext messageContext)
    {
        boolean valid = true;
        
        valid &= isValidScorecard(scorecard, resourceScope,
            messageContext);
        
        valid &= isValidReloadType(resource, reloadType, messageContext);
        
        valid &= resourceHasWorkUnderProject(resource, project, messageContext);
        
        return valid;
    }
   
    /**
     * Find out if the resource has work assigned to the project it's about to get rated on
     * @param resource
     * @param project
     * @param messageContext
     * @return
     */
    private boolean resourceHasWorkUnderProject(Resource resource, GenericProject project, 
            MessageContext messageContext)
    {
        boolean hasWork = false;
        
        String xpath = "/WorkElement[@containingProject[@id='" + project.getID() +
        "'] and @resourceTaskAssignments[@resource[@id='" + resource.getID() +"']]]";
        RpmResult results = messageContext.getFactory().getManagerCaller().load(messageContext.getSession(), xpath, null);
        if (results.isSuccessful() && results.getObjects().size() > 0)
        {
            hasWork = true;
        }
        
        return hasWork;
    }

    private boolean isValidObject(RPMObject objectToAssign,
            RPMObjectScope objectToAssignScope, MessageContext msgCtx)
    {

        boolean valid = true;

        if (!( 
                (objectToAssign instanceof Asset)
                || (objectToAssign instanceof GenericProject) || (objectToAssign instanceof AbstractScope))
                )
        {
            String[] errorParams =
            {
                    ValidationConstants.OBJECT_TO_ASSIGN_FIELD,
                    objectToAssign.getClass().getName(),
                    StringUtil.getShortClassName(Asset.class)
                            + " or "
                            + StringUtil.getShortClassName(AbstractScope.class)
                            + " or "
                            + StringUtil.getShortClassName(GenericProject.class)
            };
            RPMException e = new RPMException(ErrorCodes.INVALID_TYPE,
                errorParams, objectToAssign.getClass().getName());
            msgCtx.addExceptionNoThrow(e);

            valid = false;
        }

        if (objectToAssignScope != null)
        {
            if (!((objectToAssignScope instanceof AssetScope)
                    || (objectToAssignScope instanceof WorkElementScope) || (objectToAssignScope instanceof ScopeElementScope)))
            {
                String[] errorParams =
                {
                        ValidationConstants.OBJECT_TO_ASSIGN_SCOPE_FIELD,
                        StringUtil.getShortClassName(objectToAssignScope
                            .getClass()),
                        StringUtil.getShortClassName(AssetScope.class)
                                + " or "
                                + StringUtil
                                    .getShortClassName(WorkElementScope.class)
                                + " or "
                                + StringUtil
                                    .getShortClassName(ScopeElementScope.class),

                };
                RPMException e = new RPMException(ErrorCodes.INVALID_TYPE,
                    errorParams, objectToAssignScope.getClass().getName());
                msgCtx.addExceptionNoThrow(e);

                valid = false;
            }
        }

        return valid;
    }

    private boolean isValidContext(AbstractScorecardElement scorecard,
            RPMObject rpmObject, MessageContext msgCtx)
    {
        boolean valid = true;

        if (scorecard.getContextName() == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(scorecard.getClass())
            };

            RPMException e = new RPMException(ErrorCodes.MODULE_REQUIRED,
                errorParams, scorecard.getClass().getName());
            msgCtx.addExceptionNoThrow(e);

            valid = false;
        }
        else if (scorecard.getContextName().equals(ContextUtil.ASSET_CONTEXT)
                && (!(rpmObject instanceof Asset))
                || scorecard.getContextName().equals(ContextUtil.WBS_CONTEXT)
                && (!(rpmObject instanceof GenericProject))
                || scorecard.getContextName().equals(ContextUtil.SCOPE_CONTEXT)
                && (!(rpmObject instanceof AbstractScope)))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(scorecard.getClass()),
                    scorecard.getContextName(),
                    StringUtil.getShortClassName(rpmObject.getClass())
            };

            RPMException e = new RPMException(
                com.ibm.rpm.scorecard.constants.ErrorCodes.INCORRECT_CONTEXT_FOR_SCORECARD_ASSIGNMENT,
                errorParams, scorecard.getClass().getName());
            msgCtx.addExceptionNoThrow(e);

            valid = false;
        }

        return valid;

    }

    private boolean isValidReloadType(RPMObject rpmObject,
            ReloadType reloadType, MessageContext msgCtx)
    {
        boolean valid = true;

        if (reloadType != ReloadType.None
                && reloadType != ReloadType.ReloadResult
                && reloadType != ReloadType.SavedResult)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    reloadType.toString()
            };

            RPMException e = new RPMException(ErrorCodes.RELOAD_TYPE_INVALID,
                errorParams);
            msgCtx.addExceptionNoThrow(e);

            valid = false;
        }

        return valid;
    }

    private boolean isValidScorecard(Scorecard myScorecard,
            RPMObjectScope rpmScope, MessageContext msgCtx)
    {
        boolean valid = true;

        if (!((Scorecard) myScorecard).getActive().booleanValue())
        {
            valid &= GenericValidator.validateActiveFlag(myScorecard,
                ValidationConstants.ACTIVE_FIELD, msgCtx);
        }
        
        /* 
         * A scorecard can be validated if it has no ID, but for the method call, as per the documentation,
         * it should be previously saved by the user.
         */
        valid &= GenericValidator.validateMandatoryID(StringUtil.getShortClassName(myScorecard.getClass()), myScorecard, msgCtx);
        
        if(valid) // continue validating the scorecard itself
        {
            ScorecardCheckpoint.getInstance().validateSave(myScorecard, rpmScope,
                msgCtx);
        }

        return valid;
    }

    private boolean isValidNumberOfAssignedScorecards(RPMObject objectToAssign,
            MessageContext msgCtx)
    {
        boolean valid = true;

        if (((objectToAssign instanceof Asset) && (((Asset) objectToAssign)
            .getAssignedScorecard() != null))
                || ((objectToAssign instanceof AbstractScope) && (((AbstractScope) objectToAssign)
                    .getAssignedScorecard() != null)))    
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(objectToAssign.getClass())
            };

            RPMException e = new RPMException(
                com.ibm.rpm.scorecard.constants.ErrorCodes.INCORRECT_NUMBER_OF_SCORECARD_ASSIGNMENTS,
                errorParams, objectToAssign.getClass().getName());
            msgCtx.addExceptionNoThrow(e);
            valid = false;
        }

        return valid;
    }

    private boolean isTransferredScopeElement(RPMObject objectToAssign,
            MessageContext msgCtx)
    {
        boolean valid = true;

        if (objectToAssign instanceof AggregateScope)
        {
            AbstractAggregateScope aggregateScope = (AbstractAggregateScope) objectToAssign;

            if (aggregateScope.getTransferedUnderWbsRoot() != null
                    && aggregateScope.getTransferedUnderWbsRoot()
                        .booleanValue())
            {
                String[] errorParams =
                {
                    StringUtil.getShortClassName(objectToAssign.getClass())
                };

                RPMException e = new RPMException(
                    com.ibm.rpm.scorecard.constants.ErrorCodes.CANNOT_ASSIGN_SCORECARD_TO_TRANSFERRED_SCOPE_ELEMENT,
                    errorParams, objectToAssign.getClass().getName());
                msgCtx.addExceptionNoThrow(e);
                valid = false;
            }
        }
        return valid;
    }

}
