package com.ibm.rpm.security.managers;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.managers.ResourceRoleAssignmentManager;
import com.ibm.rpm.security.containers.ActionItemSecurityRole;
import com.ibm.rpm.security.containers.ChangeRequestSecurityRole;
import com.ibm.rpm.security.containers.DefectSecurityRole;
import com.ibm.rpm.security.containers.DeliverableSecurityRole;
import com.ibm.rpm.security.containers.DocumentSecurityRole;
import com.ibm.rpm.security.containers.FolderSecurityRole;
import com.ibm.rpm.security.containers.GenericSecurityRole;
import com.ibm.rpm.security.containers.IssueSecurityRole;
import com.ibm.rpm.security.containers.NotesMinutesSecurityRole;
import com.ibm.rpm.security.containers.PoolSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.RequirementSecurityRole;
import com.ibm.rpm.security.containers.RiskSecurityRole;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.SecurityRoleCategory;
import com.ibm.rpm.security.containers.ServiceRequestSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.security.containers.TemplateFolderSecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.security.types.ActionItemSecurityRoleType;
import com.ibm.rpm.security.types.ChangeRequestSecurityRoleType;
import com.ibm.rpm.security.types.DefectSecurityRoleType;
import com.ibm.rpm.security.types.DeliverableSecurityRoleType;
import com.ibm.rpm.security.types.DocumentSecurityRoleType;
import com.ibm.rpm.security.types.FolderSecurityRoleType;
import com.ibm.rpm.security.types.IssueSecurityRoleType;
import com.ibm.rpm.security.types.NotesMinutesSecurityRoleType;
import com.ibm.rpm.security.types.PoolSecurityRoleType;
import com.ibm.rpm.security.types.ProjectSecurityRoleType;
import com.ibm.rpm.security.types.RequirementSecurityRoleType;
import com.ibm.rpm.security.types.RiskSecurityRoleType;
import com.ibm.rpm.security.types.ServiceRequestSecurityRoleType;
import com.ibm.rpm.security.types.TaskSecurityRoleType;
import com.ibm.rpm.security.types.TemplateFolderSecurityRoleType;
import com.ibm.rpm.wbs.managers.ResourceTaskAssignmentManager;
import com.ibm.rpm.workflow.managers.WorkflowRoleMappingManager;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * Manager for TMT_GLOBAL_SECURITY and all related classes.
 * 
 * <h3>UI - API - Database correspondance</h3>
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>UI</th>
 * <th>API</th>
 * <th>Database</th>
 * </tr>
 * <tr>
 * <td>Group</td>
 * <td>SecurityRoleCategory</td>
 * <td>LEVEL_ID=0</td>
 * </tr>
 * <tr>
 * <td>Role</td>
 * <td>SecurityRole (can have children)</td>
 * <td>LEVEL_ID=1</td>
 * </tr>
 * <tr>
 * <td>Role Level</td>
 * <td>SecurityRole (no children)</td>
 * <td>LEVEL_ID=2</td>
 * </tr>
 * </table>
 */
public class SecurityRoleManager extends AbstractRPMObjectManager
{
    private static Log logger = LogFactory.getLog(SecurityRoleManager.class);

    public SecurityRoleManager()
    {
        super();
    }

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    // =============
    // CONTAINER MAP
    // =============

    private static Class[] containerClasses =
    {
            GenericSecurityRole.class, SecurityRoleCategory.class,
            SecurityRole.class, ProjectSecurityRole.class,
            DeliverableSecurityRole.class, TaskSecurityRole.class,
            ChangeRequestSecurityRole.class, IssueSecurityRole.class,
            RiskSecurityRole.class, ServiceRequestSecurityRole.class,
            RequirementSecurityRole.class, DefectSecurityRole.class,
            ActionItemSecurityRole.class, DocumentSecurityRole.class,
            NotesMinutesSecurityRole.class, PoolSecurityRole.class,
            FolderSecurityRole.class, TemplateFolderSecurityRole.class,
    };

    static
    {
        new ContainerMap(SecurityRoleManager.class, containerClasses);
    }

    /**
     * Maps a role id to the matching SecurityRoleTypeMapping.
     */
    public static final Map SECURITY_ROLE_MAP = new HashMap();

    /**
     * Maps a role class to the matching collection of role ids.
     */
    public static final Map ROLE_CLASS_TO_ROLE_IDS_MAP = new HashMap();

    private static void addSecurityRole(String roleId, Object roleType,
            Class roleClass)
    {
        SECURITY_ROLE_MAP.put(roleId, new SecurityRoleTypeMapping(roleType,
            roleClass));
        // Add the roleId to the right collection, matching the role class.
        Collection roleIds = (Collection) ROLE_CLASS_TO_ROLE_IDS_MAP
            .get(roleClass);
        if (roleIds == null)
        {
            roleIds = new HashSet();
            ROLE_CLASS_TO_ROLE_IDS_MAP.put(roleClass, roleIds);
        }
        roleIds.add(roleId);
    }

    static
    {
        addSecurityRole("SCPROJECTMANAGER",
            ProjectSecurityRoleType.ProjectManager, ProjectSecurityRole.class);
        addSecurityRole("SCCOMANAGER", ProjectSecurityRoleType.CoManager,
            ProjectSecurityRole.class);
        addSecurityRole("SCFINANCE", ProjectSecurityRoleType.Finance,
            ProjectSecurityRole.class);
        addSecurityRole("SCCOFINANCE", ProjectSecurityRoleType.CoFinance,
            ProjectSecurityRole.class);
        addSecurityRole("SCPROJECTOFFICE",
            ProjectSecurityRoleType.ProjectOffice, ProjectSecurityRole.class);
        addSecurityRole("SCMETHODAUTHOR", ProjectSecurityRoleType.MethodAuthor,
            ProjectSecurityRole.class);
        addSecurityRole("SCPROJECTASSIGNMENT",
            ProjectSecurityRoleType.ProjectAssignment,
            ProjectSecurityRole.class);
        addSecurityRole("SCPROJECTWORKFLOWVISITOR",
            ProjectSecurityRoleType.ProjectWorkflowVisitor,
            ProjectSecurityRole.class);
        addSecurityRole("SCPROJECTVISITOR",
            ProjectSecurityRoleType.ProjectVisitor, ProjectSecurityRole.class);
        addSecurityRole("SCEXECUTIVE", ProjectSecurityRoleType.ExecutiveRoles,
            ProjectSecurityRole.class);
        addSecurityRole("SCPROJECTPARTICIPANT",
            ProjectSecurityRoleType.ProjectParticipant,
            ProjectSecurityRole.class);
        addSecurityRole("SCDELIVERABLEMANAGER",
            DeliverableSecurityRoleType.DeliverableManager,
            DeliverableSecurityRole.class);
        addSecurityRole("SCDELIVERABLERESPONSIBLE",
            DeliverableSecurityRoleType.DeliverableResponsible,
            DeliverableSecurityRole.class);
        addSecurityRole("SCDELIVERABLEPARTICIPANT",
            DeliverableSecurityRoleType.DeliverableParticipant,
            DeliverableSecurityRole.class);
        addSecurityRole("SCDELIVERABLEWORKFLOWVISITOR",
            DeliverableSecurityRoleType.DeliverableWorkflowVisitor,
            DeliverableSecurityRole.class);
        addSecurityRole("SCTASKRESPONSIBLE",
            TaskSecurityRoleType.TaskResponsible, TaskSecurityRole.class);
        addSecurityRole("SCTASKPARTICIPANT",
            TaskSecurityRoleType.TaskParticipant, TaskSecurityRole.class);
        addSecurityRole("SCTASKTEAMMEMBER",
            TaskSecurityRoleType.TaskTeamMember, TaskSecurityRole.class);
        addSecurityRole("SCTASKWORKFLOWVISITOR",
            TaskSecurityRoleType.TaskWorkflowVisitor, TaskSecurityRole.class);
        addSecurityRole("SCCHANGEREQUESTRESPONSIBLE",
            ChangeRequestSecurityRoleType.ChangeRequestResponsible,
            ChangeRequestSecurityRole.class);
        addSecurityRole("SCCHANGEREQUESTPARTICIPANT",
            ChangeRequestSecurityRoleType.ChangeRequestParticipant,
            ChangeRequestSecurityRole.class);
        addSecurityRole("SCCHANGEREQUESTTEAMMEMBER",
            ChangeRequestSecurityRoleType.ChangeRequestTeamMember,
            ChangeRequestSecurityRole.class);
        addSecurityRole("SCCHANGEREQUESTVISITOR",
            ChangeRequestSecurityRoleType.ChangeRequestVisitor,
            ChangeRequestSecurityRole.class);
        addSecurityRole("SCCHANGEREQUESTWORKFLOWVISITOR",
            ChangeRequestSecurityRoleType.ChangeRequestWorkflowVisitor,
            ChangeRequestSecurityRole.class);
        addSecurityRole("SCISSUERESPONSIBLE",
            IssueSecurityRoleType.IssueResponsible, IssueSecurityRole.class);
        addSecurityRole("SCISSUEPARTICIPANT",
            IssueSecurityRoleType.IssueParticipant, IssueSecurityRole.class);
        addSecurityRole("SCISSUETEAMMEMBER",
            IssueSecurityRoleType.IssueTeamMember, IssueSecurityRole.class);
        addSecurityRole("SCISSUEVISITOR", IssueSecurityRoleType.IssueVisitor,
            IssueSecurityRole.class);
        addSecurityRole("SCISSUEWORKFLOWVISITOR",
            IssueSecurityRoleType.IssueWorkflowVisitor, IssueSecurityRole.class);
        addSecurityRole("SCRISKRESPONSIBLE",
            RiskSecurityRoleType.RiskResponsible, RiskSecurityRole.class);
        addSecurityRole("SCRISKPARTICIPANT",
            RiskSecurityRoleType.RiskParticipant, RiskSecurityRole.class);
        addSecurityRole("SCRISKTEAMMEMBER",
            RiskSecurityRoleType.RiskTeamMember, RiskSecurityRole.class);
        addSecurityRole("SCRISKVISITOR", RiskSecurityRoleType.RiskVisitor,
            RiskSecurityRole.class);
        addSecurityRole("SCRISKWORKFLOWVISITOR",
            RiskSecurityRoleType.RiskWorkflowVisitor, RiskSecurityRole.class);
        addSecurityRole("SCSERVICEREQUESTRESPONSIBLE",
            ServiceRequestSecurityRoleType.ServiceRequestResponsible,
            ServiceRequestSecurityRole.class);
        addSecurityRole("SCSERVICEREQUESTPARTICIPANT",
            ServiceRequestSecurityRoleType.ServiceRequestParticipant,
            ServiceRequestSecurityRole.class);
        addSecurityRole("SCSERVICEREQUESTTEAMMEMBER",
            ServiceRequestSecurityRoleType.ServiceRequestTeamMember,
            ServiceRequestSecurityRole.class);
        addSecurityRole("SCSERVICEREQUESTVISITOR",
            ServiceRequestSecurityRoleType.ServiceRequestVisitor,
            ServiceRequestSecurityRole.class);
        addSecurityRole("SCSERVICEREQUESTWORKFLOWVISITOR",
            ServiceRequestSecurityRoleType.ServiceRequestWorkflowVisitor,
            ServiceRequestSecurityRole.class);
        addSecurityRole("SCREQUIREMENTRESPONSIBLE",
            RequirementSecurityRoleType.RequirementResponsible,
            RequirementSecurityRole.class);
        addSecurityRole("SCREQUIREMENTPARTICIPANT",
            RequirementSecurityRoleType.RequirementParticipant,
            RequirementSecurityRole.class);
        addSecurityRole("SCREQUIREMENTTEAMMEMBER",
            RequirementSecurityRoleType.RequirementTeamMember,
            RequirementSecurityRole.class);
        addSecurityRole("SCREQUIREMENTVISITOR",
            RequirementSecurityRoleType.RequirementVisitor,
            RequirementSecurityRole.class);
        addSecurityRole("SCREQUIREMENTWORKFLOWVISITOR",
            RequirementSecurityRoleType.RequirementWorkflowVisitor,
            RequirementSecurityRole.class);
        addSecurityRole("SCDEFECTRESPONSIBLE",
            DefectSecurityRoleType.DefectResponsible, DefectSecurityRole.class);
        addSecurityRole("SCDEFECTPARTICIPANT",
            DefectSecurityRoleType.DefectParticipant, DefectSecurityRole.class);
        addSecurityRole("SCDEFECTTEAMMEMBER",
            DefectSecurityRoleType.DefectTeamMember, DefectSecurityRole.class);
        addSecurityRole("SCDEFECTVISITOR",
            DefectSecurityRoleType.DefectVisitor, DefectSecurityRole.class);
        addSecurityRole("SCDEFECTWORKFLOWVISITOR",
            DefectSecurityRoleType.DefectWorkflowVisitor,
            DefectSecurityRole.class);
        addSecurityRole("SCACTIONITEMRESPONSIBLE",
            ActionItemSecurityRoleType.ActionItemResponsible,
            ActionItemSecurityRole.class);
        addSecurityRole("SCACTIONITEMPARTICIPANT",
            ActionItemSecurityRoleType.ActionItemParticipant,
            ActionItemSecurityRole.class);
        addSecurityRole("SCACTIONITEMTEAMMEMBER",
            ActionItemSecurityRoleType.ActionItemTeamMember,
            ActionItemSecurityRole.class);
        addSecurityRole("SCACTIONITEMVISITOR",
            ActionItemSecurityRoleType.ActionItemVisitor,
            ActionItemSecurityRole.class);
        addSecurityRole("SCACTIONITEMWORKFLOWVISITOR",
            ActionItemSecurityRoleType.ActionItemWorkflowVisitor,
            ActionItemSecurityRole.class);
        addSecurityRole("SCDOCUMENTRESPONSIBLE",
            DocumentSecurityRoleType.DocumentResponsible,
            DocumentSecurityRole.class);
        addSecurityRole("SCDOCUMENTPARTICIPANT",
            DocumentSecurityRoleType.DocumentParticipant,
            DocumentSecurityRole.class);
        addSecurityRole("SCDOCUMENTVISITOR",
            DocumentSecurityRoleType.DocumentVisitor,
            DocumentSecurityRole.class);
        addSecurityRole("SCDOCUMENTWORKFLOWVISITOR",
            DocumentSecurityRoleType.DocumentWorkflowVisitor,
            DocumentSecurityRole.class);
        addSecurityRole("SCDOCUMENTUNRESTRICTEDVISITOR___",
            DocumentSecurityRoleType.DocumentUnrestrictedVisitor,
            DocumentSecurityRole.class);
        addSecurityRole("SCNOTEMINUTESRESPONSIBLE",
            NotesMinutesSecurityRoleType.NotesMinutesResponsible,
            NotesMinutesSecurityRole.class);
        addSecurityRole("SCNOTEMINUTESPARTICIPANT",
            NotesMinutesSecurityRoleType.NotesMinutesParticipant,
            NotesMinutesSecurityRole.class);
        addSecurityRole("SCNOTEMINUTESVISITOR",
            NotesMinutesSecurityRoleType.NotesMinutesVisitor,
            NotesMinutesSecurityRole.class);
        addSecurityRole("SCNOTEMINUTESWORKFLOWVISITOR",
            NotesMinutesSecurityRoleType.NotesMinutesWorkflowVisitor,
            NotesMinutesSecurityRole.class);
        addSecurityRole("SCRESOURCEMANAGER",
            PoolSecurityRoleType.ResourceManager, PoolSecurityRole.class);
        addSecurityRole("SCPOOLPARTICIPANT",
            PoolSecurityRoleType.PoolParticipant, PoolSecurityRole.class);
        addSecurityRole("SCFOLDERMANAGER",
            FolderSecurityRoleType.FolderManager, FolderSecurityRole.class);
        addSecurityRole("SCFOLDERPARTICIPANT",
            FolderSecurityRoleType.FolderParticipant, FolderSecurityRole.class);
        addSecurityRole("SCFOLDERVISITOR",
            FolderSecurityRoleType.FolderVisitor, FolderSecurityRole.class);
        addSecurityRole("SCFOLDERWORKFLOWVISITOR",
            FolderSecurityRoleType.FolderWorkflowVisitor,
            FolderSecurityRole.class);
        addSecurityRole("SCTEMPLATEFOLDERMANAGER",
            TemplateFolderSecurityRoleType.FolderManager,
            TemplateFolderSecurityRole.class);
        addSecurityRole("SCTEMPLATEFOLDERPARTICIPANT",
            TemplateFolderSecurityRoleType.FolderParticipant,
            TemplateFolderSecurityRole.class);
        addSecurityRole("SCTEMPLATEFOLDERVISITOR",
            TemplateFolderSecurityRoleType.FolderVisitor,
            TemplateFolderSecurityRole.class);
    }

    // =========
    // FIELD MAP
    // =========

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_GLOBAL_SECURITY.LEVEL_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_LEVEL_ID = "LEVEL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID, NAME_LEVEL_ID);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_GLOBAL_SECURITY.TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_GLOBAL_SECURITY.MAX_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_GLOBAL_SECURITY.CHILD_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_GLOBAL_SECURITY.DELETED_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 6;

    public static final int TYPE_PARENT_ID = Types.VARCHAR;

    // public static final String NAME_PARENT_ID =
    // "LTRIM(RTRIM(TMT_GLOBAL_SECURITY.PARENT_ID))";
    public static final String NAME_PARENT_ID = "TMT_GLOBAL_SECURITY.PARENT_ID";

    public static final int SIZE_PARENT_ID = 32;

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 7;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    // public static final String NAME_ELEMENT_ID =
    // "LTRIM(RTRIM(TMT_GLOBAL_SECURITY.ELEMENT_ID))";
    public static final String NAME_ELEMENT_ID = "TMT_GLOBAL_SECURITY.ELEMENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 8;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    // public static final String NAME_ELEMENT_NAME =
    // "LTRIM(RTRIM(TMT_GLOBAL_SECURITY.ELEMENT_NAME))";
    public static final String NAME_ELEMENT_NAME = "TMT_GLOBAL_SECURITY.ELEMENT_NAME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column NOTI_1 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_NOTI_1 = 9;

    public static final int TYPE_NOTI_1 = Types.INTEGER;

    public static final String NAME_NOTI_1 = "TMT_GLOBAL_SECURITY.NOTI_1";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_NOTI_1 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_NOTI_1,NAME_NOTI_1);}

    /**
     * Column NOTI_2 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_NOTI_2 = 10;

    public static final int TYPE_NOTI_2 = Types.INTEGER;

    public static final String NAME_NOTI_2 = "TMT_GLOBAL_SECURITY.NOTI_2";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_NOTI_2 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_NOTI_2,NAME_NOTI_2);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 11;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_GLOBAL_SECURITY.REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 12;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_GLOBAL_SECURITY.REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 13;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_GLOBAL_SECURITY.REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 14;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_GLOBAL_SECURITY.RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column INT_1 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_1 = 15;

    public static final int TYPE_INT_1 = Types.INTEGER;

    public static final String NAME_INT_1 = "TMT_GLOBAL_SECURITY.INT_1";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_1 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_1,NAME_INT_1);}

    /**
     * Column INT_2 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_2 = 16;

    public static final int TYPE_INT_2 = Types.INTEGER;

    public static final String NAME_INT_2 = "TMT_GLOBAL_SECURITY.INT_2";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_2 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_2,NAME_INT_2);}

    /**
     * Column INT_3 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_3 = 17;

    public static final int TYPE_INT_3 = Types.INTEGER;

    public static final String NAME_INT_3 = "TMT_GLOBAL_SECURITY.INT_3";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_3 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_3,NAME_INT_3);}

    /**
     * Column INT_4 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_4 = 18;

    public static final int TYPE_INT_4 = Types.INTEGER;

    public static final String NAME_INT_4 = "TMT_GLOBAL_SECURITY.INT_4";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_4 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_4,NAME_INT_4);}

    /**
     * Column INT_5 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_5 = 19;

    public static final int TYPE_INT_5 = Types.INTEGER;

    public static final String NAME_INT_5 = "TMT_GLOBAL_SECURITY.INT_5";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_5 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_5,NAME_INT_5);}

    /**
     * Column INT_6 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_6 = 20;

    public static final int TYPE_INT_6 = Types.INTEGER;

    public static final String NAME_INT_6 = "TMT_GLOBAL_SECURITY.INT_6";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_6 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_6,NAME_INT_6);}

    /**
     * Column INT_7 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_7 = 21;

    public static final int TYPE_INT_7 = Types.INTEGER;

    public static final String NAME_INT_7 = "TMT_GLOBAL_SECURITY.INT_7";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_7 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_7,NAME_INT_7);}

    /**
     * Column INT_8 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_8 = 22;

    public static final int TYPE_INT_8 = Types.INTEGER;

    public static final String NAME_INT_8 = "TMT_GLOBAL_SECURITY.INT_8";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_8 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_8,NAME_INT_8);}

    /**
     * Column INT_9 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_9 = 23;

    public static final int TYPE_INT_9 = Types.INTEGER;

    public static final String NAME_INT_9 = "TMT_GLOBAL_SECURITY.INT_9";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_9 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_9,NAME_INT_9);}

    /**
     * Column INT_10 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_10 = 24;

    public static final int TYPE_INT_10 = Types.INTEGER;

    public static final String NAME_INT_10 = "TMT_GLOBAL_SECURITY.INT_10";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_10 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_10,NAME_INT_10);}

    /**
     * Column INT_11 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_11 = 25;

    public static final int TYPE_INT_11 = Types.INTEGER;

    public static final String NAME_INT_11 = "TMT_GLOBAL_SECURITY.INT_11";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_11 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_11,NAME_INT_11);}

    /**
     * Column INT_12 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_12 = 26;

    public static final int TYPE_INT_12 = Types.INTEGER;

    public static final String NAME_INT_12 = "TMT_GLOBAL_SECURITY.INT_12";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_12 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_12,NAME_INT_12);}

    /**
     * Column INT_13 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_13 = 27;

    public static final int TYPE_INT_13 = Types.INTEGER;

    public static final String NAME_INT_13 = "TMT_GLOBAL_SECURITY.INT_13";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_13 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_13,NAME_INT_13);}

    /**
     * Column INT_14 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_14 = 28;

    public static final int TYPE_INT_14 = Types.INTEGER;

    public static final String NAME_INT_14 = "TMT_GLOBAL_SECURITY.INT_14";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_14 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_14,NAME_INT_14);}

    /**
     * Column INT_15 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_15 = 29;

    public static final int TYPE_INT_15 = Types.INTEGER;

    public static final String NAME_INT_15 = "TMT_GLOBAL_SECURITY.INT_15";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_15 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_15,NAME_INT_15);}

    /**
     * Column INT_16 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_16 = 30;

    public static final int TYPE_INT_16 = Types.INTEGER;

    public static final String NAME_INT_16 = "TMT_GLOBAL_SECURITY.INT_16";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_16 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_16,NAME_INT_16);}

    /**
     * Column INT_17 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_17 = 31;

    public static final int TYPE_INT_17 = Types.INTEGER;

    public static final String NAME_INT_17 = "TMT_GLOBAL_SECURITY.INT_17";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_17 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_17,NAME_INT_17);}

    /**
     * Column INT_18 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_18 = 32;

    public static final int TYPE_INT_18 = Types.INTEGER;

    public static final String NAME_INT_18 = "TMT_GLOBAL_SECURITY.INT_18";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_18 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_18,NAME_INT_18);}

    /**
     * Column INT_19 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_19 = 33;

    public static final int TYPE_INT_19 = Types.INTEGER;

    public static final String NAME_INT_19 = "TMT_GLOBAL_SECURITY.INT_19";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_19 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_19,NAME_INT_19);}

    /**
     * Column INT_20 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_20 = 34;

    public static final int TYPE_INT_20 = Types.INTEGER;

    public static final String NAME_INT_20 = "TMT_GLOBAL_SECURITY.INT_20";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_20 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_20,NAME_INT_20);}

    /**
     * Column INT_21 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_21 = 35;

    public static final int TYPE_INT_21 = Types.INTEGER;

    public static final String NAME_INT_21 = "TMT_GLOBAL_SECURITY.INT_21";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_21 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_21,NAME_INT_21);}

    /**
     * Column INT_22 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_22 = 36;

    public static final int TYPE_INT_22 = Types.INTEGER;

    public static final String NAME_INT_22 = "TMT_GLOBAL_SECURITY.INT_22";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_22 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_22,NAME_INT_22);}

    /**
     * Column INT_23 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_23 = 37;

    public static final int TYPE_INT_23 = Types.INTEGER;

    public static final String NAME_INT_23 = "TMT_GLOBAL_SECURITY.INT_23";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_23 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_23,NAME_INT_23);}

    /**
     * Column INT_24 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_24 = 38;

    public static final int TYPE_INT_24 = Types.INTEGER;

    public static final String NAME_INT_24 = "TMT_GLOBAL_SECURITY.INT_24";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_INT_24 = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_INT_24,NAME_INT_24);}

    /**
     * Column INT_24 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_25 = 39;

    public static final int TYPE_INT_25 = Types.INTEGER;

    public static final String NAME_INT_25 = "TMT_GLOBAL_SECURITY.INT_25";

    /**
     * Column INT_24 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INT_26 = 40;

    public static final int TYPE_INT_26 = Types.INTEGER;

    public static final String NAME_INT_26 = "TMT_GLOBAL_SECURITY.INT_26";

    /**
     * Column SECURITY_GROUP of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SECURITY_GROUP = 41;

    public static final int TYPE_SECURITY_GROUP = Types.INTEGER;

    public static final String NAME_SECURITY_GROUP = "TMT_GLOBAL_SECURITY.SECURITY_GROUP";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SECURITY_GROUP = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_SECURITY_GROUP,NAME_SECURITY_GROUP);}

    public static final String TABLE_NAME = "TMT_GLOBAL_SECURITY";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_GLOBAL_SECURITY table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_PARENT_ID, NAME_ELEMENT_ID,
            NAME_ELEMENT_NAME, NAME_NOTI_1, NAME_NOTI_2, NAME_REC_USER,
            NAME_REC_STATUS, NAME_REC_DATETIME, NAME_RANK, NAME_INT_1,
            NAME_INT_2, NAME_INT_3, NAME_INT_4, NAME_INT_5, NAME_INT_6,
            NAME_INT_7, NAME_INT_8, NAME_INT_9, NAME_INT_10, NAME_INT_11,
            NAME_INT_12, NAME_INT_13, NAME_INT_14, NAME_INT_15, NAME_INT_16,
            NAME_INT_17, NAME_INT_18, NAME_INT_19, NAME_INT_20, NAME_INT_21,
            NAME_INT_22, NAME_INT_23, NAME_INT_24, NAME_INT_25, NAME_INT_26,
            NAME_SECURITY_GROUP
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    /**
     * @param rpmObject
     * @return an array of objects that represents the primary key of the table
     *         associated with the AbstractRPMObjectManager Override if your
     *         primary contains more than one field. IMPORTANT: Always put the
     *         ID property in the primary key. The primary key array should
     *         contains a null to indicate that the object is new.
     */
    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        return new Object[]
        {
            rpmObject.getID().trim()
        // StringUtils.rightPad(rpmObject.getID(), 32)
        };
    }

    // We override here because the ID needs to be trimed otherwise
    // there is a cache problem
    protected Object[] getPrimaryKey(ResultSet resultSet) throws RPMException,
            SQLException
    {

        String[] primaryKeyArray = getPrimaryKeyArray(null);
        String[] primaryKey = new String[primaryKeyArray.length];
        for (int i = 0; i < primaryKeyArray.length; i++)
        {
            // This line removes the table name
            primaryKeyArray[i] = primaryKeyArray[i]
                .substring(primaryKeyArray[i].indexOf('.') + 1);
            primaryKey[i] = resultSet.getString(primaryKeyArray[i]);
            if (primaryKey[i] != null)
            {
                primaryKey[i] = primaryKey[i].trim();
            }
        }
        return primaryKey;

    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof SecurityRoleManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof WorkflowRoleMappingManager)
        {
            // for loading WorkflowRoleMapping.resource
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_ELEMENT_ID,
                WorkflowRoleMappingManager.NAME_SECURITY_ID,
                WorkflowRoleMappingManager.TABLE_NAME,
                WorkflowRoleMappingManager.NAME_PROJECT_ID + "=? AND "
                        + WorkflowRoleMappingManager.NAME_WKF_ROLE_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendCondition(getFinalCondition(container));
        }
        else if (manager instanceof SecurityRoleManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, getFinalCondition(container));
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ResourceTaskAssignmentManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(
                ResourceTaskAssignmentManager.NAME_SECURITY_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof ResourceRoleAssignmentManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                ResourceRoleAssignmentManager.NAME_SECURITY_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof SecurityRoleManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(NAME_ELEMENT_ID, NAME_PARENT_ID,
                TABLE_NAME, getFinalCondition(container));
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    // =======
    // FILTERS
    // =======

    private String getFinalCondition(XPathContainer container)
            throws RPMException
    {
        // Add the container filter, if required.
        SqlBuffer finalCondition = getContainerFilter(container.getType());

        return finalCondition.toString();
    }

    /**
     * Gets the right filter for the container type.
     */
    private SqlBuffer getContainerFilter(Class containerClass)
            throws RPMException
    {
        SqlBuffer filter = new SqlBuffer(getFilter());
        if (GenericSecurityRole.class.equals(containerClass))
        {
            filter.appendCondition(NAME_LEVEL_ID + ">=0");
        }
        else if (SecurityRoleCategory.class.equals(containerClass))
        {
            filter.appendEqual(NAME_LEVEL_ID, 0);
        }
        else if (SecurityRole.class.equals(containerClass))
        {
            filter.appendCondition(NAME_LEVEL_ID + ">=1");
        }
        else
        {
            Collection roleIds = (Collection) ROLE_CLASS_TO_ROLE_IDS_MAP
                .get(containerClass);
            if (roleIds != null)
            {
                filter.appendAnd();
                filter.appendOpenParenthesis();
                filter.append(NAME_PARENT_ID);
                filter.appendIn(roleIds, SIZE_PARENT_ID);
                filter.appendOr();
                filter.append(NAME_ELEMENT_ID);
                filter.appendIn(roleIds);
                filter.appendCloseParenthesis();
            }
            else
            {
                throw new RPMException("Unsupported container "
                        + containerClass);
            }
        }

        return filter;
    }

    public String getFilter()
    {
        // Programming note: "PARENT_ID!='SCDUMMYGROUP'" returns false
        // when PARENT_ID IS NULL. Hence the OR clause. SQL is so funny.
        return NAME_REC_STATUS + "!='D' AND " + NAME_ELEMENT_ID
                + "!='SCDUMMYGROUP' AND (" + NAME_PARENT_ID + " IS NULL OR "
                + NAME_PARENT_ID + "!='SCDUMMYGROUP')";
    }

    // ========
    // CREATION
    // ========

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        RPMObject container;
        int levelId = resultSet.getInt(ID_LEVEL_ID);
        if (levelId == 0)
        {
            container = new SecurityRoleCategory();
        }
        else if (levelId == 1 || levelId == 2)
        {
            int fieldId = levelId == 1 ? ID_ELEMENT_ID : ID_PARENT_ID;
            String roleId = resultSet.getString(fieldId).trim();
            container = createRole(roleId);
        }
        else
        {
            throw new RPMException("Unsupported level ID: " + levelId);
        }
        return container;
    }

    public SecurityRole createRole(String roleId) throws RPMException
    {
        SecurityRole securityRole = null;
        SecurityRoleTypeMapping srtm = (SecurityRoleTypeMapping) SECURITY_ROLE_MAP
            .get(roleId);
        if (srtm != null)
        {
            try
            {
                securityRole = (SecurityRole) srtm.getSecurityRole()
                    .newInstance();
            }
            catch (Exception x)
            {
                // Exception is handled below.
            }
        }
        if (securityRole == null)
        {
            throw new RPMException("Unsupported security role ID: " + roleId);
        }
        return securityRole;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (rpmScope != null)
        {

            if (rpmScope.getParent() != null
                    && !(rpmObject instanceof SecurityRoleCategory))
            {
                loadParent((GenericSecurityRole) rpmObject, rpmScope,
                    messageContext, relatedFields, index, isDelta);
            }

            if (rpmScope instanceof SecurityRoleScope)
            {

                SecurityRoleScope scope = (SecurityRoleScope) rpmScope;
                if (scope.getSecurityRoles() != null)
                {
                    // security roles
                    loadSecurityRoles((GenericSecurityRole) rpmObject, scope,
                        messageContext, isDelta);
                }
            }
        }

        return rpmObject;
    }

    private void loadSecurityRoles(GenericSecurityRole genericSecurityRole,
            SecurityRoleScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        String extraWhere = NAME_PARENT_ID + "='" + genericSecurityRole.getID()
                + '\'';
        SecurityRole[] children = genericSecurityRole.getSecurityRoles();
        ArrayList list = loadByForeignKeyWithSpecifiedParent(
            genericSecurityRole, children, scope.getSecurityRoles(),
            messageContext, this, null, extraWhere, null, isDelta);
        genericSecurityRole.setSecurityRoles((SecurityRole[]) ArrayUtil
            .listToArray(SecurityRole.class, list));
    }

    private void loadParent(GenericSecurityRole genericSecurityRole,
            RPMObjectScope rpmscope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (!isDelta) // Parent is never saved.
        {
            GenericSecurityRole newParent = null;
            String parentId = (String) relatedFields.get(index, ID_PARENT_ID);
            if (parentId != null)
            {
                newParent = new GenericSecurityRole();
                newParent.setID(parentId);
                newParent = (GenericSecurityRole) loadByPrimaryKey(newParent,
                    rpmscope.getParent(), messageContext, false);
            }
            genericSecurityRole.setParent(newParent);
        }
    }

    private void setSecurityType(Object securityRole, Object securityType)
            throws RPMException
    {
        Class[] argTypes =
        {
            securityType.getClass()
        };
        Object[] args =
        {
            securityType
        };
        try
        {
            securityRole.getClass().getMethod("setType", argTypes).invoke(
                securityRole, args);
        }
        catch (Exception e)
        {
            throw new RPMException(e);
        }
    }

    // private String getSecurityTypeName(Object securityRole) {
    // // This method catch the exception but don't throw them back and returns
    // null by design;
    // Class[] classes = { };
    // Object[] objects = { };
    // Object securityType = null;
    // String securityTypeName = null;
    // try {
    // securityType =
    // securityRole.getClass().getMethod("getType",classes).invoke(securityRole,objects);
    // securityTypeName =
    // (String)securityType.getClass().getMethod("getValue",classes).invoke(securityType,objects);
    // } catch (IllegalArgumentException e) {
    // logger.error(e);
    // } catch (SecurityException e) {
    // logger.error(e);
    // } catch (IllegalAccessException e) {
    // logger.error(e);
    // } catch (InvocationTargetException e) {
    // logger.error(e);
    // } catch (NoSuchMethodException e) {
    // logger.error(e);
    // }
    // return securityTypeName;
    // }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        String tempHolder;
        GenericSecurityRole genericSecurityRole = (GenericSecurityRole) rpmObject;
        tempHolder = rs.getString(ID_ELEMENT_ID);
        if (tempHolder != null)
            tempHolder = tempHolder.trim();
        genericSecurityRole.setID(tempHolder);// INTEGER 4 Yes
        if (isDelta)
        {
            genericSecurityRole.deltaTypeID(Manager.getInteger(rs, ID_TYPE_ID));// INTEGER
            // 4 No
            tempHolder = rs.getString(ID_ELEMENT_NAME);
            if (tempHolder != null)
                tempHolder = tempHolder.trim();
            genericSecurityRole.deltaName(tempHolder);// CHARACTER 50 Yes
            int level = rs.getInt(ID_LEVEL_ID);
            if (level != 0) // (level == 0 -> SecurityRoleCategory)
            {
                if (level == 1)
                {
                    SecurityRoleTypeMapping srtm = (SecurityRoleTypeMapping) SECURITY_ROLE_MAP
                        .get(genericSecurityRole.getID());
                    setSecurityType(genericSecurityRole, srtm
                        .getSecurityRoleType());
                }
                else
                {
                    tempHolder = rs.getString(ID_PARENT_ID);
                    if (tempHolder != null)
                        tempHolder = tempHolder.trim();
                    String parentID = tempHolder;
                    if (parentID != null)
                    {
                        SecurityRoleTypeMapping srtm = (SecurityRoleTypeMapping) SECURITY_ROLE_MAP
                            .get(parentID);
                        setSecurityType(genericSecurityRole, srtm
                            .getSecurityRoleType());
                    }
                }
                deltaClearanceLevel(rs, (SecurityRole) genericSecurityRole);
            }
        }
        else
        {
            genericSecurityRole
                .assignTypeID(Manager.getInteger(rs, ID_TYPE_ID));// INTEGER 4
            // No
            tempHolder = rs.getString(ID_ELEMENT_NAME);
            if (tempHolder != null)
                tempHolder = tempHolder.trim();
            genericSecurityRole.setName(tempHolder);// CHARACTER 50 Yes
            int level = rs.getInt(ID_LEVEL_ID);
            if (level != 0)// (level == 0 -> SecurityRoleCategory)
            {
                if (level == 1)
                {
                    SecurityRoleTypeMapping srtm = (SecurityRoleTypeMapping) SECURITY_ROLE_MAP
                        .get(genericSecurityRole.getID());
                    setSecurityType(genericSecurityRole, srtm
                        .getSecurityRoleType());
                }
                else
                {
                    tempHolder = rs.getString(ID_PARENT_ID);
                    if (tempHolder != null)
                        tempHolder = tempHolder.trim();
                    String parentID = tempHolder;
                    if (parentID != null)
                    {
                        SecurityRoleTypeMapping srtm = (SecurityRoleTypeMapping) SECURITY_ROLE_MAP
                            .get(parentID);
                        setSecurityType(genericSecurityRole, srtm
                            .getSecurityRoleType());
                    }
                }
                setClearanceLevel(rs, (SecurityRole) genericSecurityRole);
            }
        }

        return genericSecurityRole;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        if (parentId != null)
            parentId = parentId.trim();
        relatedFields.put(index, ID_PARENT_ID, parentId);

        return relatedFields;
    }

    private void setClearanceLevel(ResultSet rs, SecurityRole securityRole)
            throws SQLException
    {
        int[] clearanceLevel = getClearanceLevelFromResultSet(rs);
        SecurityUtil.setClearanceLevel(clearanceLevel, securityRole);
    }

    private void deltaClearanceLevel(ResultSet rs, SecurityRole securityRole)
            throws SQLException, RPMException
    {
        // Set the clearance levels in the security role object
        // from the values in the result set

        int[] clearanceLevel = getClearanceLevelFromResultSet(rs);
        SecurityUtil.deltaClearanceLevel(clearanceLevel, securityRole);
    }

    public static int[] getClearanceLevelFromResultSet(ResultSet rs)
            throws SQLException
    {
        int[] clearanceLevels = new int[SecurityUtil.SECURITY_ROLE_LEVELS_COUNT];
        for (int i = SecurityRoleManager.ID_INT_1; i <= SecurityRoleManager.ID_INT_26; i++)
        {
            clearanceLevels[i - SecurityRoleManager.ID_INT_1] = rs.getInt(i);
        }
        return clearanceLevels;
    }

    private boolean isSecurityRoleGroup(RPMObject rpmObject)
    {
        return rpmObject != null && rpmObject instanceof SecurityRoleCategory;
    }

    private boolean isSecurityRoleLevel(RPMObject rpmObject)
    {
        return rpmObject != null
                && rpmObject instanceof SecurityRole
                && rpmObject.retrieveTypeID() != null
                && rpmObject.retrieveTypeID().intValue() == SecurityRole.TYPE_ID_2;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        // beforeUpdate(pObject); // listener callback
        if (rpmObject == null)
            return null;

        // TODO move code to validation layer
        // if(isSecurityRoleGroup(rpmObject) || !isSecurityRoleLevel(rpmObject))
        // {
        // if (((GenericSecurityRole)rpmObject).testNameModified()){
        // RPMException e = new RPMException("You can change the name of a level
        // of role only.");
        // //log the error
        // logger.error(e);
        // throw e;
        // }
        // }
        if (isSecurityRoleLevel(rpmObject))
        {
            SecurityRole securityRole = (SecurityRole) rpmObject;
            if (securityRole.testNameModified())
                SP_U_SECU_NAME(securityRole, messageContext);
        }

        if (!isSecurityRoleGroup(rpmObject))
        {
            SecurityRole securityRole = (SecurityRole) rpmObject;
            if (SecurityUtil.isAnyClearanceLevelModified(securityRole))
                SP_U_SECURITY(securityRole, messageContext);
        }

        //      TODO Completer la methode, now we are saving child roles, but, not
        // children of groups

        //        if (rpmScope != null)
        //        {
        //            //we'll save children for primary roles only (others can't have children)          
        //            if (isSecurityRoleLevel(rpmObject) && SECURITY_ROLE_MAP.containskey(rpmObject.getID())
        //            {
        //                saveSecurityRoles((GenericSecurityRole) rpmObject,
        //                    (SecurityRoleScope) rpmScope, messageContext);
        //            }
        //        }

        return rpmObject;
    }
    
    

    //  private GenericSecurityRole saveSecurityRoles(GenericSecurityRole role,
    //  SecurityRoleScope scope, MessageContext messageContext)
    //  throws RPMException, SQLException, ParseException
    //{
    //SecurityRole[] roles = (SecurityRole[]) updateChildren(role, role
    //  .getSecurityRoles(), scope.getSecurityRoles(), messageContext, null);
    //role.setSecurityRoles(roles);
    //return role;
    //}

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        if (rpmObject != null)
        {
            // SecurityRole sr = (SecurityRole) rpmObject;
            // TODO move validation code
            // in the validation, we make sure that the parent is a security
            // role (SecurityRole.TYPE_ID)
            // and here we assign the current object the security role level
            // type Id (SecurityRole.TYPE_ID_2)
            // if ((sr.getName()==null) || (sr.retrieveTypeID()==null)) {
            // RPMException e = new RPMException("Missing name or type id");
            // //log the error
            // logger.error(e);
            // throw e;
            // }

            // Assign security role level type id
            rpmObject.assignTypeID(new Integer(SecurityRole.TYPE_ID_2));

            rpmObject.setID(SP_I_SECURITY((SecurityRole) rpmObject,
                messageContext));
        }
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject != null)
        {
            //TODO use direct sql instead
            SecurityRole reloadedSecurityRole = (SecurityRole)loadByPrimaryKey(rpmObject,null, messageContext, false);
            if(reloadedSecurityRole.retrieveTypeID()!=null && reloadedSecurityRole.retrieveTypeID().intValue()==SecurityRole.TYPE_ID_2)
            {
                SecurityRole sr = (SecurityRole) rpmObject;
                SP_D_SECURITY(sr, messageContext);                
            }
            else
            {
    			throw new RPMException("Security roles of level 1 cannot be deleted.");
            }

        }
    }

    public static int[] getClearanceLevelFromID(MessageContext messageContext,
            String id)
    {
        int[] clearanceLevels = new int[SecurityUtil.SECURITY_ROLE_LEVELS_COUNT];
        Connection c = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        String selectStmt = "SELECT " + ALL_FIELDS + " FROM " + TABLE_NAME
                + " WHERE (" + NAME_ELEMENT_ID + "='" + id + "')";

        logger.debug(selectStmt);

        try
        {
            c = Manager.getConnection(messageContext);

            ps = c.prepareStatement(selectStmt, ResultSet.TYPE_FORWARD_ONLY,
                ResultSet.CONCUR_READ_ONLY);
            try
            {
                rs = ManagerUtil.executeSqlQuery(messageContext, ps,
                    selectStmt, null);

            }
            catch (Exception e)
            {
                logger.error(e);
            }
            if (rs.next())
            {
                for (int i = SecurityRoleManager.ID_INT_1; i <= SecurityRoleManager.ID_INT_26; i++)
                {
                    clearanceLevels[i - SecurityRoleManager.ID_INT_1] = rs
                        .getInt(i);
                }
            }
        }
        catch (SQLException e)
        {
            logger.error(e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return clearanceLevels;
    }

    // private static void loadRoleTypeNameIdMap(MessageContext messageContext)
    // throws SQLException {
    // Connection c = null;
    // PreparedStatement ps = null;
    // ResultSet rs = null;
    // String selectStmt = "SELECT "+NAME_ELEMENT_ID+", "+NAME_ELEMENT_NAME+
    // " FROM "+TABLE_NAME+" WHERE ("+NAME_LEVEL_ID+"=1)";
    //        
    // logger.debug(selectStmt);
    //        
    // try {
    // c = Manager.getConnection(messageContext);
    //			                
    // ps = c.prepareStatement(selectStmt,
    // ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
    // try
    // {
    // rs = ps.executeQuery();
    //
    // }catch(Exception e)
    // {
    // logger.error(e);
    // }
    // while(rs.next()) {
    // ROLE_TYPE_NAME_ID_MAPPING.put(rs.getString(2).trim(),rs.getString(1).trim());
    // }
    // } catch (SQLException e) {
    // logger.error(e);
    // } finally {
    // Manager.close(rs);
    // Manager.close(ps);
    // Manager.releaseConnection(c);
    // }
    // }
    /*
     * for Application Administration/Security/WBS
     */
    private void SP_U_SECURITY(SecurityRole securityRole,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_SECURITY";
        Object args[] = new Object[28];
        args[0] = securityRole.getID();
        int[] clearanceLevels = SecurityRoleManager.getClearanceLevelFromID(
            messageContext, securityRole.getID());
        clearanceLevels = SecurityUtil.getClearanceLevels(securityRole,
            clearanceLevels);
        // TODO activate later
        for (int i = 0; i < clearanceLevels.length; i++)
        {
            args[i + 1] = new Integer(clearanceLevels[i]);
        }
        // for(int i=0; i<24;i++){
        // args[i+1] = new Integer(clearanceLevels[i]);
        // }
        // args[25] = new Integer(0);
        // args[26] = new Integer(0);
        args[27] = getUser(messageContext).getID();
        
//        SecurityRulesMaps.applyBusinessRules(args);

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_SECU_NAME(SecurityRole securityRole,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_SECU_NAME";
        Object args[] = new Object[3];
        args[0] = securityRole.getID();
        args[1] = securityRole.getName();
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private String SP_I_SECURITY(SecurityRole securityRole,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_SECURITY";
        Object args[] = new Object[4];

        // TODO move validation code to a validation class

        // if (securityRole.retrieveTypeID().intValue()!=SecurityRole.TYPE_ID_2)
        // {
        // RPMException e = new RPMException("You can only insert Level element
        // (typeid="+SecurityRole.TYPE_ID_2+".");
        // //log the error
        // logger.error(e);
        // throw e;
        // }
        // String parentName = getSecurityTypeName(securityRole);
        // if (parentName==null) {
        // RPMException e = new RPMException("Wrong SecurityRoleType.");
        // //log the error
        // logger.error(e);
        // throw e;
        // }
        // String parentID = getRoleTypeIdFromName(messageContext, parentName);
        // if (parentID==null) {
        // RPMException e = new RPMException("Wrong SecurityRoleType.");
        // //log the error
        // logger.error(e);
        // throw e;
        // }
        args[0] = securityRole.getName();
        args[1] = securityRole.getParent().getID();
        args[2] = securityRole.retrieveTypeID();
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    private String SP_D_SECURITY(SecurityRole securityRole,
            MessageContext messageContext) throws SQLException, RPMException
    {
        // TODO validate later
        // move code to validation layer

        String name = "SP_D_SECURITY";
        Object args[] = new Object[4];
        // if (securityRole.retrieveTypeID().intValue()!=SecurityRole.TYPE_ID_2)
        // {
        // RPMException e = new RPMException("You can only delete Level element
        // (typeid="+SecurityRole.TYPE_ID_2+".");
        // //log the error
        // logger.error(e);
        // throw e;
        // }
        // String parentName = getSecurityTypeName(securityRole);
        // if (parentName==null) {
        // RPMException e = new RPMException("Wrong SecurityRoleType.");
        // //log the error
        // logger.error(e);
        // throw e;
        // }
        // String parentID = getRoleTypeIdFromName(messageContext, parentName);
        // if (parentID==null) {
        // RPMException e = new RPMException("Wrong SecurityRoleType.");
        // //log the error
        // logger.error(e);
        // throw e;
        // }

        String parentId = ManagerUtil.getParentId(this, securityRole,
            messageContext, SecurityRoleManager.TABLE_NAME);
        args[0] = securityRole.getID();
        args[1] = parentId;
        args[2] = "X";
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }
}
