package com.ibm.rpm.timesheet.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class AbstractStepCheckpoint extends AbstractCheckpoint
{
    protected static AbstractStepCheckpoint instance = new AbstractStepCheckpoint();

    public static AbstractStepCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {

        // Initialize
        GenericStep genericStep = (GenericStep) rpmObject;
        
        //parent must be null
        if (genericStep.getParent() != null)
        {
            GenericValidator.validateMustAlwaysBeNull(genericStep,
                ValidationConstants.PARENT_FIELD, genericStep
                    .getParent(), msgCtx);
            return;
        }

        //validate taskAssignment mandatory if object is new
        TaskAssignment reloadedTA = null;
        if (genericStep.getID() == null)
        {

            RPMObject taskAssignment = genericStep.getTaskAssignment();

            if (!GenericValidator.validateMandatoryID(genericStep,
                ValidationConstants.TASK_ASSIGNMENT_FIELD, taskAssignment,
                msgCtx))
            {
                return;
            }

            TaskAssignment receivedTA = genericStep.getTaskAssignment();

            //Always reload the taskAssignment
            reloadedTA = (TaskAssignment) TimesheetManagerUtil
                .reloadAbstractTaskAssignment(receivedTA, msgCtx);

            if (reloadedTA != null)
            {
                genericStep.setTaskAssignment(reloadedTA);
            }
            else
            {
                //nothing can be done without a valid task assignment
                String[] errorParams =
                {
                        StringUtil.getShortClassName(receivedTA.getClass()),
                        receivedTA.getID()

                };
                RPMException e = new RPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.OBJECT_DOES_NOT_EXIST,
                    errorParams);
                addException(e, msgCtx);
                return;
            }

        }
        else
        {
            //reload the taskAssignment
            reloadedTA = loadTaskAssignment(genericStep, msgCtx);

            if (reloadedTA != null)
            {
                genericStep.setTaskAssignment(reloadedTA);
            }
            else
            {
                return;
            }
        }

        
        //Test read-only fields
        //Values are set to null to make sure we are not using these wrong ids.
        if (genericStep.testProjectModified())
        {
            GenericValidator.validateReadOnly(genericStep,
                ValidationConstants.PROJECT_FIELD, msgCtx);
            genericStep.setProject(null);
        }
        if (genericStep.testResourceModified())
        {
            GenericValidator.validateReadOnly(genericStep,
                ValidationConstants.RESOURCE_FIELD, msgCtx);
            genericStep.setResource(null);
        }
        if (genericStep.testWorkElementModified())
        {
            GenericValidator.validateReadOnly(genericStep,
                ValidationConstants.WORK_ELEMENT_FIELD, msgCtx);
            genericStep.setWorkElement(null);
        }        
    }

    private TaskAssignment loadTaskAssignment(GenericStep genericStep,
            MessageContext messageContext)
    {
        TaskAssignment taskAssignment = null;
        String taskAssignmentId = null;
        try
        {
            taskAssignmentId = TimesheetManagerUtil
                .retrieveStep_TaskAssignmentId(genericStep.getID(),
                    messageContext);

            TaskAssignment temp = new TaskAssignment();
            temp.setID(taskAssignmentId);
            if (taskAssignmentId != null)
            {
                taskAssignment = (TaskAssignment) TimesheetManagerUtil
                    .reloadAbstractTaskAssignment(temp, messageContext);
            }
            
            if (taskAssignment == null)
            {
                //nothing can be done without a valid task assignment
                String[] errorParams =
                {
                        StringUtil.getShortClassName(temp.getClass()),
                        temp.getID()

                };
                RPMException e = new RPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.OBJECT_DOES_NOT_EXIST,
                    errorParams);
                addException(e, messageContext);
            }
        }
        catch (RPMException e1)
        {
            addException(e1, messageContext);
            return null;
        }

        return taskAssignment;

    }

    protected void validateName(GenericStep genericStep,
            MessageContext messageContext)
    {

        if (genericStep.testNameModified())
        {
            GenericValidator.validateMaxLength(genericStep,
                ValidationConstants.NAME_FIELD, genericStep.getName(),
                ValidationConstants.STEP_NAME_CHARACTER_LIMIT, messageContext);
        }
    }
    
    public void validateDelete(RPMObject rpmObject,
            MessageContext messageContext)
    {

        GenericStep step = (GenericStep) rpmObject;

        long workToDate = 0;

        try
        {
            workToDate = TimesheetManagerUtil.retrieveStep_ActualWork(step
                .getID(), messageContext);

            if (workToDate > 0)
            {

                String[] params = new String[]
                {
                        StringUtil.getShortClassName(rpmObject.getClass()),
                        rpmObject.getID()
                };

                RPMException rpmException = new RPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CANNOT_DELETE_ITEM_WITH_ACTUALS,
                    params, rpmObject.getClass().getName());

                addException(rpmException, messageContext);
            }
        }
        catch (RPMException e)
        {
            addException(e, messageContext);
        }
    }
}