package com.ibm.rpm.timesheet.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.ValidateSerializer;
import com.ibm.rpm.timesheet.constants.ErrorCodes;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.StepTimesheetManager;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;

/**
 * @author shahla
 */
public class CreditTimesheetCheckpoint extends AbstractTimesheetCheckpoint
{
    private static final List TIMESHEET_PARENT_LIST = new ArrayList();

    protected static CreditTimesheetCheckpoint instance = new CreditTimesheetCheckpoint();

    static
    {
        TIMESHEET_PARENT_LIST.add(Credit.class);
    }

    public static CreditTimesheetCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {

        // validate the parent
        if (!super.isValidParent(rpmObject, messageContext, true, true,
            TIMESHEET_PARENT_LIST))
        {
            return;
        }

        super.validateSave(rpmObject, messageContext);
        if (!messageContext.isSuccessful())
            return;

        if (!GenericValidator.validateInstance(rpmObject,
            CreditTimesheet.class, messageContext))
        {
            return;
        }

        CreditTimesheet creditTimesheet = (CreditTimesheet) rpmObject;

        Credit credit = (Credit) creditTimesheet.getParent();

        // weekOf and creditWeekOf are needed before doing more validation
        if (!validWeekOf(creditTimesheet, messageContext)
                | !validCreditWeekOf(creditTimesheet, messageContext))
        {
            return;
        }
        

        //replace weekof with a date that matches the beginning of the week
        TimesheetManagerUtil.setValidWeekOf(creditTimesheet, messageContext);
        
        
        ValidateSerializer serializer = messageContext.getObjectsToValidate();
        if (!serializer.before(credit, creditTimesheet))
        {
            GenericValidator.validateMandatoryID(creditTimesheet,
                ValidationConstants.PARENT_FIELD, creditTimesheet.getParent(),
                messageContext);
        }

        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(
            creditTimesheet, messageContext);

        if (summaryTimesheet == null)
        {

            String[] params = new String[]
            {
                DateUtil.convertToString(creditTimesheet.getWeekOf())
            };

            RPMException e = new RPMException(
                ErrorCodes.NO_SUMMARY_TIMESHEET_FOR_WEEKOF, params,
                creditTimesheet.getClass().getName());
            addException(e, messageContext);
            return;
        }    

        if (!validApprovalStatus(summaryTimesheet, messageContext))
        {
            return;
        }

        //validate the existence of a stepTimesheet for the same weekOf
        validateStepTimesheets(creditTimesheet, messageContext);
                
        SummaryTimesheet creditSummaryTimesheet = loadSummaryTimesheetForCreditWeekOf(
            creditTimesheet, messageContext);

        if(creditSummaryTimesheet == null){
            return;
        }
        
        validateCurrentWeekCreditHours(creditTimesheet, summaryTimesheet, messageContext);
        
        validateCreditHours(creditTimesheet, creditSummaryTimesheet, messageContext);
        

    }

    //TODO check if possible to do a direct sql
    private SummaryTimesheet loadSummaryTimesheet(CreditTimesheet timesheet,
            MessageContext messageContext)
    {
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        try
        {
            GenericStep step = (GenericStep) timesheet.getParent();
            
            String taskAssignmentId = null;
            if (step.getTaskAssignment() != null)
            {
                taskAssignmentId = step.getTaskAssignment().getID();
            }
            else
            {
                taskAssignmentId = TimesheetManagerUtil
                    .retrieveStep_TaskAssignmentId(step.getID(), messageContext);
            }

            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(SummaryTimesheet.class);
            summaryTimesheet.setID(taskAssignmentId);
            summaryTimesheet.setWeekOf(timesheet.getWeekOf());

            summaryTimesheet = (SummaryTimesheet) manager.loadByPrimaryKey(
                summaryTimesheet, null, messageContext, false);
        }
        catch (Exception e)
        {
            RPMException rpmException = new RPMException(e);
            rpmException.assignContainer(timesheet);
            addException(rpmException, messageContext);

        }
        return summaryTimesheet;
    }

    private boolean validCreditWeekOf(CreditTimesheet timesheet,
            MessageContext msgCtx)
    {
        return GenericValidator.validateMandatory(timesheet,
            ValidationConstants.CREDIT_WEEK_OF_FIELD, timesheet.getCreditWeekOf(),
            msgCtx);
    }

    private void validateStepTimesheets(CreditTimesheet creditTimesheet,
            MessageContext messageContext)
    {
        boolean valid = false;

        Credit credit = (Credit) creditTimesheet.getParent();
        
        //A stepTimesheet must exist for this week_of
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect(StepTimesheetManager.NAME_ACTIVITY_ID);
        stmt.appendFrom(StepTimesheetManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(StepTimesheetManager.NAME_WEEK_OF);
        stmt.appendSubSelect(StepTimesheetManager.NAME_ACTIVITY_ID,
            StepManager.NAME_ELEMENT_ID, StepManager.TABLE_NAME,
            StepManager.NAME_TSK_ASSIGNMENT_ID + " = ?");

        List result = null;
        try
        {
            String taskAssignmentId = null;
            if (credit.getTaskAssignment() != null)
            {
                taskAssignmentId = credit.getTaskAssignment().getID();
            }
            else
            {
                taskAssignmentId = TimesheetManagerUtil
                    .retrieveStep_TaskAssignmentId(credit.getID(), messageContext);
            }            
            
            result = ManagerUtil.select(null, null, null, stmt, new Object[]
            {
                    DateUtil.convertToString(creditTimesheet.getWeekOf()),
                    taskAssignmentId

            }, messageContext);
        }
        catch (RPMException e)
        {
            addException(e, messageContext);
            valid = false;
        }

        valid = result != null && result.size() > 0;

        if (!valid)
        {
            String[] errorParams =
            {
                DateUtil.convertToString(creditTimesheet.getWeekOf())
            };
            RPMException e = new RPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.NO_STEP_TIMESHEET_FOR_WEEK_OF,
                errorParams, credit.getClass().getName());
            addException(e, messageContext);
        }

    }
    
    //validate that a submitted SummaryTimesheet exist for the creditWeekOf
    private SummaryTimesheet loadSummaryTimesheetForCreditWeekOf(CreditTimesheet timesheet,
            MessageContext messageContext)
    {
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        SummaryTimesheetScope scope = new SummaryTimesheetScope();
        scope.setApprovalStatus(new TimesheetApprovalStatusScope());

        try
        {
            GenericStep step = (GenericStep) timesheet.getParent();
            
            String taskAssignmentId = null;
            if (step.getTaskAssignment() != null)
            {
                taskAssignmentId = step.getTaskAssignment().getID();
            }
            else
            {
                taskAssignmentId = TimesheetManagerUtil
                    .retrieveStep_TaskAssignmentId(step.getID(), messageContext);
            }

            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(SummaryTimesheet.class);
            summaryTimesheet.setID(taskAssignmentId);
            summaryTimesheet.setWeekOf(timesheet.getCreditWeekOf());

            summaryTimesheet = (SummaryTimesheet) manager.loadByPrimaryKey(
                summaryTimesheet, scope, messageContext, false);
        }
        
        catch (Exception e)
        {
            RPMException rpmException = new RPMException(e);
            rpmException.assignContainer(timesheet);
            addException(rpmException, messageContext);

        }
        
          /*if (summaryTimesheet == null
                || summaryTimesheet.getApprovalStatus() == null
                || summaryTimesheet.getApprovalStatus().getApprovalStatus() != TimesheetStatus.Approved)
                {*/
            //TODO gmeloche complete if with || summaryTimesheet.getApprovalStatus().getApprovalStatus() != TimesheetStatus.Approved
            if (summaryTimesheet == null
                    || summaryTimesheet.getApprovalStatus() == null)
            {            
            String[] params = new String[]
            {
                DateUtil.convertToString(timesheet.getCreditWeekOf())
            };

            RPMException e = new RPMException(
                ErrorCodes.NO_APPROVED_SUMMARY_TIMESHEET_FOR_WEEKOF, params,
                timesheet.getClass().getName());
            addException(e, messageContext);
            return null;
        }
            
        return summaryTimesheet;
  
    }
    
    //checks that credit hours are not greater than the summaryTimesheet hours
    private void validateCurrentWeekCreditHours(CreditTimesheet creditTimesheet,
            SummaryTimesheet summaryTimesheet, MessageContext messageContext)
    {
        
        TimesheetManagerUtil.initWorkingHours(creditTimesheet);
        
        //monday
        validateMaxHours(creditTimesheet,
            ValidationConstants.REGULAR_MONDAY_FIELD, creditTimesheet
                .getRegularMonday(), summaryTimesheet.getRegularMonday(),
            messageContext);

        validateMaxHours(creditTimesheet,
            ValidationConstants.SPECIAL_MONDAY_FIELD, creditTimesheet
                .getSpecialMonday(), summaryTimesheet.getSpecialMonday(),
            messageContext);
        
        //tuesday
        validateMaxHours(creditTimesheet,
            ValidationConstants.REGULAR_TUESDAY_FIELD, creditTimesheet
                .getRegularTuesday(), summaryTimesheet.getRegularTuesday(),
            messageContext);

        validateMaxHours(creditTimesheet,
            ValidationConstants.SPECIAL_TUESDAY_FIELD, creditTimesheet
                .getSpecialTuesday(), summaryTimesheet.getSpecialTuesday(),
            messageContext);       

        //Wednesday
        validateMaxHours(creditTimesheet,
            ValidationConstants.REGULAR_WEDNESDAY_FIELD, creditTimesheet
                .getRegularWednesday(), summaryTimesheet.getRegularWednesday(),
            messageContext);

        validateMaxHours(creditTimesheet,
            ValidationConstants.SPECIAL_WEDNESDAY_FIELD, creditTimesheet
                .getSpecialWednesday(), summaryTimesheet.getSpecialWednesday(),
            messageContext);
        
        //Thursday
        validateMaxHours(creditTimesheet,
            ValidationConstants.REGULAR_THURSDAY_FIELD, creditTimesheet
                .getRegularThursday(), summaryTimesheet.getRegularThursday(),
            messageContext);

        validateMaxHours(creditTimesheet,
            ValidationConstants.SPECIAL_THURSDAY_FIELD, creditTimesheet
                .getSpecialThursday(), summaryTimesheet.getSpecialThursday(),
            messageContext);
        
        //Friday
        validateMaxHours(creditTimesheet,
            ValidationConstants.REGULAR_FRIDAY_FIELD, creditTimesheet
                .getRegularFriday(), summaryTimesheet.getRegularFriday(),
            messageContext);

        validateMaxHours(creditTimesheet,
            ValidationConstants.SPECIAL_FRIDAY_FIELD, creditTimesheet
                .getSpecialFriday(), summaryTimesheet.getSpecialFriday(),
            messageContext);
        
        //Saturday
        validateMaxHours(creditTimesheet,
            ValidationConstants.REGULAR_SATURDAY_FIELD, creditTimesheet
                .getRegularSaturday(), summaryTimesheet.getRegularSaturday(),
            messageContext);

        validateMaxHours(creditTimesheet,
            ValidationConstants.SPECIAL_SATURDAY_FIELD, creditTimesheet
                .getSpecialSaturday(), summaryTimesheet.getSpecialSaturday(),
            messageContext);
        
        //Sunday
        validateMaxHours(creditTimesheet,
            ValidationConstants.REGULAR_SUNDAY_FIELD, creditTimesheet
                .getRegularSunday(), summaryTimesheet.getRegularSunday(),
            messageContext);

        validateMaxHours(creditTimesheet,
            ValidationConstants.SPECIAL_SUNDAY_FIELD, creditTimesheet
                .getSpecialSunday(), summaryTimesheet.getSpecialSunday(),
            messageContext);
        
    }

    private void validateCreditHours(CreditTimesheet creditTimesheet,
            SummaryTimesheet summaryTimesheet, MessageContext messageContext)
    {
        
        TimesheetManagerUtil.initWorkingHours(creditTimesheet);
        double creditHours = TimesheetManagerUtil
        .calculateBillableRegular(creditTimesheet) + TimesheetManagerUtil
        .calculateBillableSpecial(creditTimesheet);
        
        if(creditHours == 0){
            return;
        }
    
        TimesheetManagerUtil.initWorkingHours(summaryTimesheet);
        double actualHours = TimesheetManagerUtil
        .calculateBillableRegular(summaryTimesheet) + TimesheetManagerUtil
        .calculateBillableSpecial(summaryTimesheet);
    
        if (creditHours > actualHours)
        {
            String[] errorParams =
            {
                    DateUtil.convertToString(summaryTimesheet.getWeekOf()),
                    "" + creditHours, "" + actualHours
            };
    
            RPMException e = new RPMException(ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_ACTUAL_HOURS,
                errorParams, creditTimesheet.getClass().getName());
            addException(e, messageContext);
        }
    
    }
    
    public static boolean validateMaxHours(RPMObject object, String fieldName,
            Double fieldValue1, Double fieldValue2, MessageContext msgCtx)
    {

        if (fieldValue1 != null && fieldValue1.doubleValue() > 0
                && (fieldValue1.doubleValue() > fieldValue2.doubleValue()))
        {
            String[] errorParams =
            {
                    fieldName, fieldValue1.toString(), fieldValue2.toString()
            };

            RPMException e = new RPMException(
                ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }
}