package com.ibm.rpm.timesheet.checkpoints;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.RPMObjectUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.timesheet.constants.ErrorCodes;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.StepTimesheetManager;
import com.ibm.rpm.timesheet.managers.TimesheetApprovalStatusManager;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;
import com.ibm.rpm.timesheet.util.TimesheetStatusUtil;
import com.ibm.rpm.timesheet.util.TimesheetValidator;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;
import com.ibm.rpm.wbs.managers.WbsManager;

/**
 * @author shahla
 */
public class SummaryTimesheetCheckpoint extends AbstractTimesheetCheckpoint
{

    protected static SummaryTimesheetCheckpoint instance = new SummaryTimesheetCheckpoint();

    public static SummaryTimesheetCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {

        // validate the instance and the task assignment
        super.validateSave(rpmObject, msgCtx);
        if (!msgCtx.isSuccessful())
            return;

        if (!GenericValidator.validateInstance(rpmObject,
            SummaryTimesheet.class, msgCtx))
        {
            return;
        }

        // Initialize
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;
        SummaryTimesheetScope scope = (SummaryTimesheetScope) rpmScope;

        // parent must be null
        if (summaryTimesheet.getParent() != null)
        {
            GenericValidator.validateMustAlwaysBeNull(summaryTimesheet,
                ValidationConstants.PARENT_FIELD, summaryTimesheet.getParent(),
                msgCtx);
            return;
        }

        // validate that taskAssignment is not null
        if (!GenericValidator.validateMandatory(summaryTimesheet,
            ValidationConstants.TASK_ASSIGNMENT_FIELD, summaryTimesheet
                .getTaskAssignment(), msgCtx))
        {
            return;
        }

        GenericTaskAssignment receivedTA = summaryTimesheet.getTaskAssignment();

        // Always reload the taskAssignment
        GenericTaskAssignment reloadedTA = TimesheetManagerUtil
            .reloadAbstractTaskAssignment(receivedTA, msgCtx);

        if (reloadedTA != null)
        {
            // Never override the value in EETC and %C because these are saved
            // by SummaryTimesheetManager.
            if (receivedTA instanceof TaskAssignment
                    && reloadedTA instanceof TaskAssignment)
            {
                try
                {
                    RPMObjectUtil
                        .deltaAndSet(
                            reloadedTA,
                            com.ibm.rpm.wbs.constants.ValidationConstants.EETC_HOURS_FIELD,
                            ((TaskAssignment) receivedTA).getEETCHours());
                    RPMObjectUtil
                        .deltaAndSet(
                            reloadedTA,
                            com.ibm.rpm.wbs.constants.ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                            ((TaskAssignment) receivedTA)
                                .getPercentageComplete());
                }
                catch (RPMException e)
                {
                    addException(e, msgCtx);
                }
            }
            else if (GenericTaskAssignment.class.equals(receivedTA.getClass())
                    && reloadedTA instanceof TaskAssignment)
            {
                // When original is generic, it is like a TaskAssignment
                // with null EETC and %C.
                ((TaskAssignment) reloadedTA).setEETCHours(null);
                ((TaskAssignment) reloadedTA).setPercentageComplete(null);
            }

            summaryTimesheet.setTaskAssignment(reloadedTA);
        }
        else
        {
            // nothing can be done without a valid task assignment
            String[] errorParams =
            {
                    StringUtil.getShortClassName(receivedTA.getClass()),
                    receivedTA.getID()

            };
            RPMException e = new RPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.OBJECT_DOES_NOT_EXIST,
                errorParams);
            addException(e, msgCtx);
            return;
        }

        RPMObject taskAssignment = summaryTimesheet.getTaskAssignment();

        if (taskAssignment instanceof TaskAssignment
                || taskAssignment instanceof AdministrativeTaskAssignment)
        {
            if (!GenericValidator.validateMandatoryID(summaryTimesheet,
                ValidationConstants.TASK_ASSIGNMENT_FIELD, summaryTimesheet
                    .getTaskAssignment(), msgCtx))
            {
                return;
            }
        }
        else if (taskAssignment instanceof PersonalTaskAssignment)
        {
            if (taskAssignment.getID() == null)
            {
                if (!GenericValidator
                    .validateMandatory(summaryTimesheet,
                        ValidationConstants.PERSONAL_TASK_FIELD,
                        ((PersonalTaskAssignment) taskAssignment)
                            .getPersonalTask(), msgCtx))
                {
                    return;
                }
            }
        }

        // weekOf is needed before doing more validation
        if (!validWeekOf(summaryTimesheet, msgCtx))
            return;

        if (TimesheetManagerUtil.hasNewActualReported(summaryTimesheet))
        {
            if (!validApprovalStatus(summaryTimesheet, msgCtx))
            {
                return;
            }
        }

        SummaryTimesheet loadedTimesheet = null;
        try
        {
            loadedTimesheet = TimesheetManagerUtil.loadSummaryTimesheet(
                summaryTimesheet, rpmScope, msgCtx);
        }
        catch (RPMException e)
        {
            addException(e, msgCtx);
        }

        // Test read-only fields
        // Values are set to null to make sure we are not using these wrong ids.
        if (summaryTimesheet.testProjectModified())
        {
            GenericValidator.validateReadOnly(summaryTimesheet,
                ValidationConstants.PROJECT_FIELD, msgCtx);
            summaryTimesheet.setProject(null);
        }
        if (summaryTimesheet.testResourceModified())
        {
            GenericValidator.validateReadOnly(summaryTimesheet,
                ValidationConstants.RESOURCE_FIELD, msgCtx);
            summaryTimesheet.setResource(null);
        }
        if (summaryTimesheet.testWorkElementModified())
        {
            GenericValidator.validateReadOnly(summaryTimesheet,
                ValidationConstants.WORK_ELEMENT_FIELD, msgCtx);
            summaryTimesheet.setWorkElement(null);
        }

        if (taskAssignment instanceof TaskAssignment)
        {

            // ensure that TaskAssignmnet has a resource (not a profile) and its
            // parent is of type WorkElement (not ScopeElement)
            if (!validateResourceOfTaskAssignment(summaryTimesheet, msgCtx))
                return;

            // find out if TaskAssignment has steps
            int stepsCount = 0;

            // the number of steps for this task
            int tmp = getNumberOfSteps(summaryTimesheet, msgCtx);
            if (tmp != -1)
            {
                stepsCount = tmp;
            }

            // Load the project to perform some validation
            GenericProject project = TimesheetValidator.loadProject(
                (TaskAssignment) taskAssignment, msgCtx);

            // Ensure that the project has been published to the team
            TimesheetValidator.validateProjectPublishedToTeam(summaryTimesheet,
                project, ValidationConstants.PUBLISHED_FIELD, msgCtx);

            validateTaskAssignment(summaryTimesheet, msgCtx);

            validateEETCOnTaskAssignment((TaskAssignment) taskAssignment,
                msgCtx);

            validatePercentageCompleteOnTaskAssignment(summaryTimesheet, 
                (TaskAssignment) taskAssignment, msgCtx);

            validateReadOnlyHours(summaryTimesheet, loadedTimesheet,
                stepsCount, msgCtx);
        }
        else
        {
            // test timecodes and projects are null
            String condition = " "
                    + ValidationConstants.TASK_ASSIGNMENT_FIELD
                    + " type != "
                    + StringUtil.getShortClassName(TaskAssignment.class
                        .getName());

            if (summaryTimesheet.getTimeCode1() != null)
            {
                GenericValidator.validateMustBeNull(summaryTimesheet,
                    ValidationConstants.TIMECODE1_FIELD, summaryTimesheet
                        .getTimeCode1(), condition, msgCtx);
            }
            if (summaryTimesheet.getTimeCode2() != null)
            {
                GenericValidator.validateMustBeNull(summaryTimesheet,
                    ValidationConstants.TIMECODE2_FIELD, summaryTimesheet
                        .getTimeCode2(), condition, msgCtx);
            }
            if (summaryTimesheet.getProject() != null)
            {
                GenericValidator.validateMustBeNull(summaryTimesheet,
                    ValidationConstants.PROJECT_FIELD, summaryTimesheet
                        .getProject(), condition, msgCtx);
            }

        }

        // if there are actuals reported, ForecastStartDate will be
        // read-only and null
        if (TimesheetManagerUtil.hasNewActualReported(summaryTimesheet))
        {
            if (summaryTimesheet.testForecastStartDateModified())
            {
                GenericValidator.validateReadOnly(summaryTimesheet,
                    ValidationConstants.FORECAST_START_DATE_FIELD, msgCtx);
            }
        }
        
        // if there the task has been finished, ForecastFinishDate will
        // be read-only and null
        if (summaryTimesheet.testForecastFinishDateModified())
        {
            if (summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
            {
                if (((TaskAssignment) summaryTimesheet.getTaskAssignment())
                    .getComplete() != null
                        && ((TaskAssignment) summaryTimesheet
                            .getTaskAssignment()).getComplete().booleanValue())
                {
                    GenericValidator.validateReadOnly(summaryTimesheet,
                        ValidationConstants.FORECAST_FINISH_DATE_FIELD, msgCtx);
                }
                else
                {
                    // forecastFinishDate cannot before weekOf end date
                    int year = summaryTimesheet.getWeekOf().get(Calendar.YEAR);
                    int month = summaryTimesheet.getWeekOf().get(Calendar.MONTH);
                    int day = summaryTimesheet.getWeekOf().get(Calendar.DAY_OF_MONTH);
                    
                    Calendar weekOfEndDate = DateUtil.getCalendarInstance();
                    weekOfEndDate.set(Calendar.YEAR, year);
                    weekOfEndDate.set(Calendar.MONTH, month);
                    weekOfEndDate.set(Calendar.DATE, day);
                    
                    weekOfEndDate.add(Calendar.WEEK_OF_YEAR, 1);
                    weekOfEndDate.add(Calendar.DAY_OF_YEAR, -1);
                    
                    if (summaryTimesheet.getForecastFinishDate().before(
                        weekOfEndDate))
                    {
                        String[] errorParams =
                        {
                            StringUtil.getShortClassName(summaryTimesheet
                                .getClass())
                                    + " "
                                    + ValidationConstants.FORECAST_FINISH_DATE_FIELD
                        };
                        RPMException e = new RPMException(
                            ErrorCodes.FORECAST_FINISH_DATE_CANNOT_BEFORE_TIMESHEET_WEEKOF,
                            errorParams, summaryTimesheet.getClass()
                                .getName(),
                            ValidationConstants.FORECAST_FINISH_DATE_FIELD,
                            summaryTimesheet.getID());
                        e.setSeverity(SeverityLevel.Warning);
                        addException(e, msgCtx);
                    }
                }
            }
        }
        
        if (summaryTimesheet.testTotalRegularModified())
        {
            GenericValidator.validateReadOnly(summaryTimesheet,
                ValidationConstants.TOTAL_REGULAR_FIELD, msgCtx);

            // reset the read-only field value
            summaryTimesheet.setTotalRegular(loadedTimesheet.getTotalRegular());
        }

        if (summaryTimesheet.testTotalSpecialModified())
        {
            GenericValidator.validateReadOnly(summaryTimesheet,
                ValidationConstants.TOTAL_SPECIAL_FIELD, msgCtx);

            // reset the read-only field value
            summaryTimesheet.setTotalSpecial(loadedTimesheet.getTotalSpecial());
        }

        if (summaryTimesheet.testTotalWeekModified())
        {
            GenericValidator.validateReadOnly(summaryTimesheet,
                ValidationConstants.TOTAL_WEEK_FIELD, msgCtx);

            // reset the read-only field value
            summaryTimesheet.setTotalWeek(loadedTimesheet.getTotalWeek());
        }
        
        // check TimeCode
        TimesheetValidator.validateSummaryTimesheetTimeCodes(summaryTimesheet,
            loadedTimesheet,
            getNumberOfSteps(summaryTimesheet, msgCtx), scope, msgCtx);
    }

    private boolean validWeekOf(SummaryTimesheet summaryTimesheet,
            MessageContext msgCtx)
    {
        return GenericValidator.validateMandatory(summaryTimesheet,
            ValidationConstants.WEEK_OF_FIELD, summaryTimesheet.getWeekOf(),
            msgCtx);
    }

    private void validateTaskAssignment(SummaryTimesheet summaryTimesheet,
            MessageContext msgCtx)
    {
        TaskAssignment taskAssignment = (TaskAssignment) summaryTimesheet
            .getTaskAssignment();

        // Ensure that the proposed dates have been copied to planned.
        // Don't trust the object! Query the database for the real dates.

        WorkElement workElement = loadWorkElement(taskAssignment, msgCtx);

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(WbsManager.NAME_SCHED_START);
        stmt.appendComma();
        stmt.append(WbsManager.NAME_SCHED_FINISH);
        stmt.appendFrom(WbsManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(WbsManager.NAME_ELEMENT_ID);

        try
        {
            List result = ManagerUtil.select(new Class[]
            {
                    Calendar.class, Calendar.class
            }, workElement, null, stmt, new Object[]
            {
                workElement.getID()
            }, msgCtx);
            if (result != null && result.size() == 1)
            {
                List row = (List) result.get(0);

                WbsScheduleDate plannedDate = new WbsScheduleDate();
                plannedDate.setStartDate((Calendar) row.get(0));
                plannedDate.setFinishDate((Calendar) row.get(1));
                workElement.setPlanDate(plannedDate);
            }
        }
        catch (Exception e)
        {
            RPMException rpme = null;
            if (e.getClass() == RPMException.class)
            {
                rpme = (RPMException) e;
            }
            else
            {
                rpme = new RPMException(e);
            }

            addException(rpme, msgCtx);
        }

        TimesheetValidator.validateProjectInPlannedStage(summaryTimesheet,
            workElement, ValidationConstants.PROJECT_FIELD, msgCtx);
    }
    
    private void validateEETCOnTaskAssignment(TaskAssignment taskAssignment,
            MessageContext msgCtx)
    {
        if (taskAssignment.getEETCHours() != null
                && taskAssignment.testEETCHoursModified())
        {
            GenericValidator.validateGreaterThanOrEqual(taskAssignment,
                com.ibm.rpm.wbs.constants.ValidationConstants.EETC_HOURS_FIELD,
                taskAssignment.getEETCHours(),
                com.ibm.rpm.wbs.constants.ValidationConstants.EETC_HOURS_MIN,
                msgCtx);

            GenericValidator.validateWorkingHour(taskAssignment,
                com.ibm.rpm.wbs.constants.ValidationConstants.EETC_HOURS_FIELD,
                taskAssignment.getEETCHours(), msgCtx);
        }
    }
    
    private void validatePercentageCompleteOnTaskAssignment(SummaryTimesheet summaryTimesheet, 
            TaskAssignment taskAssignment, MessageContext msgCtx)
    {
        if (taskAssignment.getPercentageComplete() != null
                && taskAssignment.testPercentageCompleteModified())
        {
            // Read only if there is no actual
            if (taskAssignment.getActualWork() == null
                    || taskAssignment.getActualWork().equals(new Double(0)))
            {
                if (!TimesheetManagerUtil
                    .hasNewActualReported(summaryTimesheet))
                {
                    GenericValidator
                        .validateReadOnly(
                            taskAssignment,
                            com.ibm.rpm.wbs.constants.ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                            msgCtx);
                    taskAssignment
                        .setPercentageComplete(new Double(
                            com.ibm.rpm.wbs.constants.ValidationConstants.PERCENT_DURATION_COMPLETED_MIN));
                }
            }
            
            GenericValidator
                .validateRange(
                    taskAssignment,
                    com.ibm.rpm.wbs.constants.ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                    taskAssignment.getPercentageComplete(),
                    com.ibm.rpm.wbs.constants.ValidationConstants.PERCENTAGE_COMPLETE_MIN,
                    com.ibm.rpm.wbs.constants.ValidationConstants.PERCENTAGE_COMPLETE_MAX,
                    msgCtx);

            GenericValidator
                .validateDecimalDigits(
                    taskAssignment,
                    com.ibm.rpm.wbs.constants.ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                    taskAssignment.getPercentageComplete(), msgCtx);
        }
    }

    private void validateReadOnlyHours(SummaryTimesheet summaryTimesheet,
            SummaryTimesheet loadedTimesheet, int stepCount,
            MessageContext msgCtx)
    {

        if (stepCount > 0)
        {
            if (summaryTimesheet.testRegularMondayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet, ValidationConstants.REGULAR_MONDAY_FIELD,
                    StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setRegularMonday(loadedTimesheet
                        .getRegularMonday());
                }
            }

            if (summaryTimesheet.testSpecialMondayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet, ValidationConstants.SPECIAL_MONDAY_FIELD,
                    StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setSpecialMonday(loadedTimesheet
                        .getSpecialMonday());
                }
            }

            if (summaryTimesheet.testRegularTuesdayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet,
                    ValidationConstants.REGULAR_TUESDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setRegularTuesday(loadedTimesheet
                        .getRegularTuesday());
                }
            }

            if (summaryTimesheet.testSpecialTuesdayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet,
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setSpecialTuesday(loadedTimesheet
                        .getSpecialTuesday());
                }
            }

            if (summaryTimesheet.testRegularWednesdayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet,
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setRegularWednesday(loadedTimesheet
                        .getRegularWednesday());
                }
            }

            if (summaryTimesheet.testSpecialWednesdayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet,
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setSpecialWednesday(loadedTimesheet
                        .getSpecialWednesday());
                }
            }

            if (summaryTimesheet.testRegularThursdayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet,
                    ValidationConstants.REGULAR_THURSDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setRegularThursday(loadedTimesheet
                        .getRegularThursday());
                }
            }

            if (summaryTimesheet.testSpecialThursdayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet,
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setSpecialThursday(loadedTimesheet
                        .getSpecialThursday());
                }
            }

            if (summaryTimesheet.testRegularFridayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet, ValidationConstants.REGULAR_FRIDAY_FIELD,
                    StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setRegularFriday(loadedTimesheet
                        .getRegularFriday());
                }
            }

            if (summaryTimesheet.testSpecialFridayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet, ValidationConstants.SPECIAL_FRIDAY_FIELD,
                    StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setSpecialFriday(loadedTimesheet
                        .getSpecialFriday());
                }
            }

            if (summaryTimesheet.testRegularSaturdayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet,
                    ValidationConstants.REGULAR_SATURDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setRegularSaturday(loadedTimesheet
                        .getRegularSaturday());
                }
            }

            if (summaryTimesheet.testSpecialSaturdayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet,
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setSpecialSaturday(loadedTimesheet
                        .getSpecialSaturday());
                }
            }

            if (summaryTimesheet.testRegularSundayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet, ValidationConstants.REGULAR_SUNDAY_FIELD,
                    StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setRegularSunday(loadedTimesheet
                        .getRegularSunday());
                }
            }

            if (summaryTimesheet.testSpecialSundayModified())
            {
                if (!GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet, ValidationConstants.SPECIAL_SUNDAY_FIELD,
                    StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx))
                {
                    // reset the read-only field value
                    summaryTimesheet.setSpecialSunday(loadedTimesheet
                        .getSpecialSunday());
                }
            }

        }
    }

    private WorkElement loadWorkElement(TaskAssignment taskAssignment,
            MessageContext messageContext)
    {
        WorkElement workElement = new WorkElement();

        // retrieve the workElementId in task_assignments table

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect(TaskAssignmentManager.NAME_TASK_ID);
        stmt.appendFrom(TaskAssignmentManager.TABLE_NAME);
        stmt.appendWhere();
        stmt
            .appendEqualQuestionMark(TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);

        List result;
        try
        {
            result = ManagerUtil.select(new Class[]
            {
                String.class
            }, taskAssignment, ValidationConstants.TASK_ASSIGNMENT_FIELD, stmt,
                new Object[]
                {
                    taskAssignment.getID()
                }, messageContext);
        }
        catch (RPMException e1)
        {
            addException(e1, messageContext);
            return null;
        }

        // The result could never be null
        String workElementId = null;
        if (result != null)
        {
            ArrayList columns = (ArrayList) result.get(0);
            workElementId = (String) columns.get(0);
        }

        // check if id exist in tmt_wbs and load the name
        stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(WbsManager.NAME_ELEMENT_NAME);
        stmt.appendFrom(WbsManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(WbsManager.NAME_ELEMENT_ID);

        try
        {
            result = ManagerUtil.select(new Class[]
            {
                String.class
            }, null, null, stmt, new Object[]
            {
                workElementId
            }, messageContext);
        }
        catch (RPMException e2)
        {
            addException(e2, messageContext);
            return null;
        }

        if (result == null || result.size() == 0)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(WorkElement.class),
                    workElementId

            };
            RPMException e = new RPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.OBJECT_DOES_NOT_EXIST,
                errorParams);
            addException(e, messageContext);
        }
        else
        {
            ArrayList columns = (ArrayList) result.get(0);
            String name = (String) columns.get(0);

            workElement = new WorkElement();
            workElement.setID(workElementId);
            workElement.setName(name);
        }

        return workElement;
    }

    /**
     * Returns -1 if an error occured. Otherwise it returns the number of steps
     * for the given TaskAsignmnet
     * 
     * @param assignment
     * @param msgCtx
     * @return
     */
    protected int getNumberOfSteps(SummaryTimesheet timesheet,
            MessageContext msgCtx)
    {
        // TODO count steps that are applicable for all weeks. If 0, count
        // stepTimesheets.

        int stepsCount = 0;

        String taskAssignmentId = timesheet.getTaskAssignment().getID();

        // be sure that REALLY there is no steps for the task
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect("COUNT(*)");
        stmt.appendFrom(StepManager.TABLE_NAME);
        stmt.appendWhere(StepManager.NAME_TSK_ASSIGNMENT_ID);
        stmt.appendEqualQuestionMark();
        stmt.appendEqual(StepManager.NAME_SHOW_STEP, "Y");

        List result = null;
        try
        {
            result = ManagerUtil.select(new Class[]
            {
                Integer.class
            }, null, null, stmt, new Object[]
            {
                taskAssignmentId
            }, msgCtx);
        }
        catch (RPMException e)
        {
            addException(e, msgCtx);
            return -1;
        }

        if (result != null && result.size() > 0)
        {
            stepsCount = ((Integer) ((ArrayList) result.get(0)).get(0))
                .intValue();
        }

        if (stepsCount == 0)
        {
            SqlBuffer condition = new SqlBuffer();
            condition
                .appendEqualQuestionMark(StepManager.NAME_TSK_ASSIGNMENT_ID);
            condition.appendEqual(StepManager.NAME_SHOW_STEP, "N");

            SqlBuffer stmt1 = new SqlBuffer();
            stmt1.appendSelect("COUNT(*)");
            stmt1.appendFrom(StepTimesheetManager.TABLE_NAME);
            stmt1.appendWhere(StepTimesheetManager.NAME_WEEK_OF);
            stmt1.appendEqualQuestionMark();
            stmt1.appendSubSelect(StepTimesheetManager.NAME_ACTIVITY_ID,
                StepManager.NAME_ELEMENT_ID, StepManager.TABLE_NAME, condition
                    .toString());

            result = null;
            try
            {
                result = ManagerUtil.select(new Class[]
                {
                    Integer.class
                }, null, null, stmt1, new Object[]
                {
                        DateUtil.convertToString(timesheet.getWeekOf()),
                        taskAssignmentId
                }, msgCtx);
            }
            catch (RPMException e)
            {
                addException(e, msgCtx);
                return -1;
            }

            if (result != null && result.size() == 1)
            {
                stepsCount = ((Integer) ((ArrayList) result.get(0)).get(0))
                    .intValue();
            }
        }

        return stepsCount;
    }

    /**
     * ResourceTaskAssignmnet must have a resource (not a profile) and must have
     * a parent of type WorkElement (not ScopeElement).
     * 
     * @param summaryTimesheet
     * @param msgCtx
     * @return
     */
    private boolean validateResourceOfTaskAssignment(
            SummaryTimesheet summaryTimesheet, MessageContext msgCtx)
    {
        boolean valid = false;

        // ensure that TaskAssignmnet has a resource (not a profile)
        // select RESOURCE_ID from RESOURCES where RESOURCES.RESOURCE_ID =
        // (select RESOURCE_ID from SUMMARY_TIMESHEETS where WEEK_OF =
        // '2006-03-04' and TSK_ASSIGNMENT_ID =
        // 'ACF8D4E3A8AD4B0E926E2FD3EE0EF722')
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect(ResourceManager.NAME_RESOURCE_ID);
        stmt.appendFrom(ResourceManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.append(ResourceManager.NAME_RESOURCE_ID);
        stmt.appendEqual();
        stmt.appendOpenParenthesis();
        stmt.appendSelect();
        stmt.append(TaskAssignmentManager.NAME_RESOURCE_ID);
        stmt.appendFrom(TaskAssignmentManager.TABLE_NAME);
        stmt.appendWhere();
        stmt
            .appendEqualQuestionMark(TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);
        stmt.appendCloseParenthesis();

        try
        {
            List result = ManagerUtil.select(null, summaryTimesheet, null,
                stmt, new Object[]
                {
                    summaryTimesheet.getTaskAssignment().getID()
                }, msgCtx);

            if (result == null || result.size() == 0)
            {
                RPMException e = new RPMException(
                    ErrorCodes.TASK_ASSIGNMENT_WITHOUT_RESOURCE, null,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.TASK_ASSIGNMENT_FIELD, summaryTimesheet
                        .getID());
                addException(e, msgCtx);
                valid = false;
            }
            else
            {
                valid = true;
            }
        }
        catch (RPMException e)
        {
            addException(e, msgCtx);
            valid = false;
        }

        // the parent of ResourceTaskAssignmnet must be a WBS element.
        // select * from TMT_WBS where ELEMENT_ID = (select TASK_ID from
        // TASK_ASSIGNMENTS where TSK_ASSIGNMENT_ID =
        // 'ACF8D4E3A8AD4B0E926E2FD3EE0EF722')
        stmt = new SqlBuffer();
        stmt.appendSelect(WbsManager.NAME_ELEMENT_ID);
        stmt.appendFrom(WbsManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.append(WbsManager.NAME_ELEMENT_ID);
        stmt.appendEqual();
        stmt.appendOpenParenthesis();
        stmt.appendSelect(TaskAssignmentManager.NAME_TASK_ID);
        stmt.appendFrom(TaskAssignmentManager.TABLE_NAME);
        stmt.appendWhere();
        stmt
            .appendEqualQuestionMark(TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);
        stmt.appendCloseParenthesis();

        try
        {
            List result = ManagerUtil.select(null, summaryTimesheet, null,
                stmt, new Object[]
                {
                    summaryTimesheet.getTaskAssignment().getID()
                }, msgCtx);

            if (result == null || result.size() == 0)
            {
                RPMException e = new RPMException(
                    ErrorCodes.SCOPE_ELEMENT_NOT_UNDER_WBS, null,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.TASK_ASSIGNMENT_FIELD, summaryTimesheet
                        .getID());
                addException(e, msgCtx);
                valid = false;
            }
            else
            {
                valid &= true;
            }
        }
        catch (RPMException e1)
        {
            addException(e1, msgCtx);
            valid = false;
        }

        return valid;
    }

    /**
     * Rule 1: 
     */
    protected boolean validApprovalStatus(SummaryTimesheet summaryTimesheet,
            MessageContext msgCtx)
    {
        boolean valid = true;
        TimesheetApprovalStatus appStatus = summaryTimesheet
            .getApprovalStatus();

        if (appStatus == null)
        {
            // load the resource id and the project id
            String taskAssignmentId = summaryTimesheet.getTaskAssignment()
                .getID();
            String resourceID = null;
            String projectID = null;

            SqlBuffer condition = new SqlBuffer();
            condition
                .appendEqualQuestionMark(TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);
            SqlBuffer stmt = new SqlBuffer();
            stmt.appendSelect();
            stmt.append(TaskAssignmentManager.NAME_RESOURCE_ID);
            stmt.appendComma();
            stmt.append(TaskAssignmentManager.NAME_PROJECT_ID);
            stmt.appendFrom(TaskAssignmentManager.TABLE_NAME);
            stmt.appendWhere(condition);

            List result = null;
            try
            {
                result = ManagerUtil.select(new Class[]
                {
                        String.class, String.class
                }, null, null, stmt, new Object[]
                {
                    taskAssignmentId
                }, msgCtx);
            }
            catch (RPMException e1)
            {
                addException(e1, msgCtx);
                return false;
            }

            if (result != null && result.size() == 1)
            {
                ArrayList row = (ArrayList) result.get(0);
                resourceID = (String) row.get(0);
                projectID = (String) row.get(1);
            }

            // check the db for an approval status
            stmt = new SqlBuffer();
            stmt.appendSelect();
            stmt.append(TimesheetApprovalStatusManager.NAME_CURRENT_STATE);
            stmt.appendComma();
            stmt.append(TimesheetApprovalStatusManager.NAME_REC_DATETIME);
            stmt.appendFrom(TimesheetApprovalStatusManager.TABLE_NAME);
            stmt.appendWhere();
            stmt
                .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_ELEMENT_ID);
            stmt
                .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_RESOURCE_ID);
            stmt
                .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_WEEK_OF);

            try
            {
                result = ManagerUtil.select(new Class[]
                {
                        Integer.class, Calendar.class
                }, null, null, stmt, new Object[]
                {
                        projectID, resourceID,
                        DateUtil.convertToString(summaryTimesheet.getWeekOf())
                }, msgCtx);
            }
            catch (RPMException e)
            {
                addException(e, msgCtx);
                result = null;
            }

            if (result != null && result.size() > 0)
            {
                ArrayList row = (ArrayList) result.get(0);

                Integer statusIndex = (Integer) row.get(0);

                appStatus = new TimesheetApprovalStatus();
                TimesheetStatus status = TimesheetStatusUtil
                    .getTimesheetStatusByIndex(statusIndex);
                appStatus.setApprovalStatus(status);
                try
                {
                    appStatus.setApprovalDate((Calendar) row.get(1));
                }
                catch (Exception e2)
                {
                    RPMException rpme = null;
                    if (e2.getClass() == RPMException.class)
                    {
                        rpme = (RPMException) e2;
                    }
                    else
                    {
                        rpme = new RPMException(e2);
                    }
                    addException(rpme, msgCtx);
                    return false;
                }
            }
        }

        if (appStatus != null)
        {
            // if the the timesheet has been really submitted for
            // approval, the approvalDate could not be null
            if (isSummaryTimesheetModified(summaryTimesheet)
                    && appStatus.getApprovalDate() != null)
            {
                TimesheetStatus status = appStatus.getApprovalStatus();

                // Design note: user should be allowed to submit a timesheet
                // while saving actuals at the same time.
                if (!appStatus.testApprovalStatusModified()
                        && (status.equals(TimesheetStatus.Submitted)
                                || status
                                        .equals(TimesheetStatus.ApprovedByProjectManager) || status
                                .equals(TimesheetStatus.ApprovedByResourceManager)))
                {
                    RPMException e = new RPMException(
                            ErrorCodes.CANNOT_MODIFY_TIMESHEET, StringUtil
                                    .getShortClassName(summaryTimesheet
                                            .getClass()),
                            ValidationConstants.APPROVAL_STATUS_FIELD,
                            summaryTimesheet.getID());
                    addException(e, msgCtx);
                    valid = false;
                }
            }

        }

        return valid;
    }

    /**
     * Determine if an editable field was modified
     * 
     * @param summaryTimesheet
     * @return
     */
    private boolean isSummaryTimesheetModified(SummaryTimesheet summaryTimesheet)
    {

        boolean isModified = false;

        // hours
        if (TimesheetManagerUtil.hasNewActualReported(summaryTimesheet)
                || summaryTimesheet.testForecastFinishDateModified()
                || summaryTimesheet.testForecastStartDateModified()
                || summaryTimesheet.testTimeCode1Modified()
                || summaryTimesheet.testTimeCode2Modified())
        {
            isModified = true;
        }

        return isModified;
    }
}