/**
 * Created on Dec 14, 2005
 * @author shahla
 */
package com.ibm.rpm.timesheet.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.ValidateSerializer;
import com.ibm.rpm.timesheet.constants.ErrorCodes;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.managers.SummaryTimesheetManager;
import com.ibm.rpm.timesheet.managers.TimesheetApprovalStatusManager;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;
import com.ibm.rpm.timesheet.util.TimesheetStatusUtil;

public class TimesheetApprovalStatusCheckpoint extends AbstractCheckpoint
{
    private static final List TIMESHEET_APPROVAL_STATUS_PARENT_LIST = new ArrayList();

    protected static TimesheetApprovalStatusCheckpoint instance = new TimesheetApprovalStatusCheckpoint();

    static
    {
        TIMESHEET_APPROVAL_STATUS_PARENT_LIST.add(SummaryTimesheet.class);
    }
    
    public static TimesheetApprovalStatusCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {
        
        if (!super.isValidParent(rpmObject, messageContext, true, true,
            TIMESHEET_APPROVAL_STATUS_PARENT_LIST)){
            return;
        }       
               
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet)rpmObject.getParent();
        
        if (!GenericValidator.validateInstance(rpmObject,
            TimesheetApprovalStatus.class, messageContext))
        {
            return;
        }

        // Initialize
        TimesheetApprovalStatus approvalStatus = (TimesheetApprovalStatus) rpmObject;
        TimesheetApprovalStatusScope scope = (TimesheetApprovalStatusScope) rpmScope;

        //TODO replace loading by a direct sql
        TimesheetApprovalStatus loadedApproval = loadTimesheetApprovalStatus(
            approvalStatus, rpmScope, messageContext);
        boolean valid = true;

        valid &= GenericValidator.validateMandatory(approvalStatus,
            ValidationConstants.APPROVAL_STATUS_FIELD, approvalStatus.getApprovalStatus(),
            messageContext); 
        if (isApprovalStatusModified(approvalStatus, loadedApproval, messageContext))
        {
            valid &= validateStatusWithDB(approvalStatus, messageContext);
        }
        
        ValidateSerializer serializer = messageContext.getObjectsToValidate();
        if (!serializer.before(summaryTimesheet, approvalStatus))
        {
            
            valid &= GenericValidator.validateMandatory(approvalStatus,
                ValidationConstants.RESOURCE_FIELD, approvalStatus.getResource(),
                messageContext);

            valid &= GenericValidator.validateMandatory(approvalStatus,
                ValidationConstants.WEEK_OF_FIELD, approvalStatus.getWeekOf(),
                messageContext);
           
            if (approvalStatus.getWeekOf() != null)
            {
                TimesheetManagerUtil.setValidWeekOf(approvalStatus,
                    messageContext);
            }

            if (valid && (approvalStatus.getID() == null))
            {
                validateSummaryTimesheet(summaryTimesheet, messageContext);
            }
        }
        

        if (valid)
        {

            if (approvalStatus.testProjectModified())
            {
                GenericValidator.validateReadOnly(approvalStatus,
                    ValidationConstants.PROJECT_FIELD, messageContext);
                approvalStatus.setProject(null);
            }
            
            if (approvalStatus.testApprovalDateModified())
            {
                GenericValidator.validateReadOnly(approvalStatus,
                    ValidationConstants.APPROVAL_DATE_FIELD, messageContext);

                // reset read-only field value
                approvalStatus
                    .setApprovalDate(loadedApproval.getApprovalDate());
            }

            if (approvalStatus.testApprovalNotesModified())
            {
                
                if(approvalStatus.getApprovalStatus() == TimesheetStatus.Submitted){
                    RPMException e = new RPMException(
                        ErrorCodes.APPROVAL_NOTES_READ_ONLY,
                        null, approvalStatus.getClass().getName(),
                        ValidationConstants.APPROVAL_NOTES_FIELD, approvalStatus
                            .getID());
                    addException(e, messageContext);
                    //reset read-only field value
                    approvalStatus.setApprovalNotes(loadedApproval.getApprovalNotes());
                }
                
                
                GenericValidator.validateMaxLength(approvalStatus,
                    ValidationConstants.APPROVAL_NOTES_FIELD, approvalStatus
                        .getApprovalNotes(),
                    ValidationConstants.APPROVAL_NOTES_MAX, messageContext);
            }

            if (scope != null && scope.getLastApprovedRejectedBy() != null
                    && approvalStatus.testLastApprovedRejectedByModified())
            {
                GenericValidator.validateReadOnly(approvalStatus,
                    ValidationConstants.LAST_APPROVED_BY_FIELD, messageContext);

                // reset read-only field value
                approvalStatus.setLastApprovedRejectedBy(loadedApproval
                    .getLastApprovedRejectedBy());
            }
        }
    }
    
    private boolean isApprovalStatusModified(
            TimesheetApprovalStatus currentApprovalStatus,
            TimesheetApprovalStatus loadedApprovalStatus, MessageContext msgCtx)
    {
        if (currentApprovalStatus == null
                || currentApprovalStatus.getApprovalStatus() == null)
        {
            return false;
        }

        if (loadedApprovalStatus == null
                || loadedApprovalStatus.getApprovalStatus() == null)
        {
            return true;
        }

        // check the ApprovalStatus
        if (currentApprovalStatus.getApprovalStatus().equals(
            loadedApprovalStatus.getApprovalStatus()))
        {
            return false;
        }

        return true;
    }
    
    // The following table shows the rules of approving timesheet.
    // Timesheet status:
    // S    - Submitted
    // Ap   - Approved
    // RM   - ApprovedByResourceManager
    // PM   - ApprovedByProjectManager
    // Rj   - Rejected
    // RjRM - RejectedByResourceManager
    // RjPM - RejectedByProjectManager
    // NS   - Not Submitted
    //
    // TMT_SETTINGS table setting values for timesheet approval
    // 0    - By project manager and resource manager
    // 1    - By project manager
    // 2    - By resource manager
    //
    // Denotations in the table
    // x    - Exception: CANNOT_APPROVE_REJECT_TIMESHEET (this is the most common one)
    // e1   - Exception: CANNOT_MODIFY_TIMESHEET
    // e2   - Exception: PROJECT_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET
    // e3   - Exception: RESOURCE_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET
    // e4   - Exception: PROJECT_MANAGER_NEEDS_TO_APPROVE_FIRST
    // e5   - Exception: SUMMARY_TIMESHEET_IS_NOT_SUBMITTED_YET
    // o    - Allowed
    //
    //                          loadedStatus from DB
    // ------------------------------------------------------------------------------------
    // Current  S        Ap        RM        PM        Rj        RjRM        RjPM        NS
    // ------------------------------------------------------------------------------------
    // S        e1       e1        e1        e1        o         o           o           o
    // Ap       o        x         x         x         x         x           x           e5
    // RM       (0)e4    x         x         (0)o      x         x           x           e5
    //          (1)e3                        (1)e3
    //          (2)o                         (2)e3
    // PM       (0)o     x         x         x         x         x           x           e5
    //          (1)o
    //          (2)e2
    // Rj       o        o         o         o         x         x           x           e5
    // RjRM     (0)o     (0)o      (0)o      (0)o      x         x           x           e5
    //          (1)e3    (1)e3     (1)e3     (1)e3
    //          (2)o     (2)o      (2)o      (2)x
    // RjPM     (0)o     (0)o      x         (0)o      x         x           x           e5
    //          (1)o     (1)o                (1)o
    //          (2)e2    (2)e3               (2)e2
    private boolean validateStatusWithDB(TimesheetApprovalStatus appStatus, MessageContext msgCtx)
    {
        boolean valid = true;
        
        TimesheetStatus loadedStatus = null;
        SqlBuffer stmt;
        List result = null;

        // Always load the approval status from the db
        // check the db for an approval status
        stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(TimesheetApprovalStatusManager.NAME_CURRENT_STATE);
        stmt.appendFrom(TimesheetApprovalStatusManager.TABLE_NAME);
        stmt.appendWhere();
        stmt
            .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_ELEMENT_ID);
        stmt
            .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_RESOURCE_ID);
        stmt
            .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_WEEK_OF);
        
        String elementId = appStatus.getID();
        String resourceId = null;
        if (appStatus.getResource() != null)
        {
            resourceId = appStatus.getResource().getID();
        }
        
        try
        {
            result = ManagerUtil.select(new Class[]
            {
                    Integer.class
            }, null, null, stmt, new Object[]
            {
                    elementId, resourceId,
                    DateUtil.convertToString(appStatus.getWeekOf())
            }, msgCtx);
        }
        catch (RPMException e)
        {
            result = null;
        }

        if (result != null && result.size() > 0)
        {
            ArrayList row = (ArrayList) result.get(0);
            loadedStatus = TimesheetStatusUtil
                .getTimesheetStatusByIndex((Integer) row.get(0));
        }

        if (appStatus != null)
        {
            // load the approval type from TMT_SETTINGS
            SqlBuffer sqlBuffer = new SqlBuffer();
            sqlBuffer
                .appendSelect(ValidationConstants.TMT_SETTINGS_SETTING_VALUE_FIELD);
            sqlBuffer.appendFrom(ValidationConstants.TMT_SETTINGS_TABLE_NAME);
            sqlBuffer.appendWhere();
            sqlBuffer
                .appendEqualQuestionMark(ValidationConstants.TMT_SETTINGS_ELEMENT_ID_FIELD);

            try
            {
                result = ManagerUtil.select(new Class[]
                {
                    Integer.class
                }, null, null, sqlBuffer, new Object[]
                {
                    ValidationConstants.TMT_SETTINGS_APPROVAL_BY_VALUE
                }, msgCtx);
            }
            catch (RPMException e)
            {
                addException(e, msgCtx);
                result = null;
            }

            int approvalType = -1;

            if (result != null && result.size() > 0)
            {
                ArrayList row = (ArrayList) result.get(0);
                approvalType = ((Integer) row.get(0)).intValue();
            }

            // if the the timesheet has been really submitted for
            // approval, the approvalDate could not be null
            if (appStatus.getApprovalDate() != null)
            {
                TimesheetStatus currentStatus = appStatus.getApprovalStatus();

                if (loadedStatus == null)
                {
                    // not submitted
                    if (!isSubmitted(currentStatus))
                    {
                        createException(
                            ErrorCodes.SUMMARY_TIMESHEET_IS_NOT_SUBMITTED_YET,
                            msgCtx);
                        valid = false;
                    }
                }
                else
                {
                    if (isSubmitted(currentStatus))
                    {
                        if (isSubmitted(loadedStatus)
                                || isApproved(loadedStatus)
                                || isApprovedByPM(loadedStatus)
                                || isApprovedByRM(loadedStatus))
                        {
                            // the timesheet has been submitted or approved
                            // already
                            createException(ErrorCodes.CANNOT_MODIFY_TIMESHEET,
                                msgCtx);
                            valid = false;
                        }
                    }
                    else if (isApproved(currentStatus))
                    {
                        if (!isSubmitted(loadedStatus))
                        {
                            // can only approve sumbitted timesheet
                            createException(
                                ErrorCodes.CANNOT_APPROVE_REJECT_TIMESHEET,
                                msgCtx);
                            valid = false;
                        }
                    }
                    else if (isApprovedByPM(currentStatus))
                    {
                        if (!isSubmitted(loadedStatus))
                        {
                            // can not approve when the timesheet status is not
                            // submitted
                            createException(
                                ErrorCodes.CANNOT_APPROVE_REJECT_TIMESHEET,
                                msgCtx);
                            valid = false;
                        }
                        else
                        {
                            if (isRM(approvalType))
                            {
                                // project manager can not approve timesheet
                                // because
                                // it has to be approved by resource manager
                                createException(
                                    ErrorCodes.PROJECT_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                    }
                    else if (isApprovedByRM(currentStatus))
                    {
                        if (isSubmitted(loadedStatus))
                        {
                            if (isPMRM(approvalType))
                            {
                                // project manager has to approve the timesheet
                                // first
                                createException(
                                    ErrorCodes.PROJECT_MANAGER_NEEDS_TO_APPROVE_FIRST,
                                    msgCtx);
                                valid = false;
                            }
                            else if (isPM(approvalType))
                            {
                                // only project manager can approve the
                                // timesheet
                                createException(
                                    ErrorCodes.RESOURCE_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else if (isApprovedByPM(loadedStatus))
                        {
                            if (isPM(approvalType) || isRM(approvalType))
                            {
                                // resource manager cannot approve the timesheet
                                createException(
                                    ErrorCodes.RESOURCE_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else
                        {
                            // the timesheet has been approved or rejected
                            createException(
                                ErrorCodes.CANNOT_APPROVE_REJECT_TIMESHEET,
                                msgCtx);
                            valid = false;
                        }
                    }
                    else if (isRejected(currentStatus))
                    {
                        if (isRejected(loadedStatus)
                                || isRejectedByPM(loadedStatus)
                                || isRejectedByRM(loadedStatus))
                        {
                            // cannot reject again
                            createException(
                                ErrorCodes.CANNOT_APPROVE_REJECT_TIMESHEET,
                                msgCtx);
                            valid = false;
                        }
                    }
                    else if (isRejectedByPM(currentStatus))
                    {
                        if (isSubmitted(loadedStatus))
                        {
                            if (isRM(approvalType))
                            {
                                // only resource manager can reject timesheet
                                createException(
                                    ErrorCodes.PROJECT_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else if (isApprovedByPM(loadedStatus))
                        {
                            if (isRM(approvalType))
                            {
                                // only resource manager can reject timesheet
                                createException(
                                    ErrorCodes.PROJECT_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else if (isApproved(loadedStatus))
                        {
                            if (isRM(approvalType))
                            {
                                // only project can reject the timesheet
                                createException(
                                    ErrorCodes.RESOURCE_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else
                        {
                            // cannot reject timesheet
                            createException(
                                ErrorCodes.CANNOT_APPROVE_REJECT_TIMESHEET,
                                msgCtx);
                            valid = false;
                        }
                    }
                    else if (isRejectedByRM(currentStatus))
                    {
                        if (isSubmitted(loadedStatus))
                        {
                            if (isPM(approvalType))
                            {
                                // only project manager can reject the timesheet
                                createException(
                                    ErrorCodes.RESOURCE_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else if (isApprovedByRM(loadedStatus))
                        {
                            if (isPM(approvalType))
                            {
                                // only project manager can reject the timesheet
                                createException(
                                    ErrorCodes.RESOURCE_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else if (isApprovedByPM(loadedStatus))
                        {
                            if (isPM(approvalType))
                            {
                                // only project manager can reject
                                createException(
                                    ErrorCodes.RESOURCE_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                            else if (isRM(approvalType))
                            {
                                // timesheet has been rejected
                                createException(
                                    ErrorCodes.CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else if (isApproved(loadedStatus))
                        {
                            if (isPM(approvalType))
                            {
                                // only project can reject the timesheet
                                createException(
                                    ErrorCodes.RESOURCE_MANAGER_CANNOT_APPROVE_REJECT_TIMESHEET,
                                    msgCtx);
                                valid = false;
                            }
                        }
                        else
                        {
                            // timesheet has been rejected
                            createException(
                                ErrorCodes.CANNOT_APPROVE_REJECT_TIMESHEET,
                                msgCtx);
                            valid = false;
                        }
                    }
                }
            }
        }
        else
        {
            // when the approvalStatus is null, there is nothing to validate
        }

        return valid;
    }
    
    private boolean isPM(int approvalType)
    {
        return approvalType == ValidationConstants.TMT_SETTINGS_SETTING_VALUE_PM;
    }

    private boolean isRM(int approvalType)
    {
        return approvalType == ValidationConstants.TMT_SETTINGS_SETTING_VALUE_RM;
    }

    private boolean isPMRM(int approvalType)
    {
        return approvalType == ValidationConstants.TMT_SETTINGS_SETTING_VALUE_PMRM;
    }

    private void createException(int errorCode, MessageContext msgCtx)
    {
        RPMException e = new RPMException(errorCode,
            ValidationConstants.APPROVAL_STATUS_FIELD);
        addException(e, msgCtx);
    }

    private boolean isSubmitted(TimesheetStatus status)
    {
        return TimesheetStatus.Submitted.equals(status);
    }

    private boolean isApproved(TimesheetStatus status)
    {
        return TimesheetStatus.Approved.equals(status);
    }

    private boolean isApprovedByPM(TimesheetStatus status)
    {
        return TimesheetStatus.ApprovedByProjectManager.equals(status);
    }

    private boolean isApprovedByRM(TimesheetStatus status)
    {
        return TimesheetStatus.ApprovedByResourceManager.equals(status);
    }

    private boolean isRejected(TimesheetStatus status)
    {
        return TimesheetStatus.Rejected.equals(status);
    }

    private boolean isRejectedByPM(TimesheetStatus status)
    {
        return TimesheetStatus.RejectedByProjectManager.equals(status);
    }

    private boolean isRejectedByRM(TimesheetStatus status)
    {
        return TimesheetStatus.RejectedByResourceManager.equals(status);
    }
    
    private void validateSummaryTimesheet(
            SummaryTimesheet summaryTimesheet, MessageContext messageContext)
    {
        
        //Validate primary key
        try
        {
            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(SummaryTimesheet.class);

            manager.isValidPrimaryKey(summaryTimesheet, messageContext);
        }
        catch (RPMException e)
        {
            addException(e, messageContext);
        }

        //Validate object exists
        if (messageContext.isSuccessful())
        {

            SqlBuffer stmt = new SqlBuffer();
            stmt.appendSelect(SummaryTimesheetManager.NAME_TSK_ASSIGNMENT_ID);
            stmt.appendFrom(SummaryTimesheetManager.TABLE_NAME);
            stmt.appendWhere();
            stmt
                .appendEqualQuestionMark(SummaryTimesheetManager.NAME_TSK_ASSIGNMENT_ID);
            stmt.appendEqualQuestionMark(SummaryTimesheetManager.NAME_WEEK_OF);

            List result = null;
            try
            {
                result = ManagerUtil.select(null, summaryTimesheet, null, stmt,
                    new Object[]
                    {
                            summaryTimesheet.getID(),
                            DateUtil.convertToString(summaryTimesheet
                                .getWeekOf()),
                    }, messageContext);
            }
            catch (RPMException e1)
            {
                addException(e1, messageContext);
            }

            if (result == null || result.size() == 0)
            {
                String[] errorParams =
                {
                        summaryTimesheet.getID(),
                        DateUtil.convertToString(summaryTimesheet.getWeekOf())
                };
                RPMException e = new RPMException(
                    ErrorCodes.NO_SUMMARY_TIMESHEET_FOR_ID_AND_WEEKOF,
                    errorParams, summaryTimesheet.getClass().getName(),
                    ValidationConstants.PARENT_FIELD);
                addException(e, messageContext);
            }
        }
    }
    private TimesheetApprovalStatus loadTimesheetApprovalStatus(
            TimesheetApprovalStatus timesheetApproval, RPMObjectScope scope,
            MessageContext messageContext)
    {
        TimesheetApprovalStatus loadedApproval = null;

        if (timesheetApproval.getID() != null)
        {
            try
            {
                //remove from cache
                messageContext.getCache().removeContainerFromCache(timesheetApproval);
                
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(TimesheetApprovalStatus.class);
                loadedApproval = (TimesheetApprovalStatus) manager
                    .loadByPrimaryKey(timesheetApproval, scope, messageContext, false);
            }
            catch (Exception e)
            {
                RPMException rpmException = new RPMException(e);
                rpmException.assignContainer(timesheetApproval);
                addException(rpmException, messageContext);
            }
        }
        if (loadedApproval == null)
        {
            loadedApproval = new TimesheetApprovalStatus();
        }

        return loadedApproval;
    }
}