/*
 * Created on Nov 15, 2005 
 * @author mmatwiy 
 * Checkpoint of common fields of Project, Proposal, and WorkOrganization.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.List;

import com.ibm.rpm.applicationadministration.checkpoints.AttributeAssignmentCheckpoint;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.util.AttributeValidator;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.financial.util.FinancialCategoryUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.wbs.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

public class GenericProjectCheckpoint extends AbstractAggregateNodeCheckpoint
{

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx, boolean parentRequired, List validParentTypes)
    {

        /*
         * If the GenericProject is save as a composite of a Portfolio, we need
         * to reset the parent field because Portfolio is not a valid parent
         * type.
         */
        RPMObject parent = rpmObject.getParent();
        if (parent != null && parent instanceof Portfolio)
        {
            if (msgCtx.getObjectsToValidate().before(parent, rpmObject))
            {
                rpmObject.setParent(null);
            }
        }
                
        
        super.validateSave(rpmObject, rpmScope, msgCtx, parentRequired,
            validParentTypes);

        GenericProject genericProject = (GenericProject) rpmObject;
        WorkElementScope scope = (WorkElementScope) rpmScope;

        if (genericProject.testNameModified())
        {
            // Check that the name doesn't exceed the max length
            GenericValidator.validateMaxLength(genericProject,
                ValidationConstants.NAME_FIELD, genericProject.getName(),
                ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT, msgCtx);
        }
        
        if (genericProject.testEarnedValuesCalculationStateModified())
        {
            GenericValidator.validateReadOnly(genericProject,
                ValidationConstants.EV_CALCULATION_ACTIVATION_STATE, msgCtx);
        }
        
        if (genericProject.testProbabilityPercentModified())
        {
            // Check that probabilityPercent is in the correct range
            // of values
            GenericValidator.validateRange(genericProject,
                ValidationConstants.PROBABILITY_PERCENT_FIELD, genericProject
                    .getProbabilityPercent(),
                ValidationConstants.PROBABILITY_PERCENT_MIN,
                ValidationConstants.PROBABILITY_PERCENT_MAX, msgCtx);
            
            // cannot be cleared on update
            GenericValidator.validateDefaultValueUpdate(genericProject,
                ValidationConstants.PROBABILITY_PERCENT_FIELD, genericProject
                    .getProbabilityPercent(), msgCtx);
        }

        if (genericProject.testReferenceNumberModified())
        {
            // Check that the reference number doesn't exceed the max length
            GenericValidator.validateMaxLength(genericProject,
                ValidationConstants.REFERENCE_NUMBER_FIELD, genericProject
                    .getReferenceNumber(),
                ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT,
                msgCtx);
        }
        if(genericProject instanceof Proposal || genericProject instanceof Project)
        {
        	if(genericProject.getID() != null) // UPDATING THE GENERIC PROJECT
        	{
        		if (genericProject.testDefaultProjectModified()) {
        			if (genericProject.getDefaultProject().booleanValue()) {
        				WorkElementScope workElementScope = new WorkElementScope();
        				workElementScope.setChildren(new WorkElementScope());

        				try {
        					WbsManager manager = (WbsManager) RPMManagerFactory
        					.getInstance().getRPMObjectManager(
        							WorkElement.class);
        					GenericProject loadedProject = (GenericProject) manager
        					.load(genericProject, workElementScope, msgCtx);
        					if (loadedProject.getChildren() != null) {
        						WorkElement[] workElements = loadedProject
        						.getChildren();
        						boolean found = false;
        						if(workElements != null)
        						{
        							for (int i = 0; i < workElements.length && !found; i++) {
        								if (workElements[i] instanceof Proposal
        										|| workElements[i] instanceof Project) {
        									found = true;
        								}
        							}
        							if(found)
        							{
        								String[] errorParams = {
        										StringUtil
        										.getShortClassName(genericProject
        												.getClass()),
        												ValidationConstants.DEFAULT_PROJECT_FIELD };
        								RPMException e = new RPMException(
        										ErrorCodes.INVALID_DEFAULT_PROJECT_CONTAINS_SUBPROJECT,
        										errorParams,
        										genericProject.getClass().getName(),
        										ValidationConstants.DEFAULT_PROJECT_FIELD,
        										genericProject.getID());
        								addException(e, msgCtx);
        							}
        						}
        					}
        				} catch (RPMException e) {
        					addException(new RPMException(e), msgCtx);
        				} catch (SQLException e) {
        					addException(new RPMException(e), msgCtx);
        				} catch (ParseException e) {
        					addException(new RPMException(e), msgCtx);
        				}
        			}
        		}
        	} else  // CREATING THE GENERIC PROJECT
        	{
        		if(parent != null && parent.getID() != null && (parent instanceof Proposal || parent instanceof Project)) {
        			try {
        				WbsManager manager = (WbsManager) RPMManagerFactory
        				.getInstance().getRPMObjectManager(
        						WorkElement.class);
        				GenericProject loadedParent = (GenericProject) manager
        				.load(parent, null, msgCtx);
        				if (loadedParent.getDefaultProject().booleanValue()) {
        					String[] errorParams = {
        							StringUtil.getShortClassName(genericProject
        									.getClass()),
        									ValidationConstants.DEFAULT_PROJECT_FIELD };
        					RPMException e = new RPMException(
        							ErrorCodes.INVALID_DEFAULT_PROJECT_CONTAINS_SUBPROJECT,
        							errorParams, genericProject.getClass()
        							.getName(),
        							ValidationConstants.DEFAULT_PROJECT_FIELD,
        							genericProject.getID());
        					addException(e, msgCtx);
        				}
        			} catch (RPMException e) {
        				addException(new RPMException(e), msgCtx);
        			} catch (SQLException e) {
        				addException(new RPMException(e), msgCtx);
        			} catch (ParseException e) {
        				addException(new RPMException(e), msgCtx);
        			}
        		}
        	}
        }      	

        if (scope != null)
        {
            if (scope.isRelatedAttributeAssignments())
            {
                if (genericProject.getActionAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getActionAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator.validateAttributeAssignmentType(
                            genericProject, assignments[i],
                            ScopeManagementModule.class,
                            ValidationConstants.ACTION_ATTR_ASGN_FIELD, msgCtx);
                    }
                }

                if (genericProject.getChangeRequestAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getChangeRequestAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator.validateAttributeAssignmentType(
                            genericProject, assignments[i],
                            ScopeManagementModule.class,
                            ValidationConstants.CHANGE_REQUEST_ATTR_ASGN_FIELD,
                            msgCtx);
                    }
                }

                if (genericProject.getDefectAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getDefectAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator
                            .validateAttributeAssignmentType(genericProject,
                                assignments[i], ScopeManagementModule.class,
                                ValidationConstants.DEFECT_ATTR_ASSGN_FIELD,
                                msgCtx);
                    }
                }

                if (genericProject.getDeliverableAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getDeliverableAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator.validateAttributeAssignmentType(
                            genericProject, assignments[i], WbsModule.class,
                            ValidationConstants.DELIVERABLE_ATTR_ASGN_FIELD,
                            msgCtx);
                    }
                }

                if (genericProject.getDocumentAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getDocumentAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator.validateAttributeAssignmentType(
                            genericProject, assignments[i],
                            DocumentModule.class,
                            ValidationConstants.DOCUMENT_ATTR_ASGN_FIELD,
                            msgCtx);
                    }
                }

                if (genericProject.getIssueAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getIssueAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator.validateAttributeAssignmentType(
                            genericProject, assignments[i],
                            ScopeManagementModule.class,
                            ValidationConstants.ISSUE_ATTR_ASGN_FIELD, msgCtx);
                    }
                }

                if (genericProject.getRequirementAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getRequirementAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator.validateAttributeAssignmentType(
                            genericProject, assignments[i],
                            ScopeManagementModule.class,
                            ValidationConstants.REQUIREMENT_ATTR_ASGN_FIELD,
                            msgCtx);
                    }
                }

                if (genericProject.getRiskAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getRiskAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator.validateAttributeAssignmentType(
                            genericProject, assignments[i],
                            ScopeManagementModule.class,
                            ValidationConstants.RISK_ATTR_ASGN_FIELD, msgCtx);
                    }
                }

                if (genericProject.getServiceRequestAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getServiceRequestAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator
                            .validateAttributeAssignmentType(
                                genericProject,
                                assignments[i],
                                ScopeManagementModule.class,
                                ValidationConstants.SERVICE_REQUEST_ATTR_ASGN_FIELD,
                                msgCtx);
                    }
                }

                if (genericProject.getTaskAttributeAssignments() != null)
                {
                    AttributeAssignment[] assignments = genericProject
                        .getTaskAttributeAssignments();

                    // check that the assigned attribute belongs to scope management
                    for (int i = 0; i < assignments.length; i++)
                    {
                        AttributeValidator.validateAttributeAssignmentType(
                            genericProject, assignments[i], WbsModule.class,
                            ValidationConstants.TASK_ATTR_ASGN_FIELD, msgCtx);
                    }
                }

                if (scope.isAttributeAssignments())
                {
                    AttributeAssignmentCheckpoint checkPoint = AttributeAssignmentCheckpoint
                        .getInstance();

                    //Attribute assignments
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getAttributeAssignments(), msgCtx);

                    //Other children attribute assignments    
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getActionAttributeAssignments(), msgCtx);
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getChangeRequestAttributeAssignments(), msgCtx);
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getDefectAttributeAssignments(), msgCtx);
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getDeliverableAttributeAssignments(), msgCtx);
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getDocumentAttributeAssignments(), msgCtx);
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getIssueAttributeAssignments(), msgCtx);
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getRequirementAttributeAssignments(), msgCtx);
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getRiskAttributeAssignments(), msgCtx);
                    checkPoint.isValidSingleSelectRule(genericProject
                        .getServiceRequestAttributeAssignments(), msgCtx);
                }

            }
        }

        if (genericProject.testPublishedModified()
                && genericProject.getPublished() == ProjectPublishedType.ToTemplate)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(genericProject.getClass()) + " "
                        + ValidationConstants.PUBLISHED_FIELD,
            };
            RPMException e = new RPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.PUBLISHED_TO_TEMPLATE_NOT_SUPPORTED,
                errorParams, genericProject.getClass().getName(),
                ValidationConstants.PUBLISHED_FIELD);
            addException(e, msgCtx);

            //TODO mmatwiy: this can be put back in once ToTemplate is supported
            //            if (genericProject.getParent() == null
            //                    || genericProject.getParent().getID() == null)
            //            {
            //                String[] errorParams =
            //                {
            //                    StringUtil.getShortClassName(genericProject.getClass())
            //                            + " " + ValidationConstants.PUBLISHED_FIELD,
            //                };
            //                RPMException e = new RPMException(
            //                    ErrorCodes.PUBLISHED_TO_TEMPLATE_PARENT_ID_MANDATORY,
            //                    errorParams, genericProject.getClass().getName(),
            //                    ValidationConstants.PUBLISHED_FIELD);
            //                addException(e, msgCtx);
            //                return false;
            //            }
        }

        // On update, the calendar and currency fields cannot be modified
        if (genericProject.testCalendarModified())
        {
            GenericValidator.validateFieldUpdateInvalidOnContainerUpdate(
                genericProject, ValidationConstants.PROJECT_CALENDAR_FIELD,
                genericProject.getCalendar(), msgCtx);
        }
        if (genericProject.testCurrencyModified())
        {
            GenericValidator.validateFieldUpdateInvalidOnContainerUpdate(
                genericProject, ValidationConstants.PROJECT_CURRENCY_FIELD,
                genericProject.getCurrency(), msgCtx);
        }


        // On update, the assignmentType, defaultProject,
        // and published fields cannot be cleared
        if (genericProject.testAssignmentTypeModified())
        {
            GenericValidator.validateDefaultValueUpdate(genericProject,
                ValidationConstants.PROJECT_ASSIGNMENT_TYPE_FIELD,
                genericProject.getAssignmentType(), msgCtx);
        }

        if (genericProject.testDefaultProjectModified())
        {
            GenericValidator.validateDefaultValueUpdate(genericProject,
                ValidationConstants.DEFAULT_PROJECT_FIELD, genericProject
                    .getDefaultProject(), msgCtx);
        }

        if (genericProject.testPublishedModified())
        {
            GenericValidator.validateDefaultValueUpdate(genericProject,
                ValidationConstants.PUBLISHED_FIELD, genericProject
                    .getPublished(), msgCtx);
        }
        
        //should not be able to save an invalid ChargeCode
        //the normal  number of chargecode is 4 but there is a possibility in future updates to
        //to have a greater number where some might be optional.
        //systemchargecodes should have AVG,HOUR,EL,FTE
        if (genericProject.getWorksheetFinancials() != null)
        {
            boolean hasSystemChargeCode = false;
            boolean isValidSystemChargeCodes = false;

            try
            {
                hasSystemChargeCode = FinancialCategoryUtil
                    .containsExpenseCategory(genericProject
                        .getWorksheetFinancials());
            }
            catch (RPMException e)
            {
                addException(e, msgCtx);
            }

            if (hasSystemChargeCode)
            {
                try
                {
                    isValidSystemChargeCodes = FinancialCategoryUtil
                        .isValidSystemExpenseChargeCodeCategory(genericProject
                            .getWorksheetFinancials());
                }
                catch (RPMException e1)
                {
                    addException(e1, msgCtx);
                }

                if (!isValidSystemChargeCodes)
                {
                    String[] errorParams =
                    {
                        StringUtil.getShortClassName(genericProject.getClass())
                                + " "
                                + ValidationConstants.WORKSHEET_FINANCIALS_FIELD,
                    };
                    RPMException e = new RPMException(
                        com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_SYSTEM_WORKSHEET_FINANCIALS,
                        errorParams, genericProject.getClass().getName(),
                        ValidationConstants.WORKSHEET_FINANCIALS_FIELD);
                    addException(e, msgCtx);

                }
            }
        }
        
    }

    public void validateProjectTemplateCreation(GenericProject inputProject,
            MessageContext msgCtx)
    {
        //validate template is in fact a template
        if (!(msgCtx.getTemplate() instanceof GenericProject)
				|| ((GenericProject) msgCtx.getTemplate()).getPublished() == null
				|| !(ProjectPublishedType.ToTemplate.getValue()
						.equals(((GenericProject) msgCtx.getTemplate())
								.getPublished().getValue()))) {
			String[] errorParams = { StringUtil.getShortClassName(inputProject
					.getClass())
					+ " " + ValidationConstants.TEMPLATE_FIELD

			};
			//invalid project template
			RPMException e = new RPMException(
					com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_PROJECT_TEMPLATE_TYPE,
					errorParams, inputProject.getClass().getName(),
					ValidationConstants.TEMPLATE_FIELD, inputProject.getID());
			addException(e, msgCtx);
		}
        else
        {        	
            //Fields Validation
        	
        	super.validateWorkElementReadOnlyFields(inputProject, msgCtx);

            //    	Field validation: AssignementType
            if (inputProject.testAssignmentTypeModified())
            {
                GenericValidator.validateReadOnly(inputProject,
                    ValidationConstants.ASSIGNMENT_TYPE_FIELD, msgCtx);
            }

            //    	Field validation: Calendar
            if (inputProject.testCalendarModified())
            {
                GenericValidator.validateReadOnly(inputProject,
                    ValidationConstants.CALENDAR_FIELD, msgCtx);
            }

            //    	Field validation: Currency
            if (inputProject.testCurrencyModified())
            {
                GenericValidator.validateReadOnly(inputProject,
                    ValidationConstants.CURRENCY_FIELD, msgCtx);
            }

            //    	Field validation: DefaultProject
            if (inputProject.testDefaultProjectModified())
            {
                GenericValidator.validateReadOnly(inputProject,
                    ValidationConstants.DEFAULT_PROJECT_FIELD, msgCtx);
            }

            //    	Field validation: Opportunity
            if (inputProject.testOpportunityModified())
            {
                GenericValidator.validateReadOnly(inputProject,
                    ValidationConstants.OPPORTUNITY_FIELD, msgCtx);
            }

            //    	Field validation: PortFolio
            if (inputProject.testPortfolioModified())
            {
                GenericValidator.validateReadOnly(inputProject,
                    ValidationConstants.PORTFOLIOS_FIELD, msgCtx);
            }

            //    	Field validation: ProjectCostCenter
            if (inputProject.testProjectCostCenterModified())
            {
                GenericValidator.validateReadOnly(inputProject,
                    ValidationConstants.PROJECT_COSTCENTER_FIELD, msgCtx);
            }
            
            //      Field validation: ReferenceID
            if (inputProject.testReferenceIDModified())
            {
                GenericValidator.validateReadOnly(inputProject,
                    ValidationConstants.REFERENCE_ID_FIELD, msgCtx);
            }

        }

    }

}
