/*
 * Created on Nov 24, 2005 
 * @author mmatwiy 
 * Checkpoint validation for container Opportunity.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkOrganization;

public class OpportunityCheckpoint extends AbstractCheckpoint
{

    private static final List OPPORTUNITY_PARENT_LIST = new ArrayList();

    protected static OpportunityCheckpoint instance = new OpportunityCheckpoint();

    static
    {
        OPPORTUNITY_PARENT_LIST.add(Proposal.class);
        OPPORTUNITY_PARENT_LIST.add(Project.class);
        OPPORTUNITY_PARENT_LIST.add(WorkOrganization.class);
    }

    public static OpportunityCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, OPPORTUNITY_PARENT_LIST);

        Opportunity opportunity = (Opportunity) rpmObject;

        if (opportunity.testBenefitBeginsMonthsModified())
        {
            // Check that benefitBeginsMonths is in the correct range
            // of values
            GenericValidator.validateRange(opportunity,
                ValidationConstants.BENEFIT_BEGINS_MONTHS_FIELD, opportunity
                    .getBenefitBeginsMonths(),
                ValidationConstants.BENEFIT_BEGINS_MONTHS_MIN,
                ValidationConstants.BENEFIT_BEGINS_MONTHS_MAX, msgCtx);
        }

        if (opportunity.testBenefitDurationMonthsModified())
        {
            // Check that benefitDurationMonths is in the correct range
            // of values
            GenericValidator.validateRange(opportunity,
                ValidationConstants.BENEFIT_DURATION_MONTHS_FIELD, opportunity
                    .getBenefitDurationMonths(),
                ValidationConstants.BENEFIT_DURATION_MONTHS_MIN,
                ValidationConstants.BENEFIT_DURATION_MONTHS_MAX, msgCtx);
        }

        if (opportunity.testContactNumberModified())
        {
            // Check that contactNumber doesn't exceed its max length
            GenericValidator.validateMaxLength(opportunity,
                ValidationConstants.CONTACT_NUMBER_FIELD, opportunity
                    .getContactNumber(),
                ValidationConstants.CONTACT_NUMBER_CHARACTER_LIMIT, msgCtx);
        }

        if (opportunity.testContactTypeModified())
        {
            // Check that contactType doesn't exceed its max length
            GenericValidator.validateMaxLength(opportunity,
                ValidationConstants.CONTACT_TYPE_FIELD, opportunity
                    .getContactType(),
                ValidationConstants.CONTACT_TYPE_CHARACTER_LIMIT, msgCtx);
        }

        if (opportunity.testCustomerNumberModified())
        {
            // Check that customerNumber doesn't exceed its max length
            GenericValidator.validateMaxLength(opportunity,
                ValidationConstants.CUSTOMER_NUMBER_FIELD, opportunity
                    .getCustomerNumber(),
                ValidationConstants.CUSTOMER_NUMBER_CHARACTER_LIMIT, msgCtx);
        }

        if (opportunity.testExternalClientModified())
        {
            // Check that externalClient doesn't exceed its max length
            GenericValidator.validateMaxLength(opportunity,
                ValidationConstants.EXTERNAL_CLIENT_FIELD, opportunity
                    .getExternalClient(),
                ValidationConstants.EXTERNAL_CLIENT_CHARACTER_LIMIT, msgCtx);
        }

        if (opportunity.testOpportunityOwnerModified())
        {
            // Check that opportunitOwner doesn't exceed its max length
            GenericValidator.validateMaxLength(opportunity,
                ValidationConstants.OPPORTUNITY_OWNER_FIELD, opportunity
                    .getOpportunityOwner(),
                ValidationConstants.OPPORTUNITY_OWNER_CHARACTER_LIMIT, msgCtx);
        }

        if (opportunity.testOpportunityTrackingNumberModified())
        {
            // Check that opportunityTrackingNumber doesn't exceed its max
            // length
            GenericValidator
                .validateMaxLength(
                    opportunity,
                    ValidationConstants.OPPORTUNITY_TRACKING_NUMBER_FIELD,
                    opportunity.getOpportunityTrackingNumber(),
                    ValidationConstants.OPPORTUNITY_TRACKING_NUMBER_CHARACTER_LIMIT,
                    msgCtx);
        }

        if (opportunity.testOrderOfMagnitudeCostModified())
        {
            // Check that orderOfMagnitudeCost is in the correct range
            // of values
            GenericValidator.validateRange(opportunity,
                ValidationConstants.ORDER_OF_MAGNITUDE_COST_FIELD, opportunity
                    .getOrderOfMagnitudeCost(),
                ValidationConstants.ORDER_OF_MAGNITUDE_COST_MIN,
                ValidationConstants.ORDER_OF_MAGNITUDE_COST_MAX, msgCtx);
        }

        if (opportunity.testOrderOfMagnitudeRevenueBenefitsModified())
        {
            // Check that orderOfMagnitudeRevenueBenefits is in the correct
            // range of values
            GenericValidator.validateRange(opportunity,
                ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_FIELD,
                opportunity.getOrderOfMagnitudeRevenueBenefits(),
                ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_MIN,
                ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_MAX,
                msgCtx);
        }

        // orderOfMagnitueGPSavingsPercent is a read-only field
        if (opportunity.testOrderOfMagnitudeGPSavingsPercentModified())
        {
            GenericValidator.validateReadOnly(opportunity,
                ValidationConstants.ORDER_OF_MAGNITUDE_GP_SAVINGS_PERCENT,
                msgCtx);

            //TODO replace loading by a direct sql 
            //(the fields required to calculategetOrderOfMagnitudeGPSavingsPercent)
            Opportunity loadedOpportunity = loadOpportunity(opportunity,
                rpmScope, msgCtx);

            // reset read-only field value
            opportunity.setOrderOfMagnitudeGPSavingsPercent(loadedOpportunity
                .getOrderOfMagnitudeGPSavingsPercent());
        }

        // On update, the benefitBeginsMonths, benefitDurationMonths, 
        // orderOfMagnitudeCost, orderOfMagnitudeRevenueBenefits, and 
        // probabilityPercent fields cannot be cleared
        GenericValidator.validateDefaultValueUpdate(opportunity,
            ValidationConstants.BENEFIT_BEGINS_MONTHS_FIELD, opportunity
                .getBenefitBeginsMonths(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(opportunity,
            ValidationConstants.BENEFIT_DURATION_MONTHS_FIELD, opportunity
                .getBenefitDurationMonths(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(opportunity,
            ValidationConstants.ORDER_OF_MAGNITUDE_COST_FIELD, opportunity
                .getOrderOfMagnitudeCost(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(opportunity,
            ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_FIELD,
            opportunity.getOrderOfMagnitudeRevenueBenefits(), msgCtx);

    }
    
    private Opportunity loadOpportunity(Opportunity opportunity,
            RPMObjectScope rpmScope, MessageContext msgCtx)
    {
        Opportunity loadedOpportunity = null;

        if (opportunity.getID() != null)
        {
            try
            {
                //TODO fix this part as this returns an exception
                //manager needs to implement isValidPrimaryKey
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(Opportunity.class);
                loadedOpportunity = (Opportunity) manager.loadByPrimaryKey(
                    opportunity, rpmScope, msgCtx, false);
            }
            catch (Exception e)
            {
                RPMException rpmException = new RPMException(e);
                rpmException.assignContainer(opportunity);
                addException(rpmException, msgCtx);
            }
        }
        if (loadedOpportunity == null)
        {
            loadedOpportunity = new Opportunity();
        }

        return loadedOpportunity;
    }
}