/*
 * Created on Feb 9, 2005 
 * @author mmatwiy 
 * Checkpoint validation of the fields common to 
 * containers Task and Milestone.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.Calendar;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestState;
import com.ibm.rpm.applicationadministration.containers.DefectState;
import com.ibm.rpm.applicationadministration.containers.IssueState;
import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.applicationadministration.containers.RiskState;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestState;
import com.ibm.rpm.applicationadministration.containers.TaskState;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.util.ScopeManagementUtil;
import com.ibm.rpm.wbs.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ConstraintType;
import com.ibm.rpm.wbs.types.TaskType;

public class SimpleNodeCheckpoint extends WorkElementCheckpoint
{
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx, boolean parentRequired, List validParentTypes)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, parentRequired,
            validParentTypes);

        SimpleNode simpleNode = (SimpleNode) rpmObject;
        WorkElementScope scope = (WorkElementScope) rpmScope;
        
        verifyState(rpmObject, msgCtx, simpleNode, scope);
        
        verifyConstraintType(msgCtx, simpleNode);
        
        /**
         * In case we set ConstraintType.AsSoonAsPossible first, then set constraintDate
         */
        if(simpleNode.testConstraintDateModified() && 
                ConstraintType.AsSoonAsPossible.equals(simpleNode.getConstraintType()))
        {
            GenericValidator
            .validateMustBeNull(simpleNode,
                ValidationConstants.CONSTRAINT_DATE_FIELD, simpleNode
                    .getConstraintDate(),
                "the constraint type is set to As soon as possible",
                msgCtx);
        }

        if (simpleNode.testNameModified())
        {
            // Check that the name doesn't exceed the max length
            GenericValidator.validateMaxLength(simpleNode,
                ValidationConstants.NAME_FIELD, simpleNode.getName(),
                ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT,
                msgCtx);
        }

        if (simpleNode.testReferenceNumberModified())
        {
            // Check that the reference number doesn't exceed the max length
            GenericValidator
                .validateMaxLength(
                    simpleNode,
                    ValidationConstants.REFERENCE_NUMBER_FIELD,
                    simpleNode.getReferenceNumber(),
                    ValidationConstants.PROJECT_ACTIVITY_REFERENCE_NUMBER_CHARACTER_LIMIT,
                    msgCtx);
        }
        
        if (simpleNode.testEffortDrivenModified()
                || simpleNode.testTaskTypeModified())
        {
            if (simpleNode.getTaskType() == TaskType.DefaultTask
                    && simpleNode.getEffortDriven() != null
                    && simpleNode.getEffortDriven().booleanValue())
            {
                // warning: effortDriven cannot be set to true if taskType is DefaultTask
                String[] errorParams =
                {
                    StringUtil.getShortClassName(simpleNode.getClass())
                };
                RPMException e = new RPMException(
                    ErrorCodes.EFFORT_DRIVEN_NOT_VALID_WITH_DEFAULT_TASK,
                    errorParams, simpleNode.getClass().getName(),
                    ValidationConstants.EFFORT_DRIVEN_FIELD, simpleNode.getID());
                e.setSeverity(SeverityLevel.Warning);
                addException(e, msgCtx);
            }
        }
        
        checkScheduleDates(msgCtx, simpleNode, scope);
    }

    private void verifyConstraintType(MessageContext msgCtx, SimpleNode simpleNode)
    {
        if (simpleNode.testConstraintTypeModified())
        {
            // in creation mode
            ConstraintType currentType = simpleNode.getConstraintType();
            if (simpleNode.getID() == null)
            {
                if (currentType == null)
                {
                    // use the default value in creation mode
                    currentType = ConstraintType.AsSoonAsPossible;
                }
            }
            else
            {
                // field may not be null during updates
                GenericValidator.validateNotNull(simpleNode,
                    ValidationConstants.CONSTRAINT_TYPE_FIELD, currentType,
                    msgCtx);
            }

            /**
             * In case we set constraintDate first, then try to also set ConstraintType.AsSoonAsPossible
             */
            if (ConstraintType.AsSoonAsPossible.equals(currentType))
            {
                GenericValidator
                    .validateMustBeNull(simpleNode,
                        ValidationConstants.CONSTRAINT_DATE_FIELD, simpleNode
                            .getConstraintDate(),
                        "the constraint type is set to As soon as possible",
                        msgCtx);
            }
        }
    }

    private void verifyState(RPMObject inputObject, MessageContext msgCtx, SimpleNode simpleNode, WorkElementScope scope)
    {
        if (scope != null && scope.isState() && simpleNode.testStateModified())
        {
            /*
             * load the scopeElement for this task
             * if it's null
             *      use standard validation
             * if it's not null
             *      cast the scopeElement to its appropriate concrete type
             *      make sure that getState() instanceof type
             * 
             * Taken from SummaryTaskCheckpoint                 
             */
            ScopeElement correspondingScopeElement = null;
            try
            {
                correspondingScopeElement = ((WorkElement) ScopeManagementUtil.loadScopeElement(
                    inputObject, msgCtx)).getScopeElement();
            }
            catch (Exception e)
            {
                msgCtx.addExceptionNoThrow(e);
            };
            
            if (correspondingScopeElement instanceof Action)
            {
                GenericValidator.validateType(inputObject, ActionState.class,
                    ValidationConstants.STATE_FIELD, simpleNode.getState(),
                    msgCtx);
            }
            else if (correspondingScopeElement instanceof ChangeRequest)
            {
                GenericValidator.validateType(inputObject,
                    ChangeRequestState.class, ValidationConstants.STATE_FIELD,
                    simpleNode.getState(), msgCtx);
            }
            else if (correspondingScopeElement instanceof Defect)
            {
                GenericValidator.validateType(inputObject, DefectState.class,
                    ValidationConstants.STATE_FIELD, simpleNode.getState(),
                    msgCtx);
            }
            else if (correspondingScopeElement instanceof Issue)
            {
                GenericValidator.validateType(inputObject, IssueState.class,
                    ValidationConstants.STATE_FIELD, simpleNode.getState(),
                    msgCtx);
            }
            else if (correspondingScopeElement instanceof Requirement)
            {
                GenericValidator.validateType(inputObject,
                    RequirementState.class, ValidationConstants.STATE_FIELD,
                    simpleNode.getState(), msgCtx);
            }
            else if (correspondingScopeElement instanceof Risk)
            {
                GenericValidator.validateType(inputObject, RiskState.class,
                    ValidationConstants.STATE_FIELD, simpleNode.getState(),
                    msgCtx);
            }
            else if (correspondingScopeElement instanceof ServiceRequest)
            {
                GenericValidator.validateType(inputObject,
                    ServiceRequestState.class, ValidationConstants.STATE_FIELD,
                    simpleNode.getState(), msgCtx);
            }
            else
            // If we get here, (correspondingScopeElement == null)
            // task.state must be of type DeliverableState
            // when the SummaryTask is not a transferred scope element
            {
                GenericValidator
                .validateType(inputObject, TaskState.class,
                    ValidationConstants.STATE_FIELD, simpleNode.getState(),
                    msgCtx);
            }
        }
    }

    private void checkScheduleDates(MessageContext msgCtx, SimpleNode simpleNode, WorkElementScope scope)
    {
        if (scope != null && scope.isScheduleDates())
        {
            // the call to workElement.setModified does not set
            // the schedule date fields to modified, we must do it here
            
        	if (simpleNode.testActualDateModified())
        	{
        		simpleNode.getActualDate().setModified();
        	}

        	if (simpleNode.getProposedDate() != null && simpleNode.testProposedDateModified())
        	{
        		simpleNode.getProposedDate().setModified();
        	}

        	if (simpleNode.testPlanDateModified())
        	{
        		simpleNode.getPlanDate().setModified();
        	}

        	if (simpleNode.testForecastDateModified())
        	{
        		simpleNode.getForecastDate().setModified();
        	}

            Calendar planStartDate = (simpleNode.getPlanDate() == null ? null
                    : simpleNode.getPlanDate().getStartDate());

            Calendar proposedStartDate = (simpleNode.getProposedDate() == null
                    ? null : simpleNode.getProposedDate().getStartDate());

            int planDurationInMinutes = (simpleNode.getPlanDate() == null ? 0
                    : (simpleNode.getPlanDate().getDurationInMinutes() == null
                            ? 0 : simpleNode.getPlanDate()
                                .getDurationInMinutes().intValue()));

            Calendar actualStartDate = (simpleNode.getActualDate() == null
                    ? null : simpleNode.getActualDate().getStartDate());

            int actualDurationInMinutes = (simpleNode.getActualDate() == null
                    ? 0
                    : (simpleNode.getActualDate().getDurationInMinutes() == null
                            ? 0 : simpleNode.getActualDate()
                                .getDurationInMinutes().intValue()));

            int forecastDurationInMinutes = (simpleNode.getForecastDate() == null
                    ? 0
                    : (simpleNode.getForecastDate().getDurationInMinutes() == null
                            ? 0 : simpleNode.getForecastDate()
                                .getDurationInMinutes().intValue()));
            
            int proposedDurationInMinutes = (simpleNode.getProposedDate() == null
                    ? 0
                    : (simpleNode.getProposedDate().getDurationInMinutes() == null
                            ? 0 : simpleNode.getProposedDate()
                                .getDurationInMinutes().intValue()));

            Object[] loadedValues = loadWbsScheduleDatesFromDB(simpleNode, msgCtx); 
            WbsScheduleDate actualDate = (WbsScheduleDate)loadedValues[0];
            WbsScheduleDate forecastDate = (WbsScheduleDate)loadedValues[1];
            WbsScheduleDate planDate = (WbsScheduleDate)loadedValues[2];
            WbsScheduleDate proposedDate = (WbsScheduleDate)loadedValues[3];

            
            verifyActualDate(msgCtx, simpleNode, planStartDate, planDurationInMinutes, actualStartDate, actualDurationInMinutes, actualDate);

            verifyForecastDate(msgCtx, simpleNode, planDurationInMinutes, actualStartDate, actualDurationInMinutes, forecastDurationInMinutes, forecastDate);

            verifyPlanDate(msgCtx, simpleNode, planStartDate, planDurationInMinutes, actualStartDate, planDate);

            verifyProposedDate(msgCtx, simpleNode, planStartDate, proposedStartDate, proposedDurationInMinutes, proposedDate);
        }
    }

    private void verifyProposedDate(MessageContext msgCtx, SimpleNode simpleNode, Calendar planStartDate, Calendar proposedStartDate, int proposedDurationInMinutes, WbsScheduleDate proposedDate)
    {
        /***********************************************************************
         * Validation of proposedDate fields
         **********************************************************************/
        if (simpleNode.getProposedDate() != null)
        {
            if (simpleNode.getProposedDate().testStartDateModified())
            {
                // proposedDate.startDate read-only once work has been assigned
                boolean validProposedStart = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_START_FIELD,
                    msgCtx);

                // proposedDate.startDate read-only when planDate.startDate is not 
                // null unless they were both entered at the same time
                if (simpleNode.getPlanDate() != null
                        && !simpleNode.getPlanDate()
                            .testStartDateModified())
                {
                    validProposedStart &= GenericValidator
                        .validateReadOnlyConditionExists(
                            simpleNode,
                            ValidationConstants.SCHEDULE_DATE_PROPOSED_START_FIELD,
                            ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                            planStartDate, msgCtx);
                }

                if (validProposedStart)
                {
                    // proposedDate.startDate cannot be cleared on update
                    GenericValidator.validateDefaultValueUpdate(simpleNode,
                        ValidationConstants.SCHEDULE_DATE_PROPOSED_START_FIELD,
                        proposedStartDate, msgCtx);
                }
                else
                {
                    // reset read-only field value
                    simpleNode.getProposedDate().setStartDate(
                        proposedDate == null ? null
                                : proposedDate
                                    .getStartDate());
                }
            }

            if (simpleNode.getProposedDate()
                .testDurationInMinutesModified())
            {
                // proposedDate.durationInMinutes read-only if work has been assigned
                boolean validProposedDuration = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_DURATION_FIELD,
                    msgCtx);

                // proposedDate.durationInMinutes read-only when planDate.startDate 
                // is not null unless they were both entered at the same time
                if (simpleNode.getPlanDate() != null
                        && !simpleNode.getPlanDate()
                            .testStartDateModified())
                {
                    validProposedDuration &= GenericValidator
                        .validateReadOnlyConditionExists(
                            simpleNode,
                            ValidationConstants.SCHEDULE_DATE_PROPOSED_DURATION_FIELD,
                            ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                            planStartDate, msgCtx);
                }

                if (validProposedDuration)
                {
                    GenericValidator.validateDurationQuarterHour(
                        simpleNode,
                        proposedDurationInMinutes,
                        ValidationConstants.SCHEDULE_DATE_PROPOSED_DURATION_FIELD,
                        msgCtx);
                }
                else
                {
                    // reset read-only field value
                    simpleNode.getProposedDate().setDurationInMinutes(
                        proposedDate == null ? null
                                : proposedDate
                                    .getDurationInMinutes());
                }
            }

            // proposedDate.finishDate is read-only if it has work assigned
            // it can be set, unlike startDate in build 116
            if (simpleNode.getProposedDate().testFinishDateModified())
            {
                boolean valid = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_FINISH_FIELD,
                    msgCtx);
                
                if(!valid)
                {
                    // reset read-only field value
                    simpleNode.getProposedDate().setFinishDate(
                        proposedDate == null ? null
                                : proposedDate
                                    .getFinishDate());
                }
            }

            // proposedDate.hours is read-only when work is assigned 
            if (simpleNode.getProposedDate().testHoursModified())
            {
                boolean valid = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_HOURS_FIELD,
                    msgCtx);
                
                if(!valid) 
                {
                    // reset read-only field value
                    simpleNode.getProposedDate().setHours(
                        proposedDate == null ? null
                                : proposedDate
                                    .getHours());
                }
            }

            // proposedDate.days is read-only when work is assigned 
            if (simpleNode.getProposedDate().testDaysModified())
            {
                boolean valid = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_DAYS_FIELD,
                    msgCtx);
                
                if(!valid)
                {
                    // reset read-only field value
                    simpleNode.getProposedDate()
                        .setDays(
                            proposedDate == null ? null
                                    : proposedDate
                                        .getDays());
                } 
            }

            // proposedDate.workHours is always read-only
            if (simpleNode.getProposedDate().testWorkHoursModified())
            {
                GenericValidator
                    .validateReadOnly(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_PROPOSED_WORK_HOURS_FIELD,
                        msgCtx);

                // reset read-only field value
                simpleNode.getProposedDate().setWorkHours(
                    proposedDate == null ? null
                            : proposedDate
                                .getWorkHours());
            }
        }
    }

    private void verifyPlanDate(MessageContext msgCtx, SimpleNode simpleNode, Calendar planStartDate, int planDurationInMinutes, Calendar actualStartDate, WbsScheduleDate planDate)
    {
        /***********************************************************************
         * Validation of planDate fields
         **********************************************************************/
        if (simpleNode.getPlanDate() != null)
        {
            if (simpleNode.getPlanDate().testStartDateModified())
            {
                // planDate.startDate read-only once work has been assigned
                boolean validPlanStart = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                    msgCtx);

                // planDate.startDate read-only when actualDate.startDate is not null
                // unless they were both modified at the same time
                if (simpleNode.getActualDate() != null
                        && !simpleNode.getActualDate()
                            .testStartDateModified())
                {
                    validPlanStart &= GenericValidator
                        .validateReadOnlyConditionExists(
                            simpleNode,
                            ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                            ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                            actualStartDate, msgCtx);
                }

                if (validPlanStart)
                {
                    // planDate.startDate cannot be cleared on update
                    GenericValidator.validateDefaultValueUpdate(simpleNode,
                        ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                        planStartDate, msgCtx);
                }
                else
                {
                    // reset read-only field value
                    simpleNode.getPlanDate().setStartDate(
                        planDate == null ? null
                                : planDate
                                    .getStartDate());
                }
            }

            if (simpleNode.getPlanDate().testDurationInMinutesModified())
            {
                // planDate.durationInMinutes read-only if work has been assigned
                boolean validPlanDuration = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                    msgCtx);

                // planDate.durationInMinutes read-only when planDate.startDate is null
                validPlanDuration = GenericValidator
                    .validateReadOnlyConditionMissing(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                        ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                        planStartDate, msgCtx);

                // planDate.durationInMinutes read-only when actualDate.startDate 
                // is not null unless they were both entered at the same time
                if (simpleNode.getActualDate() != null
                        && !simpleNode.getActualDate()
                            .testStartDateModified())
                {
                    validPlanDuration = GenericValidator
                        .validateReadOnlyConditionExists(
                            simpleNode,
                            ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                            ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                            actualStartDate, msgCtx);
                }

                if (validPlanDuration)
                {
                    GenericValidator.validateDurationQuarterHour(
                        simpleNode,
                        planDurationInMinutes,
                        ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                        msgCtx);
                }
                else
                {
                    // reset read-only field value
                    simpleNode.getPlanDate().setDurationInMinutes(
                        planDate == null ? null
                                : planDate
                                    .getDurationInMinutes());
                }

            }

            // planDate.finishDate is always read-only (calculated from
            // planDate.startDate and planDate.durationInMinutes
            if (simpleNode.getPlanDate().testFinishDateModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PLAN_FINISH_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getPlanDate().setFinishDate(
                    planDate == null ? null
                            : planDate
                                .getFinishDate());
            }

            // planDate.hours is always read-only 
            if (simpleNode.getPlanDate().testHoursModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PLAN_HOURS_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getPlanDate().setHours(
                    planDate == null ? null
                            : planDate.getHours());
            }

            // planDate.days is always read-only
            if (simpleNode.getPlanDate().testDaysModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_PLAN_DAYS_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getPlanDate().setDays(
                    planDate == null ? null
                            : planDate.getDays());
            }

            // planDate.workHours is always read-only
            if (simpleNode.getPlanDate().testWorkHoursModified())
            {
                GenericValidator
                    .validateReadOnly(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_PLAN_WORK_HOURS_FIELD,
                        msgCtx);

                // reset read-only field value
                simpleNode.getPlanDate().setWorkHours(
                    planDate == null ? null
                            : planDate
                                .getWorkHours());
            }
        }
    }

    private void verifyForecastDate(MessageContext msgCtx, SimpleNode simpleNode, int planDurationInMinutes, Calendar actualStartDate, int actualDurationInMinutes, int forecastDurationInMinutes, WbsScheduleDate forecastDate)
    {
        /***********************************************************************
         * Validation of forecastDate fields
         **********************************************************************/
        if (simpleNode.getForecastDate() != null)
        {
            if (simpleNode.getForecastDate()
                .testDurationInMinutesModified())
            {
                // forecastDate.durationInMinutes read-only if work has been assigned
                boolean forecastDurationValid = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                    msgCtx);

                // forecastDate.durationInMinutes read-only if 
                // actualDate.durationInMinutes = planDate.durationInMinutes
                if (planDurationInMinutes != 0)
                {
                    forecastDurationValid &= GenericValidator
                        .validateReadOnlyConditionalFieldsSame(
                            simpleNode,
                            ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                            ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                            ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                            actualDurationInMinutes, planDurationInMinutes,
                            msgCtx);
                }

                // forecastDate.durationInMinutes read-only if 
                // actualDate.startDate is null
                forecastDurationValid &= GenericValidator
                    .validateReadOnlyConditionMissing(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                        actualStartDate, msgCtx);

                if (forecastDurationValid)
                {
                    GenericValidator.validateDurationQuarterHour(
                        simpleNode,
                        forecastDurationInMinutes,
                        ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                        msgCtx);
                    
                    // forecastDate.durationInMinutes must be >= to 
                    // actualDate.durationInMinutes
                    GenericValidator
                        .validateGreaterThanOrEqual(
                            simpleNode,
                            ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                            new Integer(forecastDurationInMinutes),
                            ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                            new Integer(actualDurationInMinutes), msgCtx);
                }
                else
                {
                    // reset read-only field value
                    simpleNode.getForecastDate().setDurationInMinutes(
                        forecastDate == null ? null
                                : forecastDate
                                    .getDurationInMinutes());
                }

            }

            // forecastDate.finishDate is always read-only (calculated from
            // forecastDate.startDate and forecastDate.durationInMinutes
            if (simpleNode.getForecastDate().testFinishDateModified())
            {
                GenericValidator
                    .validateReadOnly(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_FORECAST_FINISH_FIELD,
                        msgCtx);

                // reset read-only field value
                simpleNode.getForecastDate().setFinishDate(
                    forecastDate == null ? null
                            : forecastDate
                                .getFinishDate());
            }

            // forecastDate.startDate is always read-only
            if (simpleNode.getForecastDate().testStartDateModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_FORECAST_START_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getForecastDate().setStartDate(
                    forecastDate == null ? null
                            : forecastDate
                                .getStartDate());
            }

            // forecastDate.hours is always read-only 
            if (simpleNode.getForecastDate().testHoursModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_FORECAST_HOURS_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getForecastDate().setHours(
                    forecastDate == null ? null
                            : forecastDate
                                .getHours());
            }

            // forecastDate.days is always read-only
            if (simpleNode.getForecastDate().testDaysModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_FORECAST_DAYS_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getForecastDate()
                    .setDays(
                        forecastDate == null ? null
                                : forecastDate
                                    .getDays());
            }

            // forecastDate.workHours is always read-only
            if (simpleNode.getForecastDate().testWorkHoursModified())
            {
                GenericValidator
                    .validateReadOnly(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_FORECAST_WORK_HOURS_FIELD,
                        msgCtx);

                // reset read-only field value
                simpleNode.getForecastDate().setWorkHours(
                    forecastDate == null ? null
                            : forecastDate
                                .getWorkHours());
            }
        }
    }

    private void verifyActualDate(MessageContext msgCtx, SimpleNode simpleNode, Calendar planStartDate, int planDurationInMinutes, Calendar actualStartDate, int actualDurationInMinutes, WbsScheduleDate actualDate)
    {
        /***********************************************************************
         * Validation of actualDate fields
         **********************************************************************/
        if (simpleNode.getActualDate() != null)
        {
            if (simpleNode.getActualDate().testStartDateModified())
            {
                // actualDate.startDate read-only if work has been assigned
                boolean actualStartValid = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                    msgCtx);

                // actualDate.startDate read-only if planDate.startDate is null
                actualStartValid &= GenericValidator
                    .validateReadOnlyConditionMissing(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                        ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                        planStartDate, msgCtx);

                // actualDate.startDate field read-only if planDate.durationInMinutes is 0
                actualStartValid &= GenericValidator
                    .validateReadOnlyConditionIsZero(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                        ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                        planDurationInMinutes, msgCtx);

                // actualDate.startDate read-only if actualDate.durationInMinutes = 
                // planDate.DurationInMinutes
                if (planDurationInMinutes != 0)
                {
                    actualStartValid &= GenericValidator
                        .validateReadOnlyConditionalFieldsSame(
                            simpleNode,
                            ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                            ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                            ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                            actualDurationInMinutes, planDurationInMinutes,
                            msgCtx);
                }

                if (!actualStartValid)
                {
                    // reset read-only field value
                    simpleNode.getActualDate().setStartDate(
                        actualDate == null ? null
                                : actualDate
                                    .getStartDate());
                }
            }

            if (simpleNode.getActualDate().testDurationInMinutesModified())
            {
                // actualDate.durationInMinutes read-only work has been assigned
                boolean actualDurationValid = validateReadOnlyWorkAssigned(
                    simpleNode,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    msgCtx);

                // actualDate.durationInMinutes read-only if planDate.startDate is null
                actualDurationValid &= GenericValidator
                    .validateReadOnlyConditionMissing(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                        ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                        planStartDate, msgCtx);

                // actualDate.durationInMinutes read-only if planDate.durationInMinutes is 0
                actualDurationValid &= GenericValidator
                    .validateReadOnlyConditionIsZero(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                        ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                        planDurationInMinutes, msgCtx);

                // actualDate.durationInMinutes read-only if actualDate.startDate is null
                actualDurationValid &= GenericValidator
                    .validateReadOnlyConditionMissing(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                        actualStartDate, msgCtx);
                
                // actualDate.durationInMinutes is read-only if planDate.startDate is NOT null.
                if(simpleNode.getPlanDate().getStartDate() != null)
                {
                    String[] errorParams =
                    {
                            StringUtil.getShortClassName(simpleNode.getClass()) + " "
                                    + ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD, ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD
                    };
                    RPMException e = new RPMException(com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_VALUE_EXISTS,
                        errorParams, simpleNode.getClass().getName(), ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                        ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD, simpleNode.getID());
                    e.setSeverity(SeverityLevel.Error);
                    addException(e, msgCtx);
                    
                    actualDurationValid = false;
                }
            

                if (actualDurationValid)
                {
                    GenericValidator.validateDurationQuarterHour(
                        simpleNode,
                        actualDurationInMinutes,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                        msgCtx);
                    
                    // actualDate.durationInMinutes must be <= planDate.durationInMinutes
                    if (planDurationInMinutes != 0)
                    {
                        GenericValidator
                            .validateLessThanOrEqual(
                                simpleNode,
                                ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                                actualDurationInMinutes,
                                ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                                planDurationInMinutes, msgCtx);
                    }
                }
                else
                {
                    // reset read-only field value
                    simpleNode.getActualDate().setDurationInMinutes(
                        actualDate == null ? null
                                : actualDate
                                    .getDurationInMinutes());
                }
            }

            // actualDate.finishDate is always read-only (calculated from
            // actualDate.startDate and actualDate.durationInMinutes
            if (simpleNode.getActualDate().testFinishDateModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_FINISH_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getActualDate().setFinishDate(
                    actualDate == null ? null
                            : actualDate
                                .getFinishDate());
            }

            // actualDate.hours is always read-only 
            if (simpleNode.getActualDate().testHoursModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_HOURS_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getActualDate().setHours(
                    actualDate == null ? null
                            : actualDate.getHours());
            }

            // actualDate.days is always read-only
            if (simpleNode.getActualDate().testDaysModified())
            {
                GenericValidator.validateReadOnly(simpleNode,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DAYS_FIELD,
                    msgCtx);

                // reset read-only field value
                simpleNode.getActualDate().setDays(
                    actualDate == null ? null
                            : actualDate.getDays());
            }

            // actualDate.workHours is always read-only
            if (simpleNode.getActualDate().testWorkHoursModified())
            {
                GenericValidator
                    .validateReadOnly(
                        simpleNode,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_WORK_HOURS_FIELD,
                        msgCtx);

                // reset read-only field value
                simpleNode.getActualDate().setWorkHours(
                    actualDate == null ? null
                            : actualDate
                                .getWorkHours());
            }
        }
    }
}
