/*
 * Created on 1-Jun-2006
 * Validation checkpoint for Workflow application methods.
 */
package com.ibm.rpm.workflow.checkpoints;

import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.workflow.constants.ErrorCodes;
import com.ibm.rpm.workflow.constants.ValidationConstants;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.util.WorkflowTypeUtil;
import com.ibm.rpm.workflow.util.WorkflowUtil;

public class WorkflowMethodCheckpoint extends AbstractCheckpoint
{
    protected static WorkflowMethodCheckpoint instance = new WorkflowMethodCheckpoint();

    public static WorkflowMethodCheckpoint getInstance()
    {
        return instance;
    }

    public void validateStartWorkflowArgs(WorkflowProcess process, String rank,
            RPMObject object, MessageContext messageContext)
            throws RPMException
    {
        // if process or the object or their IDs are null don't go further
        GenericValidator.validateMandatoryID("RPMObject", object,
            messageContext);
        GenericValidator.validateMandatoryID("WorkflowProcess", process,
            messageContext);

        if (messageContext.isSuccessful())
        {
            validateWorkflowGroupToRPMObject(ValidationConstants.PROCESS_START,
                process, rank, object, messageContext);

            if (messageContext.isSuccessful())
            {
                validateProjectAssociation(process, object,
                    ValidationConstants.PROCESS_START, messageContext);
                validateWorkflowProcessIsActive(
                    ValidationConstants.PROCESS_START, process, messageContext);
            }
        }
    }

    public void validateResumeWorkflowArgs(
            RunningWorkflowProcess runningProcess, MessageContext messageContext)
            throws RPMException
    {
        // the runningProcess and its ID must exist
        GenericValidator.validateMandatoryID("RunningWorkflowProcess",
            runningProcess, messageContext);

        if (messageContext.isSuccessful())
        {
            // need to reload the state in case the runningProcess was not
            // reloaded in between calls (ie. suspend then resume)
            WorkflowState state = WorkflowUtil.loadWorkflowProcessState(
                runningProcess, messageContext);

            if (!(state == WorkflowState.Suspended))
            {
                handleStateException(runningProcess, state,
                    ValidationConstants.PROCESS_RESUME, messageContext);
            }
        }
    }

    public void validateCancelWorkflowArgs(
            RunningWorkflowProcess runningProcess, MessageContext messageContext)
            throws RPMException
    {
        // the runningProcess and its ID must exist
        GenericValidator.validateMandatoryID("RunningWorkflowProcess",
            runningProcess, messageContext);

        if (messageContext.isSuccessful())
        {
            // need to reload the state in case the runningProcess was not
            // reloaded in between calls (ie. suspend then resume)
            WorkflowState state = WorkflowUtil.loadWorkflowProcessState(
                runningProcess, messageContext);

            if (state == WorkflowState.Canceled
                    || state == WorkflowState.Initial
                    || state == WorkflowState.Closed)
            {
                handleStateException(runningProcess, state,
                    ValidationConstants.PROCESS_CANCEL, messageContext);
            }
        }
    }

    public void validateSuspendWorkflowArgs(
            RunningWorkflowProcess runningProcess, MessageContext messageContext)
            throws RPMException
    {
        // the runningProcess and its ID must exist
        GenericValidator.validateMandatoryID("RunningWorkflowProcess",
            runningProcess, messageContext);

        if (messageContext.isSuccessful())
        {
            // need to reload the state in case the runningProcess was not
            // reloaded in between calls (ie. suspend then resume)
            WorkflowState state = WorkflowUtil.loadWorkflowProcessState(
                runningProcess, messageContext);

            if (!(state == WorkflowState.Open || state == WorkflowState.Hanged))
            {
                handleStateException(runningProcess, state,
                    ValidationConstants.PROCESS_SUSPEND, messageContext);
            }
        }
    }

    public final boolean canAssociateWorkflow(WorkflowProcess process,
            GenericProject genericProject, boolean associate,
            boolean isDefault, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canAssociateWorkflow(
                messageContext, process, genericProject, associate, isDefault).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }

    public void validateAssociateWorkflowArgs(WorkflowProcess process,
            GenericProject genericProject, boolean associate,
            boolean isDefault, MessageContext messageContext)
            throws RPMException
    {
        GenericValidator.validateMandatoryID("GenericProject", genericProject,
            messageContext);
        GenericValidator.validateMandatoryID("WorkflowProcess", process,
            messageContext);

        // can't set default without associate
        if (isDefault && !associate)
        {
            RPMException ex = new RPMException(
                ErrorCodes.WORKFLOW_DEFAULT_WITHOUT_ASSOCIATE,
                new String[]
                {
                        StringUtil.getShortClassName(process.getClass()),
                        process.getID(),
                        StringUtil.getShortClassName(genericProject.getClass()),
                        genericProject.getID()
                }, process.getClass().getName());
            addException(ex, messageContext);
        }

        if (messageContext.isSuccessful())
        {
            validateWorkflowProcessIsActive(
                ValidationConstants.PROCESS_ASSOCIATE, process, messageContext);
        }
    }

    private boolean validateWorkflowGroupToRPMObject(String desiredState,
            WorkflowProcess process, String rank, RPMObject object,
            MessageContext messageContext) throws RPMException
    {
        Integer workflowGroup = WorkflowTypeUtil.rankToWorkflowGroup(rank);
        if (object instanceof GenericProject
                && workflowGroup == WorkflowTypeUtil.WorkflowProjectGroup)
            return true;

        if ((object instanceof Deliverable || object instanceof SummaryTask || object instanceof WorkProduct)
                && workflowGroup == WorkflowTypeUtil.WorkflowDeliverableGroup)
            return true;

        if (object instanceof SimpleNode
                && workflowGroup == WorkflowTypeUtil.WorkflowTaskGroup)
            return true;

        if (object instanceof Defect
                && workflowGroup == WorkflowTypeUtil.WorkflowDefectsGroup)
            return true;

        if (object instanceof ChangeRequest
                && workflowGroup == WorkflowTypeUtil.WorkflowChangeRequestGroup)
            return true;

        if (object instanceof Risk
                && workflowGroup == WorkflowTypeUtil.WorkflowRiskGroup)
            return true;

        if (object instanceof Issue
                && workflowGroup == WorkflowTypeUtil.WorkflowIssueGroup)
            return true;

        if (object instanceof Requirement
                && workflowGroup == WorkflowTypeUtil.WorkflowRequirementGroup)
            return true;

        if (object instanceof ServiceRequest
                && workflowGroup == WorkflowTypeUtil.WorkflowServiceRequestsGroup)
            return true;

        if (object instanceof Action
                && workflowGroup == WorkflowTypeUtil.WorkflowActionItemGroup)
            return true;

        if (object instanceof AbstractDocument
                && workflowGroup == WorkflowTypeUtil.WorkflowDocumentGroup)
            return true;

        if (object instanceof NoteMinutes
                && workflowGroup == WorkflowTypeUtil.WorkflowNoteMinutesGroup)
            return true;

        String name = process.getName();
        if (name == null)
            name = WorkflowUtil.getProcessName(process, messageContext);

        RPMException ex = new RPMException(ErrorCodes.INVALID_WORKFLOW_PARAMS,
            new String[]
            {
                    desiredState, name,
                    StringUtil.getShortClassName(object.getClass())
            });
        addException(ex, messageContext);
        return false;
    }

    private boolean validateWorkflowProcessIsActive(String desiredState,
            WorkflowProcess process, MessageContext messageContext)
            throws RPMException
    {
        boolean active = false;

        if (process.getActive() != null)
        {
            active = process.getActive().booleanValue();
        }
        else
        {
            active = WorkflowUtil.isProcessActive(process, messageContext);
        }

        if (!active)
        {
            String name = process.getName();
            if (name == null)
            {
                name = WorkflowUtil.getProcessName(process, messageContext);
            }

            RPMException ex = new RPMException(ErrorCodes.INACTIVE_WORKFLOW,
                new String[]
                {
                        desiredState, name
                }, process.getClass().getName(),
                ValidationConstants.ACTIVE_FIELD, process.getID());
            addException(ex, messageContext);
        }

        return active;
    }

    private void validateProjectAssociation(WorkflowProcess process,
            RPMObject object, String processType, MessageContext messageContext)
            throws RPMException
    {
        // validate that the process is "associated" with the target object's project.
        if (!WorkflowUtil.isWorkflowAssociated(process, WorkflowUtil
            .loadAssociatedProjectID(object, messageContext), messageContext))
        {
            String name = process.getName() == null ? WorkflowUtil
                .getProcessName(process, messageContext) : process.getName();

            String[] errorParams =
            {
                    processType, name
            };
            RPMException e = new RPMException(
                ErrorCodes.WORKFLOW_PROCESS_NOT_ASSOCIATED, errorParams,
                process.getClass().getName());
            addException(e, messageContext);
        }
    }

    private void handleStateException(RunningWorkflowProcess runningProcess,
            WorkflowState currentState, String desiredState,
            MessageContext messageContext)
    {
        RPMException ex = new RPMException(ErrorCodes.INVALID_STATE_TRANSITION,
            new String[]
            {
                    desiredState, runningProcess.getID(),
                    currentState.getValue()
            }, runningProcess.getClass().getName());
        addException(ex, messageContext);
    }

    public final boolean canStartWorkflow(WorkflowProcess process,
            RunningWorkflowProcessScope scope, RPMObject object,
            MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canStartWorkflow(
                messageContext, process, object, scope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    public final boolean canSuspendWorkflow(
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canSuspendWorkflow(
                messageContext, runningProcess, scope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    public final boolean canCancelWorkflow(
            RunningWorkflowProcess runningProcess, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canCancelWorkflow(
                messageContext, runningProcess).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    public final boolean canResumeWorkflow(
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canResumeWorkflow(
                messageContext, runningProcess, scope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    
}