package com.ibm.rpm.xpathparser;

import java.lang.reflect.Field;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Stack;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContainerClassMap;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.metadata.constants.AdditionalInformation;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.metadata.model.Rule;

/**
 * @author gmeloche
 *  
 */
public class XPathUtil
{

    private static XPathUtil instance = null;

    /**
     * The relationship between two containers will be done on this field if
     * there is more than one field of the parent container compatible with the
     * child type
     */
    private static String DEFAULT_FIELD = "children";

    /**
     *  
     */
    private XPathUtil()
    {
        super();
    }

    public static XPathUtil getinstance()
    {

        if (instance == null)
        {
            instance = new XPathUtil();
        }

        return instance;
    }

    public Stack validateAndPrepareTableStack(Stack tableStack, int index)
            throws RPMException
    {

        Table table = (Table) tableStack.elementAt(index);

        String objectName = table.getObjectName();

        /*
         * Note: an object that begins with a @ is considered as a field. Else
         * if the name matches a container name, use the container else find a
         * field that matches the name
         */
        char firstChar = objectName.substring(0).charAt(0);
        boolean useFieldName = false;

        Class containerType = null;

        if (!(firstChar == '@'))
        {
            //check if this is a container
            containerType = ContainerClassMap.getInstance().getContainerClass(
                objectName);
        }
        else
        {
            //force field name
            objectName = objectName.substring(1);
            useFieldName = true;
        }

        if (containerType == null)
        {
            if (table.getParent() != null)
            {
                Class fieldType = getFieldType(objectName, table.getParent()
                    .getObjectName());
                table.getParent().setFieldName(objectName);
                table.setObjectType(fieldType);

                if (ContainerClassMap.getInstance().getContainerClass(
                    table.getObjectName()) == null)
                {
                    //field exist but container type is not supported for xpath
                    ExceptionUtil.createAndHandleRPMException(XPathUtil.class,
                        ErrorCodes.XPATH_UNSUPPORTED_FIELD_AS_CONTAINER,
                        new String[]
                        {
                                table.getParent().getObjectName(),
                                StringUtil.getShortClassName(objectName),
                                table.getObjectName()
                        });
                }
            }
            else if (index - 1 < 0)
            {
                int errorCode = ErrorCodes.XPATH_INVALID_CONTAINER_OR_FIELD;
                if (useFieldName)
                {
                    errorCode = ErrorCodes.XPATH_INVALID_FIELD;
                }

                //Cannot perform a loadFromXpath on container or field
                ExceptionUtil.createAndHandleRPMException(XPathUtil.class,
                    errorCode, new String[]
                    {
                        objectName
                    });
            }
        }
        else
        {
            table.setObjectType(containerType);
        }

        if (index - 1 >= 0)
        {
            validateAndPrepareTableStack(tableStack, index - 1);

            //Try to find the type by using the name of the parent object in
            // the stack

            Table nextTable = (Table) tableStack.elementAt(index - 1);
            if (table.getObjectType() == null)
            {
                Class fieldType = getFieldType(objectName, nextTable
                    .getObjectName());
                nextTable.setFieldName(objectName);
                table.setObjectType(fieldType);
            }

            validateJoinRelationship(nextTable, table);
        }

        ArrayList conditions = table.getConditions();
        if (conditions != null && !conditions.isEmpty())
        {
            Stack subTables = null;
            for (Iterator iter = conditions.iterator(); iter.hasNext();)
            {
                Condition condition = (Condition) iter.next();
                if (condition instanceof FieldCondition)
                {
                    validateFieldCondition((FieldCondition) condition, table);
                }
                else
                {
                    subTables = ((ContainerCondition) condition).getSubTables();
                    validateAndPrepareTableStack(subTables,
                        subTables.size() - 1);
                    Table subTable = (Table) subTables.peek();
                    validateSubTableJoinRelationship(table, subTable);
                }

            }

        }

        return tableStack;

    }

    private void validateFieldCondition(FieldCondition fieldCondition,
            Table table) throws RPMException
    {
        String fieldName = fieldCondition.getProperty();
        Class fieldType = getFieldType(fieldName, table.getObjectName());

        //validate fieldName in field condition
        if (RPMObject.class.isAssignableFrom(fieldType)
                || ((fieldCondition.getOperation() == XPathToStack.FUNC_ISNULL || fieldCondition
                    .getOperation() == XPathToStack.FUNC_ISNOTNULL) && (Boolean.class == fieldType || EnumeratedTypeUtil
                    .getInstance().isEnumeratedType(table.getObjectType(),
                        fieldType))))
        {
            ExceptionUtil.createAndHandleRPMException(XPathUtil.class,
                ErrorCodes.XPATH_UNSUPPORTED_OPERATION, new String[]
                {
                        fieldName,
                        StringUtil.getShortClassName(fieldType.getName())
                });
        }

        //validate that a timestamp exists if the field requires one
        Container metadataContainer = MetadataStore.getInstance()
            .getContainerByJavaClass(table.getObjectType());

        if (metadataContainer != null)
        {
            com.ibm.rpm.metadata.model.Field field = metadataContainer
                .getField(fieldName);

            if (field != null)
            {
                Rule rule = field
                    .getRule(AdditionalInformation.RULE_TYPE_VALIDATION);

                if (rule != null
                        && AdditionalInformation.VALIDATION_TO_SECOND
                            .equals(rule.getPredefined()))
                {
                    // If the metadata rule for this field specifies VALIDATION_TO_SECOND,
                    // the timestamp portion of the date must be provided. 
                    try
                    {
                        SimpleDateFormat dateFormatter = new SimpleDateFormat(
                            "yyyy-MM-dd HH:mm:ss");
                        Date date = dateFormatter.parse((StringUtil
                            .removeQuotationMarks((String) fieldCondition
                                .getValue())));
                        fieldCondition.setValue("'" + dateFormatter.format(date) + "'");
                    }
                    catch (ParseException e)
                    {
                        ExceptionUtil.createAndHandleRPMException(
                            XPathUtil.class,
                            ErrorCodes.XPATH_INVALID_DATE_FORMAT, new String[]
                            {
                                StringUtil.getShortClassName(metadataContainer.getType()),
                                fieldName
                            });
                    }
                }
            }
        }
    }

    private Class getFieldType(String fieldName, String containerName)
            throws RPMException
    {

        Class containerType = null;

        if (containerName != null)
        {
            //check if the parent is a valid container
            Class parentType = getValidXpathContainerType(containerName);

            if ("deleted".equalsIgnoreCase(fieldName)
                    || "parent".equalsIgnoreCase(fieldName)
                    || "class".equalsIgnoreCase(fieldName)
                    || "contextname".equalsIgnoreCase(fieldName))
            {
                ExceptionUtil.handleUnsupportedFieldInXpath(containerName,
                    fieldName);
            }

            containerType = ContainerFieldMap.getInstance().getFieldType(
                parentType, fieldName);

            if (containerType == null)
            {
                ExceptionUtil.createAndHandleRPMException(XPathUtil.class,
                    ErrorCodes.XPATH_NON_EXISTENT_FIELD, new String[]
                    {
                            fieldName, containerName
                    });
            }
        }

        return containerType;

    }

    public Class getValidXpathContainerType(String containerName)
            throws RPMException
    {

        Class containerType = ContainerClassMap.getInstance()
            .getContainerClass(containerName);

        if (containerType == null)
        {
            ExceptionUtil.createAndHandleRPMException(XPathUtil.class,
                ErrorCodes.XPATH_INVALID_CONTAINER_OR_FIELD, new String[]
                {
                    containerName
                });
        }

        return containerType;
    }

    public Integer retrieveTypeId(Table table) throws RPMException
    {
        if (table.getObjectType() == null && table.getObjectName() == null)
        {
            return null;
        }

        Integer typeId = null;

        Class containerType = table.getObjectType();

        if (containerType == null)
        {
            containerType = getValidXpathContainerType(table.getObjectName());
        }

        try
        {
            Field field = containerType.getField("TYPE_ID");

            int type = field.getInt(null);

            typeId = new Integer(type);
        }
        catch (NoSuchFieldException e)
        {
            //do nothing, the returned value will be null;
        }
        catch (Exception e)
        {
            RPMException ex;
            if (!(e instanceof RPMException))
            {
                ex = new RPMException(e.getMessage());
                Log logger = LogFactory.getLog(this.getClass());
                logger.error(ex);
            }
            else
            {
                ex = (RPMException) e;
            }
            throw ex;
        }

        return typeId;
    }

    private void validateJoinRelationship(Table nextTable, Table currentTable)
            throws RPMException
    {

        //in this case, the nextTable is the parent object

        if (nextTable.getFieldName() == null)
        {
            // caller did not specify a field name
            String fieldName = findRelationship(nextTable.getObjectType(),
                currentTable.getObjectType());
            if (fieldName != null)
            {
                nextTable.setFieldName(fieldName);
            }
            else
            {
                ExceptionUtil.createAndHandleRPMException(XPathUtil.class,
                    ErrorCodes.XPATH_INVALID_JOIN, new String[]
                    {
                            nextTable.getObjectName(),
                            currentTable.getObjectName()
                    });
            }

        }
    }

    private void validateSubTableJoinRelationship(Table currentTable,
            Table subTable) throws RPMException
    {
        //in this case, the currentTable is the parent object

        if (currentTable.getFieldName() == null)
        {
            // caller did not specify a field name
            String fieldName = findRelationship(currentTable.getObjectType(),
                subTable.getObjectType());
            if (fieldName != null)
            {
                currentTable.setFieldName(fieldName);
            }
            else
            {
                ExceptionUtil.createAndHandleRPMException(XPathUtil.class,
                    ErrorCodes.XPATH_INVALID_SUBJOIN, new String[]
                    {
                            currentTable.getObjectName(),
                            subTable.getObjectName()
                    });
            }

        }
    }

    private String findRelationship(Class parentType, Class childType)
            throws RPMException
    {

        String fieldName = null;

        HashMap fields = ContainerFieldMap.getInstance()
            .getFieldMap(parentType);

        Iterator iter = fields.keySet().iterator();
        String key = null;
        Class type = null;
        ArrayList fieldNames = new ArrayList();
        while (iter.hasNext())
        {
            key = (String) iter.next();
            type = (Class) fields.get(key);

            //Only fields of type RPMObject are evaluated
            if (RPMObject.class.isAssignableFrom(type)
                    && type.isAssignableFrom(childType))
            {
                fieldNames.add(key);
            }
        }

        if (fieldNames.size() == 1)
        {
            fieldName = (String) fieldNames.get(0);
        }
        else if (fieldNames.size() > 1)
        {
            if (fieldNames.contains(DEFAULT_FIELD.toLowerCase()))
            {
                fieldName = DEFAULT_FIELD.toLowerCase();
            }
            else
            {
                ExceptionUtil.createAndHandleRPMException(XPathUtil.class,
                    ErrorCodes.XPATH_MANDATORY_FIELD_NAME, new String[]
                    {
                            StringUtil.getShortClassName(childType),
                            StringUtil.getShortClassName(parentType),
                            fieldNames.toString()
                    });
            }
        }

        return fieldName;
    }
}
